@@ -1,4 +1,4 @@
-/* $Id: ai_grouplist.hpp 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: ai_grouplist.hpp 20563 2010-08-19 15:37:28Z yexo $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,15 +12,16 @@
 #ifndef AI_GROUPLIST_HPP
 #define AI_GROUPLIST_HPP
 
-#include "ai_abstractlist.hpp"
+#include "ai_list.hpp"
 
 /**
  * Creates a list of groups of which you are the owner.
  * @note Neither AIGroup::GROUP_ALL nor AIGroup::GROUP_DEFAULT is in this list.
  * @ingroup AIList
  */
-class AIGroupList : public AIAbstractList {
+class AIGroupList : public AIList {
 public:
+	/** Get the name of this class to identify it towards squirrel. */
 	static const char *GetClassName() { return "AIGroupList"; }
 	AIGroupList();
 };
