@@ -1,4 +1,4 @@
-/* $Id: ai_marine.hpp 20430 2010-08-09 21:48:19Z rubidium $ */
+/* $Id: ai_marine.hpp 21888 2011-01-22 10:33:16Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,7 +12,6 @@
 #ifndef AI_MARINE_HPP
 #define AI_MARINE_HPP
 
-#include "ai_object.hpp"
 #include "ai_error.hpp"
 
 /**
@@ -20,6 +19,7 @@
  */
 class AIMarine : public AIObject {
 public:
+	/** Get the name of this class to identify it towards squirrel. */
 	static const char *GetClassName() { return "AIMarine"; }
 
 	/**
@@ -37,9 +37,9 @@ public:
 	 * Types of water-related objects in the game.
 	 */
 	enum BuildType {
-		BT_DOCK,  //!< Build a dock
-		BT_DEPOT, //!< Build a ship depot
-		BT_BUOY,  //!< Build a buoy
+		BT_DOCK,  ///< Build a dock
+		BT_DEPOT, ///< Build a ship depot
+		BT_BUOY,  ///< Build a buoy
 	};
 
 	/**
