@@ -1,4 +1,4 @@
-/* $Id: ai_townlist.hpp 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: ai_townlist.hpp 20563 2010-08-19 15:37:28Z yexo $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,14 +12,15 @@
 #ifndef AI_TOWNLIST_HPP
 #define AI_TOWNLIST_HPP
 
-#include "ai_abstractlist.hpp"
+#include "ai_list.hpp"
 
 /**
  * Creates a list of towns that are currently on the map.
  * @ingroup AIList
  */
-class AITownList : public AIAbstractList {
+class AITownList : public AIList {
 public:
+	/** Get the name of this class to identify it towards squirrel. */
 	static const char *GetClassName() { return "AITownList"; }
 	AITownList();
 };
