@@ -1,20 +1,25 @@
-/* $Id: ai_vehicle.hpp 16246 2009-05-06 22:31:52Z rubidium $ */
+/* $Id: ai_vehicle.hpp 22995 2011-10-04 20:22:46Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file ai_vehicle.hpp Everything to query and build vehicles. */
 
 #ifndef AI_VEHICLE_HPP
 #define AI_VEHICLE_HPP
 
-#include "ai_object.hpp"
-#include "ai_error.hpp"
 #include "ai_road.hpp"
-#include "ai_order.hpp"
 
 /**
  * Class that handles all vehicle related functions.
  */
 class AIVehicle : public AIObject {
 public:
+	/** Get the name of this class to identify it towards squirrel. */
 	static const char *GetClassName() { return "AIVehicle"; }
 
 	/**
@@ -25,41 +30,43 @@ public:
 		ERR_VEHICLE_BASE = AIError::ERR_CAT_VEHICLE << AIError::ERR_CAT_BIT_SIZE,
 
 		/** Too many vehicles in the game, can't build any more. */
-		ERR_VEHICLE_TOO_MANY,                   // [STR_00E1_TOO_MANY_VEHICLES_IN_GAME]
+		ERR_VEHICLE_TOO_MANY,                   // [STR_ERROR_TOO_MANY_VEHICLES_IN_GAME]
 
 		/** Vehicle is not available */
-		ERR_VEHICLE_NOT_AVAILABLE,              // [STR_AIRCRAFT_NOT_AVAILABLE, STR_ROAD_VEHICLE_NOT_AVAILABLE, STR_SHIP_NOT_AVAILABLE, STR_RAIL_VEHICLE_NOT_AVAILABLE]
+		ERR_VEHICLE_NOT_AVAILABLE,              // [STR_ERROR_AIRCRAFT_NOT_AVAILABLE, STR_ERROR_ROAD_VEHICLE_NOT_AVAILABLE, STR_ERROR_SHIP_NOT_AVAILABLE, STR_ERROR_RAIL_VEHICLE_NOT_AVAILABLE]
 
 		/** Vehicle can't be build due to game settigns */
-		ERR_VEHICLE_BUILD_DISABLED,             // [STR_A008_CAN_T_BUILD_AIRCRAFT, STR_980D_CAN_T_BUILD_SHIP, STR_9009_CAN_T_BUILD_ROAD_VEHICLE, STR_882B_CAN_T_BUILD_RAILROAD_VEHICLE]
+		ERR_VEHICLE_BUILD_DISABLED,             // [STR_ERROR_CAN_T_BUY_TRAIN, STR_ERROR_CAN_T_BUY_ROAD_VEHICLE, STR_ERROR_CAN_T_BUY_SHIP, STR_ERROR_CAN_T_BUY_AIRCRAFT]
 
 		/** Vehicle can't be build in the selected depot */
-		ERR_VEHICLE_WRONG_DEPOT,                // [STR_DEPOT_WRONG_DEPOT_TYPE]
+		ERR_VEHICLE_WRONG_DEPOT,                // [STR_ERROR_DEPOT_WRONG_DEPOT_TYPE]
 
 		/** Vehicle can't return to the depot */
-		ERR_VEHICLE_CANNOT_SEND_TO_DEPOT,       // [STR_8830_CAN_T_SEND_TRAIN_TO_DEPOT, STR_9018_CAN_T_SEND_VEHICLE_TO_DEPOT, STR_9819_CAN_T_SEND_SHIP_TO_DEPOT, STR_A012_CAN_T_SEND_AIRCRAFT_TO]
+		ERR_VEHICLE_CANNOT_SEND_TO_DEPOT,       // [STR_ERROR_CAN_T_SEND_TRAIN_TO_DEPOT, STR_ERROR_CAN_T_SEND_ROAD_VEHICLE_TO_DEPOT, STR_ERROR_CAN_T_SEND_SHIP_TO_DEPOT, STR_ERROR_CAN_T_SEND_AIRCRAFT_TO_HANGAR]
 
 		/** Vehicle can't start / stop */
-		ERR_VEHICLE_CANNOT_START_STOP,          // [STR_883B_CAN_T_STOP_START_TRAIN, STR_9015_CAN_T_STOP_START_ROAD_VEHICLE, STR_9818_CAN_T_STOP_START_SHIP, STR_A016_CAN_T_STOP_START_AIRCRAFT]
+		ERR_VEHICLE_CANNOT_START_STOP,          // [STR_ERROR_CAN_T_STOP_START_TRAIN, STR_ERROR_CAN_T_STOP_START_ROAD_VEHICLE, STR_ERROR_CAN_T_STOP_START_SHIP, STR_ERROR_CAN_T_STOP_START_AIRCRAFT]
 
 		/** Vehicle can't turn */
-		ERR_VEHICLE_CANNOT_TURN,                // [STR_8869_CAN_T_REVERSE_DIRECTION, STR_9033_CAN_T_MAKE_VEHICLE_TURN]
+		ERR_VEHICLE_CANNOT_TURN,                // [STR_ERROR_CAN_T_MAKE_ROAD_VEHICLE_TURN, STR_ERROR_CAN_T_REVERSE_DIRECTION_TRAIN, STR_ERROR_CAN_T_REVERSE_DIRECTION_RAIL_VEHICLE, STR_ERROR_CAN_T_REVERSE_DIRECTION_RAIL_VEHICLE_MULTIPLE_UNITS]
 
 		/** Vehicle can't be refit */
-		ERR_VEHICLE_CANNOT_REFIT,               // [STR_RAIL_CAN_T_REFIT_VEHICLE, STR_REFIT_ROAD_VEHICLE_CAN_T, STR_9841_CAN_T_REFIT_SHIP, STR_A042_CAN_T_REFIT_AIRCRAFT]
+		ERR_VEHICLE_CANNOT_REFIT,               // [STR_ERROR_CAN_T_REFIT_TRAIN, STR_ERROR_CAN_T_REFIT_ROAD_VEHICLE, STR_ERROR_CAN_T_REFIT_SHIP, STR_ERROR_CAN_T_REFIT_AIRCRAFT]
 
 		/** Vehicle is destroyed */
-		ERR_VEHICLE_IS_DESTROYED,               // [STR_CAN_T_REFIT_DESTROYED_VEHICLE, STR_CAN_T_SELL_DESTROYED_VEHICLE]
+		ERR_VEHICLE_IS_DESTROYED,               // [STR_ERROR_VEHICLE_IS_DESTROYED]
 
 		/** Vehicle is not in a depot */
-		ERR_VEHICLE_NOT_IN_DEPOT,               // [STR_A01B_AIRCRAFT_MUST_BE_STOPPED, STR_9013_MUST_BE_STOPPED_INSIDE, STR_TRAIN_MUST_BE_STOPPED, STR_980B_SHIP_MUST_BE_STOPPED_IN]
+		ERR_VEHICLE_NOT_IN_DEPOT,               // [STR_ERROR_AIRCRAFT_MUST_BE_STOPPED_INSIDE_HANGAR, STR_ERROR_ROAD_VEHICLE_MUST_BE_STOPPED_INSIDE_DEPOT, STR_ERROR_TRAIN_MUST_BE_STOPPED_INSIDE_DEPOT, STR_ERROR_SHIP_MUST_BE_STOPPED_INSIDE_DEPOT]
 
 		/** Vehicle is flying */
-		ERR_VEHICLE_IN_FLIGHT,                  // [STR_A017_AIRCRAFT_IS_IN_FLIGHT]
+		ERR_VEHICLE_IN_FLIGHT,                  // [STR_ERROR_AIRCRAFT_IS_IN_FLIGHT]
 
 		/** Vehicle is without power */
-		ERR_VEHCILE_NO_POWER,                   // [STR_TRAIN_START_NO_CATENARY]
+		ERR_VEHICLE_NO_POWER,                   // [STR_ERROR_TRAIN_START_NO_CATENARY]
 
+		/** Vehicle would get too long during construction. */
+		ERR_VEHICLE_TOO_LONG,                   // [STR_ERROR_TRAIN_TOO_LONG]
 	};
 
 	/**
@@ -68,31 +75,29 @@ public:
 	 */
 	enum VehicleType {
 		/* Order IS important, as it now matches the internal state of the game for vehicle type */
-		VT_RAIL,           //!< Rail type vehicle.
-		VT_ROAD,           //!< Road type vehicle (bus / truck).
-		VT_WATER,          //!< Water type vehicle.
-		VT_AIR,            //!< Air type vehicle.
-		VT_INVALID = 0xFF, //!< Invalid vehicle type.
-
-#ifdef DEFINE_SCRIPT_FILES
-		VEHICLE_INVALID = -1, //!< Invalid VehicleID.
-#endif /* DEFINE_SCRIPT_FILES */
+		VT_RAIL,           ///< Rail type vehicle.
+		VT_ROAD,           ///< Road type vehicle (bus / truck).
+		VT_WATER,          ///< Water type vehicle.
+		VT_AIR,            ///< Air type vehicle.
+		VT_INVALID = 0xFF, ///< Invalid vehicle type.
 	};
 
 	/**
 	 * The different states a vehicle can be in.
 	 */
 	enum VehicleState {
-		VS_RUNNING,        //!< The vehicle is currently running.
-		VS_STOPPED,        //!< The vehicle is stopped manually.
-		VS_IN_DEPOT,       //!< The vehicle is stopped in the depot.
-		VS_AT_STATION,     //!< The vehicle is stopped at a station and is currently loading or unloading.
-		VS_BROKEN,         //!< The vehicle has broken down and will start running again in a while.
-		VS_CRASHED,        //!< The vehicle is crashed (and will never run again).
+		VS_RUNNING,        ///< The vehicle is currently running.
+		VS_STOPPED,        ///< The vehicle is stopped manually.
+		VS_IN_DEPOT,       ///< The vehicle is stopped in the depot.
+		VS_AT_STATION,     ///< The vehicle is stopped at a station and is currently loading or unloading.
+		VS_BROKEN,         ///< The vehicle has broken down and will start running again in a while.
+		VS_CRASHED,        ///< The vehicle is crashed (and will never run again).
 
-		VS_INVALID = 0xFF, //!< An invalid vehicle state.
+		VS_INVALID = 0xFF, ///< An invalid vehicle state.
 	};
 
+	static const VehicleID VEHICLE_INVALID = 0xFFFFF; ///< Invalid VehicleID.
+
 	/**
 	 * Checks whether the given vehicle is valid and owned by you.
 	 * @param vehicle_id The vehicle to check.
@@ -295,7 +300,7 @@ public:
 	 * @param engine_id The engine to use for this vehicle.
 	 * @pre The tile at depot has a depot that can build the engine and
 	 *   is owned by you.
-	 * @pre IsValidEngine(engine_id).
+	 * @pre AIEngine::IsBuildable(engine_id).
 	 * @exception AIVehicle::ERR_VEHICLE_TOO_MANY
 	 * @exception AIVehicle::ERR_VEHICLE_BUILD_DISABLED
 	 * @exception AIVehicle::ERR_VEHICLE_WRONG_DEPOT
@@ -309,7 +314,7 @@ public:
 	static VehicleID BuildVehicle(TileIndex depot, EngineID engine_id);
 
 	/**
-	 * Clones a vehicle at the given depot, copying or cloning it's orders.
+	 * Clones a vehicle at the given depot, copying or cloning its orders.
 	 * @param depot The depot where the vehicle will be build.
 	 * @param vehicle_id The vehicle to use as example for the new vehicle.
 	 * @param share_orders Should the orders be copied or shared?
@@ -452,15 +457,6 @@ public:
 	 */
 	static bool StartStopVehicle(VehicleID vehicle_id);
 
-	/**
-	 * Skips the current order of the given vehicle.
-	 * @param vehicle_id The vehicle to skip the order for.
-	 * @param order_position The selected order to which we want to skip.
-	 * @pre IsValidVehicleOrder(vehicle_id, order_position).
-	 * @return True if and only if the order has been skipped.
-	 */
-	static bool SkipToVehicleOrder(VehicleID vehicle_id, AIOrder::OrderPosition order_position);
-
 	/**
 	 * Turn the given vehicle so it'll drive the other way.
 	 * @param vehicle_id The vehicle to turn.
@@ -525,6 +521,14 @@ public:
 	 */
 	static bool HasSharedOrders(VehicleID vehicle_id);
 
+	/**
+	 * Get the current reliability of a vehicle.
+	 * @param vehicle_id The vehicle to check.
+	 * @pre IsValidVehicle(vehicle_id).
+	 * @return The current reliability (0-100%).
+	 */
+	static int GetReliability(VehicleID vehicle_id);
+
 private:
 	/**
 	 * Internal function used by SellWagon(Chain).
