@@ -1,4 +1,4 @@
-/* $Id: bridge_gui.cpp 19928 2010-06-04 21:14:22Z rubidium $ */
+/* $Id: bridge_gui.cpp 22702 2011-07-30 17:48:23Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,8 +12,6 @@
 #include "stdafx.h"
 #include "gui.h"
 #include "command_func.h"
-#include "economy_func.h"
-#include "bridge.h"
 #include "rail.h"
 #include "strings_func.h"
 #include "window_func.h"
@@ -23,6 +21,9 @@
 #include "sortlist_type.h"
 #include "widgets/dropdown_func.h"
 #include "core/geometry_func.hpp"
+#include "cmd_helper.h"
+#include "tunnelbridge_map.h"
+#include "road_gui.h"
 
 #include "table/strings.h"
 
@@ -40,22 +41,36 @@ struct BuildBridgeData {
 	Money cost;
 };
 
-typedef GUIList<BuildBridgeData> GUIBridgeList;
+typedef GUIList<BuildBridgeData> GUIBridgeList; ///< List of bridges, used in #BuildBridgeWindow.
 
 /**
  * Callback executed after a build Bridge CMD has been called
  *
- * @param result Whether the build succeded
- * @param tile The tile where the command has been executed
- * @param p1 not used
- * @param p2 not used
+ * @param result Whether the build succeeded
+ * @param end_tile End tile of the bridge.
+ * @param p1 packed start tile coords (~ dx)
+ * @param p2 various bitstuffed elements
+ * - p2 = (bit  0- 7) - bridge type (hi bh)
+ * - p2 = (bit  8-11) - rail type or road types.
+ * - p2 = (bit 15-16) - transport type.
  */
-void CcBuildBridge(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2)
+void CcBuildBridge(const CommandCost &result, TileIndex end_tile, uint32 p1, uint32 p2)
 {
-	if (result.Succeeded()) SndPlayTileFx(SND_27_BLACKSMITH_ANVIL, tile);
+	if (result.Failed()) return;
+	SndPlayTileFx(SND_27_BLACKSMITH_ANVIL, end_tile);
+
+	TransportType transport_type = Extract<TransportType, 15, 2>(p2);
+
+	if (transport_type == TRANSPORT_ROAD) {
+		DiagDirection end_direction = ReverseDiagDir(GetTunnelBridgeDirection(end_tile));
+		ConnectRoadToStructure(end_tile, end_direction);
+
+		DiagDirection start_direction = ReverseDiagDir(GetTunnelBridgeDirection(p1));
+		ConnectRoadToStructure(p1, start_direction);
+	}
 }
 
-/* Names of the build bridge selection window */
+/** Names of the widgets of the build-bridge selection window. */
 enum BuildBridgeSelectionWidgets {
 	BBSW_CAPTION,
 	BBSW_DROPDOWN_ORDER,
@@ -64,11 +79,12 @@ enum BuildBridgeSelectionWidgets {
 	BBSW_SCROLLBAR,
 };
 
+/** Window class for handling the bridge-build GUI. */
 class BuildBridgeWindow : public Window {
 private:
 	/* Runtime saved values */
-	static uint16 last_size;
-	static Listing last_sorting;
+	static uint16 last_size;     ///< Last size of the bridge GUI window.
+	static Listing last_sorting; ///< Last setting of the sort.
 
 	/* Constants for sorting the bridges */
 	static const StringID sorter_names[];
@@ -80,6 +96,7 @@ private:
 	uint32 type;
 	GUIBridgeList *bridges;
 	int bridgetext_offset; ///< Horizontal offset of the text describing the bridge properties in #BBSW_BRIDGE_LIST relative to the left edge.
+	Scrollbar *vscroll;
 
 	/** Sort the bridges by their index */
 	static int CDECL BridgeIndexSorter(const BuildBridgeData *a, const BuildBridgeData *b)
@@ -131,6 +148,7 @@ public:
 		bridges(bl)
 	{
 		this->CreateNestedTree(desc);
+		this->vscroll = this->GetScrollbar(BBSW_SCROLLBAR);
 		/* Change the data, or the caption of the gui. Set it to road or rail, accordingly. */
 		this->GetWidget<NWidgetCore>(BBSW_CAPTION)->widget_data = (GB(this->type, 15, 2) == TRANSPORT_ROAD) ? STR_SELECT_ROAD_BRIDGE_CAPTION : STR_SELECT_RAIL_BRIDGE_CAPTION;
 		this->FinishInitNested(desc, GB(br_type, 15, 2)); // Initializes 'this->bridgetext_offset'.
@@ -141,14 +159,14 @@ public:
 		this->bridges->NeedResort();
 		this->SortBridgeList();
 
-		this->vscroll.SetCount(bl->Length());
-		if (this->last_size < this->vscroll.GetCapacity()) this->last_size = this->vscroll.GetCapacity();
-		if (this->last_size > this->vscroll.GetCount()) this->last_size = this->vscroll.GetCount();
+		this->vscroll->SetCount(bl->Length());
+		if (this->last_size < this->vscroll->GetCapacity()) this->last_size = this->vscroll->GetCapacity();
+		if (this->last_size > this->vscroll->GetCount()) this->last_size = this->vscroll->GetCount();
 		/* Resize the bridge selection window if we used a bigger one the last time. */
-		if (this->last_size > this->vscroll.GetCapacity()) {
-			ResizeWindow(this, 0, (this->last_size - this->vscroll.GetCapacity()) * this->resize.step_height);
+		if (this->last_size > this->vscroll->GetCapacity()) {
+			ResizeWindow(this, 0, (this->last_size - this->vscroll->GetCapacity()) * this->resize.step_height);
 		}
-		this->GetWidget<NWidgetCore>(BBSW_BRIDGE_LIST)->widget_data = (this->vscroll.GetCapacity() << MAT_ROW_START) + (1 << MAT_COL_START);
+		this->GetWidget<NWidgetCore>(BBSW_BRIDGE_LIST)->widget_data = (this->vscroll->GetCapacity() << MAT_ROW_START) + (1 << MAT_COL_START);
 	}
 
 	~BuildBridgeWindow()
@@ -158,17 +176,12 @@ public:
 		delete bridges;
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		switch (widget) {
 			case BBSW_DROPDOWN_ORDER: {
 				Dimension d = GetStringBoundingBox(this->GetWidget<NWidgetCore>(widget)->widget_data);
-				d.width += padding.width + WD_SORTBUTTON_ARROW_WIDTH * 2; // Doubled since the word is centered, also looks nice.
+				d.width += padding.width + WD_SORTBUTTON_ARROW_WIDTH * 2; // Doubled since the string is centred and it also looks better.
 				d.height += padding.height;
 				*size = maxdim(*size, d);
 				break;
@@ -193,7 +206,7 @@ public:
 					SetDParam(2, this->bridges->Get(i)->cost);
 					SetDParam(1, b->speed);
 					SetDParam(0, b->material);
-					text_dim = maxdim(text_dim, GetStringBoundingBox(STR_SELECT_BRIDGE_INFO));
+					text_dim = maxdim(text_dim, GetStringBoundingBox(_game_mode == GM_EDITOR ? STR_SELECT_BRIDGE_SCENEDIT_INFO : STR_SELECT_BRIDGE_INFO));
 				}
 				sprite_dim.height++; // Sprite is rendered one pixel down in the matrix field.
 				text_dim.height++; // Allowing the bottom row pixels to be rendered on the edge of the matrix field.
@@ -207,6 +220,16 @@ public:
 		}
 	}
 
+	virtual Point OnInitialPosition(const WindowDesc *desc, int16 sm_width, int16 sm_height, int window_number)
+	{
+		/* Position the window so hopefully the first bridge from the list is under the mouse pointer. */
+		NWidgetBase *list = this->GetWidget<NWidgetBase>(BBSW_BRIDGE_LIST);
+		Point corner; // point of the top left corner of the window.
+		corner.y = Clamp(_cursor.pos.y - list->pos_y - 5, GetMainViewTop(), GetMainViewBottom() - sm_height);
+		corner.x = Clamp(_cursor.pos.x - list->pos_x - 5, 0, _screen.width - sm_width);
+		return corner;
+	}
+
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		switch (widget) {
@@ -216,7 +239,7 @@ public:
 
 			case BBSW_BRIDGE_LIST: {
 				uint y = r.top;
-				for (int i = this->vscroll.GetPosition(); this->vscroll.IsVisible(i) && i < (int)this->bridges->Length(); i++) {
+				for (int i = this->vscroll->GetPosition(); this->vscroll->IsVisible(i) && i < (int)this->bridges->Length(); i++) {
 					const BridgeSpec *b = this->bridges->Get(i)->spec;
 
 					SetDParam(2, this->bridges->Get(i)->cost);
@@ -224,7 +247,8 @@ public:
 					SetDParam(0, b->material);
 
 					DrawSprite(b->sprite, b->pal, r.left + WD_MATRIX_LEFT, y + this->resize.step_height - 1 - GetSpriteSize(b->sprite).height);
-					DrawStringMultiLine(r.left + this->bridgetext_offset, r.right, y + 2, y + this->resize.step_height, STR_SELECT_BRIDGE_INFO);
+					DrawStringMultiLine(r.left + this->bridgetext_offset, r.right, y + 2, y + this->resize.step_height,
+							_game_mode == GM_EDITOR ? STR_SELECT_BRIDGE_SCENEDIT_INFO : STR_SELECT_BRIDGE_INFO);
 					y += this->resize.step_height;
 				}
 				break;
@@ -249,15 +273,13 @@ public:
 		switch (widget) {
 			default: break;
 			case BBSW_BRIDGE_LIST: {
-				uint i = ((int)pt.y - this->GetWidget<NWidgetBase>(BBSW_BRIDGE_LIST)->pos_y) / this->resize.step_height;
-				if (i < this->vscroll.GetCapacity()) {
-					i += this->vscroll.GetPosition();
-					if (i < this->bridges->Length()) {
-						this->BuildBridge(i);
-						delete this;
-					}
+				uint i = this->vscroll->GetScrolledRowFromWidget(pt.y, this, BBSW_BRIDGE_LIST);
+				if (i < this->bridges->Length()) {
+					this->BuildBridge(i);
+					delete this;
 				}
-			} break;
+				break;
+			}
 
 			case BBSW_DROPDOWN_ORDER:
 				this->bridges->ToggleSortOrder();
@@ -281,26 +303,26 @@ public:
 
 	virtual void OnResize()
 	{
-		this->vscroll.SetCapacityFromWidget(this, BBSW_BRIDGE_LIST);
-		this->GetWidget<NWidgetCore>(BBSW_BRIDGE_LIST)->widget_data = (this->vscroll.GetCapacity() << MAT_ROW_START) + (1 << MAT_COL_START);
+		this->vscroll->SetCapacityFromWidget(this, BBSW_BRIDGE_LIST);
+		this->GetWidget<NWidgetCore>(BBSW_BRIDGE_LIST)->widget_data = (this->vscroll->GetCapacity() << MAT_ROW_START) + (1 << MAT_COL_START);
 
-		this->last_size = max(this->vscroll.GetCapacity(), this->last_size);
+		this->last_size = max(this->vscroll->GetCapacity(), this->last_size);
 	}
 };
 
-/* Set the default size of the Build Bridge Window */
+/** Set the default size of the Build Bridge Window. */
 uint16 BuildBridgeWindow::last_size = 4;
-/* Set the default sorting for the bridges */
-Listing BuildBridgeWindow::last_sorting = {false, 0};
+/** Set the default sorting for the bridges */
+Listing BuildBridgeWindow::last_sorting = {true, 2};
 
-/* Availible bridge sorting functions */
+/** Available bridge sorting functions. */
 GUIBridgeList::SortFunction * const BuildBridgeWindow::sorter_funcs[] = {
 	&BridgeIndexSorter,
 	&BridgePriceSorter,
 	&BridgeSpeedSorter
 };
 
-/* Names of the sorting functions */
+/** Names of the sorting functions. */
 const StringID BuildBridgeWindow::sorter_names[] = {
 	STR_SORT_BY_NUMBER,
 	STR_SORT_BY_COST,
@@ -308,6 +330,7 @@ const StringID BuildBridgeWindow::sorter_names[] = {
 	INVALID_STRING_ID
 };
 
+/** Widgets of the bridge gui. */
 static const NWidgetPart _nested_build_bridge_widgets[] = {
 	/* Header */
 	NWidget(NWID_HORIZONTAL),
@@ -323,18 +346,18 @@ static const NWidgetPart _nested_build_bridge_widgets[] = {
 				NWidget(WWT_DROPDOWN, COLOUR_DARK_GREEN, BBSW_DROPDOWN_CRITERIA), SetFill(1, 0), SetDataTip(0x0, STR_TOOLTIP_SORT_CRITERIA),
 			EndContainer(),
 			/* Matrix. */
-			NWidget(WWT_MATRIX, COLOUR_DARK_GREEN, BBSW_BRIDGE_LIST), SetFill(1, 0), SetResize(0, 22), SetDataTip(0x401, STR_SELECT_BRIDGE_SELECTION_TOOLTIP),
+			NWidget(WWT_MATRIX, COLOUR_DARK_GREEN, BBSW_BRIDGE_LIST), SetFill(1, 0), SetResize(0, 22), SetDataTip(0x401, STR_SELECT_BRIDGE_SELECTION_TOOLTIP), SetScrollbar(BBSW_SCROLLBAR),
 		EndContainer(),
 
 		/* scrollbar + resize button */
 		NWidget(NWID_VERTICAL),
-			NWidget(WWT_SCROLLBAR, COLOUR_DARK_GREEN, BBSW_SCROLLBAR),
+			NWidget(NWID_VSCROLLBAR, COLOUR_DARK_GREEN, BBSW_SCROLLBAR),
 			NWidget(WWT_RESIZEBOX, COLOUR_DARK_GREEN),
 		EndContainer(),
 	EndContainer(),
 };
 
-/* Window definition for the rail bridge selection window */
+/** Window definition for the rail bridge selection window. */
 static const WindowDesc _build_bridge_desc(
 	WDP_AUTO, 200, 114,
 	WC_BUILD_BRIDGE, WC_BUILD_TOOLBAR,
@@ -384,11 +407,11 @@ void ShowBuildBridgeWindow(TileIndex start, TileIndex end, TransportType transpo
 	/* only query bridge building possibility once, result is the same for all bridges!
 	 * returns CMD_ERROR on failure, and price on success */
 	StringID errmsg = INVALID_STRING_ID;
-	CommandCost ret = DoCommand(end, start, type, DC_AUTO | DC_QUERY_COST, CMD_BUILD_BRIDGE);
+	CommandCost ret = DoCommand(end, start, type, CommandFlagsToDCFlags(GetCommandFlags(CMD_BUILD_BRIDGE)) | DC_QUERY_COST, CMD_BUILD_BRIDGE);
 
 	GUIBridgeList *bl = NULL;
 	if (ret.Failed()) {
-		errmsg = _error_message;
+		errmsg = ret.GetErrorMessage();
 	} else {
 		/* check which bridges can be built */
 		const uint tot_bridgedata_len = CalcBridgeLenCostFactor(bridge_len + 2);
@@ -397,7 +420,11 @@ void ShowBuildBridgeWindow(TileIndex start, TileIndex end, TransportType transpo
 
 		Money infra_cost = 0;
 		switch (transport_type) {
-			case TRANSPORT_ROAD: infra_cost = (bridge_len + 2) * _price[PR_BUILD_ROAD] * 2; break;
+			case TRANSPORT_ROAD:
+				infra_cost = (bridge_len + 2) * _price[PR_BUILD_ROAD] * 2;
+				/* In case we add a new road type as well, we must be aware of those costs. */
+				if (IsBridgeTile(start)) infra_cost *= CountBits(GetRoadTypes(start) | (RoadTypes)road_rail_type);
+				break;
 			case TRANSPORT_RAIL: infra_cost = (bridge_len + 2) * RailBuildCost((RailType)road_rail_type); break;
 			default: break;
 		}
@@ -420,6 +447,6 @@ void ShowBuildBridgeWindow(TileIndex start, TileIndex end, TransportType transpo
 		new BuildBridgeWindow(&_build_bridge_desc, start, end, type, bl);
 	} else {
 		delete bl;
-		ShowErrorMessage(STR_ERROR_CAN_T_BUILD_BRIDGE_HERE, errmsg, TileX(end) * TILE_SIZE, TileY(end) * TILE_SIZE);
+		ShowErrorMessage(STR_ERROR_CAN_T_BUILD_BRIDGE_HERE, errmsg, WL_INFO, TileX(end) * TILE_SIZE, TileY(end) * TILE_SIZE);
 	}
 }
