@@ -1,4 +1,4 @@
-/* $Id: command_type.h 19603 2010-04-11 10:57:21Z rubidium $ */
+/* $Id: command_type.h 22654 2011-07-10 20:35:31Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -25,30 +25,33 @@ class CommandCost {
 	Money cost;       ///< The cost of this action
 	StringID message; ///< Warning message for when success is unset
 	bool success;     ///< Whether the comment went fine up to this moment
+	uint textref_stack_size;   ///< Number of uint32 values to put on the #TextRefStack for the error message.
+
+	static uint32 textref_stack[16];
 
 public:
 	/**
 	 * Creates a command cost return with no cost and no error
 	 */
-	CommandCost() : expense_type(INVALID_EXPENSES), cost(0), message(INVALID_STRING_ID), success(true) {}
+	CommandCost() : expense_type(INVALID_EXPENSES), cost(0), message(INVALID_STRING_ID), success(true), textref_stack_size(0) {}
 
 	/**
 	 * Creates a command return value the is failed with the given message
 	 */
-	CommandCost(StringID msg) : expense_type(INVALID_EXPENSES), cost(0), message(msg), success(false) {}
+	explicit CommandCost(StringID msg) : expense_type(INVALID_EXPENSES), cost(0), message(msg), success(false), textref_stack_size(0) {}
 
 	/**
 	 * Creates a command cost with given expense type and start cost of 0
 	 * @param ex_t the expense type
 	 */
-	CommandCost(ExpensesType ex_t) : expense_type(ex_t), cost(0), message(INVALID_STRING_ID), success(true) {}
+	explicit CommandCost(ExpensesType ex_t) : expense_type(ex_t), cost(0), message(INVALID_STRING_ID), success(true), textref_stack_size(0) {}
 
 	/**
 	 * Creates a command return value with the given start cost and expense type
 	 * @param ex_t the expense type
 	 * @param cst the initial cost of this command
 	 */
-	CommandCost(ExpensesType ex_t, const Money &cst) : expense_type(ex_t), cost(cst), message(INVALID_STRING_ID), success(true) {}
+	CommandCost(ExpensesType ex_t, const Money &cst) : expense_type(ex_t), cost(cst), message(INVALID_STRING_ID), success(true), textref_stack_size(0) {}
 
 
 	/**
@@ -90,17 +93,8 @@ public:
 	}
 
 	/**
-	 * Sets the global error message *if* this class has one.
-	 */
-	FORCEINLINE void SetGlobalErrorMessage() const
-	{
-		extern StringID _error_message;
-		if (this->message != INVALID_STRING_ID) _error_message = this->message;
-	}
-
-	/**
-	 * Makes this CommandCost behave like an error command.
-	 * @param mesasge the error message.
+	 * Makes this #CommandCost behave like an error command.
+	 * @param message The error message.
 	 */
 	void MakeError(StringID message)
 	{
@@ -109,17 +103,34 @@ public:
 		this->message = message;
 	}
 
+	void UseTextRefStack(uint num_registers);
+
+	/**
+	 * Returns the number of uint32 values for the #TextRefStack of the error message.
+	 * @return number of uint32 values.
+	 */
+	uint GetTextRefStackSize() const
+	{
+		return this->textref_stack_size;
+	}
+
+	/**
+	 * Returns a pointer to the values for the #TextRefStack of the error message.
+	 * @return uint32 values for the #TextRefStack
+	 */
+	const uint32 *GetTextRefStack() const
+	{
+		return textref_stack;
+	}
+
 	/**
 	 * Returns the error message of a command
-	 * @return the error message, if succeeded INVALID_STRING_ID
+	 * @return the error message, if succeeded #INVALID_STRING_ID
 	 */
 	StringID GetErrorMessage() const
 	{
-		extern StringID _error_message;
-
 		if (this->success) return INVALID_STRING_ID;
-		if (this->message != INVALID_STRING_ID) return this->message;
-		return _error_message;
+		return this->message;
 	}
 
 	/**
@@ -151,7 +162,7 @@ public:
  *
  * @see _command_proc_table
  */
-enum {
+enum Commands {
 	CMD_BUILD_RAILROAD_TRACK,         ///< build a rail track
 	CMD_REMOVE_RAILROAD_TRACK,        ///< remove a rail track
 	CMD_BUILD_SINGLE_RAIL,            ///< build a single rail track
@@ -163,8 +174,7 @@ enum {
 	CMD_BUILD_SIGNALS,                ///< build a signal
 	CMD_REMOVE_SIGNALS,               ///< remove a signal
 	CMD_TERRAFORM_LAND,               ///< terraform a tile
-	CMD_PURCHASE_LAND_AREA,           ///< purchase a tile
-	CMD_SELL_LAND_AREA,               ///< sell a bought tile before
+	CMD_BUILD_OBJECT,                 ///< build an object
 	CMD_BUILD_TUNNEL,                 ///< build a tunnel
 
 	CMD_REMOVE_FROM_RAIL_STATION,     ///< remove a (rectangle of) tiles from a rail station
@@ -190,15 +200,16 @@ enum {
 
 	CMD_PLANT_TREE,                   ///< plant a tree
 
-	CMD_BUILD_RAIL_VEHICLE,           ///< build a rail vehicle
+	CMD_BUILD_VEHICLE,                ///< build a vehicle
+	CMD_SELL_VEHICLE,                 ///< sell a vehicle
+	CMD_REFIT_VEHICLE,                ///< refit the cargo space of a vehicle
+	CMD_SEND_VEHICLE_TO_DEPOT,        ///< send a vehicle to a depot
+
 	CMD_MOVE_RAIL_VEHICLE,            ///< move a rail vehicle (in the depot)
-
-	CMD_SELL_RAIL_WAGON,              ///< sell a rail wagon
-
-	CMD_SEND_TRAIN_TO_DEPOT,          ///< send a train to a depot
 	CMD_FORCE_TRAIN_PROCEED,          ///< proceed a train to pass a red signal
 	CMD_REVERSE_TRAIN_DIRECTION,      ///< turn a train around
 
+	CMD_CLEAR_ORDER_BACKUP,           ///< clear the order backup of a given user/tile
 	CMD_MODIFY_ORDER,                 ///< modify an order (like set full-load)
 	CMD_SKIP_TO_ORDER,                ///< skip an order to the next of specific one
 	CMD_DELETE_ORDER,                 ///< delete an order
@@ -208,9 +219,8 @@ enum {
 
 	CMD_BUILD_INDUSTRY,               ///< build a new industry
 
-	CMD_BUILD_COMPANY_HQ,             ///< build the company headquarter
 	CMD_SET_COMPANY_MANAGER_FACE,     ///< set the manager's face of the company
-	CMD_SET_COMPANY_COLOUR,            ///< set the colour of the company
+	CMD_SET_COMPANY_COLOUR,           ///< set the colour of the company
 
 	CMD_INCREASE_LOAN,                ///< increase the loan from the bank
 	CMD_DECREASE_LOAN,                ///< decrease the loan from the bank
@@ -222,20 +232,12 @@ enum {
 	CMD_RENAME_COMPANY,               ///< change the company name
 	CMD_RENAME_PRESIDENT,             ///< change the president name
 	CMD_RENAME_STATION,               ///< rename a station
-
-	CMD_SELL_AIRCRAFT,                ///< sell an aircraft
-	CMD_BUILD_AIRCRAFT,               ///< build an aircraft
-	CMD_SEND_AIRCRAFT_TO_HANGAR,      ///< send an aircraft to a hanger
-	CMD_REFIT_AIRCRAFT,               ///< refit the cargo space of an aircraft
+	CMD_RENAME_DEPOT,                 ///< rename a depot
 
 	CMD_PLACE_SIGN,                   ///< place a sign
 	CMD_RENAME_SIGN,                  ///< rename a sign
 
-	CMD_BUILD_ROAD_VEH,               ///< build a road vehicle
-	CMD_SELL_ROAD_VEH,                ///< sell a road vehicle
-	CMD_SEND_ROADVEH_TO_DEPOT,        ///< send a road vehicle to the depot
 	CMD_TURN_ROADVEH,                 ///< turn a road vehicle around
-	CMD_REFIT_ROAD_VEH,               ///< refit the cargo space of a road vehicle
 
 	CMD_PAUSE,                        ///< pause the game
 
@@ -246,11 +248,8 @@ enum {
 	CMD_FOUND_TOWN,                   ///< found a town
 	CMD_RENAME_TOWN,                  ///< rename a town
 	CMD_DO_TOWN_ACTION,               ///< do a action from the town detail window (like advertises or bribe)
-
-	CMD_SELL_SHIP,                    ///< sell a ship
-	CMD_BUILD_SHIP,                   ///< build a new ship
-	CMD_SEND_SHIP_TO_DEPOT,           ///< send a ship to a depot
-	CMD_REFIT_SHIP,                   ///< refit the cargo space of a ship
+	CMD_EXPAND_TOWN,                  ///< expand a town
+	CMD_DELETE_TOWN,                  ///< delete a town
 
 	CMD_ORDER_REFIT,                  ///< change the refit informaction of an order (for "goto depot" )
 	CMD_CLONE_ORDER,                  ///< clone (and share) an order
@@ -262,8 +261,6 @@ enum {
 	CMD_COMPANY_CTRL,                 ///< used in multiplayer to create a new companies etc.
 	CMD_LEVEL_LAND,                   ///< level land
 
-	CMD_REFIT_RAIL_VEHICLE,           ///< refit the cargo space of a train
-	CMD_RESTORE_ORDER_INDEX,          ///< restore vehicle order-index and service interval
 	CMD_BUILD_LOCK,                   ///< build a lock
 
 	CMD_BUILD_SIGNAL_TRACK,           ///< add signals along a track (by dragging)
@@ -295,6 +292,8 @@ enum {
 	CMD_SET_VEHICLE_ON_TIME,          ///< set the vehicle on time feature (timetable)
 	CMD_AUTOFILL_TIMETABLE,           ///< autofill the timetable
 	CMD_SET_TIMETABLE_START,          ///< set the date that a timetable should start
+
+	CMD_END                           ///< Must ALWAYS be on the end of this list!! (period)
 };
 
 /**
@@ -314,8 +313,9 @@ enum DoCommandFlag {
 	DC_AUTOREPLACE           = 0x080, ///< autoreplace/autorenew is in progress, this shall disable vehicle limits when building, and ignore certain restrictions when undoing things (like vehicle attach callback)
 	DC_ALL_TILES             = 0x100, ///< allow this command also on MP_VOID tiles
 	DC_NO_MODIFY_TOWN_RATING = 0x200, ///< do not change town rating
+	DC_FORCE_CLEAR_TILE      = 0x400, ///< do not only remove the object on the tile, but also clear any water left on it
 };
-DECLARE_ENUM_AS_BIT_SET(DoCommandFlag);
+DECLARE_ENUM_AS_BIT_SET(DoCommandFlag)
 
 /**
  * Used to combine a StringID with the command.
@@ -333,7 +333,7 @@ DECLARE_ENUM_AS_BIT_SET(DoCommandFlag);
  *
  * This enumeration defines some flags which are binary-or'ed on a command.
  */
-enum {
+enum FlaggedCommands {
 	CMD_NETWORK_COMMAND       = 0x0100, ///< execute the command without sending it on the network
 	CMD_NO_TEST_IF_IN_NETWORK = 0x0200, ///< When enabled, the command will bypass the no-DC_EXEC round if in network
 	CMD_FLAGS_MASK            = 0xFF00, ///< mask for all command flags
@@ -345,7 +345,7 @@ enum {
  *
  * This enumeration defines flags for the _command_proc_table.
  */
-enum {
+enum CommandFlags {
 	CMD_SERVER    = 0x01, ///< the command can only be initiated by the server
 	CMD_SPECTATOR = 0x02, ///< the command may be initiated by a spectator
 	CMD_OFFLINE   = 0x04, ///< the command cannot be executed in a multiplayer game; single-player only
@@ -353,6 +353,30 @@ enum {
 	CMD_ALL_TILES = 0x10, ///< allow this command also on MP_VOID tiles
 	CMD_NO_TEST   = 0x20, ///< the command's output may differ between test and execute due to town rating changes etc.
 	CMD_NO_WATER  = 0x40, ///< set the DC_NO_WATER flag on this command
+	CMD_CLIENT_ID = 0x80, ///< set p2 with the ClientID of the sending client.
+};
+
+/** Types of commands we have. */
+enum CommandType {
+	CMDT_LANDSCAPE_CONSTRUCTION, ///< Construction and destruction of objects on the map.
+	CMDT_VEHICLE_CONSTRUCTION,   ///< Construction, modification (incl. refit) and destruction of vehicles.
+	CMDT_MONEY_MANAGEMENT,       ///< Management of money, i.e. loans and shares.
+	CMDT_VEHICLE_MANAGEMENT,     ///< Stopping, starting, sending to depot, turning around, replace orders etc.
+	CMDT_ROUTE_MANAGEMENT,       ///< Modifications to route management (orders, groups, etc).
+	CMDT_OTHER_MANAGEMENT,       ///< Renaming stuff, changing company colours, placing signs, etc.
+	CMDT_COMPANY_SETTING,        ///< Changing settings related to a company.
+	CMDT_SERVER_SETTING,         ///< Pausing/removing companies/server settings.
+	CMDT_CHEAT,                  ///< A cheat of some sorts.
+
+	CMDT_END,                    ///< Magic end marker.
+};
+
+/** Different command pause levels. */
+enum CommandPauseLevel {
+	CMDPL_NO_ACTIONS,      ///< No user actions may be executed.
+	CMDPL_NO_CONSTRUCTION, ///< No construction actions may be executed.
+	CMDPL_NO_LANDSCAPING,  ///< No landscaping actions may be executed.
+	CMDPL_ALL_ACTIONS,     ///< All actions may be executed.
 };
 
 /**
@@ -385,6 +409,7 @@ struct Command {
 	CommandProc *proc; ///< The procedure to actually executing
 	const char *name;  ///< A human readable name for the procedure
 	byte flags;        ///< The (command) flags to that apply to this command
+	CommandType type;  ///< The type of command.
 };
 
 /**
@@ -406,12 +431,12 @@ typedef void CommandCallback(const CommandCost &result, TileIndex tile, uint32 p
  * Structure for buffering the build command when selecting a station to join.
  */
 struct CommandContainer {
-	TileIndex tile;            ///< tile command being executed on
-	uint32 p1;                 ///< parameter p1
-	uint32 p2;                 ///< parameter p2
-	uint32 cmd;                ///< command being executed
-	CommandCallback *callback; ///< any callback function executed upon successful completion of the command
-	char text[80];             ///< possible text sent for name changes etc
+	TileIndex tile;                  ///< tile command being executed on.
+	uint32 p1;                       ///< parameter p1.
+	uint32 p2;                       ///< parameter p2.
+	uint32 cmd;                      ///< command being executed.
+	CommandCallback *callback;       ///< any callback function executed upon successful completion of the command.
+	char text[32 * MAX_CHAR_LENGTH]; ///< possible text sent for name changes etc, in bytes including '\0'.
 };
 
 #endif /* COMMAND_TYPE_H */
