@@ -1,4 +1,4 @@
-/* $Id: date.cpp 18433 2009-12-08 19:59:21Z frosch $ */
+/* $Id: date.cpp 21512 2010-12-14 14:57:51Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,15 +10,13 @@
 /** @file date.cpp Handling of dates in our native format and transforming them to something human readable. */
 
 #include "stdafx.h"
-#include "variables.h"
 #include "network/network.h"
 #include "network/network_func.h"
 #include "currency.h"
 #include "window_func.h"
-#include "functions.h"
+#include "settings_type.h"
 #include "date_func.h"
 #include "vehicle_base.h"
-#include "debug.h"
 #include "rail_gui.h"
 #include "saveload/saveload.h"
 
@@ -26,13 +24,21 @@ Year      _cur_year;   ///< Current year, starting at 0
 Month     _cur_month;  ///< Current month (0..11)
 Date      _date;       ///< Current date in days (day counter)
 DateFract _date_fract;
+uint16 _tick_counter;  ///< Ever incrementing (and sometimes wrapping) tick counter for setting off various events
 
-
-void SetDate(Date date)
+/**
+ * Set the date.
+ * @param date  New date
+ * @param fract The number of ticks that have passed on this date.
+ */
+void SetDate(Date date, DateFract fract)
 {
+	assert(fract < DAY_TICKS);
+
 	YearMonthDay ymd;
 
 	_date = date;
+	_date_fract = fract;
 	ConvertDateToYMD(date, &ymd);
 	_cur_year = ymd.year;
 	_cur_month = ymd.month;
@@ -55,7 +61,7 @@ static const uint16 _month_date_from_year_day[] = {
 };
 #undef M
 
-enum {
+enum DaysTillMonth {
 	ACCUM_JAN = 0,
 	ACCUM_FEB = ACCUM_JAN + 31,
 	ACCUM_MAR = ACCUM_FEB + 29,
@@ -70,6 +76,7 @@ enum {
 	ACCUM_DEC = ACCUM_NOV + 30,
 };
 
+/** Number of days to pass from the first day in the year before reaching the first of a month. */
 static const uint16 _accum_days_for_month[] = {
 	ACCUM_JAN, ACCUM_FEB, ACCUM_MAR, ACCUM_APR,
 	ACCUM_MAY, ACCUM_JUN, ACCUM_JUL, ACCUM_AUG,
@@ -83,8 +90,7 @@ static const uint16 _accum_days_for_month[] = {
  */
 void ConvertDateToYMD(Date date, YearMonthDay *ymd)
 {
-	/*
-	 * Year determination in multiple steps to account for leap
+	/* Year determination in multiple steps to account for leap
 	 * years. First do the large steps, then the smaller ones.
 	 */
 
@@ -153,6 +159,7 @@ Date ConvertYMDToDate(Year year, Month month, Day day)
 extern void EnginesDailyLoop();
 extern void DisasterDailyLoop();
 extern void IndustryDailyLoop();
+
 extern void CompaniesMonthlyLoop();
 extern void EnginesMonthlyLoop();
 extern void TownsMonthlyLoop();
@@ -167,6 +174,7 @@ extern void TownsYearlyLoop();
 extern void ShowEndGameChart();
 
 
+/** Available settings for autosave intervals. */
 static const Month _autosave_months[] = {
 	 0, ///< never
 	 1, ///< every month
@@ -218,15 +226,9 @@ static void OnNewYear()
  */
 static void OnNewMonth()
 {
-	if (_debug_desync_level > 2) {
-		char name[MAX_PATH];
-		snprintf(name, lengthof(name), "dmp_cmds_%08x_%08x.sav", _settings_game.game_creation.generation_seed, _date);
-		SaveOrLoad(name, SL_SAVE, AUTOSAVE_DIR);
-	}
-
 	if (_settings_client.gui.autosave != 0 && (_cur_month % _autosave_months[_settings_client.gui.autosave]) == 0) {
 		_do_autosave = true;
-		RedrawAutosave();
+		SetWindowDirty(WC_STATUS_BAR, 0);
 	}
 
 	SetWindowClassesDirty(WC_CHEATS);
@@ -247,7 +249,7 @@ static void OnNewMonth()
 static void OnNewDay()
 {
 #ifdef ENABLE_NETWORK
-	NetworkChatMessageDailyLoop();
+	if (_network_server) NetworkServerDailyLoop();
 #endif /* ENABLE_NETWORK */
 
 	DisasterDailyLoop();
