@@ -1,5 +1,14 @@
-/* $Id: elrail.cpp 12141 2008-02-14 15:59:16Z frosch $ */
-/** @file elrail.cpp
+/* $Id: elrail.cpp 22448 2011-05-13 17:57:07Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/**
+ * @file elrail.cpp
  * This file deals with displaying wires and pylons for electric railways.
  * <h2>Basics</h2>
  *
@@ -46,27 +55,16 @@
  */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "station_map.h"
 #include "viewport_func.h"
-#include "settings_type.h"
-#include "landscape.h"
-#include "rail_type.h"
-#include "debug.h"
-#include "tunnel_map.h"
-#include "road_map.h"
-#include "bridge_map.h"
-#include "bridge.h"
-#include "rail_map.h"
 #include "train.h"
 #include "rail_gui.h"
-#include "transparency.h"
 #include "tunnelbridge_map.h"
-#include "vehicle_func.h"
-#include "player_base.h"
 #include "tunnelbridge.h"
+#include "elrail_func.h"
+#include "company_base.h"
+#include "newgrf_railtype.h"
 
-#include "table/sprites.h"
 #include "table/elrail_data.h"
 
 static inline TLG GetTLG(TileIndex t)
@@ -84,32 +82,30 @@ static TrackBits GetRailTrackBitsUniversal(TileIndex t, byte *override)
 {
 	switch (GetTileType(t)) {
 		case MP_RAILWAY:
-			if (GetRailType(t) != RAILTYPE_ELECTRIC) return TRACK_BIT_NONE;
+			if (!HasCatenary(GetRailType(t))) return TRACK_BIT_NONE;
 			switch (GetRailTileType(t)) {
 				case RAIL_TILE_NORMAL: case RAIL_TILE_SIGNALS:
 					return GetTrackBits(t);
-				case RAIL_TILE_WAYPOINT:
-					return GetRailWaypointBits(t);
 				default:
 					return TRACK_BIT_NONE;
 			}
 			break;
 
 		case MP_TUNNELBRIDGE:
-			if (GetRailType(t) != RAILTYPE_ELECTRIC) return TRACK_BIT_NONE;
+			if (!HasCatenary(GetRailType(t))) return TRACK_BIT_NONE;
 			if (override != NULL && (IsTunnel(t) || GetTunnelBridgeLength(t, GetOtherBridgeEnd(t)) > 0)) {
 				*override = 1 << GetTunnelBridgeDirection(t);
 			}
-			return AxisToTrackBits(DiagDirToAxis(GetTunnelBridgeDirection(t)));
+			return DiagDirToDiagTrackBits(GetTunnelBridgeDirection(t));
 
 		case MP_ROAD:
 			if (!IsLevelCrossing(t)) return TRACK_BIT_NONE;
-			if (GetRailType(t) != RAILTYPE_ELECTRIC) return TRACK_BIT_NONE;
+			if (!HasCatenary(GetRailType(t))) return TRACK_BIT_NONE;
 			return GetCrossingRailBits(t);
 
 		case MP_STATION:
-			if (!IsRailwayStation(t)) return TRACK_BIT_NONE;
-			if (GetRailType(t) != RAILTYPE_ELECTRIC) return TRACK_BIT_NONE;
+			if (!HasStationRail(t)) return TRACK_BIT_NONE;
+			if (!HasCatenary(GetRailType(t))) return TRACK_BIT_NONE;
 			if (!IsStationTileElectrifiable(t)) return TRACK_BIT_NONE;
 			return TrackToTrackBits(GetRailStationTrack(t));
 
@@ -118,7 +114,73 @@ static TrackBits GetRailTrackBitsUniversal(TileIndex t, byte *override)
 	}
 }
 
-/** Corrects the tileh for certain tile types. Returns an effective tileh for the track on the tile.
+/**
+ * Masks out track bits when neighbouring tiles are unelectrified.
+ */
+static TrackBits MaskWireBits(TileIndex t, TrackBits tracks)
+{
+	if (!IsPlainRailTile(t)) return tracks;
+
+	TrackdirBits neighbour_tdb = TRACKDIR_BIT_NONE;
+	for (DiagDirection d = DIAGDIR_BEGIN; d < DIAGDIR_END; d++) {
+		/* If the neighbor tile is either not electrified or has no tracks that can be reached
+		 * from this tile, mark all trackdirs that can be reached from the neighbour tile
+		 * as needing no catenary. */
+		RailType rt = GetTileRailType(TileAddByDiagDir(t, d));
+		if (rt == INVALID_RAILTYPE || !HasCatenary(rt) || (TrackStatusToTrackBits(GetTileTrackStatus(TileAddByDiagDir(t, d), TRANSPORT_RAIL, 0)) & DiagdirReachesTracks(d)) == TRACK_BIT_NONE) {
+			neighbour_tdb |= DiagdirReachesTrackdirs(ReverseDiagDir(d));
+		}
+	}
+
+	/* If the tracks from either a diagonal crossing or don't overlap, both
+	 * trackdirs have to be marked to mask the corresponding track bit. Else
+	 * one marked trackdir is enough the mask the track bit. */
+	TrackBits mask;
+	if (tracks == TRACK_BIT_CROSS || !TracksOverlap(tracks)) {
+		/* If the tracks form either a diagonal crossing or don't overlap, both
+		 * trackdirs have to be marked to mask the corresponding track bit. */
+		mask = ~(TrackBits)((neighbour_tdb & (neighbour_tdb >> 8)) & TRACK_BIT_MASK);
+		/* If that results in no masked tracks and it is not a diagonal crossing,
+		 * require only one marked trackdir to mask. */
+		if (tracks != TRACK_BIT_CROSS && (mask & TRACK_BIT_MASK) == TRACK_BIT_MASK) mask = ~TrackdirBitsToTrackBits(neighbour_tdb);
+	} else {
+		/* Require only one marked trackdir to mask the track. */
+		mask = ~TrackdirBitsToTrackBits(neighbour_tdb);
+		/* If that results in an empty set, require both trackdirs for diagonal track. */
+		if ((tracks & mask) == TRACK_BIT_NONE) {
+			if ((neighbour_tdb & TRACKDIR_BIT_X_NE) == 0 || (neighbour_tdb & TRACKDIR_BIT_X_SW) == 0) mask |= TRACK_BIT_X;
+			if ((neighbour_tdb & TRACKDIR_BIT_Y_NW) == 0 || (neighbour_tdb & TRACKDIR_BIT_Y_SE) == 0) mask |= TRACK_BIT_Y;
+			/* If that still is not enough, require both trackdirs for any track. */
+			if ((tracks & mask) == TRACK_BIT_NONE) mask = ~(TrackBits)((neighbour_tdb & (neighbour_tdb >> 8)) & TRACK_BIT_MASK);
+		}
+	}
+
+	/* Mask the tracks only if at least one track bit would remain. */
+	return (tracks & mask) != TRACK_BIT_NONE ? tracks & mask : tracks;
+}
+
+/**
+ * Get the base wire sprite to use.
+ */
+static inline SpriteID GetWireBase(TileIndex tile, TileContext context = TCX_NORMAL)
+{
+	const RailtypeInfo *rti = GetRailTypeInfo(GetRailType(tile));
+	SpriteID wires = GetCustomRailSprite(rti, tile, RTSG_WIRES, context);
+	return wires == 0 ? SPR_WIRE_BASE : wires;
+}
+
+/**
+ * Get the base pylon sprite to use.
+ */
+static inline SpriteID GetPylonBase(TileIndex tile, TileContext context = TCX_NORMAL)
+{
+	const RailtypeInfo *rti = GetRailTypeInfo(GetRailType(tile));
+	SpriteID pylons = GetCustomRailSprite(rti, tile, RTSG_PYLONS, context);
+	return pylons == 0 ? SPR_PYLON_BASE : pylons;
+}
+
+/**
+ * Corrects the tileh for certain tile types. Returns an effective tileh for the track on the tile.
  * @param tile The tile to analyse
  * @param *tileh the tileh
  */
@@ -179,14 +241,14 @@ void DrawCatenaryOnTunnel(const TileInfo *ti)
 		{ 1, 0, 15, 16 }, // NW
 	};
 
-	if ((GetRailType(ti->tile) != RAILTYPE_ELECTRIC) || _patches.disable_elrails) return;
-
 	DiagDirection dir = GetTunnelBridgeDirection(ti->tile);
 
+	SpriteID wire_base = GetWireBase(ti->tile);
+
 	const SortableSpriteStruct *sss = &CatenarySpriteData_Tunnel[dir];
 	const int *BB_data = _tunnel_wire_BB[dir];
 	AddSortableSpriteToDraw(
-		sss->image, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
+		wire_base + sss->image_offset, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
 		BB_data[2] - sss->x_offset, BB_data[3] - sss->y_offset, BB_Z_SEPARATOR - sss->z_offset + 1,
 		GetTileZ(ti->tile) + sss->z_offset,
 		IsTransparencySet(TO_CATENARY),
@@ -194,7 +256,8 @@ void DrawCatenaryOnTunnel(const TileInfo *ti)
 	);
 }
 
-/** Draws wires and, if required, pylons on a given tile
+/**
+ * Draws wires and, if required, pylons on a given tile
  * @param ti The Tileinfo to draw the tile for
  */
 static void DrawCatenaryRailway(const TileInfo *ti)
@@ -203,21 +266,24 @@ static void DrawCatenaryRailway(const TileInfo *ti)
 	 * the track configuration of 2 adjacent tiles. trackconfig[0] stores the
 	 * current tile (home tile) while [1] holds the neighbour */
 	TrackBits trackconfig[TS_END];
+	TrackBits wireconfig[TS_END];
 	bool isflat[TS_END];
 	/* Note that ti->tileh has already been adjusted for Foundations */
 	Slope tileh[TS_END] = { ti->tileh, SLOPE_FLAT };
 
 	/* Half tile slopes coincide only with horizontal/vertical track.
 	 * Faking a flat slope results in the correct sprites on positions. */
-	if (IsHalftileSlope(tileh[TS_HOME])) tileh[TS_HOME] = SLOPE_FLAT;
+	Corner halftile_corner = CORNER_INVALID;
+	if (IsHalftileSlope(tileh[TS_HOME])) {
+		halftile_corner = GetHalftileSlopeCorner(tileh[TS_HOME]);
+		tileh[TS_HOME] = SLOPE_FLAT;
+	}
 
 	TLG tlg = GetTLG(ti->tile);
 	byte PCPstatus = 0;
 	byte OverridePCP = 0;
 	byte PPPpreferred[DIAGDIR_END];
 	byte PPPallowed[DIAGDIR_END];
-	DiagDirection i;
-	Track t;
 
 	/* Find which rail bits are present, and select the override points.
 	 * We don't draw a pylon:
@@ -226,25 +292,36 @@ static void DrawCatenaryRailway(const TileInfo *ti)
 	 *    because that one is drawn on the bridge. Exception is for length 0 bridges
 	 *    which have no middle tiles */
 	trackconfig[TS_HOME] = GetRailTrackBitsUniversal(ti->tile, &OverridePCP);
+	wireconfig[TS_HOME] = MaskWireBits(ti->tile, trackconfig[TS_HOME]);
 	/* If a track bit is present that is not in the main direction, the track is level */
 	isflat[TS_HOME] = ((trackconfig[TS_HOME] & (TRACK_BIT_HORZ | TRACK_BIT_VERT)) != 0);
 
 	AdjustTileh(ti->tile, &tileh[TS_HOME]);
 
-	for (i = DIAGDIR_NE; i < DIAGDIR_END; i++) {
+	SpriteID pylon_normal = GetPylonBase(ti->tile);
+	SpriteID pylon_halftile = (halftile_corner != CORNER_INVALID) ? GetPylonBase(ti->tile, TCX_UPPER_HALFTILE) : pylon_normal;
+
+	for (DiagDirection i = DIAGDIR_BEGIN; i < DIAGDIR_END; i++) {
+		static const uint edge_corners[] = {
+			1 << CORNER_N | 1 << CORNER_E, // DIAGDIR_NE
+			1 << CORNER_S | 1 << CORNER_E, // DIAGDIR_SE
+			1 << CORNER_S | 1 << CORNER_W, // DIAGDIR_SW
+			1 << CORNER_N | 1 << CORNER_W, // DIAGDIR_NW
+		};
+		SpriteID pylon_base = (halftile_corner != CORNER_INVALID && HasBit(edge_corners[i], halftile_corner)) ? pylon_halftile : pylon_normal;
 		TileIndex neighbour = ti->tile + TileOffsByDiagDir(i);
-		Foundation foundation = FOUNDATION_NONE;
-		int k;
+		byte elevation = GetPCPElevation(ti->tile, i);
 
 		/* Here's one of the main headaches. GetTileSlope does not correct for possibly
 		 * existing foundataions, so we do have to do that manually later on.*/
 		tileh[TS_NEIGHBOUR] = GetTileSlope(neighbour, NULL);
 		trackconfig[TS_NEIGHBOUR] = GetRailTrackBitsUniversal(neighbour, NULL);
-		if (IsTunnelTile(neighbour) && i != GetTunnelBridgeDirection(neighbour)) trackconfig[TS_NEIGHBOUR] = TRACK_BIT_NONE;
+		wireconfig[TS_NEIGHBOUR] = MaskWireBits(neighbour, trackconfig[TS_NEIGHBOUR]);
+		if (IsTunnelTile(neighbour) && i != GetTunnelBridgeDirection(neighbour)) wireconfig[TS_NEIGHBOUR] = trackconfig[TS_NEIGHBOUR] = TRACK_BIT_NONE;
 
 		/* If the neighboured tile does not smoothly connect to the current tile (because of a foundation),
 		 * we have to draw all pillars on the current tile. */
-		if (GetPCPElevation(ti->tile, i) != GetPCPElevation(neighbour, ReverseDiagDir(i))) trackconfig[TS_NEIGHBOUR] = TRACK_BIT_NONE;
+		if (elevation != GetPCPElevation(neighbour, ReverseDiagDir(i))) wireconfig[TS_NEIGHBOUR] = trackconfig[TS_NEIGHBOUR] = TRACK_BIT_NONE;
 
 		isflat[TS_NEIGHBOUR] = ((trackconfig[TS_NEIGHBOUR] & (TRACK_BIT_HORZ | TRACK_BIT_VERT)) != 0);
 
@@ -253,7 +330,7 @@ static void DrawCatenaryRailway(const TileInfo *ti)
 
 		/* We cycle through all the existing tracks at a PCP and see what
 		 * PPPs we want to have, or may not have at all */
-		for (k = 0; k < NUM_TRACKS_AT_PCP; k++) {
+		for (uint k = 0; k < NUM_TRACKS_AT_PCP; k++) {
 			/* Next to us, we have a bridge head, don't worry about that one, if it shows away from us */
 			if (TrackSourceTile[i][k] == TS_NEIGHBOUR &&
 			    IsBridgeTile(neighbour) &&
@@ -263,44 +340,52 @@ static void DrawCatenaryRailway(const TileInfo *ti)
 
 			/* We check whether the track in question (k) is present in the tile
 			 * (TrackSourceTile) */
-			if (HasBit(trackconfig[TrackSourceTile[i][k]], TracksAtPCP[i][k])) {
+			DiagDirection PCPpos = i;
+			if (HasBit(wireconfig[TrackSourceTile[i][k]], TracksAtPCP[i][k])) {
 				/* track found, if track is in the neighbour tile, adjust the number
 				 * of the PCP for preferred/allowed determination*/
-				DiagDirection PCPpos = (TrackSourceTile[i][k] == TS_HOME) ? i : ReverseDiagDir(i);
+				PCPpos = (TrackSourceTile[i][k] == TS_HOME) ? i : ReverseDiagDir(i);
 				SetBit(PCPstatus, i); // This PCP is in use
-
 				PPPpreferred[i] &= PreferredPPPofTrackAtPCP[TracksAtPCP[i][k]][PCPpos];
+			}
+
+			if (HasBit(trackconfig[TrackSourceTile[i][k]], TracksAtPCP[i][k])) {
 				PPPallowed[i] &= ~DisallowedPPPofTrackAtPCP[TracksAtPCP[i][k]][PCPpos];
 			}
 		}
 
 		/* Deactivate all PPPs if PCP is not used */
-		PPPpreferred[i] *= HasBit(PCPstatus, i);
-		PPPallowed[i] *= HasBit(PCPstatus, i);
+		if (!HasBit(PCPstatus, i)) {
+			PPPpreferred[i] = 0;
+			PPPallowed[i] = 0;
+		}
 
-		/* A station is always "flat", so adjust the tileh accordingly */
-		if (IsTileType(neighbour, MP_STATION)) tileh[TS_NEIGHBOUR] = SLOPE_FLAT;
+		Foundation foundation = FOUNDATION_NONE;
+
+		/* Station and road crossings are always "flat", so adjust the tileh accordingly */
+		if (IsTileType(neighbour, MP_STATION) || IsTileType(neighbour, MP_ROAD)) tileh[TS_NEIGHBOUR] = SLOPE_FLAT;
 
 		/* Read the foundataions if they are present, and adjust the tileh */
-		if (trackconfig[TS_NEIGHBOUR] != TRACK_BIT_NONE && IsTileType(neighbour, MP_RAILWAY) && GetRailType(neighbour) == RAILTYPE_ELECTRIC) foundation = GetRailFoundation(tileh[TS_NEIGHBOUR], trackconfig[TS_NEIGHBOUR]);
+		if (trackconfig[TS_NEIGHBOUR] != TRACK_BIT_NONE && IsTileType(neighbour, MP_RAILWAY) && HasCatenary(GetRailType(neighbour))) foundation = GetRailFoundation(tileh[TS_NEIGHBOUR], trackconfig[TS_NEIGHBOUR]);
 		if (IsBridgeTile(neighbour)) {
 			foundation = GetBridgeFoundation(tileh[TS_NEIGHBOUR], DiagDirToAxis(GetTunnelBridgeDirection(neighbour)));
 		}
 
 		ApplyFoundationToSlope(foundation, &tileh[TS_NEIGHBOUR]);
 
-	/* Half tile slopes coincide only with horizontal/vertical track.
-	 * Faking a flat slope results in the correct sprites on positions. */
+		/* Half tile slopes coincide only with horizontal/vertical track.
+		 * Faking a flat slope results in the correct sprites on positions. */
 		if (IsHalftileSlope(tileh[TS_NEIGHBOUR])) tileh[TS_NEIGHBOUR] = SLOPE_FLAT;
 
 		AdjustTileh(neighbour, &tileh[TS_NEIGHBOUR]);
 
 		/* If we have a straight (and level) track, we want a pylon only every 2 tiles
-		 * Delete the PCP if this is the case. */
-		/* Level means that the slope is the same, or the track is flat */
+		 * Delete the PCP if this is the case.
+		 * Level means that the slope is the same, or the track is flat */
 		if (tileh[TS_HOME] == tileh[TS_NEIGHBOUR] || (isflat[TS_HOME] && isflat[TS_NEIGHBOUR])) {
-			for (k = 0; k < NUM_IGNORE_GROUPS; k++)
+			for (uint k = 0; k < NUM_IGNORE_GROUPS; k++) {
 				if (PPPpreferred[i] == IgnoredPCP[k][tlg][i]) ClrBit(PCPstatus, i);
+			}
 		}
 
 		/* Now decide where we draw our pylons. First try the preferred PPPs, but they may not exist.
@@ -320,7 +405,7 @@ static void DrawCatenaryRailway(const TileInfo *ti)
 		}
 
 		if (PPPallowed[i] != 0 && HasBit(PCPstatus, i) && !HasBit(OverridePCP, i)) {
-			for (k = 0; k < DIR_END; k++) {
+			for (Direction k = DIR_BEGIN; k < DIR_END; k++) {
 				byte temp = PPPorder[i][GetTLG(ti->tile)][k];
 
 				if (HasBit(PPPallowed[i], temp)) {
@@ -330,19 +415,22 @@ static void DrawCatenaryRailway(const TileInfo *ti)
 					/* Don't build the pylon if it would be outside the tile */
 					if (!HasBit(OwnedPPPonPCP[i], temp)) {
 						/* We have a neighour that will draw it, bail out */
-						if (trackconfig[TS_NEIGHBOUR] != 0) break;
-						continue; /* No neighbour, go looking for a better position */
+						if (trackconfig[TS_NEIGHBOUR] != TRACK_BIT_NONE) break;
+						continue; // No neighbour, go looking for a better position
 					}
 
-					AddSortableSpriteToDraw(pylon_sprites[temp], PAL_NONE, x, y, 1, 1, BB_HEIGHT_UNDER_BRIDGE,
-							GetPCPElevation(ti->tile, i),
-							IsTransparencySet(TO_CATENARY), -1, -1);
-					break; /* We already have drawn a pylon, bail out */
+					AddSortableSpriteToDraw(pylon_base + pylon_sprites[temp], PAL_NONE, x, y, 1, 1, BB_HEIGHT_UNDER_BRIDGE,
+						elevation, IsTransparencySet(TO_CATENARY), -1, -1);
+
+					break; // We already have drawn a pylon, bail out
 				}
 			}
 		}
 	}
 
+	/* The wire above the tunnel is drawn together with the tunnel-roof (see DrawCatenaryOnTunnel()) */
+	if (IsTunnelTile(ti->tile)) return;
+
 	/* Don't draw a wire under a low bridge */
 	if (MayHaveBridgeAbove(ti->tile) && IsBridgeAbove(ti->tile) && !IsTransparencySet(TO_CATENARY)) {
 		uint height = GetBridgeHeight(GetNorthernBridgeEnd(ti->tile));
@@ -350,33 +438,50 @@ static void DrawCatenaryRailway(const TileInfo *ti)
 		if (height <= GetTileMaxZ(ti->tile) + TILE_HEIGHT) return;
 	}
 
+	SpriteID wire_normal = GetWireBase(ti->tile);
+	SpriteID wire_halftile = (halftile_corner != CORNER_INVALID) ? GetWireBase(ti->tile, TCX_UPPER_HALFTILE) : wire_normal;
+	Track halftile_track;
+	switch (halftile_corner) {
+		case CORNER_W: halftile_track = TRACK_LEFT; break;
+		case CORNER_S: halftile_track = TRACK_LOWER; break;
+		case CORNER_E: halftile_track = TRACK_RIGHT; break;
+		case CORNER_N: halftile_track = TRACK_UPPER; break;
+		default:       halftile_track = INVALID_TRACK; break;
+	}
+
 	/* Drawing of pylons is finished, now draw the wires */
-	for (t = TRACK_BEGIN; t < TRACK_END; t++) {
-		if (HasBit(trackconfig[TS_HOME], t)) {
-			if (IsTunnelTile(ti->tile)) break; // drawn together with tunnel-roof (see DrawCatenaryOnTunnel())
-			byte PCPconfig = HasBit(PCPstatus, PCPpositions[t][0]) +
-				(HasBit(PCPstatus, PCPpositions[t][1]) << 1);
+	Track t;
+	FOR_EACH_SET_TRACK(t, wireconfig[TS_HOME]) {
+		SpriteID wire_base = (t == halftile_track) ? wire_halftile : wire_normal;
+		byte PCPconfig = HasBit(PCPstatus, PCPpositions[t][0]) +
+			(HasBit(PCPstatus, PCPpositions[t][1]) << 1);
 
-			const SortableSpriteStruct *sss;
-			int tileh_selector = !(tileh[TS_HOME] % 3) * tileh[TS_HOME] / 3; /* tileh for the slopes, 0 otherwise */
+		const SortableSpriteStruct *sss;
+		int tileh_selector = !(tileh[TS_HOME] % 3) * tileh[TS_HOME] / 3; // tileh for the slopes, 0 otherwise
 
-			assert(PCPconfig != 0); /* We have a pylon on neither end of the wire, that doesn't work (since we have no sprites for that) */
-			assert(!IsSteepSlope(tileh[TS_HOME]));
-			sss = &CatenarySpriteData[Wires[tileh_selector][t][PCPconfig]];
+		assert(PCPconfig != 0); // We have a pylon on neither end of the wire, that doesn't work (since we have no sprites for that)
+		assert(!IsSteepSlope(tileh[TS_HOME]));
+		sss = &CatenarySpriteData[Wires[tileh_selector][t][PCPconfig]];
 
-			/*
-			 * The "wire"-sprite position is inside the tile, i.e. 0 <= sss->?_offset < TILE_SIZE.
-			 * Therefore it is save to use GetSlopeZ() for the elevation.
-			 * Also note, that the result of GetSlopeZ() is very special for bridge-ramps.
-			 */
-			AddSortableSpriteToDraw(sss->image, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
-				sss->x_size, sss->y_size, sss->z_size, GetSlopeZ(ti->x + sss->x_offset, ti->y + sss->y_offset) + sss->z_offset,
-				IsTransparencySet(TO_CATENARY));
-		}
+		/*
+		 * The "wire"-sprite position is inside the tile, i.e. 0 <= sss->?_offset < TILE_SIZE.
+		 * Therefore it is safe to use GetSlopeZ() for the elevation.
+		 * Also note that the result of GetSlopeZ() is very special for bridge-ramps.
+		 */
+		AddSortableSpriteToDraw(wire_base + sss->image_offset, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
+			sss->x_size, sss->y_size, sss->z_size, GetSlopeZ(ti->x + sss->x_offset, ti->y + sss->y_offset) + sss->z_offset,
+			IsTransparencySet(TO_CATENARY));
 	}
 }
 
-static void DrawCatenaryOnBridge(const TileInfo *ti)
+/**
+ * Draws wires on a tunnel tile
+ *
+ * DrawTile_TunnelBridge() calls this function to draw the wires on the bridge.
+ *
+ * @param ti The Tileinfo to draw the tile for
+ */
+void DrawCatenaryOnBridge(const TileInfo *ti)
 {
 	TileIndex end = GetSouthernBridgeEnd(ti->tile);
 	TileIndex start = GetOtherBridgeEnd(end);
@@ -402,20 +507,24 @@ static void DrawCatenaryOnBridge(const TileInfo *ti)
 
 	height = GetBridgeHeight(end);
 
-	AddSortableSpriteToDraw(sss->image, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
+	SpriteID wire_base = GetWireBase(end, TCX_ON_BRIDGE);
+
+	AddSortableSpriteToDraw(wire_base + sss->image_offset, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
 		sss->x_size, sss->y_size, sss->z_size, height + sss->z_offset,
 		IsTransparencySet(TO_CATENARY)
 	);
 
-	/* Finished with wires, draw pylons */
-	/* every other tile needs a pylon on the northern end */
+	SpriteID pylon_base = GetPylonBase(end, TCX_ON_BRIDGE);
+
+	/* Finished with wires, draw pylons
+	 * every other tile needs a pylon on the northern end */
 	if (num % 2) {
 		DiagDirection PCPpos = (axis == AXIS_X ? DIAGDIR_NE : DIAGDIR_NW);
 		Direction PPPpos = (axis == AXIS_X ? DIR_NW : DIR_NE);
 		if (HasBit(tlg, (axis == AXIS_X ? 0 : 1))) PPPpos = ReverseDir(PPPpos);
 		uint x = ti->x + x_pcp_offsets[PCPpos] + x_ppp_offsets[PPPpos];
 		uint y = ti->y + y_pcp_offsets[PCPpos] + y_ppp_offsets[PPPpos];
-		AddSortableSpriteToDraw(pylon_sprites[PPPpos], PAL_NONE, x, y, 1, 1, BB_HEIGHT_UNDER_BRIDGE, height, IsTransparencySet(TO_CATENARY), -1, -1);
+		AddSortableSpriteToDraw(pylon_base + pylon_sprites[PPPpos], PAL_NONE, x, y, 1, 1, BB_HEIGHT_UNDER_BRIDGE, height, IsTransparencySet(TO_CATENARY), -1, -1);
 	}
 
 	/* need a pylon on the southern end of the bridge */
@@ -425,30 +534,27 @@ static void DrawCatenaryOnBridge(const TileInfo *ti)
 		if (HasBit(tlg, (axis == AXIS_X ? 0 : 1))) PPPpos = ReverseDir(PPPpos);
 		uint x = ti->x + x_pcp_offsets[PCPpos] + x_ppp_offsets[PPPpos];
 		uint y = ti->y + y_pcp_offsets[PCPpos] + y_ppp_offsets[PPPpos];
-		AddSortableSpriteToDraw(pylon_sprites[PPPpos], PAL_NONE, x, y, 1, 1, BB_HEIGHT_UNDER_BRIDGE, height, IsTransparencySet(TO_CATENARY), -1, -1);
+		AddSortableSpriteToDraw(pylon_base + pylon_sprites[PPPpos], PAL_NONE, x, y, 1, 1, BB_HEIGHT_UNDER_BRIDGE, height, IsTransparencySet(TO_CATENARY), -1, -1);
 	}
 }
 
+/**
+ * Draws overhead wires and pylons for electric railways.
+ * @param ti The TileInfo struct of the tile being drawn
+ * @see DrawCatenaryRailway
+ */
 void DrawCatenary(const TileInfo *ti)
 {
-	if (_patches.disable_elrails) return;
-
-	if (MayHaveBridgeAbove(ti->tile) && IsBridgeAbove(ti->tile)) {
-		TileIndex head = GetNorthernBridgeEnd(ti->tile);
-
-		if (GetTunnelBridgeTransportType(head) == TRANSPORT_RAIL && GetRailType(head) == RAILTYPE_ELECTRIC) {
-			DrawCatenaryOnBridge(ti);
-		}
-	}
-
 	switch (GetTileType(ti->tile)) {
 		case MP_RAILWAY:
 			if (IsRailDepot(ti->tile)) {
 				const SortableSpriteStruct *sss = &CatenarySpriteData_Depot[GetRailDepotDirection(ti->tile)];
 
+				SpriteID wire_base = GetWireBase(ti->tile);
+
 				/* This wire is not visible with the default depot sprites */
 				AddSortableSpriteToDraw(
-					sss->image, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
+					wire_base + sss->image_offset, PAL_NONE, ti->x + sss->x_offset, ti->y + sss->y_offset,
 					sss->x_size, sss->y_size, sss->z_size,
 					GetTileMaxZ(ti->tile) + sss->z_offset,
 					IsTransparencySet(TO_CATENARY)
@@ -467,10 +573,10 @@ void DrawCatenary(const TileInfo *ti)
 	DrawCatenaryRailway(ti);
 }
 
-int32 SettingsDisableElrail(int32 p1)
+bool SettingsDisableElrail(int32 p1)
 {
-	Vehicle *v;
-	Player *p;
+	Company *c;
+	Train *t;
 	bool disable = (p1 != 0);
 
 	/* we will now walk through all electric train engines and change their railtypes if it is the wrong one*/
@@ -478,9 +584,9 @@ int32 SettingsDisableElrail(int32 p1)
 	const RailType new_railtype = disable ? RAILTYPE_RAIL : RAILTYPE_ELECTRIC;
 
 	/* walk through all train engines */
-	EngineID eid;
-	FOR_ALL_ENGINEIDS_OF_TYPE(eid, VEH_TRAIN) {
-		RailVehicleInfo *rv_info = &_rail_vehicle_info[eid];
+	Engine *e;
+	FOR_ALL_ENGINES_OF_TYPE(e, VEH_TRAIN) {
+		RailVehicleInfo *rv_info = &e->u.rail;
 		/* if it is an electric rail engine and its railtype is the wrong one */
 		if (rv_info->engclass == 2 && rv_info->railtype == old_railtype) {
 			/* change it to the proper one */
@@ -489,30 +595,32 @@ int32 SettingsDisableElrail(int32 p1)
 	}
 
 	/* when disabling elrails, make sure that all existing trains can run on
-	*  normal rail too */
+	 *  normal rail too */
 	if (disable) {
-		FOR_ALL_VEHICLES(v) {
-			if (v->type == VEH_TRAIN && v->u.rail.railtype == RAILTYPE_ELECTRIC) {
+		FOR_ALL_TRAINS(t) {
+			if (t->railtype == RAILTYPE_ELECTRIC) {
 				/* this railroad vehicle is now compatible only with elrail,
-				*  so add there also normal rail compatibility */
-				v->u.rail.compatible_railtypes |= RAILTYPES_RAIL;
-				v->u.rail.railtype = RAILTYPE_RAIL;
-				SetBit(v->u.rail.flags, VRF_EL_ENGINE_ALLOWED_NORMAL_RAIL);
+				 *  so add there also normal rail compatibility */
+				t->compatible_railtypes |= RAILTYPES_RAIL;
+				t->railtype = RAILTYPE_RAIL;
+				SetBit(t->flags, VRF_EL_ENGINE_ALLOWED_NORMAL_RAIL);
 			}
 		}
 	}
 
-	/* setup total power for trains */
-	FOR_ALL_VEHICLES(v) {
-		/* power is cached only for front engines */
-		if (v->type == VEH_TRAIN && IsFrontEngine(v)) TrainPowerChanged(v);
+	/* Fix the total power and acceleration for trains */
+	FOR_ALL_TRAINS(t) {
+		/* power and acceleration is cached only for front engines */
+		if (t->IsFrontEngine()) {
+			t->ConsistChanged(true);
+		}
 	}
 
-	FOR_ALL_PLAYERS(p) p->avail_railtypes = GetPlayerRailtypes(p->index);
+	FOR_ALL_COMPANIES(c) c->avail_railtypes = GetCompanyRailtypes(c->index);
 
 	/* This resets the _last_built_railtype, which will be invalid for electric
-	* rails. It may have unintended consequences if that function is ever
-	* extended, though. */
+	 * rails. It may have unintended consequences if that function is ever
+	 * extended, though. */
 	ReinitGuiAfterToggleElrail(disable);
-	return 0;
+	return true;
 }
