@@ -1,4 +1,4 @@
-/* $Id: fileio.cpp 20168 2010-07-17 11:45:42Z rubidium $ */
+/* $Id: fileio.cpp 22841 2011-08-25 13:41:06Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,7 +11,6 @@
 
 #include "stdafx.h"
 #include "fileio_func.h"
-#include "variables.h"
 #include "debug.h"
 #include "fios.h"
 #include "string_func.h"
@@ -22,9 +21,7 @@
 #include <Path.h>
 #include <storage/FindDirectory.h>
 #else
-#if defined(OPENBSD) || defined(DOS)
 #include <unistd.h>
-#endif
 #include <pwd.h>
 #endif
 #include <sys/stat.h>
@@ -56,6 +53,9 @@ static Fio _fio;
 /** Whether the working directory should be scanned. */
 static bool _do_scan_working_directory = true;
 
+extern char *_config_file;
+extern char *_highscore_file;
+
 /* Get current position in file */
 size_t FioGetPos()
 {
@@ -162,10 +162,9 @@ static inline void FioCloseFile(int slot)
 
 void FioCloseAll()
 {
-	int i;
-
-	for (i = 0; i != lengthof(_fio.handles); i++)
+	for (int i = 0; i != lengthof(_fio.handles); i++) {
 		FioCloseFile(i);
+	}
 }
 
 #if defined(LIMITED_FDS)
@@ -425,7 +424,7 @@ FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir,
  * Create a directory with the given name
  * @param name the new name of the directory
  */
-void FioCreateDirectory(const char *name)
+static void FioCreateDirectory(const char *name)
 {
 #if defined(WIN32) || defined(WINCE)
 	CreateDirectory(OTTD2FS(name), NULL);
@@ -542,7 +541,11 @@ static void SimplifyFileName(char *name)
 #endif
 }
 
-/* static */ uint TarScanner::DoScan() {
+/* static */ uint TarScanner::DoScan()
+{
+	_tar_filelist.clear();
+	_tar_list.clear();
+
 	DEBUG(misc, 1, "Scanning for tars");
 	TarScanner fs;
 	uint num = fs.Scan(".tar", DATA_DIR, false);
@@ -703,7 +706,7 @@ bool TarScanner::AddFile(const char *filename, size_t basepath_length)
 							}
 
 							/* Truncate 'dest' after last PATHSEPCHAR.
-							 * This assumes, that the truncated part is a real directory and not a link */
+							 * This assumes that the truncated part is a real directory and not a link. */
 							destpos = strrchr(dest, PATHSEPCHAR);
 							if (destpos == NULL) destpos = dest;
 						} else {
@@ -855,8 +858,9 @@ extern void DetermineBasePaths(const char *exe);
  * in the same way we remove the name from the executable name.
  * @param exe the path to the executable
  */
-void ChangeWorkingDirectory(const char *exe)
+static bool ChangeWorkingDirectoryToExecutable(const char *exe)
 {
+	bool success = false;
 #ifdef WITH_COCOA
 	char *app_bundle = strchr(exe, '.');
 	while (app_bundle != NULL && strncasecmp(app_bundle, ".app", 4) != 0) app_bundle = strchr(&app_bundle[1], '.');
@@ -870,12 +874,17 @@ void ChangeWorkingDirectory(const char *exe)
 		/* If we want to go to the root, we can't use cd C:, but we must use '/' */
 		if (s[-1] == ':') chdir("/");
 #endif
-		if (chdir(exe) != 0) DEBUG(misc, 0, "Directory with the binary does not exist?");
+		if (chdir(exe) != 0) {
+			DEBUG(misc, 0, "Directory with the binary does not exist?");
+		} else {
+			success = true;
+		}
 		*s = PATHSEPCHAR;
 	}
 #ifdef WITH_COCOA
 	if (app_bundle != NULL) app_bundle[0] = '.';
 #endif /* WITH_COCOA */
+	return success;
 }
 
 /**
@@ -953,14 +962,19 @@ void DetermineBasePaths(const char *exe)
 	_do_scan_working_directory = DoScanWorkingDirectory();
 
 	/* Change the working directory to that one of the executable */
-	ChangeWorkingDirectory(exe);
-	if (getcwd(tmp, MAX_PATH) == NULL) *tmp = '\0';
-	AppendPathSeparator(tmp, MAX_PATH);
-	_searchpaths[SP_BINARY_DIR] = strdup(tmp);
+	if (ChangeWorkingDirectoryToExecutable(exe)) {
+		if (getcwd(tmp, MAX_PATH) == NULL) *tmp = '\0';
+		AppendPathSeparator(tmp, MAX_PATH);
+		_searchpaths[SP_BINARY_DIR] = strdup(tmp);
+	} else {
+		_searchpaths[SP_BINARY_DIR] = NULL;
+	}
 
 	if (_searchpaths[SP_WORKING_DIR] != NULL) {
 		/* Go back to the current working directory. */
-		ChangeWorkingDirectory(_searchpaths[SP_WORKING_DIR]);
+		if (chdir(_searchpaths[SP_WORKING_DIR]) != 0) {
+			DEBUG(misc, 0, "Failed to return to working directory!");
+		}
 	}
 
 #if defined(__MORPHOS__) || defined(__AMIGA__) || defined(DOS) || defined(OS2)
@@ -1032,7 +1046,8 @@ void DeterminePaths(const char *exe)
 	DEBUG(misc, 3, "%s found as personal directory", _personal_dir);
 
 	_highscore_file = str_fmt("%shs.dat", _personal_dir);
-	_log_file = str_fmt("%sopenttd.log",  _personal_dir);
+	extern char *_hotkeys_file;
+	_hotkeys_file = str_fmt("%shotkeys.cfg",  _personal_dir);
 
 	/* Make the necessary folders */
 #if !defined(__MORPHOS__) && !defined(__AMIGA__) && defined(WITH_PERSONAL_DIR)
@@ -1061,6 +1076,9 @@ void DeterminePaths(const char *exe)
 		FioCreateDirectory(tmp);
 		free(tmp);
 	}
+
+	extern char *_log_file;
+	_log_file = str_fmt("%sopenttd.log",  _personal_dir);
 #else /* ENABLE_NETWORK */
 	/* If we don't have networking, we don't need to make the directory. But
 	 * if it exists we keep it, otherwise remove it from the search paths. */
@@ -1118,7 +1136,7 @@ void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 
 
 /**
- * Scan a single directory (and recursively it's children) and add
+ * Scan a single directory (and recursively its children) and add
  * any graphics sets that are found.
  * @param fs              the file scanner to add the files to
  * @param extension       the extension of files to search for.
