@@ -1,4 +1,4 @@
-/* $Id: fontcache.cpp 19856 2010-05-18 21:38:09Z rubidium $ */
+/* $Id: fontcache.cpp 23053 2011-10-22 21:06:58Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -39,13 +39,18 @@ static int _ascender[FS_END];
 
 FreeTypeSettings _freetype;
 
-enum {
-	FACE_COLOUR = 1,
-	SHADOW_COLOUR = 2,
-};
+static const byte FACE_COLOUR   = 1;
+static const byte SHADOW_COLOUR = 2;
+
+/**
+ * Get the font loaded into a Freetype face by using a font-name.
+ * If no appropiate font is found, the function returns an error
+ */
+
+/* ========================================================================================
+ * Windows support
+ * ======================================================================================== */
 
-/** Get the font loaded into a Freetype face by using a font-name.
- * If no appropiate font is found, the function returns an error */
 #ifdef WIN32
 #include <windows.h>
 #include <shlobj.h> /* SHGetFolderPath */
@@ -265,7 +270,7 @@ static const char *GetEnglishFontName(const ENUMLOGFONTEX *logfont)
 		font_name[length] = '\0';
 
 		if ((platformId == 1 && languageId == 0) ||      // Macintosh English
-			(platformId == 3 && languageId == 0x0409)) { // Microsoft English (US)
+				(platformId == 3 && languageId == 0x0409)) { // Microsoft English (US)
 			ret_font_name = font_name;
 			break;
 		}
@@ -276,15 +281,15 @@ err3:
 err2:
 	SelectObject(dc, oldfont);
 	ReleaseDC(NULL, dc);
-err1:
 	DeleteObject(font);
-
+err1:
 	return ret_font_name == NULL ? WIDE_TO_MB((const TCHAR*)logfont->elfFullName) : ret_font_name;
 }
 
 struct EFCParam {
 	FreeTypeSettings *settings;
 	LOCALESIGNATURE  locale;
+	SetFallbackFontCallback *callback;
 };
 
 static int CALLBACK EnumFontCallback(const ENUMLOGFONTEX *logfont, const NEWTEXTMETRICEX *metric, DWORD type, LPARAM lParam)
@@ -341,14 +346,15 @@ static int CALLBACK EnumFontCallback(const ENUMLOGFONTEX *logfont, const NEWTEXT
 
 	if (!found) return 1;
 
-	DEBUG(freetype, 1, "Fallback font: %s (%s)", font_name, english_name);
 	strecpy(info->settings->small_font,  font_name, lastof(info->settings->small_font));
 	strecpy(info->settings->medium_font, font_name, lastof(info->settings->medium_font));
 	strecpy(info->settings->large_font,  font_name, lastof(info->settings->large_font));
+	if (info->callback(NULL)) return 1;
+	DEBUG(freetype, 1, "Fallback font: %s (%s)", font_name, english_name);
 	return 0; // stop enumerating
 }
 
-bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str)
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, SetFallbackFontCallback *callback)
 {
 	DEBUG(freetype, 1, "Trying fallback fonts");
 	EFCParam langInfo;
@@ -358,6 +364,7 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 		return false;
 	}
 	langInfo.settings = settings;
+	langInfo.callback = callback;
 
 	LOGFONT font;
 	/* Enumerate all fonts. */
@@ -371,7 +378,10 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 	return ret == 0;
 }
 
-#elif defined(__APPLE__)
+#elif defined(__APPLE__) /* end ifdef Win32 */
+/* ========================================================================================
+ * OSX support
+ * ======================================================================================== */
 
 #include "os/macosx/macos.h"
 #include <ApplicationServices/ApplicationServices.h>
@@ -418,10 +428,13 @@ FT_Error GetFontByFaceName(const char *font_name, FT_Face *face)
 	return err;
 }
 
-bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str)
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, SetFallbackFontCallback *callback)
 {
+	const char *str;
 	bool result = false;
 
+	callback(&str);
+
 #if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
 	if (MacOSVersionIsAtLeast(10, 5, 0)) {
 		/* Determine fallback font using CoreText. This uses the language isocode
@@ -469,8 +482,8 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 						CFRelease(font_name);
 						/* Skip some inappropriate or ugly looking fonts that have better alternatives. */
 						if (strncmp(name, "Courier", 7) == 0 || strncmp(name, "Apple Symbols", 13) == 0 ||
-							strncmp(name, ".Aqua", 5) == 0 || strncmp(name, "LastResort", 10) == 0 ||
-							strncmp(name, "GB18030 Bitmap", 14) == 0) continue;
+								strncmp(name, ".Aqua", 5) == 0 || strncmp(name, "LastResort", 10) == 0 ||
+								strncmp(name, "GB18030 Bitmap", 14) == 0) continue;
 
 						/* Save result. */
 						strecpy(settings->small_font,  name, lastof(settings->small_font));
@@ -497,8 +510,15 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 		 * ATSUI was deprecated with 10.6 and is only partially available in
 		 * 64-bit mode. */
 
+		/* Remove all control characters in the range from SCC_CONTROL_START to
+		 * SCC_CONTROL_END as well as all ASCII < 0x20 from the string as it will
+		 * mess with the automatic font detection */
+		char buff[256]; // This length is enough to find a suitable replacement font
+		strecpy(buff, str, lastof(buff));
+		str_validate(buff, lastof(buff), true, false);
+
 		/* Extract a UniChar represenation of the sample string. */
-		CFStringRef cf_str = CFStringCreateWithCString(kCFAllocatorDefault, str, kCFStringEncodingUTF8);
+		CFStringRef cf_str = CFStringCreateWithCString(kCFAllocatorDefault, buff, kCFStringEncodingUTF8);
 		if (cf_str == NULL) {
 			/* Something went wrong. Corrupt/invalid sample string? */
 			return false;
@@ -536,7 +556,7 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 				ATSUSetAttributes(style, 1, &tag, &size, &val);
 				offset += run_len;
 			}
-			/* Exit if the end of the string is reached or some other error occured. */
+			/* Exit if the end of the string is reached or some other error occurred. */
 		} while (os_err == kATSUFontsMatched && offset < (UniCharArrayOffset)str_len);
 
 		if (os_err == noErr || os_err == kATSUFontsMatched) {
@@ -589,10 +609,14 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 		}
 	 }
 
+	callback(NULL);
 	return result;
 }
 
-#elif defined(WITH_FONTCONFIG)
+#elif defined(WITH_FONTCONFIG) /* end ifdef __APPLE__ */
+/* ========================================================================================
+ * FontConfig (unix) support
+ * ======================================================================================== */
 static FT_Error GetFontByFaceName(const char *font_name, FT_Face *face)
 {
 	FT_Error err = FT_Err_Cannot_Open_Resource;
@@ -659,7 +683,7 @@ static FT_Error GetFontByFaceName(const char *font_name, FT_Face *face)
 	return err;
 }
 
-bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str)
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, SetFallbackFontCallback *callback)
 {
 	if (!FcInit()) return false;
 
@@ -669,79 +693,78 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 	 * before the _ of e.g. en_GB is used, so "remove" everything after
 	 * the _. */
 	char lang[16];
-	strecpy(lang, language_isocode, lastof(lang));
+	seprintf(lang, lastof(lang), ":lang=%s", language_isocode);
 	char *split = strchr(lang, '_');
 	if (split != NULL) *split = '\0';
 
-	FcPattern *pat;
-	FcPattern *match;
-	FcResult result;
-	FcChar8 *file;
-	FcFontSet *fs;
-	FcValue val;
-	val.type = FcTypeString;
-	val.u.s = (FcChar8*)lang;
+	/* First create a pattern to match the wanted language. */
+	FcPattern *pat = FcNameParse((FcChar8*)lang);
+	/* We only want to know the filename. */
+	FcObjectSet *os = FcObjectSetBuild(FC_FILE, NULL);
+	/* Get the list of filenames matching the wanted language. */
+	FcFontSet *fs = FcFontList(NULL, pat, os);
 
-	/* First create a pattern to match the wanted language */
-	pat = FcPatternCreate();
-	/* And fill it with the language and other defaults */
-	if (pat == NULL ||
-			!FcPatternAdd(pat, "lang", val, false) ||
-			!FcConfigSubstitute(0, pat, FcMatchPattern)) {
-		goto error_pattern;
+	/* We don't need these anymore. */
+	FcObjectSetDestroy(os);
+	FcPatternDestroy(pat);
+
+	if (fs != NULL) {
+		for (int i = 0; i < fs->nfont; i++) {
+			FcPattern *font = fs->fonts[i];
+
+			FcChar8 *file = NULL;
+			FcResult res = FcPatternGetString(font, FC_FILE, 0, &file);
+			if (res != FcResultMatch || file == NULL) {
+				continue;
+			}
+
+			strecpy(settings->small_font,  (const char*)file, lastof(settings->small_font));
+			strecpy(settings->medium_font, (const char*)file, lastof(settings->medium_font));
+			strecpy(settings->large_font,  (const char*)file, lastof(settings->large_font));
+
+			bool missing = callback(NULL);
+			DEBUG(freetype, 1, "Font \"%s\" misses%s glyphs", file, missing ? "" : " no");
+
+			if (!missing) {
+				ret = true;
+				break;
+			}
+		}
+
+		/* Clean up the list of filenames. */
+		FcFontSetDestroy(fs);
 	}
 
-	FcDefaultSubstitute(pat);
-
-	/* Then create a font set and match that */
-	match = FcFontMatch(0, pat, &result);
-
-	if (match == NULL) {
-		goto error_pattern;
-	}
-
-	/* Find all fonts that do match */
-	fs = FcFontSetCreate();
-	FcFontSetAdd(fs, match);
-
-	/* And take the first, if it exists */
-	if (fs->nfont <= 0 || FcPatternGetString(fs->fonts[0], FC_FILE, 0, &file)) {
-		goto error_fontset;
-	}
-
-	strecpy(settings->small_font,  (const char*)file, lastof(settings->small_font));
-	strecpy(settings->medium_font, (const char*)file, lastof(settings->medium_font));
-	strecpy(settings->large_font,  (const char*)file, lastof(settings->large_font));
-
-	ret = true;
-
-error_fontset:
-	FcFontSetDestroy(fs);
-error_pattern:
-	if (pat != NULL) FcPatternDestroy(pat);
 	FcFini();
 	return ret;
 }
 
 #else /* without WITH_FONTCONFIG */
 FT_Error GetFontByFaceName(const char *font_name, FT_Face *face) {return FT_Err_Cannot_Open_Resource;}
-bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str) { return false; }
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, SetFallbackFontCallback *callback) { return false; }
 #endif /* WITH_FONTCONFIG */
 
 static void SetFontGeometry(FT_Face face, FontSize size, int pixels)
 {
-	FT_Set_Pixel_Sizes(face, 0, pixels);
+	FT_Error err = FT_Set_Pixel_Sizes(face, 0, pixels);
+	if (err == FT_Err_Invalid_Pixel_Size) {
 
-	if (FT_IS_SCALABLE(face)) {
-		int asc = face->ascender * pixels / face->units_per_EM;
-		int dec = face->descender * pixels / face->units_per_EM;
+		/* Find nearest size to that requested */
+		FT_Bitmap_Size *bs = face->available_sizes;
+		int i = face->num_fixed_sizes;
+		int n = bs->height;
+		for (; --i; bs++) {
+			if (abs(pixels - bs->height) < abs(pixels - n)) n = bs->height;
+		}
 
-		_ascender[size] = asc;
-		_font_height[size] = asc - dec;
-	} else {
-		_ascender[size] = pixels;
-		_font_height[size] = pixels;
+		FT_Set_Pixel_Sizes(face, 0, n);
 	}
+
+	int asc = face->size->metrics.ascender >> 6;
+	int dec = face->size->metrics.descender >> 6;
+
+	_ascender[size] = asc;
+	_font_height[size] = asc - dec;
 }
 
 /**
@@ -873,6 +896,7 @@ static FT_Face GetFontFace(FontSize size)
 struct GlyphEntry {
 	Sprite *sprite;
 	byte width;
+	bool duplicate;
 };
 
 
@@ -900,6 +924,7 @@ static void ResetGlyphCache()
 			if (_glyph_ptr[i][j] == NULL) continue;
 
 			for (int k = 0; k < 256; k++) {
+				if (_glyph_ptr[i][j][k].duplicate) continue;
 				free(_glyph_ptr[i][j][k].sprite);
 			}
 
@@ -919,7 +944,7 @@ static GlyphEntry *GetGlyphPtr(FontSize size, WChar key)
 }
 
 
-static void SetGlyphPtr(FontSize size, WChar key, const GlyphEntry *glyph)
+static void SetGlyphPtr(FontSize size, WChar key, const GlyphEntry *glyph, bool duplicate = false)
 {
 	if (_glyph_ptr[size] == NULL) {
 		DEBUG(freetype, 3, "Allocating root glyph cache for size %u", size);
@@ -932,8 +957,9 @@ static void SetGlyphPtr(FontSize size, WChar key, const GlyphEntry *glyph)
 	}
 
 	DEBUG(freetype, 4, "Set glyph for unicode character 0x%04X, size %u", key, size);
-	_glyph_ptr[size][GB(key, 8, 8)][GB(key, 0, 8)].sprite = glyph->sprite;
-	_glyph_ptr[size][GB(key, 8, 8)][GB(key, 0, 8)].width  = glyph->width;
+	_glyph_ptr[size][GB(key, 8, 8)][GB(key, 0, 8)].sprite    = glyph->sprite;
+	_glyph_ptr[size][GB(key, 8, 8)][GB(key, 0, 8)].width     = glyph->width;
+	_glyph_ptr[size][GB(key, 8, 8)][GB(key, 0, 8)].duplicate = duplicate;
 }
 
 static void *AllocateFont(size_t size)
@@ -986,7 +1012,26 @@ const Sprite *GetGlyph(FontSize size, WChar key)
 
 	bool aa = GetFontAAState(size);
 
-	FT_Load_Char(face, key, FT_LOAD_DEFAULT);
+	FT_UInt glyph_index = FT_Get_Char_Index(face, key);
+	if (glyph_index == 0) {
+		if (key == '?') {
+			/* The font misses the '?' character. Use sprite font. */
+			SpriteID sprite = GetUnicodeGlyph(size, key);
+			Sprite *spr = (Sprite*)GetRawSprite(sprite, ST_FONT, AllocateFont);
+			assert(spr != NULL);
+			new_glyph.sprite = spr;
+			new_glyph.width  = spr->width + (size != FS_NORMAL);
+			SetGlyphPtr(size, key, &new_glyph, false);
+			return new_glyph.sprite;
+		} else {
+			/* Use '?' for missing characters. */
+			GetGlyph(size, '?');
+			glyph = GetGlyphPtr(size, '?');
+			SetGlyphPtr(size, key, glyph, true);
+			return glyph->sprite;
+		}
+	}
+	FT_Load_Glyph(face, glyph_index, FT_LOAD_DEFAULT);
 	FT_Render_Glyph(face->glyph, aa ? FT_RENDER_MODE_NORMAL : FT_RENDER_MODE_MONO);
 
 	/* Despite requesting a normal glyph, FreeType may have returned a bitmap */
@@ -996,6 +1041,9 @@ const Sprite *GetGlyph(FontSize size, WChar key)
 	width  = max(1, slot->bitmap.width + (size == FS_NORMAL));
 	height = max(1, slot->bitmap.rows  + (size == FS_NORMAL));
 
+	/* Limit glyph size to prevent overflows later on. */
+	if (width > 256 || height > 256) usererror("Font glyph is too large");
+
 	/* FreeType has rendered the glyph, now we allocate a sprite and copy the image into it */
 	sprite.AllocateData(width * height);
 	sprite.width = width;
