@@ -1,4 +1,4 @@
-/* $Id: gamelog.cpp 20519 2010-08-16 22:20:20Z rubidium $ */
+/* $Id: gamelog.cpp 21890 2011-01-22 14:52:20Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,15 +10,13 @@
 /** @file gamelog.cpp Definition of functions used for logging of important changes in the game */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "saveload/saveload.h"
-#include "core/alloc_func.hpp"
-#include "variables.h"
 #include "string_func.h"
 #include "settings_type.h"
 #include "gamelog_internal.h"
 #include "console_func.h"
 #include "debug.h"
+#include "date_func.h"
 #include "rev.h"
 
 #include <stdarg.h>
@@ -39,7 +37,8 @@ uint _gamelog_actions         = 0;           ///< number of actions
 static LoggedAction *_current_action = NULL; ///< current action we are logging, NULL when there is no action active
 
 
-/** Stores information about new action, but doesn't allocate it
+/**
+ * Stores information about new action, but doesn't allocate it
  * Action is allocated only when there is at least one change
  * @param at type of action
  */
@@ -49,7 +48,8 @@ void GamelogStartAction(GamelogActionType at)
 	_gamelog_action_type = at;
 }
 
-/** Stops logging of any changes
+/**
+ * Stops logging of any changes
  */
 void GamelogStopAction()
 {
@@ -63,7 +63,8 @@ void GamelogStopAction()
 	if (print) GamelogPrintDebug(5);
 }
 
-/** Resets and frees all memory allocated - used before loading or starting a new game
+/**
+ * Resets and frees all memory allocated - used before loading or starting a new game
  */
 void GamelogReset()
 {
@@ -85,11 +86,9 @@ void GamelogReset()
 	_current_action  = NULL;
 }
 
-enum {
-	GAMELOG_BUF_LEN = 1024 ///< length of buffer for one line of text
-};
+static const uint GAMELOG_BUF_LEN = 1024; ///< length of buffer for one line of text
 
-static int _dbgofs = 0; ///< offset in current output buffer
+static uint _dbgofs = 0; ///< offset in current output buffer
 
 static void AddDebugText(char *buf, const char *s, ...) WARN_FORMAT(2, 3);
 
@@ -105,34 +104,34 @@ static void AddDebugText(char *buf, const char *s, ...)
 }
 
 
-/** Prints GRF filename if found
- * @param buf The location in the _dbgofs buffer to draw
- * @param grfid GRF which filename to print
- */
-static void PrintGrfFilename(char *buf, uint grfid)
-{
-	const GRFConfig *gc = FindGRFConfig(grfid);
-
-	if (gc == NULL) return;
-
-	AddDebugText(buf, ", filename: %s", gc->filename);
-}
-
-/** Prints GRF ID, checksum and filename if found
+/**
+ * Prints GRF ID, checksum and filename if found
  * @param buf The location in the _dbgofs buffer to draw
  * @param grfid GRF ID
- * @param md5sum array of md5sum to print
+ * @param md5sum array of md5sum to print, if known
+ * @param gc GrfConfig, if known
  */
-static void PrintGrfInfo(char *buf, uint grfid, const uint8 *md5sum)
+static void PrintGrfInfo(char *buf, uint grfid, const uint8 *md5sum, const GRFConfig *gc)
 {
 	char txt[40];
 
-	md5sumToString(txt, lastof(txt), md5sum);
-
-	AddDebugText(buf, "GRF ID %08X, checksum %s", BSWAP32(grfid), txt);
-
-	PrintGrfFilename(buf, grfid);
+	if (md5sum != NULL) {
+		md5sumToString(txt, lastof(txt), md5sum);
+		AddDebugText(buf, "GRF ID %08X, checksum %s", BSWAP32(grfid), txt);
+	} else {
+		AddDebugText(buf, "GRF ID %08X", BSWAP32(grfid));
+	}
 
+	if (gc != NULL) {
+		AddDebugText(buf, ", filename: %s (md5sum matches)", gc->filename);
+	} else {
+		gc = FindGRFConfig(grfid, FGCM_ANY);
+		if (gc != NULL) {
+			AddDebugText(buf, ", filename: %s (matches GRFID only)", gc->filename);
+		} else {
+			AddDebugText(buf, ", unknown GRF");
+		}
+	}
 	return;
 }
 
@@ -150,6 +149,20 @@ static const char * const la_text[] = {
 
 assert_compile(lengthof(la_text) == GLAT_END);
 
+/**
+ * Information about the presence of a Grf at a certain point during gamelog history
+ * Note about missing Grfs:
+ * Changes to missing Grfs are not logged including manual removal of the Grf.
+ * So if the gamelog tells a Grf is missing we do not know whether it was readded or completely removed
+ * at some later point.
+ */
+struct GRFPresence{
+	const GRFConfig *gc;  ///< GRFConfig, if known
+	bool was_missing;     ///< Grf was missing during some gameload in the past
+
+	GRFPresence(const GRFConfig *gc) : gc(gc), was_missing(false) {}
+};
+typedef SmallMap<uint32, GRFPresence> GrfIDMapping;
 
 /**
  * Prints active gamelog
@@ -158,6 +171,7 @@ assert_compile(lengthof(la_text) == GLAT_END);
 void GamelogPrint(GamelogPrintProc *proc)
 {
 	char buf[GAMELOG_BUF_LEN];
+	GrfIDMapping grf_names;
 
 	proc("---- gamelog start ----");
 
@@ -229,40 +243,71 @@ void GamelogPrint(GamelogPrintProc *proc)
 					AddDebugText(buf, "Setting changed: %s : %d -> %d", lc->setting.name, lc->setting.oldval, lc->setting.newval);
 					break;
 
-				case GLCT_GRFADD:
+				case GLCT_GRFADD: {
+					const GRFConfig *gc = FindGRFConfig(lc->grfadd.grfid, FGCM_EXACT, lc->grfadd.md5sum);
 					AddDebugText(buf, "Added NewGRF: ");
-					PrintGrfInfo(buf, lc->grfadd.grfid, lc->grfadd.md5sum);
+					PrintGrfInfo(buf, lc->grfadd.grfid, lc->grfadd.md5sum, gc);
+					GrfIDMapping::Pair *gm = grf_names.Find(lc->grfrem.grfid);
+					if (gm != grf_names.End() && !gm->second.was_missing) AddDebugText(buf, ". Gamelog inconsistency: GrfID was already added!");
+					grf_names[lc->grfadd.grfid] = gc;
 					break;
+				}
 
-				case GLCT_GRFREM:
-					AddDebugText(buf, "Removed NewGRF: %08X", BSWAP32(lc->grfrem.grfid));
-					PrintGrfFilename(buf, lc->grfrem.grfid);
+				case GLCT_GRFREM: {
+					GrfIDMapping::Pair *gm = grf_names.Find(lc->grfrem.grfid);
+					AddDebugText(buf, la->at == GLAT_LOAD ? "Missing NewGRF: " : "Removed NewGRF: ");
+					PrintGrfInfo(buf, lc->grfrem.grfid, NULL, gm != grf_names.End() ? gm->second.gc : NULL);
+					if (gm == grf_names.End()) {
+						AddDebugText(buf, ". Gamelog inconsistency: GrfID was never added!");
+					} else {
+						if (la->at == GLAT_LOAD) {
+							/* Missing grfs on load are not removed from the configuration */
+							gm->second.was_missing = true;
+						} else {
+							grf_names.Erase(gm);
+						}
+					}
 					break;
+				}
 
-				case GLCT_GRFCOMPAT:
+				case GLCT_GRFCOMPAT: {
+					const GRFConfig *gc = FindGRFConfig(lc->grfadd.grfid, FGCM_EXACT, lc->grfadd.md5sum);
 					AddDebugText(buf, "Compatible NewGRF loaded: ");
-					PrintGrfInfo(buf, lc->grfcompat.grfid, lc->grfcompat.md5sum);
+					PrintGrfInfo(buf, lc->grfcompat.grfid, lc->grfcompat.md5sum, gc);
+					if (!grf_names.Contains(lc->grfcompat.grfid)) AddDebugText(buf, ". Gamelog inconsistency: GrfID was never added!");
+					grf_names[lc->grfcompat.grfid] = gc;
 					break;
+				}
 
-				case GLCT_GRFPARAM:
-					AddDebugText(buf, "GRF parameter changed: %08X", BSWAP32(lc->grfparam.grfid));
-					PrintGrfFilename(buf, lc->grfparam.grfid);
+				case GLCT_GRFPARAM: {
+					GrfIDMapping::Pair *gm = grf_names.Find(lc->grfrem.grfid);
+					AddDebugText(buf, "GRF parameter changed: ");
+					PrintGrfInfo(buf, lc->grfparam.grfid, NULL, gm != grf_names.End() ? gm->second.gc : NULL);
+					if (gm == grf_names.End()) AddDebugText(buf, ". Gamelog inconsistency: GrfID was never added!");
 					break;
+				}
 
-				case GLCT_GRFMOVE:
+				case GLCT_GRFMOVE: {
+					GrfIDMapping::Pair *gm = grf_names.Find(lc->grfrem.grfid);
 					AddDebugText(buf, "GRF order changed: %08X moved %d places %s",
 						BSWAP32(lc->grfmove.grfid), abs(lc->grfmove.offset), lc->grfmove.offset >= 0 ? "down" : "up" );
-					PrintGrfFilename(buf, lc->grfmove.grfid);
+					PrintGrfInfo(buf, lc->grfmove.grfid, NULL, gm != grf_names.End() ? gm->second.gc : NULL);
+					if (gm == grf_names.End()) AddDebugText(buf, ". Gamelog inconsistency: GrfID was never added!");
 					break;
+				}
 
-				case GLCT_GRFBUG:
+				case GLCT_GRFBUG: {
+					GrfIDMapping::Pair *gm = grf_names.Find(lc->grfrem.grfid);
 					switch (lc->grfbug.bug) {
 						default: NOT_REACHED();
 						case GBUG_VEH_LENGTH:
 							AddDebugText(buf, "Rail vehicle changes length outside a depot: GRF ID %08X, internal ID 0x%X", BSWAP32(lc->grfbug.grfid), (uint)lc->grfbug.data);
-							PrintGrfFilename(buf, lc->grfbug.grfid);
 							break;
 					}
+					PrintGrfInfo(buf, lc->grfbug.grfid, NULL, gm != grf_names.End() ? gm->second.gc : NULL);
+					if (gm == grf_names.End()) AddDebugText(buf, ". Gamelog inconsistency: GrfID was never added!");
+					break;
+				}
 
 				case GLCT_EMERGENCY:
 					break;
@@ -294,7 +339,8 @@ static void GamelogPrintDebugProc(const char *s)
 }
 
 
-/** Prints gamelog to debug output. Code is executed even when
+/**
+ * Prints gamelog to debug output. Code is executed even when
  * there will be no output. It is called very seldom, so it
  * doesn't matter that much. At least it gives more uniform code...
  * @param level debug level we need to print stuff
@@ -306,7 +352,8 @@ void GamelogPrintDebug(int level)
 }
 
 
-/** Allocates new LoggedChange and new LoggedAction if needed.
+/**
+ * Allocates new LoggedChange and new LoggedAction if needed.
  * If there is no action active, NULL is returned.
  * @param ct type of change
  * @return new LoggedChange, or NULL if there is no action active
@@ -334,7 +381,8 @@ static LoggedChange *GamelogChange(GamelogChangeType ct)
 }
 
 
-/** Logs a emergency savegame
+/**
+ * Logs a emergency savegame
  */
 void GamelogEmergency()
 {
@@ -345,7 +393,8 @@ void GamelogEmergency()
 	GamelogStopAction();
 }
 
-/** Finds out if current game is a loaded emergency savegame.
+/**
+ * Finds out if current game is a loaded emergency savegame.
  */
 bool GamelogTestEmergency()
 {
@@ -362,7 +411,8 @@ bool GamelogTestEmergency()
 	return (emergency != NULL);
 }
 
-/** Logs a change in game revision
+/**
+ * Logs a change in game revision
  */
 void GamelogRevision()
 {
@@ -378,7 +428,8 @@ void GamelogRevision()
 	lc->revision.newgrf = _openttd_newgrf_version;
 }
 
-/** Logs a change in game mode (scenario editor or game)
+/**
+ * Logs a change in game mode (scenario editor or game)
  */
 void GamelogMode()
 {
@@ -391,7 +442,8 @@ void GamelogMode()
 	lc->mode.landscape = _settings_game.game_creation.landscape;
 }
 
-/** Logs loading from savegame without gamelog
+/**
+ * Logs loading from savegame without gamelog
  */
 void GamelogOldver()
 {
@@ -404,7 +456,8 @@ void GamelogOldver()
 	lc->oldver.version = (_savegame_type == SGT_OTTD ? ((uint32)_sl_version << 8 | _sl_minor_version) : _ttdp_version);
 }
 
-/** Logs change in game settings. Only non-networksafe settings are logged
+/**
+ * Logs change in game settings. Only non-networksafe settings are logged
  * @param name setting name
  * @param oldval old setting value
  * @param newval new setting value
@@ -422,7 +475,8 @@ void GamelogSetting(const char *name, int32 oldval, int32 newval)
 }
 
 
-/** Finds out if current revision is different than last revision stored in the savegame.
+/**
+ * Finds out if current revision is different than last revision stored in the savegame.
  * Appends GLCT_REVISION when the revision string changed
  */
 void GamelogTestRevision()
@@ -444,7 +498,8 @@ void GamelogTestRevision()
 	}
 }
 
-/** Finds last stored game mode or landscape.
+/**
+ * Finds last stored game mode or landscape.
  * Any change is logged
  */
 void GamelogTestMode()
@@ -463,7 +518,8 @@ void GamelogTestMode()
 }
 
 
-/** Logs triggered GRF bug.
+/**
+ * Logs triggered GRF bug.
  * @param grfid ID of problematic GRF
  * @param bug type of bug, @see enum GRFBugs
  * @param data additional data
@@ -480,7 +536,8 @@ static void GamelogGRFBug(uint32 grfid, byte bug, uint64 data)
 	lc->grfbug.bug   = bug;
 }
 
-/** Logs GRF bug - rail vehicle has different length after reversing.
+/**
+ * Logs GRF bug - rail vehicle has different length after reversing.
  * Ensures this is logged only once for each GRF and engine type
  * This check takes some time, but it is called pretty seldom, so it
  * doesn't matter that much (ideally it shouldn't be called at all).
@@ -509,7 +566,8 @@ bool GamelogGRFBugReverse(uint32 grfid, uint16 internal_id)
 }
 
 
-/** Decides if GRF should be logged
+/**
+ * Decides if GRF should be logged
  * @param g grf to determine
  * @return true iff GRF is not static and is loaded
  */
@@ -518,7 +576,8 @@ static inline bool IsLoggableGrfConfig(const GRFConfig *g)
 	return !HasBit(g->flags, GCF_STATIC) && g->status != GCS_NOT_FOUND;
 }
 
-/** Logs removal of a GRF
+/**
+ * Logs removal of a GRF
  * @param grfid ID of removed GRF
  */
 void GamelogGRFRemove(uint32 grfid)
@@ -531,7 +590,8 @@ void GamelogGRFRemove(uint32 grfid)
 	lc->grfrem.grfid = grfid;
 }
 
-/** Logs adding of a GRF
+/**
+ * Logs adding of a GRF
  * @param newg added GRF
  */
 void GamelogGRFAdd(const GRFConfig *newg)
@@ -543,10 +603,11 @@ void GamelogGRFAdd(const GRFConfig *newg)
 	LoggedChange *lc = GamelogChange(GLCT_GRFADD);
 	if (lc == NULL) return;
 
-	memcpy(&lc->grfadd, newg, sizeof(GRFIdentifier));
+	lc->grfadd = newg->ident;
 }
 
-/** Logs loading compatible GRF
+/**
+ * Logs loading compatible GRF
  * (the same ID, but different MD5 hash)
  * @param newg new (updated) GRF
  */
@@ -557,10 +618,11 @@ void GamelogGRFCompatible(const GRFIdentifier *newg)
 	LoggedChange *lc = GamelogChange(GLCT_GRFCOMPAT);
 	if (lc == NULL) return;
 
-	memcpy(&lc->grfcompat, newg, sizeof(GRFIdentifier));
+	lc->grfcompat = *newg;
 }
 
-/** Logs changing GRF order
+/**
+ * Logs changing GRF order
  * @param grfid GRF that is moved
  * @param offset how far it is moved, positive = moved down
  */
@@ -575,7 +637,8 @@ static void GamelogGRFMove(uint32 grfid, int32 offset)
 	lc->grfmove.offset = offset;
 }
 
-/** Logs change in GRF parameters.
+/**
+ * Logs change in GRF parameters.
  * Details about parameters changed are not stored
  * @param grfid ID of GRF to store
  */
@@ -589,7 +652,8 @@ static void GamelogGRFParameters(uint32 grfid)
 	lc->grfparam.grfid = grfid;
 }
 
-/** Logs adding of list of GRFs.
+/**
+ * Logs adding of list of GRFs.
  * Useful when old savegame is loaded or when new game is started
  * @param newg head of GRF linked list
  */
@@ -608,7 +672,8 @@ struct GRFList {
 	const GRFConfig *grf[];
 };
 
-/** Generates GRFList
+/**
+ * Generates GRFList
  * @param grfc head of GRF linked list
  */
 static GRFList *GenerateGRFList(const GRFConfig *grfc)
@@ -628,7 +693,8 @@ static GRFList *GenerateGRFList(const GRFConfig *grfc)
 	return list;
 }
 
-/** Compares two NewGRF lists and logs any change
+/**
+ * Compares two NewGRF lists and logs any change
  * @param oldc original GRF list
  * @param newc new GRF list
  */
@@ -643,10 +709,10 @@ void GamelogGRFUpdate(const GRFConfig *oldc, const GRFConfig *newc)
 		const GRFConfig *og = ol->grf[o];
 		const GRFConfig *ng = nl->grf[n];
 
-		if (og->grfid != ng->grfid) {
+		if (og->ident.grfid != ng->ident.grfid) {
 			uint oi, ni;
 			for (oi = 0; oi < ol->n; oi++) {
-				if (ol->grf[oi]->grfid == nl->grf[n]->grfid) break;
+				if (ol->grf[oi]->ident.grfid == nl->grf[n]->ident.grfid) break;
 			}
 			if (oi < o) {
 				/* GRF was moved, this change has been logged already */
@@ -659,7 +725,7 @@ void GamelogGRFUpdate(const GRFConfig *oldc, const GRFConfig *newc)
 				continue;
 			}
 			for (ni = 0; ni < nl->n; ni++) {
-				if (nl->grf[ni]->grfid == ol->grf[o]->grfid) break;
+				if (nl->grf[ni]->ident.grfid == ol->grf[o]->ident.grfid) break;
 			}
 			if (ni < n) {
 				/* GRF was moved, this change has been logged already */
@@ -668,7 +734,7 @@ void GamelogGRFUpdate(const GRFConfig *oldc, const GRFConfig *newc)
 			}
 			if (ni == nl->n) {
 				/* GRF couldn't be found in the NEW list, GRF was REMOVED */
-				GamelogGRFRemove(ol->grf[o++]->grfid);
+				GamelogGRFRemove(ol->grf[o++]->ident.grfid);
 				continue;
 			}
 
@@ -682,18 +748,18 @@ void GamelogGRFUpdate(const GRFConfig *oldc, const GRFConfig *newc)
 
 			if (ni >= oi) { // prefer the one that is moved further
 				/* GRF was moved down */
-				GamelogGRFMove(ol->grf[o++]->grfid, ni);
+				GamelogGRFMove(ol->grf[o++]->ident.grfid, ni);
 			} else {
-				GamelogGRFMove(nl->grf[n++]->grfid, -(int)oi);
+				GamelogGRFMove(nl->grf[n++]->ident.grfid, -(int)oi);
 			}
 		} else {
-			if (memcmp(og->md5sum, ng->md5sum, sizeof(og->md5sum)) != 0) {
+			if (memcmp(og->ident.md5sum, ng->ident.md5sum, sizeof(og->ident.md5sum)) != 0) {
 				/* md5sum changed, probably loading 'compatible' GRF */
-				GamelogGRFCompatible(nl->grf[n]);
+				GamelogGRFCompatible(&nl->grf[n]->ident);
 			}
 
 			if (og->num_params != ng->num_params || memcmp(og->param, ng->param, og->num_params * sizeof(og->param[0])) != 0) {
-				GamelogGRFParameters(ol->grf[o]->grfid);
+				GamelogGRFParameters(ol->grf[o]->ident.grfid);
 			}
 
 			o++;
@@ -701,36 +767,9 @@ void GamelogGRFUpdate(const GRFConfig *oldc, const GRFConfig *newc)
 		}
 	}
 
-	while (o < ol->n) GamelogGRFRemove(ol->grf[o++]->grfid); // remaining GRFs were removed ...
-	while (n < nl->n) GamelogGRFAdd   (nl->grf[n++]);    // ... or added
+	while (o < ol->n) GamelogGRFRemove(ol->grf[o++]->ident.grfid); // remaining GRFs were removed ...
+	while (n < nl->n) GamelogGRFAdd   (nl->grf[n++]);              // ... or added
 
 	free(ol);
 	free(nl);
 }
-
-/**
- * Get the MD5 checksum of the original NewGRF that was loaded.
- * @param grfid the GRF ID to search for
- * @param md5sum the MD5 checksum to write to.
- */
-void GamelogGetOriginalGRFMD5Checksum(uint32 grfid, byte *md5sum)
-{
-	const LoggedAction *la = &_gamelog_action[_gamelog_actions - 1];
-	/* There should always be a "start game" action */
-	assert(_gamelog_actions > 0);
-
-	do {
-		const LoggedChange *lc = &la->change[la->changes - 1];
-		/* There should always be at least one change per action */
-		assert(la->changes > 0);
-
-		do {
-			if (lc->ct == GLCT_GRFADD && lc->grfadd.grfid == grfid) {
-				memcpy(md5sum, lc->grfadd.md5sum, sizeof(lc->grfadd.md5sum));
-				return;
-			}
-		} while (lc-- != la->change);
-	} while (la-- != _gamelog_action);
-
-	DEBUG(grf, 0, "The NewGRF with GRF ID %08X is not listed in the game's log. Can't recover the original MD5 checksum.", BSWAP32(grfid));
-}
