@@ -1,4 +1,4 @@
-/* $Id: graph_gui.cpp 20576 2010-08-20 10:31:20Z rubidium $ */
+/* $Id: graph_gui.cpp 22252 2011-03-14 19:03:17Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,7 +10,6 @@
 /** @file graph_gui.cpp GUI that shows performance graphs. */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "graph_gui.h"
 #include "window_gui.h"
 #include "company_base.h"
@@ -23,6 +22,8 @@
 #include "gfx_func.h"
 #include "sortlist_type.h"
 #include "core/geometry_func.hpp"
+#include "math.h"
+#include "currency.h"
 
 #include "table/strings.h"
 #include "table/sprites.h"
@@ -59,11 +60,6 @@ struct GraphLegendWindow : Window {
 		}
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		if (!IsInsideMM(widget, GLW_FIRST_COMPANY, MAX_COMPANIES + GLW_FIRST_COMPANY)) return;
@@ -72,7 +68,7 @@ struct GraphLegendWindow : Window {
 
 		if (!Company::IsValidID(cid)) return;
 
-		bool rtl = _dynlang.text_dir == TD_RTL;
+		bool rtl = _current_text_dir == TD_RTL;
 
 		DrawCompanyIcon(cid, rtl ? r.right - 16 : r.left + 2, r.top + 2 + (FONT_HEIGHT_NORMAL - 10) / 2);
 
@@ -95,8 +91,14 @@ struct GraphLegendWindow : Window {
 		InvalidateWindowData(WC_COMPANY_VALUE, 0);
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
+		if (!gui_scope) return;
 		if (Company::IsValidID(data)) return;
 
 		SetBit(_legend_excluded_companies, data);
@@ -154,6 +156,12 @@ static void ShowGraphLegend()
 	AllocateWindowDescFront<GraphLegendWindow>(&_graph_legend_desc, 0);
 }
 
+/** Contains the interval of a graph's data. */
+struct ValuesInterval {
+	OverflowSafeInt64 highest; ///< Highest value of this interval. Must be zero or greater.
+	OverflowSafeInt64 lowest;  ///< Lowest value of this interval. Must be zero or less.
+};
+
 /******************/
 /* BASE OF GRAPHS */
 /*****************/
@@ -162,25 +170,22 @@ static void ShowGraphLegend()
 enum CompanyValueWidgets {
 	BGW_KEY_BUTTON,
 	BGW_BACKGROUND,
+	BGW_GRAPH,
+	BGW_RESIZE,
 };
 
 struct BaseGraphWindow : Window {
 protected:
-	enum {
-		GRAPH_MAX_DATASETS = 32,
-		GRAPH_AXIS_LINE_COLOUR  = 215,
-		GRAPH_NUM_MONTHS = 24, ///< Number of months displayed in the graph.
+	static const int GRAPH_MAX_DATASETS     =  32;
+	static const int GRAPH_AXIS_LINE_COLOUR = 215;
+	static const int GRAPH_NUM_MONTHS       =  24; ///< Number of months displayed in the graph.
 
-		GRAPH_NUM_LINES_Y = 9, ///< How many horizontal lines to draw.
-		/* 9 is convenient as that means the distance between them is the gd_height of the graph / 8,
-		 * which is the same
-		 * as height >> 3. */
-	};
+	static const int MIN_GRAPH_NUM_LINES_Y  =   9; ///< Minimal number of horizontal lines to draw.
+	static const int MIN_GRID_PIXEL_SIZE    =  20; ///< Minimum distance between graph lines.
 
 	uint excluded_data; ///< bitmask of the datasets that shouldn't be displayed.
 	byte num_dataset;
 	byte num_on_x_axis;
-	bool has_negative_values;
 	byte num_vert_lines;
 	static const TextColour graph_axis_label_colour = TC_BLACK; ///< colour of the graph axis label.
 
@@ -200,13 +205,16 @@ protected:
 	OverflowSafeInt64 cost[GRAPH_MAX_DATASETS][GRAPH_NUM_MONTHS]; ///< Stored costs for the last #GRAPH_NUM_MONTHS months
 
 	/**
-	 * Get the highest value of the graph's data. Excluded data is ignored to allow showing smaller values in
+	 * Get the interval that contains the graph's data. Excluded data is ignored to show smaller values in
 	 * better detail when disabling higher ones.
-	 * @return Highest value of the graph (ignoring disabled data).
+	 * @param num_hori_lines Number of horizontal lines to be drawn.
+	 * @return Highest and lowest values of the graph (ignoring disabled data).
 	 */
-	int64 GetHighestValue() const
+	ValuesInterval GetValuesInterval(int num_hori_lines) const
 	{
-		OverflowSafeInt64 highest_value = 0;
+		ValuesInterval current_interval;
+		current_interval.highest = INT64_MIN;
+		current_interval.lowest  = INT64_MAX;
 
 		for (int i = 0; i < this->num_dataset; i++) {
 			if (HasBit(this->excluded_data, i)) continue;
@@ -214,39 +222,60 @@ protected:
 				OverflowSafeInt64 datapoint = this->cost[i][j];
 
 				if (datapoint != INVALID_DATAPOINT) {
-					/* For now, if the graph has negative values the scaling is
-					 * symmetrical about the x axis, so take the absolute value
-					 * of each data point. */
-					highest_value = max(highest_value, abs(datapoint));
+					current_interval.highest = max(current_interval.highest, datapoint);
+					current_interval.lowest  = min(current_interval.lowest, datapoint);
 				}
 			}
 		}
 
-		/* Prevent showing the highest value too close to the graph upper limit. */
-		highest_value = (11 * highest_value) / 10;
-		/* Avoid using zero as the highest value. */
-		if (highest_value == 0) highest_value = GRAPH_NUM_LINES_Y - 1;
-		/* Round up highest_value so that it will divide cleanly into the number of
-		 * axis labels used. */
-		int round_val = highest_value % (GRAPH_NUM_LINES_Y - 1);
-		if (round_val != 0) highest_value += (GRAPH_NUM_LINES_Y - 1 - round_val);
+		/* Prevent showing values too close to the graph limits. */
+		current_interval.highest = (11 * current_interval.highest) / 10;
+		current_interval.lowest =  (11 * current_interval.lowest) / 10;
 
-		return highest_value;
+		/* Always include zero in the shown range. */
+		double abs_lower  = (current_interval.lowest > 0) ? 0 : (double)abs(current_interval.lowest);
+		double abs_higher = (current_interval.highest < 0) ? 0 : (double)current_interval.highest;
+
+		int num_pos_grids;
+		int64 grid_size;
+
+		if (abs_lower != 0 || abs_higher != 0) {
+			/* The number of grids to reserve for the positive part is: */
+			num_pos_grids = (int)floor(0.5 + num_hori_lines * abs_higher / (abs_higher + abs_lower));
+
+			/* If there are any positive or negative values, force that they have at least one grid. */
+			if (num_pos_grids == 0 && abs_higher != 0) num_pos_grids++;
+			if (num_pos_grids == num_hori_lines && abs_lower != 0) num_pos_grids--;
+
+			/* Get the required grid size for each side and use the maximum one. */
+			int64 grid_size_higher = (abs_higher > 0) ? ((int64)abs_higher + num_pos_grids - 1) / num_pos_grids : 0;
+			int64 grid_size_lower = (abs_lower > 0) ? ((int64)abs_lower + num_hori_lines - num_pos_grids - 1) / (num_hori_lines - num_pos_grids) : 0;
+			grid_size = max(grid_size_higher, grid_size_lower);
+		} else {
+			/* If both values are zero, show an empty graph. */
+			num_pos_grids = num_hori_lines / 2;
+			grid_size = 1;
+		}
+
+		current_interval.highest = num_pos_grids * grid_size;
+		current_interval.lowest = -(num_hori_lines - num_pos_grids) * grid_size;
+		return current_interval;
 	}
 
-	uint GetYLabelWidth(int64 highest_value) const
+	/**
+	 * Get width for Y labels.
+	 * @param current_interval Interval that contains all of the graph data.
+	 * @param num_hori_lines Number of horizontal lines to be drawn.
+	 */
+	uint GetYLabelWidth(ValuesInterval current_interval, int num_hori_lines) const
 	{
 		/* draw text strings on the y axis */
-		int64 y_label = highest_value;
-		int64 y_label_separation = highest_value / (GRAPH_NUM_LINES_Y - 1);
-
-		/* If there are negative values, the graph goes from highest_value to
-		 * -highest_value, not highest_value to 0. */
-		if (this->has_negative_values) y_label_separation *= 2;
+		int64 y_label = current_interval.highest;
+		int64 y_label_separation = (current_interval.highest - current_interval.lowest) / num_hori_lines;
 
 		uint max_width = 0;
 
-		for (int i = 0; i < GRAPH_NUM_LINES_Y; i++) {
+		for (int i = 0; i < (num_hori_lines + 1); i++) {
 			SetDParam(0, this->format_str_y_axis);
 			SetDParam(1, y_label);
 			Dimension d = GetStringBoundingBox(STR_GRAPH_Y_LABEL);
@@ -264,9 +293,9 @@ protected:
 	 */
 	void DrawGraph(Rect r) const
 	{
-		uint x, y;                       ///< Reused whenever x and y coordinates are needed.
-		OverflowSafeInt64 highest_value; ///< Highest value to be drawn.
-		int x_axis_offset;               ///< Distance from the top of the graph to the x axis.
+		uint x, y;               ///< Reused whenever x and y coordinates are needed.
+		ValuesInterval interval; ///< Interval that contains all of the graph data.
+		int x_axis_offset;       ///< Distance from the top of the graph to the x axis.
 
 		/* the colours and cost array of GraphDrawer must accomodate
 		 * both values for cargo and companies. So if any are higher, quit */
@@ -282,24 +311,29 @@ protected:
 		r.left   += 9;
 		r.right  -= 5;
 
-		highest_value = GetHighestValue();
+		/* Initial number of horizontal lines. */
+		int num_hori_lines = 160 / MIN_GRID_PIXEL_SIZE;
+		/* For the rest of the height, the number of horizontal lines will increase more slowly. */
+		int resize = (r.bottom - r.top - 160) / (2 * MIN_GRID_PIXEL_SIZE);
+		if (resize > 0) num_hori_lines += resize;
 
-		/* Get width for Y labels */
-		int label_width = GetYLabelWidth(highest_value);
+		interval = GetValuesInterval(num_hori_lines);
+
+		int label_width = GetYLabelWidth(interval, num_hori_lines);
 
 		r.left += label_width;
 
 		int x_sep = (r.right - r.left) / this->num_vert_lines;
-		int y_sep = (r.bottom - r.top) / (GRAPH_NUM_LINES_Y - 1);
+		int y_sep = (r.bottom - r.top) / num_hori_lines;
 
 		/* Redetermine right and bottom edge of graph to fit with the integer
 		 * separation values. */
 		r.right = r.left + x_sep * this->num_vert_lines;
-		r.bottom = r.top + y_sep * (GRAPH_NUM_LINES_Y - 1);
+		r.bottom = r.top + y_sep * num_hori_lines;
 
-		/* Where to draw the X axis */
-		x_axis_offset = r.bottom - r.top;
-		if (this->has_negative_values) x_axis_offset /= 2;
+		OverflowSafeInt64 interval_size = interval.highest + abs(interval.lowest);
+		/* Where to draw the X axis. Use floating point to avoid overflowing and results of zero. */
+		x_axis_offset = (int)((r.bottom - r.top) * (double)interval.highest / (double)interval_size);
 
 		/* Draw the vertical grid lines. */
 
@@ -314,7 +348,7 @@ protected:
 		/* Draw the horizontal grid lines. */
 		y = r.bottom;
 
-		for (int i = 0; i < GRAPH_NUM_LINES_Y; i++) {
+		for (int i = 0; i < (num_hori_lines + 1); i++) {
 			GfxFillRect(r.left - 3, y, r.left - 1, y, GRAPH_AXIS_LINE_COLOUR);
 			GfxFillRect(r.left, y, r.right, y, grid_colour);
 			y -= y_sep;
@@ -328,23 +362,18 @@ protected:
 		GfxFillRect(r.left, y, r.right, y, GRAPH_AXIS_LINE_COLOUR);
 
 		/* Find the largest value that will be drawn. */
-		if (this->num_on_x_axis == 0)
-			return;
+		if (this->num_on_x_axis == 0) return;
 
 		assert(this->num_on_x_axis > 0);
 		assert(this->num_dataset > 0);
 
 		/* draw text strings on the y axis */
-		int64 y_label = highest_value;
-		int64 y_label_separation = highest_value / (GRAPH_NUM_LINES_Y - 1);
-
-		/* If there are negative values, the graph goes from highest_value to
-		 * -highest_value, not highest_value to 0. */
-		if (this->has_negative_values) y_label_separation *= 2;
+		int64 y_label = interval.highest;
+		int64 y_label_separation = abs(interval.highest - interval.lowest) / num_hori_lines;
 
 		y = r.top - GetCharacterHeight(FS_SMALL) / 2;
 
-		for (int i = 0; i < GRAPH_NUM_LINES_Y; i++) {
+		for (int i = 0; i < (num_hori_lines + 1); i++) {
 			SetDParam(0, this->format_str_y_axis);
 			SetDParam(1, y_label);
 			DrawString(r.left - label_width - 4, r.left - 4, y, STR_GRAPH_Y_LABEL, graph_axis_label_colour, SA_RIGHT);
@@ -380,7 +409,7 @@ protected:
 
 			for (int i = 0; i < this->num_on_x_axis; i++) {
 				SetDParam(0, label);
-				DrawString(x + 1, x + x_sep - 1, y, STR_GRAPH_Y_LABEL_NUMBER, graph_axis_label_colour, SA_CENTER);
+				DrawString(x + 1, x + x_sep - 1, y, STR_GRAPH_Y_LABEL_NUMBER, graph_axis_label_colour, SA_HOR_CENTER);
 
 				label += this->x_values_increment;
 				x += x_sep;
@@ -421,8 +450,7 @@ protected:
 						} else {
 							datapoint >>= reduce_range;
 						}
-
-						y = r.top + x_axis_offset - (x_axis_offset * datapoint) / (highest_value >> reduce_range);
+						y = r.top + x_axis_offset - ((r.bottom - r.top) * datapoint) / (interval_size >> reduce_range);
 
 						/* Draw the point. */
 						GfxFillRect(x - 1, y - 1, x + 1, y + 1, colour);
@@ -444,8 +472,8 @@ protected:
 	}
 
 
-	BaseGraphWindow(int widget, bool has_negative_values, StringID format_str_y_axis) :
-			Window(), has_negative_values(has_negative_values),
+	BaseGraphWindow(int widget, StringID format_str_y_axis) :
+			Window(),
 			format_str_y_axis(format_str_y_axis)
 	{
 		SetWindowDirty(WC_GRAPH_LEGEND, 0);
@@ -462,11 +490,6 @@ protected:
 	}
 
 public:
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		if (widget != this->graph_widget) return;
@@ -499,7 +522,7 @@ public:
 		uint y_label_width = GetStringBoundingBox(STR_GRAPH_Y_LABEL).width;
 
 		size->width  = max<uint>(size->width,  5 + y_label_width + this->num_on_x_axis * (x_label_width + 5) + 9);
-		size->height = max<uint>(size->height, 5 + (1 + GRAPH_NUM_LINES_Y * 2 + (this->month != 0xFF ? 3 : 1)) * FONT_HEIGHT_SMALL + 4);
+		size->height = max<uint>(size->height, 5 + (1 + MIN_GRAPH_NUM_LINES_Y * 2 + (this->month != 0xFF ? 3 : 1)) * FONT_HEIGHT_SMALL + 4);
 		size->height = max<uint>(size->height, size->width / 3);
 	}
 
@@ -526,8 +549,14 @@ public:
 		this->UpdateStatistics(false);
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
+		if (!gui_scope) return;
 		this->UpdateStatistics(true);
 	}
 
@@ -592,7 +621,7 @@ public:
 
 struct OperatingProfitGraphWindow : BaseGraphWindow {
 	OperatingProfitGraphWindow(const WindowDesc *desc, WindowNumber window_number) :
-			BaseGraphWindow(BGW_BACKGROUND, true, STR_JUST_CURRCOMPACT)
+			BaseGraphWindow(BGW_GRAPH, STR_JUST_CURRCOMPACT)
 	{
 		this->InitializeWindow(desc, window_number);
 	}
@@ -611,7 +640,15 @@ static const NWidgetPart _nested_operating_profit_widgets[] = {
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND), SetMinimalSize(576, 160), EndContainer(),
+	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(WWT_EMPTY, COLOUR_GREY, BGW_GRAPH), SetMinimalSize(576, 160), SetFill(1, 1), SetResize(1, 1),
+			NWidget(NWID_VERTICAL),
+				NWidget(NWID_SPACER), SetFill(0, 1), SetResize(0, 1),
+				NWidget(WWT_RESIZEBOX, COLOUR_GREY, BGW_RESIZE),
+			EndContainer(),
+		EndContainer(),
+	EndContainer(),
 };
 
 static const WindowDesc _operating_profit_desc(
@@ -634,7 +671,7 @@ void ShowOperatingProfitGraph()
 
 struct IncomeGraphWindow : BaseGraphWindow {
 	IncomeGraphWindow(const WindowDesc *desc, WindowNumber window_number) :
-			BaseGraphWindow(BGW_BACKGROUND, false, STR_JUST_CURRCOMPACT)
+			BaseGraphWindow(BGW_GRAPH, STR_JUST_CURRCOMPACT)
 	{
 		this->InitializeWindow(desc, window_number);
 	}
@@ -653,7 +690,15 @@ static const NWidgetPart _nested_income_graph_widgets[] = {
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND), SetMinimalSize(576, 128), EndContainer(),
+	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(WWT_EMPTY, COLOUR_GREY, BGW_GRAPH), SetMinimalSize(576, 128), SetFill(1, 1), SetResize(1, 1),
+			NWidget(NWID_VERTICAL),
+				NWidget(NWID_SPACER), SetFill(0, 1), SetResize(0, 1),
+				NWidget(WWT_RESIZEBOX, COLOUR_GREY, BGW_RESIZE),
+			EndContainer(),
+		EndContainer(),
+	EndContainer(),
 };
 
 
@@ -675,7 +720,7 @@ void ShowIncomeGraph()
 
 struct DeliveredCargoGraphWindow : BaseGraphWindow {
 	DeliveredCargoGraphWindow(const WindowDesc *desc, WindowNumber window_number) :
-			BaseGraphWindow(BGW_BACKGROUND, false, STR_JUST_COMMA)
+			BaseGraphWindow(BGW_GRAPH, STR_JUST_COMMA)
 	{
 		this->InitializeWindow(desc, window_number);
 	}
@@ -694,7 +739,15 @@ static const NWidgetPart _nested_delivered_cargo_graph_widgets[] = {
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND), SetMinimalSize(576, 128), EndContainer(),
+	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(WWT_EMPTY, COLOUR_GREY, BGW_GRAPH), SetMinimalSize(576, 128), SetFill(1, 1), SetResize(1, 1),
+			NWidget(NWID_VERTICAL),
+				NWidget(NWID_SPACER), SetFill(0, 1), SetResize(0, 1),
+				NWidget(WWT_RESIZEBOX, COLOUR_GREY, BGW_RESIZE),
+			EndContainer(),
+		EndContainer(),
+	EndContainer(),
 };
 
 static const WindowDesc _delivered_cargo_graph_desc(
@@ -718,11 +771,13 @@ enum PerformanceHistoryGraphWidgets {
 	PHW_KEY,
 	PHW_DETAILED_PERFORMANCE,
 	PHW_BACKGROUND,
+	PHW_GRAPH,
+	PHW_RESIZE,
 };
 
 struct PerformanceHistoryGraphWindow : BaseGraphWindow {
 	PerformanceHistoryGraphWindow(const WindowDesc *desc, WindowNumber window_number) :
-			BaseGraphWindow(PHW_BACKGROUND, false, STR_JUST_COMMA)
+			BaseGraphWindow(PHW_GRAPH, STR_JUST_COMMA)
 	{
 		this->InitializeWindow(desc, window_number);
 	}
@@ -748,7 +803,15 @@ static const NWidgetPart _nested_performance_history_widgets[] = {
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, PHW_BACKGROUND), SetMinimalSize(576, 224), EndContainer(),
+	NWidget(WWT_PANEL, COLOUR_GREY, PHW_BACKGROUND),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(WWT_EMPTY, COLOUR_GREY, PHW_GRAPH), SetMinimalSize(576, 224), SetFill(1, 1), SetResize(1, 1),
+			NWidget(NWID_VERTICAL),
+				NWidget(NWID_SPACER), SetFill(0, 1), SetResize(0, 1),
+				NWidget(WWT_RESIZEBOX, COLOUR_GREY, PHW_RESIZE),
+			EndContainer(),
+		EndContainer(),
+	EndContainer(),
 };
 
 static const WindowDesc _performance_history_desc(
@@ -769,7 +832,7 @@ void ShowPerformanceHistoryGraph()
 
 struct CompanyValueGraphWindow : BaseGraphWindow {
 	CompanyValueGraphWindow(const WindowDesc *desc, WindowNumber window_number) :
-			BaseGraphWindow(BGW_BACKGROUND, false, STR_JUST_CURRCOMPACT)
+			BaseGraphWindow(BGW_GRAPH, STR_JUST_CURRCOMPACT)
 	{
 		this->InitializeWindow(desc, window_number);
 	}
@@ -788,7 +851,15 @@ static const NWidgetPart _nested_company_value_graph_widgets[] = {
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND), SetMinimalSize(576, 224), EndContainer(),
+	NWidget(WWT_PANEL, COLOUR_GREY, BGW_BACKGROUND),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(WWT_EMPTY, COLOUR_GREY, BGW_GRAPH), SetMinimalSize(576, 224), SetFill(1, 1), SetResize(1, 1),
+			NWidget(NWID_VERTICAL),
+				NWidget(NWID_SPACER), SetFill(0, 1), SetResize(0, 1),
+				NWidget(WWT_RESIZEBOX, COLOUR_GREY, BGW_RESIZE),
+			EndContainer(),
+		EndContainer(),
+	EndContainer(),
 };
 
 static const WindowDesc _company_value_graph_desc(
@@ -812,14 +883,19 @@ enum CargoPaymentRatesWidgets {
 	CPW_BACKGROUND,
 	CPW_HEADER,
 	CPW_GRAPH,
+	CPW_RESIZE,
 	CPW_FOOTER,
+	CPW_ENABLE_CARGOS,
+	CPW_DISABLE_CARGOS,
 	CPW_CARGO_FIRST,
 };
 
 struct PaymentRatesGraphWindow : BaseGraphWindow {
+	bool first_init; ///< This value is true until the first initialization of the window has finished.
 	PaymentRatesGraphWindow(const WindowDesc *desc, WindowNumber window_number) :
-			BaseGraphWindow(CPW_GRAPH, false, STR_JUST_CURRCOMPACT)
+			BaseGraphWindow(CPW_GRAPH, STR_JUST_CURRCOMPACT)
 	{
+		this->first_init = true;
 		this->num_on_x_axis = 20;
 		this->num_vert_lines = 20;
 		this->month = 0xFF;
@@ -831,14 +907,40 @@ struct PaymentRatesGraphWindow : BaseGraphWindow {
 
 		this->InitNested(desc, window_number);
 
+		this->UpdateLoweredWidgets();
+	}
+
+	virtual void OnInit()
+	{
+		/* UpdateLoweredWidgets needs to be called after a language or NewGRF change, but it can't be called before
+		 * InitNested is done. On the first init these functions are called in the correct order by the constructor. */
+		if (!this->first_init) {
+			/* Initialise the dataset */
+			this->OnHundredthTick();
+			this->UpdateLoweredWidgets();
+		}
+		this->first_init = false;
+	}
+
+	void UpdateExcludedData()
+	{
+		this->excluded_data = 0;
+
 		int i = 0;
 		const CargoSpec *cs;
-		FOR_ALL_CARGOSPECS(cs) {
-			this->SetWidgetLoweredState(CPW_CARGO_FIRST + cs->Index(), !HasBit(_legend_excluded_cargo, i));
+		FOR_ALL_SORTED_STANDARD_CARGOSPECS(cs) {
+			if (HasBit(_legend_excluded_cargo, cs->Index())) SetBit(this->excluded_data, i);
 			i++;
 		}
 	}
 
+	void UpdateLoweredWidgets()
+	{
+		for (int i = 0; i < _sorted_standard_cargo_specs_size; i++) {
+			this->SetWidgetLoweredState(CPW_CARGO_FIRST + i, !HasBit(this->excluded_data, i));
+		}
+	}
+
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		if (widget < CPW_CARGO_FIRST) {
@@ -846,7 +948,7 @@ struct PaymentRatesGraphWindow : BaseGraphWindow {
 			return;
 		}
 
-		const CargoSpec *cs = CargoSpec::Get(widget - CPW_CARGO_FIRST);
+		const CargoSpec *cs = _sorted_cargo_specs[widget - CPW_CARGO_FIRST];
 		SetDParam(0, cs->name);
 		Dimension d = GetStringBoundingBox(STR_GRAPH_CARGO_PAYMENT_CARGO);
 		d.width += 14; // colour field
@@ -862,8 +964,8 @@ struct PaymentRatesGraphWindow : BaseGraphWindow {
 			return;
 		}
 
-		const CargoSpec *cs = CargoSpec::Get(widget - CPW_CARGO_FIRST);
-		bool rtl = _dynlang.text_dir == TD_RTL;
+		const CargoSpec *cs = _sorted_cargo_specs[widget - CPW_CARGO_FIRST];
+		bool rtl = _current_text_dir == TD_RTL;
 
 		/* Since the buttons have no text, no images,
 		 * both the text and the coloured box have to be manually painted.
@@ -883,18 +985,38 @@ struct PaymentRatesGraphWindow : BaseGraphWindow {
 
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
-		if (widget >= CPW_CARGO_FIRST) {
-			int i = 0;
-			const CargoSpec *cs;
-			FOR_ALL_CARGOSPECS(cs) {
-				if (cs->Index() + CPW_CARGO_FIRST == widget) break;
-				i++;
+		switch (widget) {
+			case CPW_ENABLE_CARGOS:
+				/* Remove all cargos from the excluded lists. */
+				_legend_excluded_cargo = 0;
+				this->excluded_data = 0;
+				this->UpdateLoweredWidgets();
+				this->SetDirty();
+				break;
+
+			case CPW_DISABLE_CARGOS: {
+				/* Add all cargos to the excluded lists. */
+				int i = 0;
+				const CargoSpec *cs;
+				FOR_ALL_SORTED_STANDARD_CARGOSPECS(cs) {
+					SetBit(_legend_excluded_cargo, cs->Index());
+					SetBit(this->excluded_data, i);
+					i++;
+				}
+				this->UpdateLoweredWidgets();
+				this->SetDirty();
+				break;
 			}
 
-			ToggleBit(_legend_excluded_cargo, i);
-			this->ToggleWidgetLoweredState(widget);
-			this->excluded_data = _legend_excluded_cargo;
-			this->SetDirty();
+			default:
+				if (widget >= CPW_CARGO_FIRST) {
+					int i = widget - CPW_CARGO_FIRST;
+					ToggleBit(_legend_excluded_cargo, _sorted_cargo_specs[i]->Index());
+					this->ToggleWidgetLoweredState(widget);
+					this->UpdateExcludedData();
+					this->SetDirty();
+				}
+				break;
 		}
 	}
 
@@ -903,23 +1025,28 @@ struct PaymentRatesGraphWindow : BaseGraphWindow {
 		/* Override default OnTick */
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
+		if (!gui_scope) return;
 		this->OnHundredthTick();
 	}
 
 	virtual void OnHundredthTick()
 	{
-		this->excluded_data = _legend_excluded_cargo;
+		this->UpdateExcludedData();
 
 		int i = 0;
 		const CargoSpec *cs;
-		FOR_ALL_CARGOSPECS(cs) {
+		FOR_ALL_SORTED_STANDARD_CARGOSPECS(cs) {
 			this->colours[i] = cs->legend_colour;
 			for (uint j = 0; j != 20; j++) {
 				this->cost[i][j] = GetTransportedGoodsIncome(10, 20, j * 4 + 4, cs->Index());
 			}
-
 			i++;
 		}
 		this->num_dataset = i;
@@ -931,15 +1058,14 @@ static NWidgetBase *MakeCargoButtons(int *biggest_index)
 {
 	NWidgetVertical *ver = new NWidgetVertical;
 
-	const CargoSpec *cs;
-	FOR_ALL_CARGOSPECS(cs) {
-		*biggest_index = CPW_CARGO_FIRST + cs->Index();
-		NWidgetBackground *leaf = new NWidgetBackground(WWT_PANEL, COLOUR_ORANGE, *biggest_index, NULL);
+	for (int i = 0; i < _sorted_standard_cargo_specs_size; i++) {
+		NWidgetBackground *leaf = new NWidgetBackground(WWT_PANEL, COLOUR_ORANGE, CPW_CARGO_FIRST + i, NULL);
 		leaf->tool_tip = STR_GRAPH_CARGO_PAYMENT_TOGGLE_CARGO;
 		leaf->SetFill(1, 0);
 		leaf->SetLowered(true);
 		ver->Add(leaf);
 	}
+	*biggest_index = CPW_CARGO_FIRST + _sorted_standard_cargo_specs_size - 1;
 	return ver;
 }
 
@@ -951,27 +1077,29 @@ static const NWidgetPart _nested_cargo_payment_rates_widgets[] = {
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, CPW_BACKGROUND), SetMinimalSize(568, 128), SetResize(0, 1),
-		NWidget(NWID_VERTICAL),
-			NWidget(NWID_HORIZONTAL),
-				NWidget(NWID_SPACER), SetFill(1, 0),
-				NWidget(WWT_TEXT, COLOUR_GREY, CPW_HEADER), SetMinimalSize(0, 6), SetPadding(2, 0, 2, 0), SetDataTip(STR_GRAPH_CARGO_PAYMENT_RATES_TITLE, STR_NULL),
-				NWidget(NWID_SPACER), SetFill(1, 0),
-			EndContainer(),
-			NWidget(NWID_HORIZONTAL),
-				NWidget(WWT_EMPTY, COLOUR_GREY, CPW_GRAPH), SetMinimalSize(495, 0), SetFill(1, 1),
-				NWidget(NWID_VERTICAL),
-					NWidget(NWID_SPACER), SetMinimalSize(0, 24), SetFill(0, 0),
-						NWidgetFunction(MakeCargoButtons),
-					NWidget(NWID_SPACER), SetMinimalSize(0, 24), SetFill(0, 1),
-				EndContainer(),
-				NWidget(NWID_SPACER), SetMinimalSize(5, 0), SetFill(0, 1),
-			EndContainer(),
-			NWidget(NWID_HORIZONTAL),
-				NWidget(NWID_SPACER), SetFill(1, 0),
-				NWidget(WWT_TEXT, COLOUR_GREY, CPW_FOOTER), SetMinimalSize(0, 6), SetPadding(2, 0, 2, 0), SetDataTip(STR_GRAPH_CARGO_PAYMENT_RATES_X_LABEL, STR_NULL),
-				NWidget(NWID_SPACER), SetFill(1, 0),
+	NWidget(WWT_PANEL, COLOUR_GREY, CPW_BACKGROUND), SetMinimalSize(568, 128),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(NWID_SPACER), SetFill(1, 0), SetResize(1, 0),
+			NWidget(WWT_TEXT, COLOUR_GREY, CPW_HEADER), SetMinimalSize(0, 6), SetPadding(2, 0, 2, 0), SetDataTip(STR_GRAPH_CARGO_PAYMENT_RATES_TITLE, STR_NULL),
+			NWidget(NWID_SPACER), SetFill(1, 0), SetResize(1, 0),
+		EndContainer(),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(WWT_EMPTY, COLOUR_GREY, CPW_GRAPH), SetMinimalSize(495, 0), SetFill(1, 1), SetResize(1, 1),
+			NWidget(NWID_VERTICAL),
+				NWidget(NWID_SPACER), SetMinimalSize(0, 24), SetFill(0, 0), SetResize(0, 1),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_ORANGE, CPW_ENABLE_CARGOS), SetDataTip(STR_GRAPH_CARGO_ENABLE_ALL, STR_GRAPH_CARGO_TOOLTIP_ENABLE_ALL), SetFill(1, 0),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_ORANGE, CPW_DISABLE_CARGOS), SetDataTip(STR_GRAPH_CARGO_DISABLE_ALL, STR_GRAPH_CARGO_TOOLTIP_DISABLE_ALL), SetFill(1, 0),
+				NWidget(NWID_SPACER), SetMinimalSize(0, 4),
+				NWidgetFunction(MakeCargoButtons),
+				NWidget(NWID_SPACER), SetMinimalSize(0, 24), SetFill(0, 1), SetResize(0, 1),
 			EndContainer(),
+			NWidget(NWID_SPACER), SetMinimalSize(5, 0), SetFill(0, 1), SetResize(0, 1),
+		EndContainer(),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(NWID_SPACER), SetMinimalSize(WD_RESIZEBOX_WIDTH, 0), SetFill(1, 0), SetResize(1, 0),
+			NWidget(WWT_TEXT, COLOUR_GREY, CPW_FOOTER), SetMinimalSize(0, 6), SetPadding(2, 0, 2, 0), SetDataTip(STR_GRAPH_CARGO_PAYMENT_RATES_X_LABEL, STR_NULL),
+			NWidget(NWID_SPACER), SetFill(1, 0), SetResize(1, 0),
+			NWidget(WWT_RESIZEBOX, COLOUR_GREY, CPW_RESIZE),
 		EndContainer(),
 	EndContainer(),
 };
@@ -979,7 +1107,7 @@ static const NWidgetPart _nested_cargo_payment_rates_widgets[] = {
 static const WindowDesc _cargo_payment_rates_desc(
 	WDP_AUTO, 0, 0,
 	WC_PAYMENT_RATES, WC_NONE,
-	0,
+	WDF_UNCLICK_BUTTONS,
 	_nested_cargo_payment_rates_widgets, lengthof(_nested_cargo_payment_rates_widgets)
 );
 
@@ -1049,7 +1177,7 @@ private:
 	/** Sort the company league by performance history */
 	static int CDECL PerformanceSorter(const Company * const *c1, const Company * const *c2)
 	{
-		return (*c2)->old_economy[1].performance_history - (*c1)->old_economy[1].performance_history;
+		return (*c2)->old_economy[0].performance_history - (*c1)->old_economy[0].performance_history;
 	}
 
 public:
@@ -1075,7 +1203,7 @@ public:
 		uint y = r.top + WD_FRAMERECT_TOP;
 		int icon_y_offset = 1 + (FONT_HEIGHT_NORMAL - 10) / 2;
 
-		bool rtl = _dynlang.text_dir == TD_RTL;
+		bool rtl = _current_text_dir == TD_RTL;
 		uint ordinal_left  = rtl ? r.right - WD_FRAMERECT_LEFT - this->ordinal_width : r.left + WD_FRAMERECT_LEFT;
 		uint ordinal_right = rtl ? r.right - WD_FRAMERECT_LEFT : r.left + WD_FRAMERECT_LEFT + this->ordinal_width;
 		uint icon_left     = r.left + WD_FRAMERECT_LEFT + WD_FRAMERECT_RIGHT + (rtl ? this->text_width : this->ordinal_width);
@@ -1090,7 +1218,7 @@ public:
 
 			SetDParam(0, c->index);
 			SetDParam(1, c->index);
-			SetDParam(2, GetPerformanceTitleFromValue(c->old_economy[1].performance_history));
+			SetDParam(2, GetPerformanceTitleFromValue(c->old_economy[0].performance_history));
 			DrawString(text_left, text_right, y, STR_COMPANY_LEAGUE_COMPANY_NAME);
 			y += FONT_HEIGHT_NORMAL;
 		}
@@ -1137,9 +1265,15 @@ public:
 		}
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
 		if (data == 0) {
+			/* This needs to be done in command-scope to enforce rebuilding before resorting invalid data */
 			this->companies.ForceRebuild();
 		} else {
 			this->companies.ForceResort();
@@ -1234,8 +1368,24 @@ struct PerformanceRatingDetailWindow : Window {
 
 				/* At this number we are roughly at the max; it can become wider,
 				 * but then you need at 1000 times more money. At that time you're
-				 * not that interested anymore in the last few digits anyway. */
-				uint max = 999999999; // nine 9s
+				 * not that interested anymore in the last few digits anyway.
+				 * The 500 is because 999 999 500 to 999 999 999 are rounded to
+				 * 1 000 M, and not 999 999 k. Use negative numbers to account for
+				 * the negative income/amount of money etc. as well. */
+				int max = -(999999999 - 500);
+
+				/* Scale max for the display currency. Prior to rendering the value
+				 * is converted into the display currency, which may cause it to
+				 * raise significantly. We need to compensate for that since {{CURRCOMPACT}}
+				 * is used, which can produce quite short renderings of very large
+				 * values. Otherwise the calculated width could be too narrow.
+				 * Note that it doesn't work if there was a currency with an exchange
+				 * rate greater than max.
+				 * When the currency rate is more than 1000, the 999 999 k becomes at
+				 * least 999 999 M which roughly is equally long. Furthermore if the
+				 * exchange rate is that high, 999 999 k is usually not enough anymore
+				 * to show the different currency numbers. */
+				if (_currency->rate < 1000) max /= _currency->rate;
 				SetDParam(0, max);
 				SetDParam(1, max);
 				uint score_detail_width = GetStringBoundingBox(STR_PERFORMANCE_DETAIL_AMOUNT_CURRENCY).width;
@@ -1244,7 +1394,7 @@ struct PerformanceRatingDetailWindow : Window {
 				uint left  = 7;
 				uint right = size->width - 7;
 
-				bool rtl = _dynlang.text_dir == TD_RTL;
+				bool rtl = _current_text_dir == TD_RTL;
 				this->score_info_left  = rtl ? right - score_info_width : left;
 				this->score_info_right = rtl ? right : left + score_info_width;
 
@@ -1257,12 +1407,6 @@ struct PerformanceRatingDetailWindow : Window {
 		}
 	}
 
-	virtual void OnPaint()
-	{
-		/* Draw standard stuff */
-		this->DrawWidgets();
-	}
-
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		/* No need to draw when there's nothing to draw */
@@ -1307,7 +1451,7 @@ struct PerformanceRatingDetailWindow : Window {
 
 		/* Calculate the %-bar */
 		uint x = Clamp(val, 0, needed) * this->bar_width / needed;
-		bool rtl = _dynlang.text_dir == TD_RTL;
+		bool rtl = _current_text_dir == TD_RTL;
 		if (rtl) {
 			x = this->bar_right - x;
 		} else {
@@ -1320,7 +1464,7 @@ struct PerformanceRatingDetailWindow : Window {
 
 		/* Draw it */
 		SetDParam(0, Clamp(val, 0, needed) * 100 / needed);
-		DrawString(this->bar_left, this->bar_right, text_top, STR_PERFORMANCE_DETAIL_PERCENT, TC_FROMSTRING, SA_CENTER);
+		DrawString(this->bar_left, this->bar_right, text_top, STR_PERFORMANCE_DETAIL_PERCENT, TC_FROMSTRING, SA_HOR_CENTER);
 
 		/* SCORE_LOAN is inversed */
 		if (score_type == SCORE_LOAN) val = needed - val;
@@ -1368,11 +1512,13 @@ struct PerformanceRatingDetailWindow : Window {
 	}
 
 	/**
-	 * Invalidate the data of this window.
+	 * Some data on this window has become invalid.
 	 * @param data the company ID of the company that is going to be removed
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
 	 */
-	virtual void OnInvalidateData(int data)
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
+		if (!gui_scope) return;
 		/* Disable the companies who are not active */
 		for (CompanyID i = COMPANY_FIRST; i < MAX_COMPANIES; i++) {
 			this->SetWidgetDisabledState(i + PRW_COMPANY_FIRST, !Company::IsValidID(i));
@@ -1400,7 +1546,8 @@ struct PerformanceRatingDetailWindow : Window {
 
 CompanyID PerformanceRatingDetailWindow::company = INVALID_COMPANY;
 
-/** Make a vertical list of panels for outputting score details.
+/**
+ * Make a vertical list of panels for outputting score details.
  * @param biggest_index Storage for collecting the biggest index used in the returned tree.
  * @return Panel with performance details.
  * @post \c *biggest_index contains the largest used index in the tree.
@@ -1433,55 +1580,10 @@ static NWidgetBase *MakePerformanceDetailPanels(int *biggest_index)
 	return vert;
 }
 
-/**
- * Make a number of rows with button-like graphics, for enabling/disabling each company.
- * @param biggest_index Storage for collecting the biggest index used in the returned tree.
- * @return Panel with rows of company buttons.
- * @post \c *biggest_index contains the largest used index in the tree.
- */
-static NWidgetBase *MakeCompanyButtonRows(int *biggest_index)
+/** Make a number of rows with buttons for each company for the performance rating detail window. */
+NWidgetBase *MakeCompanyButtonRowsGraphGUI(int *biggest_index)
 {
-	static const int MAX_LENGTH = 8; // Maximal number of company buttons in one row.
-	NWidgetVertical *vert = NULL; // Storage for all rows.
-	NWidgetHorizontal *hor = NULL; // Storage for buttons in one row.
-	int hor_length = 0;
-
-	Dimension sprite_size = GetSpriteSize(SPR_COMPANY_ICON);
-	sprite_size.width  += WD_MATRIX_LEFT + WD_MATRIX_RIGHT;
-	sprite_size.height += WD_MATRIX_TOP + WD_MATRIX_BOTTOM + 1; // 1 for the 'offset' of being pressed
-
-	for (int widnum = PRW_COMPANY_FIRST; widnum <= PRW_COMPANY_LAST; widnum++) {
-		/* Ensure there is room in 'hor' for another button. */
-		if (hor_length == MAX_LENGTH) {
-			if (vert == NULL) vert = new NWidgetVertical();
-			vert->Add(hor);
-			hor = NULL;
-			hor_length = 0;
-		}
-		if (hor == NULL) {
-			hor = new NWidgetHorizontal();
-			hor_length = 0;
-		}
-
-		NWidgetBackground *panel = new NWidgetBackground(WWT_PANEL, COLOUR_GREY, widnum);
-		panel->SetMinimalSize(sprite_size.width, sprite_size.height);
-		panel->SetFill(1, 0);
-		panel->SetDataTip(0x0, STR_PERFORMANCE_DETAIL_SELECT_COMPANY_TOOLTIP);
-		hor->Add(panel);
-		hor_length++;
-	}
-	*biggest_index = PRW_COMPANY_LAST;
-	if (vert == NULL) return hor; // All buttons fit in a single row.
-
-	if (hor_length > 0 && hor_length < MAX_LENGTH) {
-		/* Last row is partial, add a spacer at the end to force all buttons to the left. */
-		NWidgetSpacer *spc = new NWidgetSpacer(0, 0);
-		spc->SetMinimalSize(sprite_size.width, sprite_size.height);
-		spc->SetFill(1, 0);
-		hor->Add(spc);
-	}
-	if (hor != NULL) vert->Add(hor);
-	return vert;
+	return MakeCompanyButtonRows(biggest_index, PRW_COMPANY_FIRST, PRW_COMPANY_LAST, 8, STR_PERFORMANCE_DETAIL_SELECT_COMPANY_TOOLTIP);
 }
 
 static const NWidgetPart _nested_performance_rating_detail_widgets[] = {
@@ -1492,7 +1594,7 @@ static const NWidgetPart _nested_performance_rating_detail_widgets[] = {
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(WWT_PANEL, COLOUR_GREY),
-		NWidgetFunction(MakeCompanyButtonRows), SetPadding(0, 1, 1, 2),
+		NWidgetFunction(MakeCompanyButtonRowsGraphGUI), SetPadding(0, 1, 1, 2),
 	EndContainer(),
 	NWidgetFunction(MakePerformanceDetailPanels),
 };
