@@ -1,4 +1,4 @@
-/* $Id: music_gui.cpp 22252 2011-03-14 19:03:17Z rubidium $ */
+/* $Id: music_gui.cpp 23053 2011-10-22 21:06:58Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -78,19 +78,24 @@ static byte * const _playlists[] = {
 
 /**
  * Validate a playlist.
- * @param playlist the playlist to validate
+ * @param playlist The playlist to validate.
+ * @param last The last location in the list.
  */
-void ValidatePlaylist(byte *playlist)
+void ValidatePlaylist(byte *playlist, byte *last)
 {
-	while (*playlist != 0) {
-		if (*playlist <= BaseMusic::GetUsedSet()->num_available) {
+	while (*playlist != 0 && playlist <= last) {
+		/* Song indices are saved off-by-one so 0 is "nothing". */
+		if (*playlist <= NUM_SONGS_AVAILABLE && !StrEmpty(GetSongName(*playlist - 1))) {
 			playlist++;
 			continue;
 		}
-		for (byte *p = playlist; *p != 0; p++) {
+		for (byte *p = playlist; *p != 0 && p <= last; p++) {
 			p[0] = p[1];
 		}
 	}
+
+	/* Make sure the list is null terminated. */
+	*last = 0;
 }
 
 /** Initialize the playlists */
@@ -116,8 +121,8 @@ void InitializeMusic()
 		_playlists[k + 1][j] = 0;
 	}
 
-	ValidatePlaylist(_msf.custom_1);
-	ValidatePlaylist(_msf.custom_2);
+	ValidatePlaylist(_msf.custom_1, lastof(_msf.custom_1));
+	ValidatePlaylist(_msf.custom_2, lastof(_msf.custom_2));
 
 	if (BaseMusic::GetUsedSet()->num_available < _music_wnd_cursong) {
 		/* If there are less songs than the currently played song,
