@@ -1,4 +1,4 @@
-/* $Id: network_func.h 19858 2010-05-18 21:49:59Z rubidium $ */
+/* $Id: network_func.h 22838 2011-08-25 13:24:32Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -36,12 +36,11 @@ extern StringList _network_ban_list;
 byte NetworkSpectatorCount();
 void NetworkUpdateClientName();
 bool NetworkCompanyHasClients(CompanyID company);
-const char *NetworkChangeCompanyPassword(const char *);
+const char *NetworkChangeCompanyPassword(CompanyID company_id, const char *password);
 void NetworkReboot();
-void NetworkDisconnect(bool blocking = false);
+void NetworkDisconnect(bool blocking = false, bool close_admins = true);
 void NetworkGameLoop();
 void NetworkUDPGameLoop();
-void NetworkUDPCloseAll();
 void ParseConnectionString(const char **company, const char **port, char *connection_string);
 void NetworkStartDebugLog(NetworkAddress address);
 void NetworkPopulateCompanyStats(NetworkCompanyStats *stats);
@@ -60,6 +59,7 @@ void NetworkPrintClients();
 void NetworkHandlePauseChange(PauseMode prev_mode, PauseMode changed_mode);
 
 /*** Commands ran by the server ***/
+void NetworkServerDailyLoop();
 void NetworkServerMonthlyLoop();
 void NetworkServerYearlyLoop();
 void NetworkServerSendConfigUpdate();
@@ -68,23 +68,19 @@ bool NetworkServerStart();
 void NetworkServerUpdateCompanyPassworded(CompanyID company_id, bool passworded);
 bool NetworkServerChangeClientName(ClientID client_id, const char *new_name);
 
-NetworkClientInfo *NetworkFindClientInfoFromIndex(ClientIndex index);
-NetworkClientInfo *NetworkFindClientInfoFromClientID(ClientID client_id);
-NetworkClientInfo *NetworkFindClientInfoFromIP(const char *ip);
-const char *GetClientIP(NetworkClientInfo *ci);
 
 void NetworkServerDoMove(ClientID client_id, CompanyID company_id);
-void NetworkServerSendRcon(ClientID client_id, ConsoleColour colour_code, const char *string);
-void NetworkServerSendError(ClientID client_id, NetworkErrorCode error);
-void NetworkServerSendChat(NetworkAction action, DestType type, int dest, const char *msg, ClientID from_id, int64 data = 0);
+void NetworkServerSendRcon(ClientID client_id, TextColour colour_code, const char *string);
+void NetworkServerSendChat(NetworkAction action, DestType type, int dest, const char *msg, ClientID from_id, int64 data = 0, bool from_admin = false);
 
 void NetworkServerKickClient(ClientID client_id);
+uint NetworkServerKickOrBanIP(ClientID client_id, bool ban);
 uint NetworkServerKickOrBanIP(const char *ip, bool ban);
 
 void NetworkInitChatMessage();
-void CDECL NetworkAddChatMessage(TextColour colour, uint8 duration, const char *message, ...) WARN_FORMAT(3, 4);
+void CDECL NetworkAddChatMessage(TextColour colour, uint duration, const char *message, ...) WARN_FORMAT(3, 4);
 void NetworkUndrawChatMessage();
-void NetworkChatMessageDailyLoop();
+void NetworkChatMessageLoop();
 
 void NetworkAfterNewGRFScan();
 
