@@ -1,4 +1,4 @@
-/* $Id: newgrf_house.h 19855 2010-05-18 21:30:56Z rubidium $ */
+/* $Id: newgrf_house.h 20627 2010-08-26 17:01:17Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -42,9 +42,9 @@ void DecreaseBuildingCount(Town *t, HouseID house_id);
 
 void DrawNewHouseTile(TileInfo *ti, HouseID house_id);
 void AnimateNewHouseTile(TileIndex tile);
-void ChangeHouseAnimationFrame(const struct GRFFile *file, TileIndex tile, uint16 callback_result);
+void AnimateNewHouseConstruction(TileIndex tile);
 
-uint16 GetHouseCallback(CallbackID callback, uint32 param1, uint32 param2, HouseID house_id, Town *town, TileIndex tile, bool not_yet_constructed = false);
+uint16 GetHouseCallback(CallbackID callback, uint32 param1, uint32 param2, HouseID house_id, const Town *town, TileIndex tile, bool not_yet_constructed = false, uint8 initial_random_bits = 0);
 
 bool CanDeleteHouse(TileIndex tile);
 
