@@ -1,4 +1,4 @@
-/* $Id: newgrf_industrytiles.cpp 19855 2010-05-18 21:30:56Z rubidium $ */
+/* $Id: newgrf_industrytiles.cpp 22838 2011-08-25 13:24:32Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,23 +10,19 @@
 /** @file newgrf_industrytiles.cpp NewGRF handling of industry tiles. */
 
 #include "stdafx.h"
-#include "variables.h"
 #include "debug.h"
 #include "viewport_func.h"
 #include "landscape.h"
 #include "newgrf.h"
-#include "newgrf_commons.h"
-#include "newgrf_industries.h"
 #include "newgrf_industrytiles.h"
 #include "newgrf_sound.h"
 #include "newgrf_text.h"
 #include "industry.h"
-#include "functions.h"
 #include "town.h"
 #include "command_func.h"
-#include "animated_tile_func.h"
 #include "water.h"
 #include "sprite.h"
+#include "newgrf_animation_base.h"
 
 #include "table/strings.h"
 
@@ -35,17 +31,19 @@
  * @param parameter from callback.  It's in fact a pair of coordinates
  * @param tile TileIndex from which the callback was initiated
  * @param index of the industry been queried for
+ * @param signed_offsets Are the x and y offset encoded in parameter signed?
  * @return a construction of bits obeying the newgrf format
  */
-uint32 GetNearbyIndustryTileInformation(byte parameter, TileIndex tile, IndustryID index)
+uint32 GetNearbyIndustryTileInformation(byte parameter, TileIndex tile, IndustryID index, bool signed_offsets)
 {
-	if (parameter != 0) tile = GetNearbyTile(parameter, tile); // only perform if it is required
+	if (parameter != 0) tile = GetNearbyTile(parameter, tile, signed_offsets); // only perform if it is required
 	bool is_same_industry = (IsTileType(tile, MP_INDUSTRY) && GetIndustryIndex(tile) == index);
 
 	return GetNearbyTileInformation(tile) | (is_same_industry ? 1 : 0) << 8;
 }
 
-/** This is the position of the tile relative to the northernmost tile of the industry.
+/**
+ * This is the position of the tile relative to the northernmost tile of the industry.
  * Format: 00yxYYXX
  * Variable  Content
  * x         the x offset from the northernmost tile
@@ -55,7 +53,7 @@ uint32 GetNearbyIndustryTileInformation(byte parameter, TileIndex tile, Industry
  * @param tile TileIndex of the tile to evaluate
  * @param ind_tile northernmost tile of the industry
  */
-static uint32 GetRelativePosition(TileIndex tile, TileIndex ind_tile)
+uint32 GetRelativePosition(TileIndex tile, TileIndex ind_tile)
 {
 	byte x = TileX(tile) - TileX(ind_tile);
 	byte y = TileY(tile) - TileY(ind_tile);
@@ -86,7 +84,7 @@ static uint32 IndustryTileGetVariable(const ResolverObject *object, byte variabl
 		case 0x43: return GetRelativePosition(tile, inds->location.tile);
 
 		/* Animation frame. Like house variable 46 but can contain anything 0..FF. */
-		case 0x44: return (IsTileType(tile, MP_INDUSTRY)) ? GetIndustryAnimationState(tile) : 0;
+		case 0x44: return (IsTileType(tile, MP_INDUSTRY)) ? GetAnimationFrame(tile) : 0;
 
 		/* Land info of nearby tiles */
 		case 0x60: return GetNearbyIndustryTileInformation(parameter, tile, inds == NULL ? (IndustryID)INVALID_INDUSTRY : inds->index);
@@ -95,7 +93,7 @@ static uint32 IndustryTileGetVariable(const ResolverObject *object, byte variabl
 		case 0x61:
 			tile = GetNearbyTile(parameter, tile);
 			if (IsTileType(tile, MP_INDUSTRY) && Industry::GetByTile(tile) == inds) {
-				return GetIndustryAnimationState(tile);
+				return GetAnimationFrame(tile);
 			}
 			return UINT_MAX;
 
@@ -103,7 +101,7 @@ static uint32 IndustryTileGetVariable(const ResolverObject *object, byte variabl
 		case 0x62: return GetIndustryIDAtOffset(GetNearbyTile(parameter, tile), inds, object->grffile->grfid);
 	}
 
-	DEBUG(grf, 1, "Unhandled industry tile property 0x%X", variable);
+	DEBUG(grf, 1, "Unhandled industry tile variable 0x%X", variable);
 
 	*available = false;
 	return UINT_MAX;
@@ -167,10 +165,7 @@ static void NewIndustryTileResolver(ResolverObject *res, IndustryGfx gfx, TileIn
 	res->callback        = CBID_NO_CALLBACK;
 	res->callback_param1 = 0;
 	res->callback_param2 = 0;
-	res->last_value      = 0;
-	res->trigger         = 0;
-	res->reseed          = 0;
-	res->count           = 0;
+	res->ResetState();
 
 	const IndustryTileSpec *its = GetIndustryTileSpec(gfx);
 	res->grffile         = (its != NULL ? its->grf_prop.grffile : NULL);
@@ -188,7 +183,7 @@ static void IndustryDrawTileLayout(const TileInfo *ti, const TileLayoutSpriteGro
 	if (GB(image, 0, SPRITE_WIDTH) != 0) {
 		/* If the ground sprite is the default flat water sprite, draw also canal/river borders
 		 * Do not do this if the tile's WaterClass is 'land'. */
-		if (image == SPR_FLAT_WATER_TILE && IsIndustryTileOnWater(ti->tile)) {
+		if (image == SPR_FLAT_WATER_TILE && IsTileOnWater(ti->tile)) {
 			DrawWaterClassGround(ti);
 		} else {
 			DrawGroundSprite(image, GroundSpritePaletteTransform(image, pal, GENERAL_SPRITE_COLOUR(rnd_colour)));
@@ -211,7 +206,7 @@ uint16 GetIndustryTileCallback(CallbackID callback, uint32 param1, uint32 param2
 	object.callback_param1 = param1;
 	object.callback_param2 = param2;
 
-	group = SpriteGroup::Resolve(GetIndustryTileSpec(gfx_id)->grf_prop.spritegroup, &object);
+	group = SpriteGroup::Resolve(GetIndustryTileSpec(gfx_id)->grf_prop.spritegroup[0], &object);
 	if (group == NULL || group->type != SGT_CALLBACK) return CALLBACK_FAILED;
 
 	return group->GetCallbackResult();
@@ -226,7 +221,7 @@ bool DrawNewIndustryTile(TileInfo *ti, Industry *i, IndustryGfx gfx, const Indus
 		bool draw_old_one = true;
 		if (HasBit(inds->callback_mask, CBM_INDT_DRAW_FOUNDATIONS)) {
 			/* Called to determine the type (if any) of foundation to draw for industry tile */
-			uint32 callback_res = GetIndustryTileCallback(CBID_INDUSTRY_DRAW_FOUNDATIONS, 0, 0, gfx, i, ti->tile);
+			uint32 callback_res = GetIndustryTileCallback(CBID_INDTILE_DRAW_FOUNDATIONS, 0, 0, gfx, i, ti->tile);
 			draw_old_one = (callback_res != 0);
 		}
 
@@ -235,14 +230,14 @@ bool DrawNewIndustryTile(TileInfo *ti, Industry *i, IndustryGfx gfx, const Indus
 
 	NewIndustryTileResolver(&object, gfx, ti->tile, i);
 
-	group = SpriteGroup::Resolve(inds->grf_prop.spritegroup, &object);
+	group = SpriteGroup::Resolve(inds->grf_prop.spritegroup[0], &object);
 	if (group == NULL || group->type != SGT_TILELAYOUT) {
 		return false;
 	} else {
 		/* Limit the building stage to the number of stages supplied. */
 		const TileLayoutSpriteGroup *tlgroup = (const TileLayoutSpriteGroup *)group;
 		byte stage = GetIndustryConstructionStage(ti->tile);
-		stage = Clamp(stage - 4 + tlgroup->num_building_stages, 0, tlgroup->num_building_stages - 1);
+		stage = tlgroup->GetConstructionStageOffset(stage);
 		IndustryDrawTileLayout(ti, tlgroup, i->random_colour, stage, gfx);
 		return true;
 	}
@@ -250,129 +245,84 @@ bool DrawNewIndustryTile(TileInfo *ti, Industry *i, IndustryGfx gfx, const Indus
 
 extern bool IsSlopeRefused(Slope current, Slope refused);
 
-bool PerformIndustryTileSlopeCheck(TileIndex ind_base_tile, TileIndex ind_tile, const IndustryTileSpec *its, IndustryType type, IndustryGfx gfx, uint itspec_index)
+/**
+ * Check the slope of a tile of a new industry.
+ * @param ind_base_tile Base tile of the industry.
+ * @param ind_tile      Tile to check.
+ * @param its           Tile specification.
+ * @param type          Industry type.
+ * @param gfx           Gfx of the tile.
+ * @param itspec_index  Layout.
+ * @param initial_random_bits Random bits of industry after construction
+ * @param founder       Industry founder
+ * @param creation_type The circumstances the industry is created under.
+ * @return Suceeded or failed command.
+ */
+CommandCost PerformIndustryTileSlopeCheck(TileIndex ind_base_tile, TileIndex ind_tile, const IndustryTileSpec *its, IndustryType type, IndustryGfx gfx, uint itspec_index, uint16 initial_random_bits, Owner founder, IndustryAvailabilityCallType creation_type)
 {
 	Industry ind;
 	ind.index = INVALID_INDUSTRY;
 	ind.location.tile = ind_base_tile;
 	ind.location.w = 0;
 	ind.type = type;
+	ind.random = initial_random_bits;
+	ind.founder = founder;
 
-	uint16 callback_res = GetIndustryTileCallback(CBID_INDTILE_SHAPE_CHECK, 0, itspec_index, gfx, &ind, ind_tile);
+	uint16 callback_res = GetIndustryTileCallback(CBID_INDTILE_SHAPE_CHECK, 0, creation_type << 8 | itspec_index, gfx, &ind, ind_tile);
 	if (callback_res == CALLBACK_FAILED) {
-		return !IsSlopeRefused(GetTileSlope(ind_tile, NULL), its->slopes_refused);
+		if (!IsSlopeRefused(GetTileSlope(ind_tile, NULL), its->slopes_refused)) return CommandCost();
+		return_cmd_error(STR_ERROR_SITE_UNSUITABLE);
 	}
 	if (its->grf_prop.grffile->grf_version < 7) {
-		return callback_res != 0;
+		if (callback_res != 0) return CommandCost();
+		return_cmd_error(STR_ERROR_SITE_UNSUITABLE);
+	}
+	if (callback_res == 0x400) return CommandCost();
+
+	CommandCost res;
+	switch (callback_res) {
+		case 0x401: res = CommandCost(STR_ERROR_SITE_UNSUITABLE); break;
+		case 0x402: res = CommandCost(STR_ERROR_CAN_ONLY_BE_BUILT_IN_RAINFOREST); break;
+		case 0x403: res = CommandCost(STR_ERROR_CAN_ONLY_BE_BUILT_IN_DESERT); break;
+		default:    res = CommandCost(GetGRFStringID(its->grf_prop.grffile->grfid, 0xD000 + callback_res)); break;
 	}
-	if (callback_res == 0x400) return true;
 
 	/* Copy some parameters from the registers to the error message text ref. stack */
-	SwitchToErrorRefStack();
-	PrepareTextRefStackUsage(4);
-	SwitchToNormalRefStack();
+	res.UseTextRefStack(4);
 
-	switch (callback_res) {
-		case 0x401: _error_message = STR_ERROR_SITE_UNSUITABLE;                 return false;
-		case 0x402: _error_message = STR_ERROR_CAN_ONLY_BE_BUILT_IN_RAINFOREST; return false;
-		case 0x403: _error_message = STR_ERROR_CAN_ONLY_BE_BUILT_IN_DESERT;     return false;
-		default: _error_message = GetGRFStringID(its->grf_prop.grffile->grfid, 0xD000 + callback_res); return false;
-	}
+	return res;
 }
 
+/* Simple wrapper for GetHouseCallback to keep the animation unified. */
+uint16 GetSimpleIndustryCallback(CallbackID callback, uint32 param1, uint32 param2, const IndustryTileSpec *spec, const Industry *ind, TileIndex tile)
+{
+	return GetIndustryTileCallback(callback, param1, param2, spec - GetIndustryTileSpec(0), const_cast<Industry *>(ind), tile);
+}
+
+/** Helper class for animation control. */
+struct IndustryAnimationBase : public AnimationBase<IndustryAnimationBase, IndustryTileSpec, Industry, GetSimpleIndustryCallback> {
+	static const CallbackID cb_animation_speed      = CBID_INDTILE_ANIMATION_SPEED;
+	static const CallbackID cb_animation_next_frame = CBID_INDTILE_ANIM_NEXT_FRAME;
+
+	static const IndustryTileCallbackMask cbm_animation_speed      = CBM_INDT_ANIM_SPEED;
+	static const IndustryTileCallbackMask cbm_animation_next_frame = CBM_INDT_ANIM_NEXT_FRAME;
+};
+
 void AnimateNewIndustryTile(TileIndex tile)
 {
-	Industry *ind = Industry::GetByTile(tile);
-	IndustryGfx gfx = GetIndustryGfx(tile);
-	const IndustryTileSpec *itspec = GetIndustryTileSpec(gfx);
-	byte animation_speed = itspec->animation_speed;
+	const IndustryTileSpec *itspec = GetIndustryTileSpec(GetIndustryGfx(tile));
+	if (itspec == NULL) return;
 
-	if (HasBit(itspec->callback_mask, CBM_INDT_ANIM_SPEED)) {
-		uint16 callback_res = GetIndustryTileCallback(CBID_INDTILE_ANIMATION_SPEED, 0, 0, gfx, ind, tile);
-		if (callback_res != CALLBACK_FAILED) animation_speed = Clamp(callback_res & 0xFF, 0, 16);
-	}
-
-	/* An animation speed of 2 means the animation frame changes 4 ticks, and
-	 * increasing this value by one doubles the wait. 0 is the minimum value
-	 * allowed for animation_speed, which corresponds to 30ms, and 16 is the
-	 * maximum, corresponding to around 33 minutes. */
-	if ((_tick_counter % (1 << animation_speed)) != 0) return;
-
-	bool frame_set_by_callback = false;
-	byte frame = GetIndustryAnimationState(tile);
-	uint16 num_frames = GB(itspec->animation_info, 0, 8);
-
-	if (HasBit(itspec->callback_mask, CBM_INDT_ANIM_NEXT_FRAME)) {
-		uint16 callback_res = GetIndustryTileCallback(CBID_INDTILE_ANIM_NEXT_FRAME,
-				(itspec->special_flags & INDTILE_SPECIAL_NEXTFRAME_RANDOMBITS) ? Random() : 0, 0, gfx, ind, tile);
-
-		if (callback_res != CALLBACK_FAILED) {
-			frame_set_by_callback = true;
-
-			switch (callback_res & 0xFF) {
-				case 0xFF:
-					DeleteAnimatedTile(tile);
-					break;
-				case 0xFE:
-					/* Carry on as normal. */
-					frame_set_by_callback = false;
-					break;
-				default:
-					frame = callback_res & 0xFF;
-					break;
-			}
-
-			/* If the lower 7 bits of the upper byte of the callback
-			 * result are not empty, it is a sound effect. */
-			if (GB(callback_res, 8, 7) != 0) PlayTileSound(itspec->grf_prop.grffile, GB(callback_res, 8, 7), tile);
-		}
-	}
-
-	if (!frame_set_by_callback) {
-		if (frame < num_frames) {
-			frame++;
-		} else if (frame == num_frames && GB(itspec->animation_info, 8, 8) == 1) {
-			/* This animation loops, so start again from the beginning */
-			frame = 0;
-		} else {
-			/* This animation doesn't loop, so stay here */
-			DeleteAnimatedTile(tile);
-		}
-	}
-
-	SetIndustryAnimationState(tile, frame);
-	MarkTileDirtyByTile(tile);
-}
-
-static void ChangeIndustryTileAnimationFrame(const IndustryTileSpec *itspec, TileIndex tile, IndustryAnimationTrigger iat, uint32 random_bits, IndustryGfx gfx, Industry *ind)
-{
-	uint16 callback_res = GetIndustryTileCallback(CBID_INDTILE_ANIM_START_STOP, random_bits, iat, gfx, ind, tile);
-	if (callback_res == CALLBACK_FAILED) return;
-
-	switch (callback_res & 0xFF) {
-		case 0xFD: /* Do nothing. */         break;
-		case 0xFE: AddAnimatedTile(tile);    break;
-		case 0xFF: DeleteAnimatedTile(tile); break;
-		default:
-			SetIndustryAnimationState(tile, callback_res & 0xFF);
-			AddAnimatedTile(tile);
-			break;
-	}
-
-	/* If the lower 7 bits of the upper byte of the callback
-	 * result are not empty, it is a sound effect. */
-	if (GB(callback_res, 8, 7) != 0) PlayTileSound(itspec->grf_prop.grffile, GB(callback_res, 8, 7), tile);
+	IndustryAnimationBase::AnimateTile(itspec, Industry::GetByTile(tile), tile, (itspec->special_flags & INDTILE_SPECIAL_NEXTFRAME_RANDOMBITS) != 0);
 }
 
 bool StartStopIndustryTileAnimation(TileIndex tile, IndustryAnimationTrigger iat, uint32 random)
 {
-	IndustryGfx gfx = GetIndustryGfx(tile);
-	const IndustryTileSpec *itspec = GetIndustryTileSpec(gfx);
+	const IndustryTileSpec *itspec = GetIndustryTileSpec(GetIndustryGfx(tile));
 
-	if (!HasBit(itspec->animation_triggers, iat)) return false;
+	if (!HasBit(itspec->animation.triggers, iat)) return false;
 
-	Industry *ind = Industry::GetByTile(tile);
-	ChangeIndustryTileAnimationFrame(itspec, tile, iat, random, gfx, ind);
+	IndustryAnimationBase::ChangeAnimationFrame(CBID_INDTILE_ANIM_START_STOP, itspec, Industry::GetByTile(tile), tile, random, iat);
 	return true;
 }
 
@@ -393,7 +343,14 @@ bool StartStopIndustryTileAnimation(const Industry *ind, IndustryAnimationTrigge
 	return ret;
 }
 
-static void DoTriggerIndustryTile(TileIndex tile, IndustryTileTrigger trigger, Industry *ind)
+/**
+ * Trigger random triggers for an industry tile and reseed its random bits.
+ * @param tile Industry tile to trigger.
+ * @param trigger Trigger to trigger.
+ * @param ind Industry of the tile.
+ * @param [in,out] reseed_industry Collects bits to reseed for the industry.
+ */
+static void DoTriggerIndustryTile(TileIndex tile, IndustryTileTrigger trigger, Industry *ind, uint32 &reseed_industry)
 {
 	ResolverObject object;
 
@@ -402,34 +359,75 @@ static void DoTriggerIndustryTile(TileIndex tile, IndustryTileTrigger trigger, I
 	IndustryGfx gfx = GetIndustryGfx(tile);
 	const IndustryTileSpec *itspec = GetIndustryTileSpec(gfx);
 
-	if (itspec->grf_prop.spritegroup == NULL) return;
+	if (itspec->grf_prop.spritegroup[0] == NULL) return;
 
 	NewIndustryTileResolver(&object, gfx, tile, ind);
 
 	object.callback = CBID_RANDOM_TRIGGER;
 	object.trigger = trigger;
 
-	const SpriteGroup *group = SpriteGroup::Resolve(itspec->grf_prop.spritegroup, &object);
+	const SpriteGroup *group = SpriteGroup::Resolve(itspec->grf_prop.spritegroup[0], &object);
 	if (group == NULL) return;
 
 	byte new_random_bits = Random();
 	byte random_bits = GetIndustryRandomBits(tile);
-	random_bits &= ~object.reseed;
-	random_bits |= new_random_bits & object.reseed;
+	random_bits &= ~object.reseed[VSG_SCOPE_SELF];
+	random_bits |= new_random_bits & object.reseed[VSG_SCOPE_SELF];
 	SetIndustryRandomBits(tile, random_bits);
 	MarkTileDirtyByTile(tile);
+
+	reseed_industry |= object.reseed[VSG_SCOPE_PARENT];
 }
 
+/**
+ * Reseeds the random bits of an industry.
+ * @param ind Industry.
+ * @param reseed Bits to reseed.
+ */
+static void DoReseedIndustry(Industry *ind, uint32 reseed)
+{
+	if (reseed == 0 || ind == NULL) return;
+
+	uint16 random_bits = Random();
+	ind->random &= reseed;
+	ind->random |= random_bits & reseed;
+}
+
+/**
+ * Trigger a random trigger for a single industry tile.
+ * @param tile Industry tile to trigger.
+ * @param trigger Trigger to trigger.
+ */
 void TriggerIndustryTile(TileIndex tile, IndustryTileTrigger trigger)
 {
-	DoTriggerIndustryTile(tile, trigger, Industry::GetByTile(tile));
+	uint32 reseed_industry = 0;
+	Industry *ind = Industry::GetByTile(tile);
+	DoTriggerIndustryTile(tile, trigger, ind, reseed_industry);
+	DoReseedIndustry(ind, reseed_industry);
 }
 
+/**
+ * Trigger a random trigger for all industry tiles.
+ * @param ind Industry to trigger.
+ * @param trigger Trigger to trigger.
+ */
 void TriggerIndustry(Industry *ind, IndustryTileTrigger trigger)
 {
+	uint32 reseed_industry = 0;
 	TILE_AREA_LOOP(tile, ind->location) {
 		if (IsTileType(tile, MP_INDUSTRY) && GetIndustryIndex(tile) == ind->index) {
-			DoTriggerIndustryTile(tile, trigger, ind);
+			DoTriggerIndustryTile(tile, trigger, ind, reseed_industry);
 		}
 	}
+	DoReseedIndustry(ind, reseed_industry);
+}
+
+/**
+ * Resolve a industry tile's spec and such so we can get a variable.
+ * @param ro    The resolver object to fill.
+ * @param index The industry tile to get the data from.
+ */
+void GetIndustryTileResolver(ResolverObject *ro, uint index)
+{
+	NewIndustryTileResolver(ro, GetIndustryGfx(index), index, Industry::GetByTile(index));
 }
