@@ -1,4 +1,4 @@
-/* $Id: openttd.cpp 19938 2010-06-05 21:13:35Z rubidium $ */
+/* $Id: openttd.cpp 22886 2011-09-03 18:56:34Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,11 +11,6 @@
 
 #include "stdafx.h"
 
-#define VARDEF
-#include "variables.h"
-#undef VARDEF
-
-
 #include "blitter/factory.hpp"
 #include "sound/sound_driver.hpp"
 #include "music/music_driver.hpp"
@@ -28,7 +23,6 @@
 
 #include "base_media_base.h"
 #include "saveload/saveload.h"
-#include "landscape.h"
 #include "company_func.h"
 #include "command_func.h"
 #include "news_func.h"
@@ -36,6 +30,7 @@
 #include "aircraft.h"
 #include "roadveh.h"
 #include "train.h"
+#include "ship.h"
 #include "console_func.h"
 #include "screenshot.h"
 #include "network/network.h"
@@ -50,22 +45,21 @@
 #include "date_func.h"
 #include "vehicle_func.h"
 #include "gamelog.h"
-#include "cheat_type.h"
 #include "animated_tile_func.h"
 #include "roadstop_base.h"
-#include "functions.h"
 #include "elrail_func.h"
 #include "rev.h"
 #include "highscore.h"
 #include "thread/thread.h"
 #include "station_base.h"
 #include "crashlog.h"
-#include "company_base.h"
-#include "engine_base.h"
 #include "engine_func.h"
 #include "core/random_func.hpp"
+#include "rail_gui.h"
+#include "core/backup_type.hpp"
+#include "hotkeys.h"
+#include "newgrf.h"
 
-#include "newgrf_commons.h"
 
 #include "town.h"
 #include "industry.h"
@@ -81,13 +75,12 @@ void IncreaseDate();
 void DoPaletteAnimations();
 void MusicLoop();
 void ResetMusic();
-void ProcessAsyncSaveFinish();
 void CallWindowTickEvent();
 
 extern void SetDifficultyLevel(int mode, DifficultySettings *gm_opt);
 extern Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY);
 extern void ShowOSErrorBox(const char *buf, bool system);
-extern void InitializeRailGUI();
+extern char *_config_file;
 
 /**
  * Error handling for fatal user errors.
@@ -207,8 +200,9 @@ static void ShowHelp()
 	p = BlitterFactoryBase::GetBlittersInfo(p, lastof(buf));
 
 	/* We need to initialize the AI, so it finds the AIs */
+	TarScanner::DoScan();
 	AI::Initialize();
-	p = AI::GetConsoleList(p, lastof(buf));
+	p = AI::GetConsoleList(p, lastof(buf), true);
 	AI::Uninitialize(true);
 
 	/* ShowInfo put output to stderr, but version information should go
@@ -241,8 +235,9 @@ struct MyGetOptData {
 static int MyGetOpt(MyGetOptData *md)
 {
 	char *s = md->cont;
-	if (s != NULL)
+	if (s != NULL) {
 		goto md_continue_here;
+	}
 
 	for (;;) {
 		if (--md->numleft < 0) return -1;
@@ -265,8 +260,7 @@ md_continue_here:;
 						/* It comes as a separate arg. Check if out of args? */
 						if (--md->numleft < 0 || *(t = *md->argv) == '-') {
 							/* Check if item is optional? */
-							if (r[2] != ':')
-								return -2;
+							if (r[2] != ':') return -2;
 							md->numleft++;
 							t = NULL;
 						} else {
@@ -306,17 +300,9 @@ static void ParseResolution(Dimension *res, const char *s)
 	res->height = max(strtoul(t + 1, NULL, 0), 64UL);
 }
 
-static void InitializeDynamicVariables()
-{
-	/* Dynamic stuff needs to be initialized somewhere... */
-	_engine_mngr.ResetToDefaultMapping();
-	_house_mngr.ResetMapping();
-	_industry_mngr.ResetMapping();
-	_industile_mngr.ResetMapping();
-}
 
-
-/** Unitializes drivers, frees allocated memory, cleans pools, ...
+/**
+ * Unitializes drivers, frees allocated memory, cleans pools, ...
  * Generally, prepares the game for shutting down
  */
 static void ShutdownGame()
@@ -332,9 +318,6 @@ static void ShutdownGame()
 	/* stop the AI */
 	AI::Uninitialize(false);
 
-	/* Uninitialize airport state machines */
-	UnInitializeAirports();
-
 	/* Uninitialize variables that are allocated dynamically */
 	GamelogReset();
 	_town_pool.CleanPool();
@@ -349,7 +332,11 @@ static void ShutdownGame()
 	_engine_pool.CleanPool();
 	_company_pool.CleanPool();
 
+#ifdef ENABLE_NETWORK
 	free(_config_file);
+#endif
+
+	ResetNewGRFData();
 
 	/* Close all and any open filehandles */
 	FioCloseAll();
@@ -377,6 +364,7 @@ static void LoadIntroGame()
 	_pause_mode = PM_UNPAUSED;
 	_cursor.fix_at = false;
 
+	CheckForMissingSprites();
 	CheckForMissingGlyphsInLoadedLanguagePack();
 
 	/* Play main theme */
@@ -393,7 +381,10 @@ void MakeNewgameSettingsLive()
 	}
 #endif /* ENABLE_AI */
 
+	/* Copy newgame settings to active settings.
+	 * Also initialise old settings needed for savegame conversion. */
 	_settings_game = _settings_newgame;
+	_old_vds = _settings_client.company.vehicle;
 
 #ifdef ENABLE_AI
 	for (CompanyID c = COMPANY_FIRST; c < MAX_COMPANIES; c++) {
@@ -405,7 +396,6 @@ void MakeNewgameSettingsLive()
 #endif /* ENABLE_AI */
 }
 
-byte _savegame_sort_order;
 #if defined(UNIX) && !defined(__MORPHOS__)
 extern void DedicatedFork();
 #endif
@@ -434,12 +424,14 @@ int ttd_main(int argc, char *argv[])
 	uint16 dedicated_port = 0;
 	char *join_server_password = NULL;
 	char *join_company_password = NULL;
+
+	extern bool _dedicated_forks;
+	_dedicated_forks = false;
 #endif /* ENABLE_NETWORK */
 
 	_game_mode = GM_MENU;
 	_switch_mode = SM_MENU;
 	_switch_mode_errorstr = INVALID_STRING_ID;
-	_dedicated_forks = false;
 	_config_file = NULL;
 
 	/* The last param of the following function means this:
@@ -507,7 +499,8 @@ int ttd_main(int argc, char *argv[])
 				CreateConsole();
 #endif
 				if (mgo.opt != NULL) SetDebugString(mgo.opt);
-			} break;
+				break;
+			}
 		case 'e': _switch_mode = SM_EDITOR; break;
 		case 'i':
 			/* there is an argument, it is not empty, and it is exactly 1 char long */
@@ -565,17 +558,18 @@ int ttd_main(int argc, char *argv[])
 	BaseSounds::FindSets();
 	BaseMusic::FindSets();
 
-#if defined(UNIX) && !defined(__MORPHOS__)
+#if defined(ENABLE_NETWORK) && defined(UNIX) && !defined(__MORPHOS__)
 	/* We must fork here, or we'll end up without some resources we need (like sockets) */
-	if (_dedicated_forks)
-		DedicatedFork();
+	if (_dedicated_forks) DedicatedFork();
 #endif
 
+	TarScanner::DoScan();
 	AI::Initialize();
 	LoadFromConfig();
 	AI::Uninitialize(true);
 	CheckConfig();
 	LoadFromHighScore();
+	LoadHotkeysFromConfig();
 
 	if (resolution.width != 0) { _cur_resolution = resolution; }
 	if (startyear != INVALID_YEAR) _settings_newgame.game_creation.starting_year = startyear;
@@ -583,13 +577,15 @@ int ttd_main(int argc, char *argv[])
 
 	/*
 	 * The width and height must be at least 1 pixel and width times
-	 * height must still fit within a 32 bits integer, this way all
-	 * internal drawing routines work correctly.
+	 * height times bytes per pixel must still fit within a 32 bits
+	 * integer, even for 32 bpp video modes. This way all internal
+	 * drawing routines work correctly.
 	 */
-	_cur_resolution.width  = ClampU(_cur_resolution.width,  1, UINT16_MAX);
-	_cur_resolution.height = ClampU(_cur_resolution.height, 1, UINT16_MAX);
+	_cur_resolution.width  = ClampU(_cur_resolution.width,  1, UINT16_MAX / 2);
+	_cur_resolution.height = ClampU(_cur_resolution.height, 1, UINT16_MAX / 2);
 
 #if defined(ENABLE_NETWORK)
+	if (dedicated) DEBUG(net, 0, "Starting dedicated version %s", _openttd_revision);
 	if (dedicated_host) {
 		_network_bind_list.Clear();
 		*_network_bind_list.Append() = strdup(dedicated_host);
@@ -604,12 +600,6 @@ int ttd_main(int argc, char *argv[])
 	/* initialize screenshot formats */
 	InitializeScreenshotFormats();
 
-	/* initialize airport state machines */
-	InitializeAirports();
-
-	/* initialize all variables that are allocated dynamically */
-	InitializeDynamicVariables();
-
 	/* Initialize FreeType */
 	InitFreeType();
 
@@ -648,10 +638,11 @@ int ttd_main(int argc, char *argv[])
 
 	DEBUG(misc, 1, "Loading blitter...");
 	if (blitter == NULL && _ini_blitter != NULL) blitter = strdup(_ini_blitter);
-	if (BlitterFactoryBase::SelectBlitter(blitter) == NULL)
+	if (BlitterFactoryBase::SelectBlitter(blitter) == NULL) {
 		StrEmpty(blitter) ?
 			usererror("Failed to autoprobe blitter") :
 			usererror("Failed to select requested blitter '%s'; does it exist?", blitter);
+	}
 	free(blitter);
 
 	DEBUG(driver, 1, "Loading drivers...");
@@ -683,12 +674,11 @@ int ttd_main(int argc, char *argv[])
 	}
 	free(musicdriver);
 
-	_savegame_sort_order = SORT_BY_DATE | SORT_DESCENDING;
 	/* Initialize the zoom level of the screen to normal */
 	_screen.zoom = ZOOM_LVL_NORMAL;
 
 	/* restore saved music volume */
-	_music_driver->SetVolume(msf.music_vol);
+	_music_driver->SetVolume(_msf.music_vol);
 
 	NetworkStartUp(); // initialize network-core
 
@@ -763,6 +753,7 @@ int ttd_main(int argc, char *argv[])
 	/* only save config if we have to */
 	if (save_config) {
 		SaveToConfig();
+		SaveHotkeysToConfig();
 		SaveToHighScore();
 	}
 
@@ -812,7 +803,6 @@ static void MakeNewGameDone()
 	IConsoleCmdExec("exec scripts/game_start.scr 0");
 
 	SetLocalCompany(COMPANY_FIRST);
-	_current_company = _local_company;
 
 	InitializeRailGUI();
 
@@ -820,7 +810,7 @@ static void MakeNewGameDone()
 	/* We are the server, we start a new company (not dedicated),
 	 * so set the default password *if* needed. */
 	if (_network_server && !StrEmpty(_settings_client.network.default_company_pass)) {
-		NetworkChangeCompanyPassword(_settings_client.network.default_company_pass);
+		NetworkChangeCompanyPassword(_local_company, _settings_client.network.default_company_pass);
 	}
 #endif /* ENABLE_NETWORK */
 
@@ -834,7 +824,6 @@ static void MakeNewGame(bool from_heightmap, bool reset_settings)
 	_game_mode = GM_NORMAL;
 
 	ResetGRFConfig(true);
-	InitializeDynamicVariables();
 
 	GenerateWorldSetCallback(&MakeNewGameDone);
 	GenerateWorld(from_heightmap ? GWM_HEIGHTMAP : GWM_NEWGAME, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y, reset_settings);
@@ -855,76 +844,45 @@ static void MakeNewEditorWorld()
 	GenerateWorld(GWM_EMPTY, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
 }
 
-void StartupCompanies();
-void StartupDisasters();
-extern void StartupEconomy();
-
 /**
- * Start Scenario starts a new game based on a scenario.
- * Eg 'New Game' --> select a preset scenario
- * This starts a scenario based on your current difficulty settings
- */
-static void StartScenario()
-{
-	_game_mode = GM_NORMAL;
-
-	/* invalid type */
-	if (_file_to_saveload.mode == SL_INVALID) {
-		DEBUG(sl, 0, "Savegame is obsolete or invalid format: '%s'", _file_to_saveload.name);
-		SetDParamStr(0, GetSaveLoadErrorString());
-		ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
-		_game_mode = GM_MENU;
-		return;
-	}
-
-	/* Reinitialize windows */
-	ResetWindowSystem();
-
-	SetupColoursAndInitialWindow();
-
-	ResetGRFConfig(true);
-
-	/* Load game */
-	if (SaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, SCENARIO_DIR) != SL_OK) {
-		LoadIntroGame();
-		SetDParamStr(0, GetSaveLoadErrorString());
-		ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
-	}
-
-	_settings_game.difficulty = _settings_newgame.difficulty;
-
-	/* Inititalize data */
-	StartupEconomy();
-	StartupCompanies();
-	StartupEngines();
-	StartupDisasters();
-
-	SetLocalCompany(COMPANY_FIRST);
-	_current_company = _local_company;
-	Company *c = Company::Get(COMPANY_FIRST);
-	c->settings = _settings_client.company;
-
-	MarkWholeScreenDirty();
-}
-
-/** Load the specified savegame but on error do different things.
+ * Load the specified savegame but on error do different things.
  * If loading fails due to corrupt savegame, bad version, etc. go back to
  * a previous correct state. In the menu for example load the intro game again.
- * @param filename file to be loaded
  * @param mode mode of loading, either SL_LOAD or SL_OLD_LOAD
  * @param newgm switch to this mode of loading fails due to some unknown error
+ * @param filename file to be loaded
  * @param subdir default directory to look for filename, set to 0 if not needed
+ * @param lf Load filter to use, if NULL: use filename + subdir.
  */
-bool SafeSaveOrLoad(const char *filename, int mode, GameMode newgm, Subdirectory subdir)
+bool SafeLoad(const char *filename, int mode, GameMode newgm, Subdirectory subdir, struct LoadFilter *lf = NULL)
 {
+	assert(mode == SL_LOAD || (lf == NULL && mode == SL_OLD_LOAD));
 	GameMode ogm = _game_mode;
 
 	_game_mode = newgm;
-	assert(mode == SL_LOAD || mode == SL_OLD_LOAD);
-	switch (SaveOrLoad(filename, mode, subdir)) {
+
+	switch (lf == NULL ? SaveOrLoad(filename, mode, subdir) : LoadWithFilter(lf)) {
 		case SL_OK: return true;
 
 		case SL_REINIT:
+#ifdef ENABLE_NETWORK
+			if (_network_dedicated) {
+				/*
+				 * We need to reinit a network map...
+				 * We can't simply load the intro game here as that game has many
+				 * special cases which make clients desync immediately. So we fall
+				 * back to just generating a new game with the current settings.
+				 */
+				DEBUG(net, 0, "Loading game failed, so a new (random) game will be started!");
+				MakeNewGame(false, true);
+				return false;
+			}
+			if (_network_server) {
+				/* We can't load the intro game as server, so disconnect first. */
+				NetworkDisconnect();
+			}
+#endif /* ENABLE_NETWORK */
+
 			switch (ogm) {
 				default:
 				case GM_MENU:   LoadIntroGame();      break;
@@ -988,23 +946,13 @@ void SwitchToMode(SwitchMode new_mode)
 			MakeNewGame(false, new_mode == SM_NEWGAME);
 			break;
 
-		case SM_START_SCENARIO: // New Game --> Choose one of the preset scenarios
-#ifdef ENABLE_NETWORK
-			if (_network_server) {
-				snprintf(_network_game_info.map_name, lengthof(_network_game_info.map_name), "%s (Loaded scenario)", _file_to_saveload.title);
-			}
-#endif /* ENABLE_NETWORK */
-			StartScenario();
-			break;
-
 		case SM_LOAD: { // Load game, Play Scenario
 			ResetGRFConfig(true);
 			ResetWindowSystem();
 
-			if (!SafeSaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_NORMAL, NO_DIRECTORY)) {
-				LoadIntroGame();
+			if (!SafeLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_NORMAL, NO_DIRECTORY)) {
 				SetDParamStr(0, GetSaveLoadErrorString());
-				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
+				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, WL_ERROR);
 			} else {
 				if (_saveload_mode == SLD_LOAD_SCENARIO) {
 					StartupEngines();
@@ -1042,14 +990,14 @@ void SwitchToMode(SwitchMode new_mode)
 			break;
 
 		case SM_LOAD_SCENARIO: { // Load scenario from scenario editor
-			if (SafeSaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_EDITOR, NO_DIRECTORY)) {
+			if (SafeLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_EDITOR, NO_DIRECTORY)) {
 				SetLocalCompany(OWNER_NONE);
 				_settings_newgame.game_creation.starting_year = _cur_year;
 				/* Cancel the saveload pausing */
 				DoCommandP(0, PM_PAUSED_SAVELOAD, 0, CMD_PAUSE);
 			} else {
 				SetDParamStr(0, GetSaveLoadErrorString());
-				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
+				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, WL_ERROR);
 			}
 			break;
 		}
@@ -1057,7 +1005,7 @@ void SwitchToMode(SwitchMode new_mode)
 		case SM_MENU: // Switch to game intro menu
 			LoadIntroGame();
 			if (BaseSounds::ini_set == NULL && BaseSounds::GetUsedSet()->fallback) {
-				ShowErrorMessage(STR_WARNING_FALLBACK_SOUNDSET, INVALID_STRING_ID, 0, 0, true);
+				ShowErrorMessage(STR_WARNING_FALLBACK_SOUNDSET, INVALID_STRING_ID, WL_CRITICAL);
 				BaseSounds::ini_set = strdup(BaseSounds::GetUsedSet()->name);
 			}
 			break;
@@ -1066,7 +1014,7 @@ void SwitchToMode(SwitchMode new_mode)
 			/* Make network saved games on pause compatible to singleplayer */
 			if (SaveOrLoad(_file_to_saveload.name, SL_SAVE, NO_DIRECTORY) != SL_OK) {
 				SetDParamStr(0, GetSaveLoadErrorString());
-				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
+				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, WL_ERROR);
 			} else {
 				DeleteWindowById(WC_SAVELOAD, 0);
 			}
@@ -1083,7 +1031,8 @@ void SwitchToMode(SwitchMode new_mode)
 	}
 
 	if (_switch_mode_errorstr != INVALID_STRING_ID) {
-		ShowErrorMessage(_switch_mode_errorstr, INVALID_STRING_ID, 0, 0, true);
+		ShowErrorMessage(_switch_mode_errorstr, INVALID_STRING_ID, WL_CRITICAL);
+		_switch_mode_errorstr = INVALID_STRING_ID;
 	}
 }
 
@@ -1112,58 +1061,77 @@ static void CheckCaches()
 
 	Vehicle *v;
 	FOR_ALL_VEHICLES(v) {
-		if (v != v->First() || v->vehstatus & VS_CRASHED) continue;
+		extern void FillNewGRFVehicleCache(const Vehicle *v);
+		if (v != v->First() || v->vehstatus & VS_CRASHED || !v->IsPrimaryVehicle()) continue;
+
+		uint length = 0;
+		for (const Vehicle *u = v; u != NULL; u = u->Next()) length++;
+
+		NewGRFCache        *grf_cache = CallocT<NewGRFCache>(length);
+		VehicleCache       *veh_cache = CallocT<VehicleCache>(length);
+		GroundVehicleCache *gro_cache = CallocT<GroundVehicleCache>(length);
+		TrainCache         *tra_cache = CallocT<TrainCache>(length);
+
+		length = 0;
+		for (const Vehicle *u = v; u != NULL; u = u->Next()) {
+			FillNewGRFVehicleCache(u);
+			grf_cache[length] = u->grf_cache;
+			veh_cache[length] = u->vcache;
+			switch (u->type) {
+				case VEH_TRAIN:
+					gro_cache[length] = Train::From(u)->gcache;
+					tra_cache[length] = Train::From(u)->tcache;
+					break;
+				case VEH_ROAD:
+					gro_cache[length] = RoadVehicle::From(u)->gcache;
+					break;
+				default:
+					break;
+			}
+			length++;
+		}
 
 		switch (v->type) {
-			case VEH_ROAD: {
-				RoadVehicle *rv = RoadVehicle::From(v);
-				RoadVehicleCache cache;
-				memset(&cache, 0, sizeof(cache));
-				cache = rv->rcache;
-				RoadVehUpdateCache(rv);
-
-				if (memcmp(&cache, &rv->rcache, sizeof(RoadVehicleCache)) != 0) {
-					DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i", v->index, (int)v->owner, v->unitnumber);
-				}
-			} break;
-
-			case VEH_TRAIN: {
-				uint length = 0;
-				Train *t = Train::From(v);
-				for (Vehicle *u = t; u != NULL; u = u->Next()) length++;
-
-				TrainCache *wagons = CallocT<TrainCache>(length);
-				length = 0;
-				for (Train *u = t; u != NULL; u = u->Next()) wagons[length++] = u->tcache;
-
-				t->ConsistChanged(true);
-
-				length = 0;
-				for (Train *u = t; u != NULL; u = u->Next()) {
-					if (memcmp(&wagons[length], &u->tcache, sizeof(TrainCache)) != 0) {
-						DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i, wagon %i", v->index, (int)v->owner, v->unitnumber, length);
-					}
-					length++;
-				}
-
-				free(wagons);
-			} break;
-
-			case VEH_AIRCRAFT: {
-				Aircraft *a = Aircraft::From(v);
-				AircraftCache cache;
-				memset(&cache, 0, sizeof(cache));
-				cache = a->acache;
-				UpdateAircraftCache(a);
-
-				if (memcmp(&cache, &a->acache, sizeof(AircraftCache)) != 0) {
-					DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i", v->index, (int)v->owner, v->unitnumber);
-				}
-			} break;
-
-			default:
-				break;
+			case VEH_TRAIN:    Train::From(v)->ConsistChanged(true);     break;
+			case VEH_ROAD:     RoadVehUpdateCache(RoadVehicle::From(v)); break;
+			case VEH_AIRCRAFT: UpdateAircraftCache(Aircraft::From(v));   break;
+			case VEH_SHIP:     Ship::From(v)->UpdateCache();             break;
+			default: break;
 		}
+
+		length = 0;
+		for (const Vehicle *u = v; u != NULL; u = u->Next()) {
+			FillNewGRFVehicleCache(u);
+			if (memcmp(&grf_cache[length], &u->grf_cache, sizeof(NewGRFCache)) != 0) {
+				DEBUG(desync, 2, "newgrf cache mismatch: type %i, vehicle %i, company %i, unit number %i, wagon %i", (int)v->type, v->index, (int)v->owner, v->unitnumber, length);
+			}
+			if (memcmp(&veh_cache[length], &u->vcache, sizeof(VehicleCache)) != 0) {
+				DEBUG(desync, 2, "vehicle cache mismatch: type %i, vehicle %i, company %i, unit number %i, wagon %i", (int)v->type, v->index, (int)v->owner, v->unitnumber, length);
+			}
+			switch (u->type) {
+				case VEH_TRAIN:
+					if (memcmp(&gro_cache[length], &Train::From(u)->gcache, sizeof(GroundVehicleCache)) != 0) {
+						DEBUG(desync, 2, "train ground vehicle cache mismatch: vehicle %i, company %i, unit number %i, wagon %i", v->index, (int)v->owner, v->unitnumber, length);
+					}
+					if (memcmp(&tra_cache[length], &Train::From(u)->tcache, sizeof(TrainCache)) != 0) {
+						DEBUG(desync, 2, "train cache mismatch: vehicle %i, company %i, unit number %i, wagon %i", v->index, (int)v->owner, v->unitnumber, length);
+					}
+					break;
+				case VEH_ROAD:
+					if (memcmp(&gro_cache[length], &RoadVehicle::From(u)->gcache, sizeof(GroundVehicleCache)) != 0) {
+						DEBUG(desync, 2, "road vehicle ground vehicle cache mismatch: vehicle %i, company %i, unit number %i, wagon %i", v->index, (int)v->owner, v->unitnumber, length);
+					}
+					break;
+				default:
+					break;
+			}
+			length++;
+		}
+
+		free(grf_cache);
+		free(veh_cache);
+		free(gro_cache);
+		free(tra_cache);
 	}
 
 	/* Check whether the caches are still valid */
@@ -1194,6 +1162,7 @@ void StateGameLoop()
 {
 	/* dont execute the state loop during pause */
 	if (_pause_mode != PM_UNPAUSED) {
+		UpdateLandscapingLimits();
 		CallWindowTickEvent();
 		return;
 	}
@@ -1206,6 +1175,7 @@ void StateGameLoop()
 		CallVehicleTicks();
 		CallLandscapeTick();
 		ClearStorageChanges(true);
+		UpdateLandscapingLimits();
 
 		CallWindowTickEvent();
 		NewsLoop();
@@ -1214,15 +1184,14 @@ void StateGameLoop()
 			/* Save the desync savegame if needed. */
 			char name[MAX_PATH];
 			snprintf(name, lengthof(name), "dmp_cmds_%08x_%08x.sav", _settings_game.game_creation.generation_seed, _date);
-			SaveOrLoad(name, SL_SAVE, AUTOSAVE_DIR);
+			SaveOrLoad(name, SL_SAVE, AUTOSAVE_DIR, false);
 		}
 
 		CheckCaches();
 
 		/* All these actions has to be done from OWNER_NONE
 		 *  for multiplayer compatibility */
-		CompanyID old_company = _current_company;
-		_current_company = OWNER_NONE;
+		Backup<CompanyByte> cur_company(_current_company, OWNER_NONE, FILE_LINE);
 
 		AnimateAnimatedTiles();
 		IncreaseDate();
@@ -1232,15 +1201,20 @@ void StateGameLoop()
 		ClearStorageChanges(true);
 
 		AI::GameLoop();
+		UpdateLandscapingLimits();
 
 		CallWindowTickEvent();
 		NewsLoop();
-		_current_company = old_company;
+		cur_company.Restore();
 	}
+
+	assert(IsLocalCompany());
 }
 
-/** Create an autosave. The default name is "autosave#.sav". However with
- * the setting 'keep_all_autosave' the name defaults to company-name + date */
+/**
+ * Create an autosave. The default name is "autosave#.sav". However with
+ * the setting 'keep_all_autosave' the name defaults to company-name + date
+ */
 static void DoAutosave()
 {
 	char buf[MAX_PATH];
@@ -1264,7 +1238,7 @@ static void DoAutosave()
 
 	DEBUG(sl, 2, "Autosaving to '%s'", buf);
 	if (SaveOrLoad(buf, SL_SAVE, AUTOSAVE_DIR) != SL_OK) {
-		ShowErrorMessage(STR_ERROR_AUTOSAVE_FAILED, INVALID_STRING_ID, 0, 0);
+		ShowErrorMessage(STR_ERROR_AUTOSAVE_FAILED, INVALID_STRING_ID, WL_ERROR);
 	}
 }
 
@@ -1276,7 +1250,7 @@ void GameLoop()
 	if (_do_autosave) {
 		_do_autosave = false;
 		DoAutosave();
-		RedrawAutosave();
+		SetWindowDirty(WC_STATUS_BAR, 0);
 	}
 
 	/* switch game mode? */
@@ -1290,7 +1264,6 @@ void GameLoop()
 
 	extern int _caret_timer;
 	_caret_timer += 3;
-	_palette_animation_counter += 8;
 	CursorTick();
 
 #ifdef ENABLE_NETWORK
@@ -1309,13 +1282,20 @@ void GameLoop()
 		/* Singleplayer */
 		StateGameLoop();
 	}
+
+	/* Check chat messages roughly once a second. */
+	static uint check_message = 0;
+	if (++check_message > 1000 / MILLISECONDS_PER_TICK) {
+		check_message = 0;
+		NetworkChatMessageLoop();
+	}
 #else
 	StateGameLoop();
 #endif /* ENABLE_NETWORK */
 
 	if (!_pause_mode && HasBit(_display_opt, DO_FULL_ANIMATION)) DoPaletteAnimations();
 
-	if (!_pause_mode || _cheats.build_in_pause.value) MoveAllTextEffects();
+	if (!_pause_mode || _game_mode == GM_EDITOR || _settings_game.construction.command_pause_level > CMDPL_NO_CONSTRUCTION) MoveAllTextEffects();
 
 	InputLoop();
 
