@@ -1,4 +1,4 @@
-/* $Id: yapf_costrail.hpp 19927 2010-06-04 21:10:18Z rubidium $ */
+/* $Id: yapf_costrail.hpp 21815 2011-01-15 22:40:54Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -9,8 +9,8 @@
 
 /** @file yapf_costrail.hpp Cost determination for rails. */
 
-#ifndef  YAPF_COSTRAIL_HPP
-#define  YAPF_COSTRAIL_HPP
+#ifndef YAPF_COSTRAIL_HPP
+#define YAPF_COSTRAIL_HPP
 
 #include "../../pbs.h"
 
@@ -258,8 +258,8 @@ public:
 		const Train *v = Yapf().GetVehicle();
 		assert(v != NULL);
 		assert(v->type == VEH_TRAIN);
-		assert(v->tcache.cached_total_length != 0);
-		int missing_platform_length = (v->tcache.cached_total_length + TILE_SIZE - 1) / TILE_SIZE - platform_length;
+		assert(v->gcache.cached_total_length != 0);
+		int missing_platform_length = CeilDiv(v->gcache.cached_total_length, TILE_SIZE) - platform_length;
 		if (missing_platform_length < 0) {
 			/* apply penalty for longer platform than needed */
 			cost += Yapf().PfGetSettings().rail_longer_platform_penalty + Yapf().PfGetSettings().rail_longer_platform_per_tile_penalty * -missing_platform_length;
@@ -276,9 +276,11 @@ public:
 		m_max_cost = max_cost;
 	}
 
-	/** Called by YAPF to calculate the cost from the origin to the given node.
+	/**
+	 * Called by YAPF to calculate the cost from the origin to the given node.
 	 *  Calculates only the cost of given node, adds it to the parent node cost
-	 *  and stores the result into Node::m_cost member */
+	 *  and stores the result into Node::m_cost member
+	 */
 	FORCEINLINE bool PfCalcCost(Node &n, const TrackFollower *tf)
 	{
 		assert(!n.flags_u.flags_s.m_targed_seen);
@@ -433,10 +435,14 @@ no_entry_cost: // jump here at the beginning if the node has no parent (it is th
 						/* If this is a safe waiting position we're done searching for it */
 						if (IsSafeWaitingPosition(v, t, td, true, _settings_game.pf.forbid_90_deg)) break;
 					}
+
+					/* In the case this platform is (possibly) occupied we add penalty so the
+					 * other platforms of this waypoint are evaluated as well, i.e. we assume
+					 * that there is a red signal in the waypoint when it's occupied. */
 					if (td == INVALID_TRACKDIR ||
 							!IsSafeWaitingPosition(v, t, td, true, _settings_game.pf.forbid_90_deg) ||
 							!IsWaitingPositionFree(v, t, td, _settings_game.pf.forbid_90_deg)) {
-						extra_cost += Yapf().PfGetSettings().rail_lastred_exit_penalty;
+						extra_cost += Yapf().PfGetSettings().rail_lastred_penalty;
 					}
 				}
 				/* Waypoint is also a good reason to finish. */
@@ -464,11 +470,12 @@ no_entry_cost: // jump here at the beginning if the node has no parent (it is th
 			{
 				int min_speed = 0;
 				int max_speed = tf->GetSpeedLimit(&min_speed);
-				if (max_speed < v->max_speed) {
-					extra_cost += YAPF_TILE_LENGTH * (v->max_speed - max_speed) * (4 + tf->m_tiles_skipped) / v->max_speed;
+				int max_veh_speed = v->GetDisplayMaxSpeed();
+				if (max_speed < max_veh_speed) {
+					extra_cost += YAPF_TILE_LENGTH * (max_veh_speed - max_speed) * (4 + tf->m_tiles_skipped) / max_veh_speed;
 				}
-				if (min_speed > v->max_speed) {
-					extra_cost += YAPF_TILE_LENGTH * (min_speed - v->max_speed);
+				if (min_speed > max_veh_speed) {
+					extra_cost += YAPF_TILE_LENGTH * (min_speed - max_veh_speed);
 				}
 			}
 
@@ -508,10 +515,13 @@ no_entry_cost: // jump here at the beginning if the node has no parent (it is th
 			TILE next(tf_local.m_new_tile, (Trackdir)FindFirstBit2x64(tf_local.m_new_td_bits));
 
 			if (TrackFollower::DoTrackMasking() && IsTileType(next.tile, MP_RAILWAY)) {
-				if ((HasSignalOnTrackdir(next.tile, next.td) && IsPbsSignal(GetSignalType(next.tile, TrackdirToTrack(next.td)))) ||
-						(HasSignalOnTrackdir(next.tile, ReverseTrackdir(next.td)) && GetSignalType(next.tile, TrackdirToTrack(next.td)) == SIGTYPE_PBS_ONEWAY)) {
+				if (HasSignalOnTrackdir(next.tile, next.td) && IsPbsSignal(GetSignalType(next.tile, TrackdirToTrack(next.td)))) {
 					/* Possible safe tile. */
 					end_segment_reason |= ESRB_SAFE_TILE;
+				} else if (HasSignalOnTrackdir(next.tile, ReverseTrackdir(next.td)) && GetSignalType(next.tile, TrackdirToTrack(next.td)) == SIGTYPE_PBS_ONEWAY) {
+					/* Possible safe tile, but not so good as it's the back of a signal... */
+					end_segment_reason |= ESRB_SAFE_TILE | ESRB_DEAD_END;
+					extra_cost += Yapf().PfGetSettings().rail_lastred_exit_penalty;
 				}
 			}
 
@@ -580,8 +590,8 @@ no_entry_cost: // jump here at the beginning if the node has no parent (it is th
 				if (n.m_last_red_signal_type == SIGTYPE_EXIT) {
 					/* last signal was red pre-signal-exit */
 					extra_cost += Yapf().PfGetSettings().rail_lastred_exit_penalty;
-				} else {
-					/* last signal was red, but not exit */
+				} else if (!IsPbsSignal(n.m_last_red_signal_type)) {
+					/* Last signal was red, but not exit or path signal. */
 					extra_cost += Yapf().PfGetSettings().rail_lastred_penalty;
 				}
 			}
