@@ -1,4 +1,4 @@
-/* $Id: road_gui.cpp 19665 2010-04-17 22:27:49Z rubidium $ */
+/* $Id: road_gui.cpp 22252 2011-03-14 19:03:17Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,18 +17,18 @@
 #include "viewport_func.h"
 #include "command_func.h"
 #include "road_cmd.h"
-#include "road_map.h"
 #include "station_func.h"
-#include "functions.h"
 #include "window_func.h"
 #include "vehicle_func.h"
 #include "sound_func.h"
 #include "company_func.h"
 #include "tunnelbridge.h"
+#include "tunnelbridge_map.h"
 #include "tilehighlight_func.h"
 #include "company_base.h"
+#include "hotkeys.h"
+#include "road_gui.h"
 
-#include "table/sprites.h"
 #include "table/strings.h"
 
 static void ShowRVStationPicker(Window *parent, RoadStopType rs);
@@ -50,7 +50,7 @@ enum RoadFlags {
 	RF_START_HALFROAD_X = 0x08,    // The start tile in X-dir should have only a half road
 	RF_END_HALFROAD_X   = 0x10,    // The end tile in X-dir should have only a half road
 };
-DECLARE_ENUM_AS_BIT_SET(RoadFlags);
+DECLARE_ENUM_AS_BIT_SET(RoadFlags)
 
 static RoadFlags _place_road_flag;
 
@@ -65,62 +65,41 @@ void CcPlaySound1D(const CommandCost &result, TileIndex tile, uint32 p1, uint32
 }
 
 /**
- * Set the initial flags for the road constuction.
- * The flags are:
- * @li The direction is the X-dir
- * @li The first tile has a partitial RoadBit (true or false)
- *
- * @param tile The start tile
+ * Callback to start placing a bridge.
+ * @param tile Start tile of the bridge.
  */
-static void PlaceRoad_X_Dir(TileIndex tile)
+static void PlaceRoad_Bridge(TileIndex tile, Window *w)
 {
-	_place_road_flag = RF_DIR_X;
-	if (_tile_fract_coords.x >= 8) _place_road_flag |= RF_START_HALFROAD_X;
-	VpStartPlaceSizing(tile, VPM_FIX_Y, DDSP_PLACE_ROAD_X_DIR);
+	if (IsBridgeTile(tile)) {
+		TileIndex other_tile = GetOtherTunnelBridgeEnd(tile);
+		Point pt = {0, 0};
+		w->OnPlaceMouseUp(VPM_X_OR_Y, DDSP_BUILD_BRIDGE, pt, tile, other_tile);
+	} else {
+		VpStartPlaceSizing(tile, VPM_X_OR_Y, DDSP_BUILD_BRIDGE);
+	}
 }
 
 /**
- * Set the initial flags for the road constuction.
- * The flags are:
- * @li The direction is the Y-dir
- * @li The first tile has a partitial RoadBit (true or false)
+ * Callback executed after a build road tunnel command has been called.
  *
- * @param tile The start tile
+ * @param result Whether the build succeeded.
+ * @param start_tile Starting tile of the tunnel.
+ * @param p1 bit 0-3 railtype or roadtypes
+ *           bit 8-9 transport type
+ * @param p2 unused
  */
-static void PlaceRoad_Y_Dir(TileIndex tile)
-{
-	_place_road_flag = RF_DIR_Y;
-	if (_tile_fract_coords.y >= 8) _place_road_flag |= RF_START_HALFROAD_Y;
-	VpStartPlaceSizing(tile, VPM_FIX_X, DDSP_PLACE_ROAD_Y_DIR);
-}
-
-/**
- * Set the initial flags for the road constuction.
- * The flags are:
- * @li The direction is not set.
- * @li The first tile has a partitial RoadBit (true or false)
- *
- * @param tile The start tile
- */
-static void PlaceRoad_AutoRoad(TileIndex tile)
-{
-	_place_road_flag = RF_NONE;
-	if (_tile_fract_coords.x >= 8) _place_road_flag |= RF_START_HALFROAD_X;
-	if (_tile_fract_coords.y >= 8) _place_road_flag |= RF_START_HALFROAD_Y;
-	VpStartPlaceSizing(tile, VPM_X_OR_Y, DDSP_PLACE_AUTOROAD);
-}
-
-static void PlaceRoad_Bridge(TileIndex tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_OR_Y, DDSP_BUILD_BRIDGE);
-}
-
-
-void CcBuildRoadTunnel(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2)
+void CcBuildRoadTunnel(const CommandCost &result, TileIndex start_tile, uint32 p1, uint32 p2)
 {
 	if (result.Succeeded()) {
-		SndPlayTileFx(SND_20_SPLAT_2, tile);
+		SndPlayTileFx(SND_20_SPLAT_2, start_tile);
 		if (!_settings_client.gui.persistent_buildingtools) ResetObjectToPlace();
+
+		DiagDirection start_direction = ReverseDiagDir(GetTunnelBridgeDirection(start_tile));
+		ConnectRoadToStructure(start_tile, start_direction);
+
+		TileIndex end_tile = GetOtherTunnelBridgeEnd(start_tile);
+		DiagDirection end_direction = ReverseDiagDir(GetTunnelBridgeDirection(end_tile));
+		ConnectRoadToStructure(end_tile, end_direction);
 	} else {
 		SetRedErrorSquare(_build_tunnel_endtile);
 	}
@@ -172,12 +151,12 @@ static const RoadTypeInfo _road_type_infos[] = {
 	},
 };
 
-static void PlaceRoad_Tunnel(TileIndex tile)
-{
-	DoCommandP(tile, RoadTypeToRoadTypes(_cur_roadtype) | (TRANSPORT_ROAD << 8), 0, CMD_BUILD_TUNNEL | CMD_MSG(STR_ERROR_CAN_T_BUILD_TUNNEL_HERE), CcBuildRoadTunnel);
-}
-
-static void BuildRoadOutsideStation(TileIndex tile, DiagDirection direction)
+/**
+ * If required, connects a new structure to an existing road or tram by building the missing roadbit.
+ * @param tile Tile containing the structure to connect.
+ * @param direction Direction to check.
+ */
+void ConnectRoadToStructure(TileIndex tile, DiagDirection direction)
 {
 	tile += TileOffsByDiagDir(direction);
 	/* if there is a roadpiece just outside of the station entrance, build a connecting route */
@@ -195,44 +174,97 @@ void CcRoadDepot(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2
 	DiagDirection dir = (DiagDirection)GB(p1, 0, 2);
 	SndPlayTileFx(SND_1F_SPLAT, tile);
 	if (!_settings_client.gui.persistent_buildingtools) ResetObjectToPlace();
-	BuildRoadOutsideStation(tile, dir);
-	/* For a drive-through road stop build connecting road for other entrance */
-	if (HasBit(p2, 1)) BuildRoadOutsideStation(tile, ReverseDiagDir(dir));
+	ConnectRoadToStructure(tile, dir);
 }
 
-static void PlaceRoad_Depot(TileIndex tile)
+/**
+ * Command callback for building road stops.
+ * @param result Result of the build road stop command.
+ * @param tile Start tile.
+ * @param p1 bit 0..7: Width of the road stop.
+ *           bit 8..15: Length of the road stop.
+ * @param p2 bit 0: 0 For bus stops, 1 for truck stops.
+ *           bit 1: 0 For normal stops, 1 for drive-through.
+ *           bit 2..3: The roadtypes.
+ *           bit 5: Allow stations directly adjacent to other stations.
+ *           bit 6..7: Entrance direction (#DiagDirection).
+ *           bit 16..31: Station ID to join (NEW_STATION if build new one).
+ * @see CmdBuildRoadStop
+ */
+void CcRoadStop(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2)
 {
-	DoCommandP(tile, _cur_roadtype << 2 | _road_depot_orientation, 0, CMD_BUILD_ROAD_DEPOT | CMD_MSG(_road_type_infos[_cur_roadtype].err_depot), CcRoadDepot);
+	if (result.Failed()) return;
+
+	DiagDirection dir = (DiagDirection)GB(p2, 6, 2);
+	SndPlayTileFx(SND_1F_SPLAT, tile);
+	if (!_settings_client.gui.persistent_buildingtools) ResetObjectToPlace();
+	TileArea roadstop_area(tile, GB(p1, 0, 8), GB(p1, 8, 8));
+	TILE_AREA_LOOP(cur_tile, roadstop_area) {
+		ConnectRoadToStructure(cur_tile, dir);
+		/* For a drive-through road stop build connecting road for other entrance. */
+		if (HasBit(p2, 1)) ConnectRoadToStructure(cur_tile, ReverseDiagDir(dir));
+	}
 }
 
-static void PlaceRoadStop(TileIndex tile, uint32 p2, uint32 cmd)
+/**
+ * Place a new road stop.
+ * @param start_tile First tile of the area.
+ * @param end_tile Last tile of the area.
+ * @param p2 bit 0: 0 For bus stops, 1 for truck stops.
+ *           bit 2..3: The roadtypes.
+ *           bit 5: Allow stations directly adjacent to other stations.
+ * @param cmd Command to use.
+ * @see CcRoadStop()
+ */
+static void PlaceRoadStop(TileIndex start_tile, TileIndex end_tile, uint32 p2, uint32 cmd)
 {
-	uint32 p1 = _road_station_picker_orientation;
+	uint8 ddir = _road_station_picker_orientation;
 	SB(p2, 16, 16, INVALID_STATION); // no station to join
 
-	if (p1 >= DIAGDIR_END) {
-		SetBit(p2, 1); // It's a drive-through stop
-		p1 -= DIAGDIR_END; // Adjust picker result to actual direction
+	if (ddir >= DIAGDIR_END) {
+		SetBit(p2, 1); // It's a drive-through stop.
+		ddir -= DIAGDIR_END; // Adjust picker result to actual direction.
 	}
-	CommandContainer cmdcont = { tile, p1, p2, cmd, CcRoadDepot, "" };
-	ShowSelectStationIfNeeded(cmdcont, TileArea(tile, 1, 1));
+	p2 |= ddir << 6; // Set the DiagDirecion into p2 bits 6 and 7.
+
+	TileArea ta(start_tile, end_tile);
+	CommandContainer cmdcont = { ta.tile, ta.w | ta.h << 8, p2, cmd, CcRoadStop, "" };
+	ShowSelectStationIfNeeded(cmdcont, ta);
 }
 
+/**
+ * Callback for placing a bus station.
+ * @param tile Position to place the station.
+ */
 static void PlaceRoad_BusStation(TileIndex tile)
 {
 	if (_remove_button_clicked) {
-		DoCommandP(tile, 0, ROADSTOP_BUS, CMD_REMOVE_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_remove_station[ROADSTOP_BUS]), CcPlaySound1D);
+		VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_REMOVE_BUSSTOP);
 	} else {
-		PlaceRoadStop(tile, (_ctrl_pressed << 5) | RoadTypeToRoadTypes(_cur_roadtype) << 2 | ROADSTOP_BUS, CMD_BUILD_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_build_station[ROADSTOP_BUS]));
+		if (_road_station_picker_orientation < DIAGDIR_END) { // Not a drive-through stop.
+			VpStartPlaceSizing(tile, (DiagDirToAxis(_road_station_picker_orientation) == AXIS_X) ? VPM_X_LIMITED : VPM_Y_LIMITED, DDSP_BUILD_BUSSTOP);
+		} else {
+			VpStartPlaceSizing(tile, VPM_X_AND_Y_LIMITED, DDSP_BUILD_BUSSTOP);
+		}
+		VpSetPlaceSizingLimit(_settings_game.station.station_spread);
 	}
 }
 
+/**
+ * Callback for placing a truck station.
+ * @param tile Position to place the station.
+ */
 static void PlaceRoad_TruckStation(TileIndex tile)
 {
 	if (_remove_button_clicked) {
-		DoCommandP(tile, 0, ROADSTOP_TRUCK, CMD_REMOVE_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_remove_station[ROADSTOP_TRUCK]), CcPlaySound1D);
+		VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_REMOVE_TRUCKSTOP);
 	} else {
-		PlaceRoadStop(tile, (_ctrl_pressed << 5) | RoadTypeToRoadTypes(_cur_roadtype) << 2 | ROADSTOP_TRUCK, CMD_BUILD_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_build_station[ROADSTOP_TRUCK]));
+		if (_road_station_picker_orientation < DIAGDIR_END) { // Not a drive-through stop.
+			VpStartPlaceSizing(tile, (DiagDirToAxis(_road_station_picker_orientation) == AXIS_X) ? VPM_X_LIMITED : VPM_Y_LIMITED, DDSP_BUILD_TRUCKSTOP);
+		} else {
+			VpStartPlaceSizing(tile, VPM_X_AND_Y_LIMITED, DDSP_BUILD_TRUCKSTOP);
+		}
+		VpSetPlaceSizingLimit(_settings_game.station.station_spread);
 	}
 }
 
@@ -254,7 +286,8 @@ enum RoadToolbarWidgets {
 typedef void OnButtonClick(Window *w);
 
 
-/** Toogles state of the Remove button of Build road toolbar
+/**
+ * Toogles state of the Remove button of Build road toolbar
  * @param w window the button belongs to
  */
 static void ToggleRoadButton_Remove(Window *w)
@@ -265,7 +298,8 @@ static void ToggleRoadButton_Remove(Window *w)
 	SetSelectionRed(_remove_button_clicked);
 }
 
-/** Updates the Remove button because of Ctrl state change
+/**
+ * Updates the Remove button because of Ctrl state change
  * @param w window the button belongs to
  * @return true iff the remove buton was changed
  */
@@ -284,140 +318,20 @@ static bool RoadToolbar_CtrlChanged(Window *w)
 	return false;
 }
 
-
-/**
- * Function that handles the click on the
- *  X road placement button.
- *
- * @param w The current window
- */
-static void BuildRoadClick_X_Dir(Window *w)
-{
-	HandlePlacePushButton(w, RTW_ROAD_X, _road_type_infos[_cur_roadtype].cursor_nwse, HT_RECT, PlaceRoad_X_Dir);
-}
-
-/**
- * Function that handles the click on the
- *  Y road placement button.
- *
- * @param w The current window
- */
-static void BuildRoadClick_Y_Dir(Window *w)
-{
-	HandlePlacePushButton(w, RTW_ROAD_Y, _road_type_infos[_cur_roadtype].cursor_nesw, HT_RECT, PlaceRoad_Y_Dir);
-}
-
-/**
- * Function that handles the click on the
- *  autoroad placement button.
- *
- * @param w The current window
- */
-static void BuildRoadClick_AutoRoad(Window *w)
-{
-	HandlePlacePushButton(w, RTW_AUTOROAD, _road_type_infos[_cur_roadtype].cursor_autoroad, HT_RECT, PlaceRoad_AutoRoad);
-}
-
-static void BuildRoadClick_Demolish(Window *w)
-{
-	HandlePlacePushButton(w, RTW_DEMOLISH, ANIMCURSOR_DEMOLISH, HT_RECT, PlaceProc_DemolishArea);
-}
-
-static void BuildRoadClick_Depot(Window *w)
-{
-	if (_game_mode == GM_EDITOR || !CanBuildVehicleInfrastructure(VEH_ROAD)) return;
-	if (HandlePlacePushButton(w, RTW_DEPOT, SPR_CURSOR_ROAD_DEPOT, HT_RECT, PlaceRoad_Depot)) ShowRoadDepotPicker(w);
-}
-
-static void BuildRoadClick_BusStation(Window *w)
-{
-	if (_game_mode == GM_EDITOR || !CanBuildVehicleInfrastructure(VEH_ROAD)) return;
-	if (HandlePlacePushButton(w, RTW_BUS_STATION, SPR_CURSOR_BUS_STATION, HT_RECT, PlaceRoad_BusStation)) ShowRVStationPicker(w, ROADSTOP_BUS);
-}
-
-static void BuildRoadClick_TruckStation(Window *w)
-{
-	if (_game_mode == GM_EDITOR || !CanBuildVehicleInfrastructure(VEH_ROAD)) return;
-	if (HandlePlacePushButton(w, RTW_TRUCK_STATION, SPR_CURSOR_TRUCK_STATION, HT_RECT, PlaceRoad_TruckStation)) ShowRVStationPicker(w, ROADSTOP_TRUCK);
-}
-
-/**
- * Function that handles the click on the
- *  one way road button.
- *
- * @param w The current window
- */
-static void BuildRoadClick_OneWay(Window *w)
-{
-	if (w->IsWidgetDisabled(RTW_ONE_WAY)) return;
-	w->SetDirty();
-	w->ToggleWidgetLoweredState(RTW_ONE_WAY);
-	SetSelectionRed(false);
-}
-
-static void BuildRoadClick_Bridge(Window *w)
-{
-	HandlePlacePushButton(w, RTW_BUILD_BRIDGE, SPR_CURSOR_BRIDGE, HT_RECT, PlaceRoad_Bridge);
-}
-
-static void BuildRoadClick_Tunnel(Window *w)
-{
-	HandlePlacePushButton(w, RTW_BUILD_TUNNEL, SPR_CURSOR_ROAD_TUNNEL, HT_SPECIAL, PlaceRoad_Tunnel);
-}
-
-static void BuildRoadClick_Remove(Window *w)
-{
-	if (w->IsWidgetDisabled(RTW_REMOVE)) return;
-
-	DeleteWindowById(WC_SELECT_STATION, 0);
-	ToggleRoadButton_Remove(w);
-	SndPlayFx(SND_15_BEEP);
-}
-
-/** Array with the handlers of the button-clicks for the road-toolbar */
-static OnButtonClick * const _build_road_button_proc[] = {
-	BuildRoadClick_X_Dir,
-	BuildRoadClick_Y_Dir,
-	BuildRoadClick_AutoRoad,
-	BuildRoadClick_Demolish,
-	BuildRoadClick_Depot,
-	BuildRoadClick_BusStation,
-	BuildRoadClick_TruckStation,
-	BuildRoadClick_OneWay,
-	BuildRoadClick_Bridge,
-	BuildRoadClick_Tunnel,
-	BuildRoadClick_Remove
-};
-
-/** Array with the keycode of the button-clicks for the road-toolbar */
-static const uint16 _road_keycodes[] = {
-	'1',
-	'2',
-	'3',
-	'4',
-	'5',
-	'6',
-	'7',
-	'8',
-	'B',
-	'T',
-	'R',
-};
-
+/** Road toolbar window handler. */
 struct BuildRoadToolbarWindow : Window {
+	int last_started_action; ///< Last started user action.
+
 	BuildRoadToolbarWindow(const WindowDesc *desc, WindowNumber window_number) : Window()
 	{
 		this->InitNested(desc, window_number);
 		this->SetWidgetsDisabledState(true,
-			RTW_REMOVE,
-			RTW_ONE_WAY,
-			WIDGET_LIST_END);
+				RTW_REMOVE,
+				RTW_ONE_WAY,
+				WIDGET_LIST_END);
 
-		this->SetWidgetsDisabledState(!CanBuildVehicleInfrastructure(VEH_ROAD),
-			RTW_DEPOT,
-			RTW_BUS_STATION,
-			RTW_TRUCK_STATION,
-			WIDGET_LIST_END);
+		this->OnInvalidateData();
+		this->last_started_action = WIDGET_LIST_END;
 
 		if (_settings_client.gui.link_terraform_toolbar) ShowTerraformToolbar(this);
 	}
@@ -427,6 +341,21 @@ struct BuildRoadToolbarWindow : Window {
 		if (_settings_client.gui.link_terraform_toolbar) DeleteWindowById(WC_SCEN_LAND_GEN, 0, false);
 	}
 
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
+	{
+		if (!gui_scope) return;
+		this->SetWidgetsDisabledState(!CanBuildVehicleInfrastructure(VEH_ROAD),
+				RTW_DEPOT,
+				RTW_BUS_STATION,
+				RTW_TRUCK_STATION,
+				WIDGET_LIST_END);
+	}
+
 	/**
 	 * Update the remove button lowered state of the road toolbar
 	 *
@@ -458,37 +387,101 @@ struct BuildRoadToolbarWindow : Window {
 			case RTW_ROAD_Y:
 			case RTW_AUTOROAD:
 				this->SetWidgetsDisabledState(!this->IsWidgetLowered(clicked_widget),
-					RTW_REMOVE,
-					RTW_ONE_WAY,
-					WIDGET_LIST_END);
+						RTW_REMOVE,
+						RTW_ONE_WAY,
+						WIDGET_LIST_END);
 				break;
 
 			default:
 				/* When any other buttons than road/station, raise and
 				 * disable the removal button */
 				this->SetWidgetsDisabledState(true,
-					RTW_REMOVE,
-					RTW_ONE_WAY,
-					WIDGET_LIST_END);
+						RTW_REMOVE,
+						RTW_ONE_WAY,
+						WIDGET_LIST_END);
 				this->SetWidgetsLoweredState(false,
-					RTW_REMOVE,
-					RTW_ONE_WAY,
-					WIDGET_LIST_END);
+						RTW_REMOVE,
+						RTW_ONE_WAY,
+						WIDGET_LIST_END);
 				break;
 		}
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
-		if (widget >= RTW_ROAD_X) {
-			_remove_button_clicked = false;
-			_one_way_button_clicked = false;
-			_build_road_button_proc[widget - RTW_ROAD_X](this);
+		_remove_button_clicked = false;
+		_one_way_button_clicked = false;
+		switch (widget) {
+			case RTW_ROAD_X:
+				HandlePlacePushButton(this, RTW_ROAD_X, _road_type_infos[_cur_roadtype].cursor_nwse, HT_RECT);
+				this->last_started_action = widget;
+				break;
+
+			case RTW_ROAD_Y:
+				HandlePlacePushButton(this, RTW_ROAD_Y, _road_type_infos[_cur_roadtype].cursor_nesw, HT_RECT);
+				this->last_started_action = widget;
+				break;
+
+			case RTW_AUTOROAD:
+				HandlePlacePushButton(this, RTW_AUTOROAD, _road_type_infos[_cur_roadtype].cursor_autoroad, HT_RECT);
+				this->last_started_action = widget;
+				break;
+
+			case RTW_DEMOLISH:
+				HandlePlacePushButton(this, RTW_DEMOLISH, ANIMCURSOR_DEMOLISH, HT_RECT | HT_DIAGONAL);
+				this->last_started_action = widget;
+				break;
+
+			case RTW_DEPOT:
+				if (_game_mode == GM_EDITOR || !CanBuildVehicleInfrastructure(VEH_ROAD)) return;
+				if (HandlePlacePushButton(this, RTW_DEPOT, SPR_CURSOR_ROAD_DEPOT, HT_RECT)) {
+					ShowRoadDepotPicker(this);
+					this->last_started_action = widget;
+				}
+				break;
+
+			case RTW_BUS_STATION:
+				if (_game_mode == GM_EDITOR || !CanBuildVehicleInfrastructure(VEH_ROAD)) return;
+				if (HandlePlacePushButton(this, RTW_BUS_STATION, SPR_CURSOR_BUS_STATION, HT_RECT)) {
+					ShowRVStationPicker(this, ROADSTOP_BUS);
+					this->last_started_action = widget;
+				}
+				break;
+
+			case RTW_TRUCK_STATION:
+				if (_game_mode == GM_EDITOR || !CanBuildVehicleInfrastructure(VEH_ROAD)) return;
+				if (HandlePlacePushButton(this, RTW_TRUCK_STATION, SPR_CURSOR_TRUCK_STATION, HT_RECT)) {
+					ShowRVStationPicker(this, ROADSTOP_TRUCK);
+					this->last_started_action = widget;
+				}
+				break;
+
+			case RTW_ONE_WAY:
+				if (this->IsWidgetDisabled(RTW_ONE_WAY)) return;
+				this->SetDirty();
+				this->ToggleWidgetLoweredState(RTW_ONE_WAY);
+				SetSelectionRed(false);
+				break;
+
+			case RTW_BUILD_BRIDGE:
+				HandlePlacePushButton(this, RTW_BUILD_BRIDGE, SPR_CURSOR_BRIDGE, HT_RECT);
+				this->last_started_action = widget;
+				break;
+
+			case RTW_BUILD_TUNNEL:
+				HandlePlacePushButton(this, RTW_BUILD_TUNNEL, SPR_CURSOR_ROAD_TUNNEL, HT_SPECIAL);
+				this->last_started_action = widget;
+				break;
+
+			case RTW_REMOVE:
+				if (this->IsWidgetDisabled(RTW_REMOVE)) return;
+
+				DeleteWindowById(WC_SELECT_STATION, 0);
+				ToggleRoadButton_Remove(this);
+				SndPlayFx(SND_15_BEEP);
+				break;
+
+			default: NOT_REACHED();
 		}
 		this->UpdateOptionWidgetStatus((RoadToolbarWidgets)widget);
 		if (_ctrl_pressed) RoadToolbar_CtrlChanged(this);
@@ -496,36 +489,74 @@ struct BuildRoadToolbarWindow : Window {
 
 	virtual EventState OnKeyPress(uint16 key, uint16 keycode)
 	{
-		EventState state = ES_NOT_HANDLED;
-		for (uint i = 0; i != lengthof(_road_keycodes); i++) {
-			if (keycode == _road_keycodes[i]) {
-				_remove_button_clicked = false;
-				_one_way_button_clicked = false;
-				_build_road_button_proc[i](this);
-				this->UpdateOptionWidgetStatus((RoadToolbarWidgets)(i + RTW_ROAD_X));
-				if (_ctrl_pressed) RoadToolbar_CtrlChanged(this);
-				state = ES_HANDLED;
-				break;
-			}
-		}
+		int num = CheckHotkeyMatch(roadtoolbar_hotkeys, keycode, this);
+		if (num == -1 || this->GetWidget<NWidgetBase>(num) == NULL) return ES_NOT_HANDLED;
+		this->OnClick(Point(), num, 1);
 		MarkTileDirtyByTile(TileVirtXY(_thd.pos.x, _thd.pos.y)); // redraw tile selection
-		return state;
+		return ES_HANDLED;
 	}
 
 	virtual void OnPlaceObject(Point pt, TileIndex tile)
 	{
 		_remove_button_clicked = this->IsWidgetLowered(RTW_REMOVE);
 		_one_way_button_clicked = this->IsWidgetLowered(RTW_ONE_WAY);
-		_place_proc(tile);
+		switch (this->last_started_action) {
+			case RTW_ROAD_X:
+				_place_road_flag = RF_DIR_X;
+				if (_tile_fract_coords.x >= 8) _place_road_flag |= RF_START_HALFROAD_X;
+				VpStartPlaceSizing(tile, VPM_FIX_Y, DDSP_PLACE_ROAD_X_DIR);
+				break;
+
+			case RTW_ROAD_Y:
+				_place_road_flag = RF_DIR_Y;
+				if (_tile_fract_coords.y >= 8) _place_road_flag |= RF_START_HALFROAD_Y;
+				VpStartPlaceSizing(tile, VPM_FIX_X, DDSP_PLACE_ROAD_Y_DIR);
+				break;
+
+			case RTW_AUTOROAD:
+				_place_road_flag = RF_NONE;
+				if (_tile_fract_coords.x >= 8) _place_road_flag |= RF_START_HALFROAD_X;
+				if (_tile_fract_coords.y >= 8) _place_road_flag |= RF_START_HALFROAD_Y;
+				VpStartPlaceSizing(tile, VPM_X_OR_Y, DDSP_PLACE_AUTOROAD);
+				break;
+
+			case RTW_DEMOLISH:
+				PlaceProc_DemolishArea(tile);
+				break;
+
+			case RTW_DEPOT:
+				DoCommandP(tile, _cur_roadtype << 2 | _road_depot_orientation, 0,
+						CMD_BUILD_ROAD_DEPOT | CMD_MSG(_road_type_infos[_cur_roadtype].err_depot), CcRoadDepot);
+				break;
+
+			case RTW_BUS_STATION:
+				PlaceRoad_BusStation(tile);
+				break;
+
+			case RTW_TRUCK_STATION:
+				PlaceRoad_TruckStation(tile);
+				break;
+
+			case RTW_BUILD_BRIDGE:
+				PlaceRoad_Bridge(tile, this);
+				break;
+
+			case RTW_BUILD_TUNNEL:
+				DoCommandP(tile, RoadTypeToRoadTypes(_cur_roadtype) | (TRANSPORT_ROAD << 8), 0,
+						CMD_BUILD_TUNNEL | CMD_MSG(STR_ERROR_CAN_T_BUILD_TUNNEL_HERE), CcBuildRoadTunnel);
+				break;
+
+			default: NOT_REACHED();
+		}
 	}
 
 	virtual void OnPlaceObjectAbort()
 	{
 		this->RaiseButtons();
 		this->SetWidgetsDisabledState(true,
-			RTW_REMOVE,
-			RTW_ONE_WAY,
-			WIDGET_LIST_END);
+				RTW_REMOVE,
+				RTW_ONE_WAY,
+				WIDGET_LIST_END);
 		this->SetWidgetDirty(RTW_REMOVE);
 		this->SetWidgetDirty(RTW_ONE_WAY);
 
@@ -603,10 +634,30 @@ struct BuildRoadToolbarWindow : Window {
 					_place_road_flag = (RoadFlags)((_place_road_flag & RF_DIR_Y) ? (_place_road_flag & 0x07) : (_place_road_flag >> 3));
 
 					DoCommandP(start_tile, end_tile, _place_road_flag | (_cur_roadtype << 3) | (_one_way_button_clicked << 5),
-						_remove_button_clicked ?
-						CMD_REMOVE_LONG_ROAD | CMD_MSG(_road_type_infos[_cur_roadtype].err_remove_road) :
-						CMD_BUILD_LONG_ROAD | CMD_MSG(_road_type_infos[_cur_roadtype].err_build_road), CcPlaySound1D);
+							_remove_button_clicked ?
+							CMD_REMOVE_LONG_ROAD | CMD_MSG(_road_type_infos[_cur_roadtype].err_remove_road) :
+							CMD_BUILD_LONG_ROAD | CMD_MSG(_road_type_infos[_cur_roadtype].err_build_road), CcPlaySound1D);
 					break;
+
+				case DDSP_BUILD_BUSSTOP:
+					PlaceRoadStop(start_tile, end_tile, (_ctrl_pressed << 5) | RoadTypeToRoadTypes(_cur_roadtype) << 2 | ROADSTOP_BUS, CMD_BUILD_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_build_station[ROADSTOP_BUS]));
+					break;
+
+				case DDSP_BUILD_TRUCKSTOP:
+					PlaceRoadStop(start_tile, end_tile, (_ctrl_pressed << 5) | RoadTypeToRoadTypes(_cur_roadtype) << 2 | ROADSTOP_TRUCK, CMD_BUILD_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_build_station[ROADSTOP_TRUCK]));
+					break;
+
+				case DDSP_REMOVE_BUSSTOP: {
+					TileArea ta(start_tile, end_tile);
+					DoCommandP(ta.tile, ta.w | ta.h << 8, ROADSTOP_BUS, CMD_REMOVE_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_remove_station[ROADSTOP_BUS]), CcPlaySound1D);
+					break;
+				}
+
+				case DDSP_REMOVE_TRUCKSTOP: {
+					TileArea ta(start_tile, end_tile);
+					DoCommandP(ta.tile, ta.w | ta.h << 8, ROADSTOP_TRUCK, CMD_REMOVE_ROAD_STOP | CMD_MSG(_road_type_infos[_cur_roadtype].err_remove_station[ROADSTOP_TRUCK]), CcPlaySound1D);
+					break;
+				}
 			}
 		}
 	}
@@ -622,8 +673,27 @@ struct BuildRoadToolbarWindow : Window {
 		if (RoadToolbar_CtrlChanged(this)) return ES_HANDLED;
 		return ES_NOT_HANDLED;
 	}
+
+	static Hotkey<BuildRoadToolbarWindow> roadtoolbar_hotkeys[];
 };
 
+Hotkey<BuildRoadToolbarWindow> BuildRoadToolbarWindow::roadtoolbar_hotkeys[] = {
+	Hotkey<BuildRoadToolbarWindow>('1', "build_x", RTW_ROAD_X),
+	Hotkey<BuildRoadToolbarWindow>('2', "build_y", RTW_ROAD_Y),
+	Hotkey<BuildRoadToolbarWindow>('3', "autoroad", RTW_AUTOROAD),
+	Hotkey<BuildRoadToolbarWindow>('4', "demolish", RTW_DEMOLISH),
+	Hotkey<BuildRoadToolbarWindow>('5', "depot", RTW_DEPOT),
+	Hotkey<BuildRoadToolbarWindow>('6', "bus_station", RTW_BUS_STATION),
+	Hotkey<BuildRoadToolbarWindow>('7', "truck_station", RTW_TRUCK_STATION),
+	Hotkey<BuildRoadToolbarWindow>('8', "oneway", RTW_ONE_WAY),
+	Hotkey<BuildRoadToolbarWindow>('B', "bridge", RTW_BUILD_BRIDGE),
+	Hotkey<BuildRoadToolbarWindow>('T', "tunnel", RTW_BUILD_TUNNEL),
+	Hotkey<BuildRoadToolbarWindow>('R', "remove", RTW_REMOVE),
+	HOTKEY_LIST_END(BuildRoadToolbarWindow)
+};
+Hotkey<BuildRoadToolbarWindow> *_roadtoolbar_hotkeys = BuildRoadToolbarWindow::roadtoolbar_hotkeys;
+
+
 static const NWidgetPart _nested_build_road_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_DARK_GREEN),
@@ -703,13 +773,30 @@ static const WindowDesc _build_tramway_desc(
 	_nested_build_tramway_widgets, lengthof(_nested_build_tramway_widgets)
 );
 
-void ShowBuildRoadToolbar(RoadType roadtype)
+/**
+ * Open the build road toolbar window
+ *
+ * If the terraform toolbar is linked to the toolbar, that window is also opened.
+ *
+ * @return newly opened road toolbar, or NULL if the toolbar could not be opened.
+ */
+Window *ShowBuildRoadToolbar(RoadType roadtype)
 {
-	if (!Company::IsValidID(_local_company)) return;
+	if (!Company::IsValidID(_local_company)) return NULL;
 	_cur_roadtype = roadtype;
 
 	DeleteWindowByClass(WC_BUILD_TOOLBAR);
-	AllocateWindowDescFront<BuildRoadToolbarWindow>(roadtype == ROADTYPE_ROAD ? &_build_road_desc : &_build_tramway_desc, TRANSPORT_ROAD);
+	return AllocateWindowDescFront<BuildRoadToolbarWindow>(roadtype == ROADTYPE_ROAD ? &_build_road_desc : &_build_tramway_desc, TRANSPORT_ROAD);
+}
+
+EventState RoadToolbarGlobalHotkeys(uint16 key, uint16 keycode)
+{
+	extern RoadType _last_built_roadtype;
+	int num = CheckHotkeyMatch<BuildRoadToolbarWindow>(_roadtoolbar_hotkeys, keycode, NULL, true);
+	if (num == -1) return ES_NOT_HANDLED;
+	Window *w = ShowBuildRoadToolbar(_last_built_roadtype);
+	if (w == NULL) return ES_NOT_HANDLED;
+	return w->OnKeyPress(key, keycode);
 }
 
 static const NWidgetPart _nested_build_road_scen_widgets[] = {
@@ -746,10 +833,19 @@ static const WindowDesc _build_road_scen_desc(
 	_nested_build_road_scen_widgets, lengthof(_nested_build_road_scen_widgets)
 );
 
-void ShowBuildRoadScenToolbar()
+Window *ShowBuildRoadScenToolbar()
 {
 	_cur_roadtype = ROADTYPE_ROAD;
-	AllocateWindowDescFront<BuildRoadToolbarWindow>(&_build_road_scen_desc, 0);
+	return AllocateWindowDescFront<BuildRoadToolbarWindow>(&_build_road_scen_desc, 0);
+}
+
+EventState RoadToolbarEditorGlobalHotkeys(uint16 key, uint16 keycode)
+{
+	int num = CheckHotkeyMatch<BuildRoadToolbarWindow>(_roadtoolbar_hotkeys, keycode, NULL, true);
+	if (num == -1) return ES_NOT_HANDLED;
+	Window *w = ShowBuildRoadScenToolbar();
+	if (w == NULL) return ES_NOT_HANDLED;
+	return w->OnKeyPress(key, keycode);
 }
 
 /** Enum referring to the widgets of the build road depot window */
@@ -775,11 +871,6 @@ struct BuildRoadDepotWindow : public PickerWindowBase {
 		this->FinishInitNested(desc, TRANSPORT_ROAD);
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		if (!IsInsideMM(widget, BRDW_DEPOT_NE, BRDW_DEPOT_NW + 1)) return;
@@ -874,11 +965,11 @@ struct BuildRoadStationWindow : public PickerWindowBase {
 			_road_station_picker_orientation = DIAGDIR_END;
 		}
 		this->SetWidgetsDisabledState(_cur_roadtype == ROADTYPE_TRAM,
-			BRSW_STATION_NE,
-			BRSW_STATION_SE,
-			BRSW_STATION_SW,
-			BRSW_STATION_NW,
-			WIDGET_LIST_END);
+				BRSW_STATION_NE,
+				BRSW_STATION_SE,
+				BRSW_STATION_SW,
+				BRSW_STATION_NW,
+				WIDGET_LIST_END);
 
 		this->GetWidget<NWidgetCore>(BRSW_CAPTION)->widget_data = _road_type_infos[_cur_roadtype].picker_title[rs];
 		for (uint i = BRSW_STATION_NE; i < BRSW_LT_OFF; i++) this->GetWidget<NWidgetCore>(i)->tool_tip = _road_type_infos[_cur_roadtype].picker_tooltip[rs];
