@@ -1,4 +1,4 @@
-/* $Id: roadveh_cmd.cpp 20094 2010-07-08 19:52:28Z rubidium $ */
+/* $Id: roadveh_cmd.cpp 22520 2011-05-29 17:55:15Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,7 +10,6 @@
 /** @file roadveh_cmd.cpp Handling of road vehicles. */
 
 #include "stdafx.h"
-#include "landscape.h"
 #include "roadveh.h"
 #include "command_func.h"
 #include "news_func.h"
@@ -19,31 +18,24 @@
 #include "company_func.h"
 #include "vehicle_gui.h"
 #include "articulated_vehicles.h"
-#include "newgrf_engine.h"
 #include "newgrf_sound.h"
 #include "pathfinder/yapf/yapf.h"
 #include "strings_func.h"
 #include "tunnelbridge_map.h"
-#include "functions.h"
 #include "window_func.h"
 #include "date_func.h"
 #include "vehicle_func.h"
 #include "sound_func.h"
-#include "autoreplace_gui.h"
 #include "ai/ai.hpp"
 #include "depot_map.h"
 #include "effectvehicle_func.h"
-#include "effectvehicle_base.h"
 #include "roadstop_base.h"
-#include "cargotype.h"
 #include "spritecache.h"
 #include "core/random_func.hpp"
-#include "engine_base.h"
 #include "company_base.h"
-#include "engine_func.h"
+#include "core/backup_type.hpp"
 
 #include "table/strings.h"
-#include "table/sprites.h"
 
 static const uint16 _roadveh_images[63] = {
 	0xCD4, 0xCDC, 0xCE4, 0xCEC, 0xCF4, 0xCFC, 0xD0C, 0xD14,
@@ -91,7 +83,7 @@ static const Trackdir _roadveh_depot_exit_trackdir[DIAGDIR_END] = {
  */
 bool RoadVehicle::IsBus() const
 {
-	assert(this->IsRoadVehFront());
+	assert(this->IsFrontEngine());
 	return IsCargoInClass(this->cargo_type, CC_PASSENGERS);
 }
 
@@ -108,7 +100,7 @@ int RoadVehicle::GetDisplayImageWidth(Point *offset) const
 		offset->x = reference_width / 2;
 		offset->y = 0;
 	}
-	return this->rcache.cached_veh_length * reference_width / 8;
+	return this->gcache.cached_veh_length * reference_width / VEHICLE_LENGTH;
 }
 
 static SpriteID GetRoadVehIcon(EngineID engine)
@@ -145,6 +137,15 @@ SpriteID RoadVehicle::GetImage(Direction direction) const
 	return sprite;
 }
 
+/**
+ * Draw a road vehicle engine.
+ * @param left Left edge to draw within.
+ * @param right Right edge to draw within.
+ * @param preferred_x Preferred position of the engine.
+ * @param y Vertical position of the engine.
+ * @param engine Engine to draw
+ * @param pal Palette to use.
+ */
 void DrawRoadVehEngine(int left, int right, int preferred_x, int y, EngineID engine, PaletteID pal)
 {
 	SpriteID sprite = GetRoadVehIcon(engine);
@@ -153,88 +154,77 @@ void DrawRoadVehEngine(int left, int right, int preferred_x, int y, EngineID eng
 	DrawSprite(sprite, pal, preferred_x, y);
 }
 
+/**
+ * Get length of a road vehicle.
+ * @param v Road vehicle to query length.
+ * @return Length of the given road vehicle.
+ */
 static uint GetRoadVehLength(const RoadVehicle *v)
 {
-	uint length = 8;
+	uint length = VEHICLE_LENGTH;
 
 	uint16 veh_len = GetVehicleCallback(CBID_VEHICLE_LENGTH, 0, 0, v->engine_type, v);
 	if (veh_len != CALLBACK_FAILED) {
-		length -= Clamp(veh_len, 0, 7);
+		length -= Clamp(veh_len, 0, VEHICLE_LENGTH - 1);
 	}
 
 	return length;
 }
 
+/**
+ * Update the cache of a road vehicle.
+ * @param v Road vehicle needing an update of its cache.
+ * @pre \a v must be first road vehicle.
+ */
 void RoadVehUpdateCache(RoadVehicle *v)
 {
 	assert(v->type == VEH_ROAD);
-	assert(v->IsRoadVehFront());
+	assert(v->IsFrontEngine());
 
 	v->InvalidateNewGRFCacheOfChain();
 
-	v->rcache.cached_total_length = 0;
+	v->gcache.cached_total_length = 0;
 
 	for (RoadVehicle *u = v; u != NULL; u = u->Next()) {
 		/* Check the v->first cache. */
 		assert(u->First() == v);
 
 		/* Update the 'first engine' */
-		u->rcache.first_engine = (v == u) ? INVALID_ENGINE : v->engine_type;
+		u->gcache.first_engine = (v == u) ? INVALID_ENGINE : v->engine_type;
 
 		/* Update the length of the vehicle. */
-		u->rcache.cached_veh_length = GetRoadVehLength(u);
-		v->rcache.cached_total_length += u->rcache.cached_veh_length;
+		u->gcache.cached_veh_length = GetRoadVehLength(u);
+		v->gcache.cached_total_length += u->gcache.cached_veh_length;
+
+		/* Update visual effect */
+		v->UpdateVisualEffect();
 
 		/* Invalidate the vehicle colour map */
 		u->colourmap = PAL_NONE;
 	}
+
+	uint max_speed = GetVehicleProperty(v, PROP_ROADVEH_SPEED, 0);
+	v->vcache.cached_max_speed = (max_speed != 0) ? max_speed * 4 : RoadVehInfo(v->engine_type)->max_speed;
 }
 
-/** Build a road vehicle.
- * @param tile tile of depot where road vehicle is built
- * @param flags operation to perform
- * @param p1 bus/truck type being built (engine)
- * @param p2 unused
- * @param text unused
- * @return the cost of this operation or an error
+/**
+ * Build a road vehicle.
+ * @param tile     tile of the depot where road vehicle is built.
+ * @param flags    type of operation.
+ * @param e        the engine to build.
+ * @param data     unused.
+ * @param ret[out] the vehicle that has been built.
+ * @return the cost of this operation or an error.
  */
-CommandCost CmdBuildRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
+CommandCost CmdBuildRoadVehicle(TileIndex tile, DoCommandFlag flags, const Engine *e, uint16 data, Vehicle **ret)
 {
-	EngineID eid = GB(p1, 0, 16);
-	if (!IsEngineBuildable(eid, VEH_ROAD, _current_company)) return_cmd_error(STR_ERROR_ROAD_VEHICLE_NOT_AVAILABLE);
-
-	const Engine *e = Engine::Get(eid);
-	/* Engines without valid cargo should not be available */
-	if (e->GetDefaultCargoType() == CT_INVALID) return CMD_ERROR;
-
-	CommandCost cost(EXPENSES_NEW_VEHICLES, e->GetCost());
-	if (flags & DC_QUERY_COST) return cost;
-
-	/* The ai_new queries the vehicle cost before building the route,
-	 * so we must check against cheaters no sooner than now. --pasky */
-	if (!IsRoadDepotTile(tile)) return CMD_ERROR;
-	if (!IsTileOwner(tile, _current_company)) return CMD_ERROR;
-
 	if (HasTileRoadType(tile, ROADTYPE_TRAM) != HasBit(e->info.misc_flags, EF_ROAD_TRAM)) return_cmd_error(STR_ERROR_DEPOT_WRONG_DEPOT_TYPE);
 
-	uint num_vehicles = 1 + CountArticulatedParts(eid, false);
-
-	/* Allow for the front and the articulated parts */
-	if (!Vehicle::CanAllocateItem(num_vehicles)) {
-		return_cmd_error(STR_ERROR_TOO_MANY_VEHICLES_IN_GAME);
-	}
-
-	/* find the first free roadveh id */
-	UnitID unit_num = (flags & DC_AUTOREPLACE) ? 0 : GetFreeUnitNumber(VEH_ROAD);
-	if (unit_num > _settings_game.vehicle.max_roadveh) {
-		return_cmd_error(STR_ERROR_TOO_MANY_VEHICLES_IN_GAME);
-	}
-
 	if (flags & DC_EXEC) {
 		const RoadVehicleInfo *rvi = &e->u.road;
 
 		RoadVehicle *v = new RoadVehicle();
-		v->unitnumber = unit_num;
+		*ret = v;
 		v->direction = DiagDirToDir(GetRoadDepotDirection(tile));
 		v->owner = _current_company;
 
@@ -251,12 +241,10 @@ CommandCost CmdBuildRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 		v->spritenum = rvi->image_index;
 		v->cargo_type = e->GetDefaultCargoType();
 		v->cargo_cap = rvi->capacity;
-		v->value = cost.GetCost();
 
 		v->last_station_visited = INVALID_STATION;
-		v->max_speed = rvi->max_speed;
-		v->engine_type = eid;
-		v->rcache.first_engine = INVALID_ENGINE; // needs to be set before first callback
+		v->engine_type = e->index;
+		v->gcache.first_engine = INVALID_ENGINE; // needs to be set before first callback
 
 		v->reliability = e->reliability;
 		v->reliability_spd_dec = e->reliability_spd_dec;
@@ -270,11 +258,11 @@ CommandCost CmdBuildRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 
 		v->cur_image = SPR_IMG_QUERY;
 		v->random_bits = VehicleRandomBits();
-		v->SetRoadVehFront();
+		v->SetFrontEngine();
 
 		v->roadtype = HasBit(e->info.misc_flags, EF_ROAD_TRAM) ? ROADTYPE_TRAM : ROADTYPE_ROAD;
 		v->compatible_roadtypes = RoadTypeToRoadTypes(v->roadtype);
-		v->rcache.cached_veh_length = 8;
+		v->gcache.cached_veh_length = VEHICLE_LENGTH;
 
 		if (e->flags & ENGINE_EXCLUSIVE_PREVIEW) SetBit(v->vehicle_flags, VF_BUILT_AS_PROTOTYPE);
 
@@ -288,22 +276,15 @@ CommandCost CmdBuildRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 			u->InvalidateNewGRFCache();
 		}
 		RoadVehUpdateCache(v);
+		/* Initialize cached values for realistic acceleration. */
+		if (_settings_game.vehicle.roadveh_acceleration_model != AM_ORIGINAL) v->CargoChanged();
 
 		VehicleMove(v, false);
 
-		InvalidateWindowData(WC_VEHICLE_DEPOT, v->tile);
-		InvalidateWindowClassesData(WC_ROADVEH_LIST, 0);
-		SetWindowDirty(WC_COMPANY, v->owner);
-		if (IsLocalCompany()) {
-			InvalidateAutoreplaceWindow(v->engine_type, v->group_id); // updates the replace Road window
-		}
-
-		Company::Get(_current_company)->num_engines[eid]++;
-
 		CheckConsistencyOfArticulatedVehicle(v);
 	}
 
-	return cost;
+	return CommandCost();
 }
 
 bool RoadVehicle::IsStoppedInDepot() const
@@ -311,7 +292,7 @@ bool RoadVehicle::IsStoppedInDepot() const
 	TileIndex tile = this->tile;
 
 	if (!IsRoadDepotTile(tile)) return false;
-	if (this->IsRoadVehFront() && !(this->vehstatus & VS_STOPPED)) return false;
+	if (this->IsFrontEngine() && !(this->vehstatus & VS_STOPPED)) return false;
 
 	for (const RoadVehicle *v = this; v != NULL; v = v->Next()) {
 		if (v->state != RVSB_IN_DEPOT || v->tile != tile) return false;
@@ -319,34 +300,6 @@ bool RoadVehicle::IsStoppedInDepot() const
 	return true;
 }
 
-/** Sell a road vehicle.
- * @param tile unused
- * @param flags operation to perform
- * @param p1 vehicle ID to be sold
- * @param p2 unused
- * @param text unused
- * @return the cost of this operation or an error
- */
-CommandCost CmdSellRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
-{
-	RoadVehicle *v = RoadVehicle::GetIfValid(p1);
-	if (v == NULL || !CheckOwnership(v->owner)) return CMD_ERROR;
-
-	if (v->vehstatus & VS_CRASHED) return_cmd_error(STR_ERROR_CAN_T_SELL_DESTROYED_VEHICLE);
-
-	if (!v->IsStoppedInDepot()) {
-		return_cmd_error(STR_ERROR_ROAD_VEHICLE_MUST_BE_STOPPED_INSIDE_DEPOT);
-	}
-
-	CommandCost ret(EXPENSES_NEW_VEHICLES, -v->value);
-
-	if (flags & DC_EXEC) {
-		delete v;
-	}
-
-	return ret;
-}
-
 static FindDepotData FindClosestRoadDepot(const RoadVehicle *v, int max_distance)
 {
 	if (IsRoadDepotTile(v->tile)) return FindDepotData(v->tile, 0);
@@ -370,31 +323,8 @@ bool RoadVehicle::FindClosestDepot(TileIndex *location, DestinationID *destinati
 	return true;
 }
 
-/** Send a road vehicle to the depot.
- * @param tile unused
- * @param flags operation to perform
- * @param p1 vehicle ID to send to the depot
- * @param p2 various bitmasked elements
- * - p2 bit 0-3 - DEPOT_ flags (see vehicle.h)
- * - p2 bit 8-10 - VLW flag (for mass goto depot)
- * @param text unused
- * @return the cost of this operation or an error
- */
-CommandCost CmdSendRoadVehToDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
-{
-	if (p2 & DEPOT_MASS_SEND) {
-		/* Mass goto depot requested */
-		if (!ValidVLWFlags(p2 & VLW_MASK)) return CMD_ERROR;
-		return SendAllVehiclesToDepot(VEH_ROAD, flags, p2 & DEPOT_SERVICE, _current_company, (p2 & VLW_MASK), p1);
-	}
-
-	RoadVehicle *v = RoadVehicle::GetIfValid(p1);
-	if (v == NULL) return CMD_ERROR;
-
-	return v->SendToDepot(flags, (DepotCommand)(p2 & DEPOT_COMMAND_MASK));
-}
-
-/** Turn a roadvehicle around.
+/**
+ * Turn a roadvehicle around.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 vehicle ID to turn
@@ -405,7 +335,10 @@ CommandCost CmdSendRoadVehToDepot(TileIndex tile, DoCommandFlag flags, uint32 p1
 CommandCost CmdTurnRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
 	RoadVehicle *v = RoadVehicle::GetIfValid(p1);
-	if (v == NULL || !CheckOwnership(v->owner)) return CMD_ERROR;
+	if (v == NULL) return CMD_ERROR;
+
+	CommandCost ret = CheckOwnership(v->owner);
+	if (ret.Failed()) return ret;
 
 	if ((v->vehstatus & VS_STOPPED) ||
 			(v->vehstatus & VS_CRASHED) ||
@@ -429,9 +362,10 @@ CommandCost CmdTurnRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 
 void RoadVehicle::MarkDirty()
 {
-	for (Vehicle *v = this; v != NULL; v = v->Next()) {
+	for (RoadVehicle *v = this; v != NULL; v = v->Next()) {
 		v->UpdateViewport(false, false);
 	}
+	this->CargoChanged();
 }
 
 void RoadVehicle::UpdateDeltaXY(Direction direction)
@@ -457,6 +391,33 @@ void RoadVehicle::UpdateDeltaXY(Direction direction)
 	this->z_extent      = 6;
 }
 
+/**
+ * Calculates the maximum speed of the vehicle under its current conditions.
+ * @return Maximum speed of the vehicle.
+ */
+FORCEINLINE int RoadVehicle::GetCurrentMaxSpeed() const
+{
+	if (_settings_game.vehicle.roadveh_acceleration_model == AM_ORIGINAL) return this->vcache.cached_max_speed;
+
+	int max_speed = this->vcache.cached_max_speed;
+
+	/* Limit speed to 50% while reversing, 75% in curves. */
+	for (const RoadVehicle *u = this; u != NULL; u = u->Next()) {
+		if (this->state <= RVSB_TRACKDIR_MASK && IsReversingRoadTrackdir((Trackdir)this->state)) {
+			max_speed = this->vcache.cached_max_speed / 2;
+			break;
+		} else if ((u->direction & 1) == 0) {
+			max_speed = this->vcache.cached_max_speed * 3 / 4;
+		}
+	}
+
+	return max_speed;
+}
+
+/**
+ * Delete last vehicle of a chain road vehicles.
+ * @param v First roadvehicle.
+ */
 static void DeleteLastRoadVeh(RoadVehicle *v)
 {
 	Vehicle *u = v;
@@ -469,22 +430,6 @@ static void DeleteLastRoadVeh(RoadVehicle *v)
 	delete v;
 }
 
-static byte SetRoadVehPosition(RoadVehicle *v, int x, int y, bool turned)
-{
-	byte new_z, old_z;
-
-	/* need this hint so it returns the right z coordinate on bridges. */
-	v->x_pos = x;
-	v->y_pos = y;
-	new_z = GetSlopeZ(x, y);
-
-	old_z = v->z_pos;
-	v->z_pos = new_z;
-
-	v->UpdateViewport(true, turned);
-	return old_z;
-}
-
 static void RoadVehSetRandomDirection(RoadVehicle *v)
 {
 	static const DirDiff delta[] = {
@@ -495,10 +440,15 @@ static void RoadVehSetRandomDirection(RoadVehicle *v)
 		uint32 r = Random();
 
 		v->direction = ChangeDir(v->direction, delta[r & 3]);
-		SetRoadVehPosition(v, v->x_pos, v->y_pos, true);
+		v->UpdateViewport(true, true);
 	} while ((v = v->Next()) != NULL);
 }
 
+/**
+ * Road vehicle chain has crashed.
+ * @param v First roadvehicle.
+ * @return whether the chain still exists.
+ */
 static bool RoadVehIsCrashed(RoadVehicle *v)
 {
 	v->crashed_ctr++;
@@ -515,22 +465,26 @@ static bool RoadVehIsCrashed(RoadVehicle *v)
 	return true;
 }
 
+/**
+ * Check routine whether a road and a train vehicle have collided.
+ * @param v    %Train vehicle to test.
+ * @param data Road vehicle to test.
+ * @return %Train vehicle if the vehicles collided, else \c NULL.
+ */
 static Vehicle *EnumCheckRoadVehCrashTrain(Vehicle *v, void *data)
 {
 	const Vehicle *u = (Vehicle*)data;
 
-	return
-		v->type == VEH_TRAIN &&
-		abs(v->z_pos - u->z_pos) <= 6 &&
-		abs(v->x_pos - u->x_pos) <= 4 &&
-		abs(v->y_pos - u->y_pos) <= 4 ?
-			v : NULL;
+	return (v->type == VEH_TRAIN &&
+			abs(v->z_pos - u->z_pos) <= 6 &&
+			abs(v->x_pos - u->x_pos) <= 4 &&
+			abs(v->y_pos - u->y_pos) <= 4) ? v : NULL;
 }
 
 uint RoadVehicle::Crash(bool flooded)
 {
-	uint pass = Vehicle::Crash(flooded);
-	if (this->IsRoadVehFront()) {
+	uint pass = this->GroundVehicleBase::Crash(flooded);
+	if (this->IsFrontEngine()) {
 		pass += 1; // driver
 
 		/* If we're in a drive through road stop we ought to leave it */
@@ -578,39 +532,6 @@ static bool RoadVehCheckTrainCrash(RoadVehicle *v)
 	return false;
 }
 
-static void HandleBrokenRoadVeh(RoadVehicle *v)
-{
-	if (v->breakdown_ctr != 1) {
-		v->breakdown_ctr = 1;
-		v->cur_speed = 0;
-
-		if (v->breakdowns_since_last_service != 255)
-			v->breakdowns_since_last_service++;
-
-		v->MarkDirty();
-		SetWindowDirty(WC_VEHICLE_VIEW, v->index);
-		SetWindowDirty(WC_VEHICLE_DETAILS, v->index);
-
-		if (!PlayVehicleSound(v, VSE_BREAKDOWN)) {
-			SndPlayVehicleFx((_settings_game.game_creation.landscape != LT_TOYLAND) ?
-				SND_0F_VEHICLE_BREAKDOWN : SND_35_COMEDY_BREAKDOWN, v);
-		}
-
-		if (!(v->vehstatus & VS_HIDDEN)) {
-			EffectVehicle *u = CreateEffectVehicleRel(v, 4, 4, 5, EV_BREAKDOWN_SMOKE);
-			if (u != NULL) u->animation_state = v->breakdown_delay * 2;
-		}
-	}
-
-	if ((v->tick_counter & 1) == 0) {
-		if (--v->breakdown_delay == 0) {
-			v->breakdown_ctr = 0;
-			v->MarkDirty();
-			SetWindowDirty(WC_VEHICLE_VIEW, v->index);
-		}
-	}
-}
-
 TileIndex RoadVehicle::GetOrderStationLocation(StationID station)
 {
 	if (station == this->last_station_visited) this->last_station_visited = INVALID_STATION;
@@ -618,7 +539,7 @@ TileIndex RoadVehicle::GetOrderStationLocation(StationID station)
 	const Station *st = Station::Get(station);
 	if (!CanVehicleUseStation(this, st)) {
 		/* There is no stop left at the station, so don't even TRY to go there */
-		this->IncrementOrderIndex();
+		this->IncrementRealOrderIndex();
 		return 0;
 	}
 
@@ -629,8 +550,9 @@ static void StartRoadVehSound(const RoadVehicle *v)
 {
 	if (!PlayVehicleSound(v, VSE_START)) {
 		SoundID s = RoadVehInfo(v->engine_type)->sfx;
-		if (s == SND_19_BUS_START_PULL_AWAY && (v->tick_counter & 3) == 0)
+		if (s == SND_19_BUS_START_PULL_AWAY && (v->tick_counter & 3) == 0) {
 			s = SND_1A_BUS_START_PULL_AWAY_WITH_HORN;
+		}
 		SndPlayVehicleFx(s, v);
 	}
 }
@@ -708,6 +630,11 @@ static RoadVehicle *RoadVehFindCloseTo(RoadVehicle *v, int x, int y, Direction d
 	return RoadVehicle::From(rvf.best);
 }
 
+/**
+ * A road vehicle arrives at a station. If it is the first time, create a news item.
+ * @param v  Road vehicle that arrived.
+ * @param st Station where the road vehicle arrived.
+ */
 static void RoadVehArrivesAt(const RoadVehicle *v, Station *st)
 {
 	if (v->IsBus()) {
@@ -739,39 +666,23 @@ static void RoadVehArrivesAt(const RoadVehicle *v, Station *st)
 	}
 }
 
-static int RoadVehAccelerate(RoadVehicle *v)
+/**
+ * This function looks at the vehicle and updates its speed (cur_speed
+ * and subspeed) variables. Furthermore, it returns the distance that
+ * the vehicle can drive this tick. #Vehicle::GetAdvanceDistance() determines
+ * the distance to drive before moving a step on the map.
+ * @return distance to drive.
+ */
+int RoadVehicle::UpdateSpeed()
 {
-	uint oldspeed = v->cur_speed;
-	uint accel = 256 + (v->overtaking != 0 ? 256 : 0);
-	uint spd = v->subspeed + accel;
+	switch (_settings_game.vehicle.roadveh_acceleration_model) {
+		default: NOT_REACHED();
+		case AM_ORIGINAL:
+			return this->DoUpdateSpeed(this->overtaking != 0 ? 512 : 256, 0, this->GetCurrentMaxSpeed());
 
-	v->subspeed = (uint8)spd;
-
-	int tempmax = v->max_speed;
-	if (v->cur_speed > v->max_speed) {
-		tempmax = v->cur_speed - (v->cur_speed / 10) - 1;
+		case AM_REALISTIC:
+			return this->DoUpdateSpeed(this->GetAcceleration() + (this->overtaking != 0 ? 256 : 0), this->GetAccelerationStatus() == AS_BRAKE ? 0 : 4, this->GetCurrentMaxSpeed());
 	}
-
-	v->cur_speed = spd = Clamp(v->cur_speed + ((int)spd >> 8), 0, tempmax);
-
-	/* Apply bridge speed limit */
-	if (v->state == RVSB_WORMHOLE && !(v->vehstatus & VS_HIDDEN)) {
-		v->cur_speed = min(v->cur_speed, GetBridgeSpec(GetBridgeType(v->tile))->speed * 2);
-	}
-
-	/* Update statusbar only if speed has changed to save CPU time */
-	if (oldspeed != v->cur_speed) {
-		if (_settings_client.gui.vehicle_speed) {
-			SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
-		}
-	}
-
-	/* Speed is scaled in the same manner as for trains. @see train_cmd.cpp */
-	int scaled_spd = spd * 3 >> 2;
-
-	scaled_spd += v->progress;
-	v->progress = 0;
-	return scaled_spd;
 }
 
 static Direction RoadVehGetNewDirection(const RoadVehicle *v, int x, int y)
@@ -811,9 +722,7 @@ static Vehicle *EnumFindVehBlockingOvertake(Vehicle *v, void *data)
 {
 	const OvertakeData *od = (OvertakeData*)data;
 
-	return
-		v->type == VEH_ROAD && v->First() == v && v != od->u && v != od->v ?
-			v : NULL;
+	return (v->type == VEH_ROAD && v->First() == v && v != od->u && v != od->v) ? v : NULL;
 }
 
 /**
@@ -843,7 +752,7 @@ static void RoadVehCheckOvertake(RoadVehicle *v, RoadVehicle *u)
 	od.v = v;
 	od.u = u;
 
-	if (u->max_speed >= v->max_speed &&
+	if (u->vcache.cached_max_speed >= v->vcache.cached_max_speed &&
 			!(u->vehstatus & VS_STOPPED) &&
 			u->cur_speed != 0) {
 		return;
@@ -878,25 +787,21 @@ static void RoadVehCheckOvertake(RoadVehicle *v, RoadVehicle *u)
 	od.tile = v->tile + TileOffsByDiagDir(DirToDiagDir(v->direction));
 	if (CheckRoadBlockedForOvertaking(&od)) return;
 
-	if (od.u->cur_speed == 0 || (od.u->vehstatus & VS_STOPPED)) {
-		v->overtaking_ctr = 0x11;
-		v->overtaking = 0x10;
-	} else {
-//		if (CheckRoadBlockedForOvertaking(&od)) return;
-		v->overtaking_ctr = 0;
-		v->overtaking = 0x10;
-	}
+	/* When the vehicle in front of us is stopped we may only take
+	 * half the time to pass it than when the vehicle is moving. */
+	v->overtaking_ctr = (od.u->cur_speed == 0 || (od.u->vehstatus & VS_STOPPED)) ? RV_OVERTAKE_TIMEOUT / 2 : 0;
+	v->overtaking = RVSB_DRIVE_SIDE;
 }
 
 static void RoadZPosAffectSpeed(RoadVehicle *v, byte old_z)
 {
-	if (old_z == v->z_pos) return;
+	if (old_z == v->z_pos || _settings_game.vehicle.roadveh_acceleration_model != AM_ORIGINAL) return;
 
 	if (old_z < v->z_pos) {
 		v->cur_speed = v->cur_speed * 232 / 256; // slow down by ~10%
 	} else {
 		uint16 spd = v->cur_speed + 2;
-		if (spd <= v->max_speed) v->cur_speed = spd;
+		if (spd <= v->vcache.cached_max_speed) v->cur_speed = spd;
 	}
 }
 
@@ -923,6 +828,7 @@ static Trackdir RoadFindPathToDest(RoadVehicle *v, TileIndex tile, DiagDirection
 
 	TileIndex desttile;
 	Trackdir best_track;
+	bool path_found = true;
 
 	TrackStatus ts = GetTileTrackStatus(tile, TRANSPORT_ROAD, v->compatible_roadtypes);
 	TrackdirBits red_signals = TrackStatusToRedSignals(ts); // crossing
@@ -998,11 +904,12 @@ static Trackdir RoadFindPathToDest(RoadVehicle *v, TileIndex tile, DiagDirection
 	}
 
 	switch (_settings_game.pf.pathfinder_for_roadvehs) {
-		case VPF_NPF: return_track(NPFRoadVehicleChooseTrack(v, tile, enterdir, trackdirs));
-		case VPF_YAPF: return_track(YapfRoadVehicleChooseTrack(v, tile, enterdir, trackdirs));
+		case VPF_NPF:  best_track = NPFRoadVehicleChooseTrack(v, tile, enterdir, trackdirs, path_found); break;
+		case VPF_YAPF: best_track = YapfRoadVehicleChooseTrack(v, tile, enterdir, trackdirs, path_found); break;
 
 		default: NOT_REACHED();
 	}
+	v->HandlePathfindingResult(path_found);
 
 found_best_track:;
 
@@ -1017,13 +924,6 @@ struct RoadDriveEntry {
 
 #include "table/roadveh_movement.h"
 
-static const byte _road_veh_data_1[] = {
-	20, 20, 16, 16, 0, 0, 0, 0,
-	19, 19, 15, 15, 0, 0, 0, 0,
-	16, 16, 12, 12, 0, 0, 0, 0,
-	15, 15, 11, 11
-};
-
 static bool RoadVehLeaveDepot(RoadVehicle *v, bool first)
 {
 	/* Don't leave if not all the wagons are in the depot. */
@@ -1041,6 +941,12 @@ static bool RoadVehLeaveDepot(RoadVehicle *v, bool first)
 	int y = TileY(v->tile) * TILE_SIZE + (rdp[RVC_DEPOT_START_FRAME].y & 0xF);
 
 	if (first) {
+		/* We are leaving a depot, but have to go to the exact same one; re-enter */
+		if (v->current_order.IsType(OT_GOTO_DEPOT) && v->tile == v->dest_tile) {
+			VehicleEnterDepot(v);
+			return true;
+		}
+
 		if (RoadVehFindCloseTo(v, x, y, v->direction, false) != NULL) return true;
 
 		VehicleServiceInDepot(v);
@@ -1055,7 +961,9 @@ static bool RoadVehLeaveDepot(RoadVehicle *v, bool first)
 	v->state = tdir;
 	v->frame = RVC_DEPOT_START_FRAME;
 
-	SetRoadVehPosition(v, x, y, true);
+	v->x_pos = x;
+	v->y_pos = y;
+	v->UpdateInclination(true, true);
 
 	InvalidateWindowData(WC_VEHICLE_DEPOT, v->tile);
 
@@ -1088,7 +996,7 @@ static Trackdir FollowPreviousRoadVehicle(const RoadVehicle *v, const RoadVehicl
 		if (already_reversed && prev->tile != tile) {
 			/*
 			 * The vehicle has reversed, but did not go straight back.
-			 * It immediatelly turn onto another tile. This means that
+			 * It immediately turn onto another tile. This means that
 			 * the roadstate of the previous vehicle cannot be used
 			 * as the direction we have to go with this vehicle.
 			 *
@@ -1138,12 +1046,11 @@ static Trackdir FollowPreviousRoadVehicle(const RoadVehicle *v, const RoadVehicl
 static bool CanBuildTramTrackOnTile(CompanyID c, TileIndex t, RoadBits r)
 {
 	/* The 'current' company is not necessarily the owner of the vehicle. */
-	CompanyID original_company = _current_company;
-	_current_company = c;
+	Backup<CompanyByte> cur_company(_current_company, c, FILE_LINE);
 
 	CommandCost ret = DoCommand(t, ROADTYPE_TRAM << 4 | r, 0, DC_NONE, CMD_BUILD_ROAD);
 
-	_current_company = original_company;
+	cur_company.Restore();
 	return ret.Succeeded();
 }
 
@@ -1153,7 +1060,7 @@ static bool IndividualRoadVehicleController(RoadVehicle *v, const RoadVehicle *p
 		if (IsTileType(v->tile, MP_STATION)) {
 			/* Force us to be not overtaking! */
 			v->overtaking = 0;
-		} else if (++v->overtaking_ctr >= 35) {
+		} else if (++v->overtaking_ctr >= RV_OVERTAKE_TIMEOUT) {
 			/* If overtaking just aborts at a random moment, we can have a out-of-bound problem,
 			 *  if the vehicle started a corner. To protect that, only allow an abort of
 			 *  overtake if we are on straight roads */
@@ -1172,7 +1079,13 @@ static bool IndividualRoadVehicleController(RoadVehicle *v, const RoadVehicle *p
 		/* Vehicle is entering a depot or is on a bridge or in a tunnel */
 		GetNewVehiclePosResult gp = GetNewVehiclePos(v);
 
-		if (v->IsRoadVehFront()) {
+		/* Apply bridge speed limit */
+		if (!(v->vehstatus & VS_HIDDEN)) {
+			RoadVehicle *first = v->First();
+			first->cur_speed = min(first->cur_speed, GetBridgeSpec(GetBridgeType(v->tile))->speed * 2);
+		}
+
+		if (v->IsFrontEngine()) {
 			const Vehicle *u = RoadVehFindCloseTo(v, gp.x, gp.y, v->direction);
 			if (u != NULL) {
 				v->cur_speed = u->First()->cur_speed;
@@ -1182,7 +1095,9 @@ static bool IndividualRoadVehicleController(RoadVehicle *v, const RoadVehicle *p
 
 		if (IsTileType(gp.new_tile, MP_TUNNELBRIDGE) && HasBit(VehicleEnterTile(v, gp.new_tile, gp.x, gp.y), VETS_ENTERED_WORMHOLE)) {
 			/* Vehicle has just entered a bridge or tunnel */
-			SetRoadVehPosition(v, gp.x, gp.y, true);
+			v->x_pos = gp.x;
+			v->y_pos = gp.y;
+			v->UpdateInclination(true, true);
 			return true;
 		}
 
@@ -1203,7 +1118,7 @@ static bool IndividualRoadVehicleController(RoadVehicle *v, const RoadVehicle *p
 		TileIndex tile = v->tile + TileOffsByDiagDir((DiagDirection)(rd.x & 3));
 		Trackdir dir;
 
-		if (v->IsRoadVehFront()) {
+		if (v->IsFrontEngine()) {
 			/* If this is the front engine, look for the right path. */
 			dir = RoadFindPathToDest(v, tile, (DiagDirection)(rd.x & 3));
 		} else {
@@ -1211,7 +1126,7 @@ static bool IndividualRoadVehicleController(RoadVehicle *v, const RoadVehicle *p
 		}
 
 		if (dir == INVALID_TRACKDIR) {
-			if (!v->IsRoadVehFront()) error("Disconnecting road vehicle.");
+			if (!v->IsFrontEngine()) error("Disconnecting road vehicle.");
 			v->cur_speed = 0;
 			return false;
 		}
@@ -1219,6 +1134,9 @@ static bool IndividualRoadVehicleController(RoadVehicle *v, const RoadVehicle *p
 again:
 		uint start_frame = RVC_DEFAULT_START_FRAME;
 		if (IsReversingRoadTrackdir(dir)) {
+			/* When turning around we can't be overtaking. */
+			v->overtaking = 0;
+
 			/* Turning around */
 			if (v->roadtype == ROADTYPE_TRAM) {
 				/* Determine the road bits the tram needs to be able to turn around
@@ -1232,7 +1150,7 @@ again:
 					case TRACKDIR_RVREV_NW: needed = ROAD_SE; break;
 				}
 				if ((v->Previous() != NULL && v->Previous()->tile == tile) ||
-						(v->IsRoadVehFront() && IsNormalRoadTile(tile) && !HasRoadWorks(tile) &&
+						(v->IsFrontEngine() && IsNormalRoadTile(tile) && !HasRoadWorks(tile) &&
 							(needed & GetRoadBits(tile, ROADTYPE_TRAM)) != ROAD_NONE)) {
 					/*
 					 * Taking the 'big' corner for trams only happens when:
@@ -1244,7 +1162,7 @@ again:
 					 *   going to cause the tram to split up.
 					 * - Or the front of the tram can drive over the next tile.
 					 */
-				} else if (!v->IsRoadVehFront() || !CanBuildTramTrackOnTile(v->owner, tile, needed) || ((~needed & GetAnyRoadBits(v->tile, ROADTYPE_TRAM, false)) == ROAD_NONE)) {
+				} else if (!v->IsFrontEngine() || !CanBuildTramTrackOnTile(v->owner, tile, needed) || ((~needed & GetAnyRoadBits(v->tile, ROADTYPE_TRAM, false)) == ROAD_NONE)) {
 					/*
 					 * Taking the 'small' corner for trams only happens when:
 					 * - We are not the from vehicle of an articulated tram.
@@ -1278,7 +1196,7 @@ again:
 		int y = TileY(tile) * TILE_SIZE + rdp[start_frame].y;
 
 		Direction new_dir = RoadVehGetSlidingDirection(v, x, y);
-		if (v->IsRoadVehFront()) {
+		if (v->IsFrontEngine()) {
 			Vehicle *u = RoadVehFindCloseTo(v, x, y, new_dir);
 			if (u != NULL) {
 				v->cur_speed = u->First()->cur_speed;
@@ -1330,10 +1248,11 @@ again:
 		}
 		if (new_dir != v->direction) {
 			v->direction = new_dir;
-			v->cur_speed -= v->cur_speed >> 2;
+			if (_settings_game.vehicle.roadveh_acceleration_model == AM_ORIGINAL) v->cur_speed -= v->cur_speed >> 2;
 		}
-
-		RoadZPosAffectSpeed(v, SetRoadVehPosition(v, x, y, true));
+		v->x_pos = x;
+		v->y_pos = y;
+		RoadZPosAffectSpeed(v, v->UpdateInclination(true, true));
 		return true;
 	}
 
@@ -1342,8 +1261,7 @@ again:
 		Trackdir dir;
 		uint turn_around_start_frame = RVC_TURN_AROUND_START_FRAME;
 
-		RoadBits tram;
-		if (v->roadtype == ROADTYPE_TRAM && !IsRoadDepotTile(v->tile) && HasExactlyOneBit(tram = GetAnyRoadBits(v->tile, ROADTYPE_TRAM, true))) {
+		if (v->roadtype == ROADTYPE_TRAM && !IsRoadDepotTile(v->tile) && HasExactlyOneBit(GetAnyRoadBits(v->tile, ROADTYPE_TRAM, true))) {
 			/*
 			 * The tram is turning around with one tram 'roadbit'. This means that
 			 * it is using the 'big' corner 'drive data'. However, to support the
@@ -1362,7 +1280,7 @@ again:
 				case DIAGDIR_SW: dir = TRACKDIR_RVREV_NE; break;
 			}
 		} else {
-			if (v->IsRoadVehFront()) {
+			if (v->IsFrontEngine()) {
 				/* If this is the front engine, look for the right path. */
 				dir = RoadFindPathToDest(v, v->tile, (DiagDirection)(rd.x & 3));
 			} else {
@@ -1381,7 +1299,7 @@ again:
 		int y = TileY(v->tile) * TILE_SIZE + rdp[turn_around_start_frame].y;
 
 		Direction new_dir = RoadVehGetSlidingDirection(v, x, y);
-		if (v->IsRoadVehFront() && RoadVehFindCloseTo(v, x, y, new_dir) != NULL) return false;
+		if (v->IsFrontEngine() && RoadVehFindCloseTo(v, x, y, new_dir) != NULL) return false;
 
 		uint32 r = VehicleEnterTile(v, v->tile, x, y);
 		if (HasBit(r, VETS_CANNOT_ENTER)) {
@@ -1394,10 +1312,12 @@ again:
 
 		if (new_dir != v->direction) {
 			v->direction = new_dir;
-			v->cur_speed -= v->cur_speed >> 2;
+			if (_settings_game.vehicle.roadveh_acceleration_model == AM_ORIGINAL) v->cur_speed -= v->cur_speed >> 2;
 		}
 
-		RoadZPosAffectSpeed(v, SetRoadVehPosition(v, x, y, true));
+		v->x_pos = x;
+		v->y_pos = y;
+		RoadZPosAffectSpeed(v, v->UpdateInclination(true, true));
 		return true;
 	}
 
@@ -1405,7 +1325,7 @@ again:
 	 * it's on a depot tile, check if it's time to activate the next vehicle in
 	 * the chain yet. */
 	if (v->Next() != NULL && IsRoadDepotTile(v->tile)) {
-		if (v->frame == v->rcache.cached_veh_length + RVC_DEPOT_START_FRAME) {
+		if (v->frame == v->gcache.cached_veh_length + RVC_DEPOT_START_FRAME) {
 			RoadVehLeaveDepot(v->Next(), false);
 		}
 	}
@@ -1416,7 +1336,7 @@ again:
 
 	Direction new_dir = RoadVehGetSlidingDirection(v, x, y);
 
-	if (v->IsRoadVehFront() && !IsInsideMM(v->state, RVSB_IN_ROAD_STOP, RVSB_IN_ROAD_STOP_END)) {
+	if (v->IsFrontEngine() && !IsInsideMM(v->state, RVSB_IN_ROAD_STOP, RVSB_IN_ROAD_STOP_END)) {
 		/* Vehicle is not in a road stop.
 		 * Check for another vehicle to overtake */
 		RoadVehicle *u = RoadVehFindCloseTo(v, x, y, new_dir);
@@ -1444,10 +1364,10 @@ again:
 	Direction old_dir = v->direction;
 	if (new_dir != old_dir) {
 		v->direction = new_dir;
-		v->cur_speed -= (v->cur_speed >> 2);
+		if (_settings_game.vehicle.roadveh_acceleration_model == AM_ORIGINAL) v->cur_speed -= v->cur_speed >> 2;
 		if (old_dir != v->state) {
 			/* The vehicle is in a road stop */
-			SetRoadVehPosition(v, v->x_pos, v->y_pos, true);
+			v->UpdateInclination(false, true);
 			/* Note, return here means that the frame counter is not incremented
 			 * for vehicles changing direction in a road stop. This causes frames to
 			 * be repeated. (XXX) Is this intended? */
@@ -1460,8 +1380,8 @@ again:
 	 * and it's the correct type of stop (bus or truck) and the frame equals the stop frame...
 	 * (the station test and stop type test ensure that other vehicles, using the road stop as
 	 * a through route, do not stop) */
-	if (v->IsRoadVehFront() && ((IsInsideMM(v->state, RVSB_IN_ROAD_STOP, RVSB_IN_ROAD_STOP_END) &&
-			_road_veh_data_1[v->state - RVSB_IN_ROAD_STOP + (_settings_game.vehicle.road_side << RVS_DRIVE_SIDE)] == v->frame) ||
+	if (v->IsFrontEngine() && ((IsInsideMM(v->state, RVSB_IN_ROAD_STOP, RVSB_IN_ROAD_STOP_END) &&
+			_road_stop_stop_frame[v->state - RVSB_IN_ROAD_STOP + (_settings_game.vehicle.road_side << RVS_DRIVE_SIDE)] == v->frame) ||
 			(IsInsideMM(v->state, RVSB_IN_DT_ROAD_STOP, RVSB_IN_DT_ROAD_STOP_END) &&
 			v->current_order.ShouldStopAtStation(v, GetStationIndex(v->tile)) &&
 			v->owner == GetTileOwner(v->tile) &&
@@ -1474,7 +1394,7 @@ again:
 		/* Vehicle is at the stop position (at a bay) in a road stop.
 		 * Note, if vehicle is loading/unloading it has already been handled,
 		 * so if we get here the vehicle has just arrived or is just ready to leave. */
-		if (!v->current_order.IsType(OT_LEAVESTATION)) {
+		if (!HasBit(v->state, RVS_ENTERED_STOP)) {
 			/* Vehicle has arrived at a bay in a road stop */
 
 			if (IsDriveThroughStopTile(v->tile)) {
@@ -1483,12 +1403,15 @@ again:
 				/* Check if next inline bay is free and has compatible road. */
 				if (RoadStop::IsDriveThroughRoadStopContinuation(v->tile, next_tile) && (GetRoadTypes(next_tile) & v->compatible_roadtypes) != 0) {
 					v->frame++;
-					RoadZPosAffectSpeed(v, SetRoadVehPosition(v, x, y, false));
+					v->x_pos = x;
+					v->y_pos = y;
+					RoadZPosAffectSpeed(v, v->UpdateInclination(true, false));
 					return true;
 				}
 			}
 
 			rs->SetEntranceBusy(false);
+			SetBit(v->state, RVS_ENTERED_STOP);
 
 			v->last_station_visited = st->index;
 
@@ -1504,7 +1427,7 @@ again:
 				v->cur_speed = 0;
 				return false;
 			}
-			v->current_order.Free();
+			if (v->current_order.IsType(OT_LEAVESTATION)) v->current_order.Free();
 		}
 
 		if (IsStandardRoadStopTile(v->tile)) rs->SetEntranceBusy(true);
@@ -1528,8 +1451,9 @@ again:
 	/* Move to next frame unless vehicle arrived at a stop position
 	 * in a depot or entered a tunnel/bridge */
 	if (!HasBit(r, VETS_ENTERED_WORMHOLE)) v->frame++;
-
-	RoadZPosAffectSpeed(v, SetRoadVehPosition(v, x, y, true));
+	v->x_pos = x;
+	v->y_pos = y;
+	RoadZPosAffectSpeed(v, v->UpdateInclination(false, true));
 	return true;
 }
 
@@ -1546,14 +1470,7 @@ static bool RoadVehController(RoadVehicle *v)
 	}
 
 	/* road vehicle has broken down? */
-	if (v->breakdown_ctr != 0) {
-		if (v->breakdown_ctr <= 2) {
-			HandleBrokenRoadVeh(v);
-			return true;
-		}
-		if (!v->current_order.IsType(OT_LOADING)) v->breakdown_ctr--;
-	}
-
+	if (v->HandleBreakdown()) return true;
 	if (v->vehstatus & VS_STOPPED) return true;
 
 	ProcessOrders(v);
@@ -1563,10 +1480,12 @@ static bool RoadVehController(RoadVehicle *v)
 
 	if (v->IsInDepot() && RoadVehLeaveDepot(v, true)) return true;
 
-	/* Check how far the vehicle needs to proceed */
-	int j = RoadVehAccelerate(v);
+	v->ShowVisualEffect();
 
-	int adv_spd = (v->direction & 1) ? 192 : 256;
+	/* Check how far the vehicle needs to proceed */
+	int j = v->UpdateSpeed();
+
+	int adv_spd = v->GetAdvanceDistance();
 	bool blocked = false;
 	while (j >= adv_spd) {
 		j -= adv_spd;
@@ -1580,13 +1499,15 @@ static bool RoadVehController(RoadVehicle *v)
 		}
 		if (blocked) break;
 
-		/* 192 spd used for going straight, 256 for going diagonally. */
-		adv_spd = (v->direction & 1) ? 192 : 256;
+		/* Determine distance to next map position */
+		adv_spd = v->GetAdvanceDistance();
 
 		/* Test for a collision, but only if another movement will occur. */
 		if (j >= adv_spd && RoadVehCheckTrainCrash(v)) break;
 	}
 
+	v->SetLastSpeed();
+
 	for (RoadVehicle *u = v; u != NULL; u = u->Next()) {
 		if ((u->vehstatus & VS_HIDDEN) != 0) continue;
 
@@ -1609,12 +1530,12 @@ Money RoadVehicle::GetRunningCost() const
 	uint cost_factor = GetVehicleProperty(this, PROP_ROADVEH_RUNNING_COST_FACTOR, e->u.road.running_cost);
 	if (cost_factor == 0) return 0;
 
-	return GetPrice(e->u.road.running_cost_class, cost_factor, e->grffile);
+	return GetPrice(e->u.road.running_cost_class, cost_factor, e->grf_prop.grffile);
 }
 
 bool RoadVehicle::Tick()
 {
-	if (this->IsRoadVehFront()) {
+	if (this->IsFrontEngine()) {
 		if (!(this->vehstatus & VS_STOPPED)) this->running_ticks++;
 		return RoadVehController(this);
 	}
@@ -1659,8 +1580,7 @@ static void CheckIfRoadVehNeedsService(RoadVehicle *v)
 		return;
 	}
 
-	if (v->current_order.IsType(OT_LOADING)) v->LeaveStation();
-
+	SetBit(v->gv_flags, GVF_SUPPRESS_IMPLICIT_ORDERS);
 	v->current_order.MakeGoToDepot(depot, ODTFB_SERVICE);
 	v->dest_tile = rfdd.tile;
 	SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
@@ -1668,7 +1588,7 @@ static void CheckIfRoadVehNeedsService(RoadVehicle *v)
 
 void RoadVehicle::OnNewDay()
 {
-	if (!this->IsRoadVehFront()) return;
+	if (!this->IsFrontEngine()) return;
 
 	if ((++this->day_counter & 7) == 0) DecreaseVehicleValue(this);
 	if (this->blocked_ctr == 0) CheckVehicleBreakdown(this);
@@ -1712,44 +1632,3 @@ Trackdir RoadVehicle::GetVehicleTrackdir() const
 	 * otherwise transform it into a valid track direction */
 	return (Trackdir)((IsReversingRoadTrackdir((Trackdir)this->state)) ? (this->state - 6) : this->state);
 }
-
-
-/** Refit a road vehicle to the specified cargo type
- * @param tile unused
- * @param flags operation to perform
- * @param p1 Vehicle ID of the vehicle to refit
- * @param p2 Bitstuffed elements
- * - p2 = (bit 0-7) - the new cargo type to refit to
- * - p2 = (bit 8-15) - the new cargo subtype to refit to
- * - p2 = (bit 16) - refit only this vehicle
- * @param text unused
- * @return the cost of this operation or an error
- */
-CommandCost CmdRefitRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
-{
-	CargoID new_cid = GB(p2, 0, 8);
-	byte new_subtype = GB(p2, 8, 8);
-	bool only_this = HasBit(p2, 16);
-
-	RoadVehicle *v = RoadVehicle::GetIfValid(p1);
-
-	if (v == NULL || !CheckOwnership(v->owner)) return CMD_ERROR;
-	if (!v->IsStoppedInDepot()) return_cmd_error(STR_ERROR_ROAD_VEHICLE_MUST_BE_STOPPED_INSIDE_DEPOT);
-	if (v->vehstatus & VS_CRASHED) return_cmd_error(STR_ERROR_CAN_T_REFIT_DESTROYED_VEHICLE);
-
-	if (new_cid >= NUM_CARGO) return CMD_ERROR;
-
-	CommandCost cost = RefitVehicle(v, only_this, new_cid, new_subtype, flags);
-
-	if (flags & DC_EXEC) {
-		RoadVehicle *front = v->First();
-		RoadVehUpdateCache(front);
-		InvalidateWindowData(WC_VEHICLE_DETAILS, front->index);
-		SetWindowDirty(WC_VEHICLE_DEPOT, front->tile);
-		InvalidateWindowClassesData(WC_ROADVEH_LIST, 0);
-	} else {
-		v->InvalidateNewGRFCacheOfChain(); // always invalidate; querycost might have filled it
-	}
-
-	return cost;
-}
