@@ -1,4 +1,4 @@
-/* $Id: ai_sl.cpp 19430 2010-03-15 22:52:39Z rubidium $ */
+/* $Id: ai_sl.cpp 22884 2011-09-03 18:50:20Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -66,6 +66,8 @@ static void Load_AIPL()
 
 	CompanyID index;
 	while ((index = (CompanyID)SlIterateArray()) != (CompanyID)-1) {
+		if (index >= MAX_COMPANIES) SlErrorCorrupt("Too many AI configs");
+
 		_ai_saveload_version = -1;
 		SlObject(NULL, _ai_company);
 
@@ -121,7 +123,7 @@ static void Save_AIPL()
 }
 
 extern const ChunkHandler _ai_chunk_handlers[] = {
-	{ 'AIPL', Save_AIPL, Load_AIPL, NULL, CH_ARRAY | CH_LAST},
+	{ 'AIPL', Save_AIPL, Load_AIPL, NULL, NULL, CH_ARRAY | CH_LAST},
 };
 #else
 
@@ -178,7 +180,7 @@ static bool LoadObjects()
 		case SQSL_ARRAY_TABLE_END:
 			return false;
 
-		default: NOT_REACHED();
+		default: SlErrorCorrupt("Invalid AI data type");
 	}
 }
 
@@ -197,6 +199,6 @@ static void Load_AIPL()
 }
 
 extern const ChunkHandler _ai_chunk_handlers[] = {
-	{ 'AIPL', NULL, Load_AIPL, NULL, CH_ARRAY | CH_LAST},
+	{ 'AIPL', NULL, Load_AIPL, NULL, NULL, CH_ARRAY | CH_LAST},
 };
 #endif /* ENABLE_AI */
