@@ -1,4 +1,4 @@
-/* $Id: oldloader_sl.cpp 20430 2010-08-09 21:48:19Z rubidium $ */
+/* $Id: oldloader_sl.cpp 22597 2011-06-18 18:33:20Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -24,11 +24,7 @@
 #include "../depot_base.h"
 #include "../date_func.h"
 #include "../vehicle_func.h"
-#include "../variables.h"
 #include "../effectvehicle_base.h"
-#include "../core/mem_func.hpp"
-#include "../core/alloc_type.hpp"
-#include "../engine_base.h"
 #include "../engine_func.h"
 #include "../company_base.h"
 #include "saveload_internal.h"
@@ -38,9 +34,11 @@
 #include "../table/engines.h"
 #include "../table/townname.h"
 
-static bool   _read_ttdpatch_flags;
+static bool _read_ttdpatch_flags;    ///< Have we (tried to) read TTDPatch extra flags?
+static uint16 _old_extra_chunk_nums; ///< Number of extra TTDPatch chunks
+static byte _old_vehicle_multiplier; ///< TTDPatch vehicle multiplier
 
-static uint8  *_old_map3;
+static uint8 *_old_map3;
 
 void FixOldMapArray()
 {
@@ -110,7 +108,7 @@ static void FixTTDDepots()
 {
 	const Depot *d;
 	FOR_ALL_DEPOTS_FROM(d, 252) {
-		if (!IsRoadDepotTile(d->xy) && !IsRailDepotTile(d->xy) && !IsShipDepotTile(d->xy) && !IsHangarTile(d->xy)) {
+		if (!IsDepotTile(d->xy) || GetDepotIndex(d->xy) != d->index) {
 			/** Workaround for SVXConverter bug, depots 252-255 could be invalid */
 			delete d;
 		}
@@ -165,6 +163,11 @@ static void FixOldTowns()
 
 static StringID *_old_vehicle_names;
 
+/**
+ * Convert the old style vehicles into something that resembles
+ * the old new style savegames. Then #AfterLoadGame can handle
+ * the rest of the conversion.
+ */
 void FixOldVehicles()
 {
 	Vehicle *v;
@@ -193,6 +196,10 @@ void FixOldVehicles()
 			RoadVehicle *rv = RoadVehicle::From(v);
 			if (rv->state != RVSB_IN_DEPOT && rv->state != RVSB_WORMHOLE) {
 				ClrBit(rv->state, 2);
+				if (IsTileType(rv->tile, MP_STATION) && _m[rv->tile].m5 >= 168) {
+					/* Update the vehicle's road state to show we're in a drive through road stop. */
+					SetBit(rv->state, RVS_IN_DT_ROAD_STOP);
+				}
 			}
 		}
 
@@ -324,7 +331,7 @@ static bool FixTTOMapArray()
 				}
 				break;
 
-			case MP_UNMOVABLE:
+			case MP_OBJECT:
 				_m[t].m2 = 0;
 				_m[t].m3 = 0;
 				break;
@@ -348,14 +355,14 @@ static bool FixTTOEngines()
 	static const EngineID ttd_to_tto[] = {
 		  0, 255, 255, 255, 255, 255, 255, 255,   5,   7,   8,   9,  10,  11,  12,  13,
 		255, 255, 255, 255, 255, 255,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
-		25,   26,  27,  28,  29,  30, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+		25,   26,  27,  29,  28,  30, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255,  31, 255,  32,  33,  34,  35,  36,  37,  38,
 		 39,  40,  41,  42, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255,  44,  45,  46, 255, 255, 255, 255,  47,  48, 255,  49,  50,
-		255, 255, 255, 255,  51,  52, 255,  53,  54, 255,  55,  56, 255,  57,  58, 255,
-		 59,  60, 255,  61,  62, 255,  63,  64, 255,  65,  66, 255, 255, 255, 255, 255,
+		255, 255, 255, 255,  51,  52, 255,  53,  54, 255,  55,  56, 255,  57,  59, 255,
+		 58,  60, 255,  61,  62, 255,  63,  64, 255,  65,  66, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  67,  68,  69,  70,
@@ -367,8 +374,8 @@ static bool FixTTOEngines()
 	/** TTO->TTD remapping of engines. SVXConverter uses the same table. */
 	static const EngineID tto_to_ttd[] = {
 		  0,   0,   8,   8,   8,   8,   8,   9,  10,  11,  12,  13,  14,  15,  15,  22,
-		 23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  55,
-		 57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67, 116, 116, 117, 118, 123,
+		 23,  24,  25,  26,  27,  29,  28,  30,  31,  32,  33,  34,  35,  36,  37,  55,
+		 57,  59,  58,  60,  61,  62,  63,  64,  65,  66,  67, 116, 116, 117, 118, 123,
 		124, 126, 127, 132, 133, 135, 136, 138, 139, 141, 142, 144, 145, 147, 148, 150,
 		151, 153, 154, 204, 205, 206, 207, 208, 211, 212, 211, 212, 211, 212, 215, 216,
 		217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232,
@@ -483,11 +490,9 @@ extern TileIndex *_animated_tile_list;
 extern uint _animated_tile_count;
 extern char *_old_name_array;
 
-static byte   _old_vehicle_multiplier;
 static uint32 _old_town_index;
 static uint16 _old_string_id;
 static uint16 _old_string_id_2;
-static uint16 _old_extra_chunk_nums;
 
 static void ReadTTDPatchFlags()
 {
@@ -495,10 +500,13 @@ static void ReadTTDPatchFlags()
 
 	_read_ttdpatch_flags = true;
 
-	if (_savegame_type == SGT_TTO) {
-		_old_vehicle_multiplier = 1;
-		return;
-	}
+	/* Set default values */
+	_old_vehicle_multiplier = 1;
+	_ttdp_version = 0;
+	_old_extra_chunk_nums = 0;
+	_bump_assert_value = 0;
+
+	if (_savegame_type == SGT_TTO) return;
 
 	/* TTDPatch misuses _old_map3 for flags.. read them! */
 	_old_vehicle_multiplier = _old_map3[0];
@@ -662,7 +670,10 @@ static bool LoadOldDepot(LoadgameState *ls, int num)
 	if (!LoadChunk(ls, d, depot_chunk)) return false;
 
 	if (d->xy != 0) {
-		d->town_index = RemapTownIndex(_old_town_index);
+		/* In some cases, there could be depots referencing invalid town. */
+		Town *t = Town::GetIfValid(RemapTownIndex(_old_town_index));
+		if (t == NULL) t = Town::GetRandom();
+		d->town = t;
 	} else {
 		delete d;
 	}
@@ -699,7 +710,7 @@ static bool LoadOldGood(LoadgameState *ls, int num)
 
 	SB(ge->acceptance_pickup, GoodsEntry::ACCEPTANCE, 1, HasBit(_waiting_acceptance, 15));
 	SB(ge->acceptance_pickup, GoodsEntry::PICKUP, 1, _cargo_source != 0xFF);
-	if (GB(_waiting_acceptance, 0, 12) != 0) {
+	if (GB(_waiting_acceptance, 0, 12) != 0 && CargoPacket::CanAllocateItem()) {
 		ge->cargo.Append(new CargoPacket(GB(_waiting_acceptance, 0, 12), _cargo_days, (_cargo_source == 0xFF) ? INVALID_STATION : _cargo_source, 0, 0));
 	}
 
@@ -712,7 +723,7 @@ static const OldChunks station_chunk[] = {
 
 	OCL_NULL( 4 ), ///< bus/lorry tile
 	OCL_SVAR(   OC_TILE, Station, train_station.tile ),
-	OCL_SVAR(   OC_TILE, Station, airport_tile ),
+	OCL_SVAR(   OC_TILE, Station, airport.tile ),
 	OCL_SVAR(   OC_TILE, Station, dock_tile ),
 	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Station, train_station.w ),
 
@@ -732,11 +743,11 @@ static const OldChunks station_chunk[] = {
 	OCL_SVAR(  OC_UINT8, Station, delete_ctr ),
 	OCL_SVAR(  OC_UINT8, Station, owner ),
 	OCL_SVAR(  OC_UINT8, Station, facilities ),
-	OCL_SVAR( OC_TTD | OC_UINT8, Station, airport_type ),
-	OCL_SVAR( OC_TTO | OC_FILE_U16 | OC_VAR_U64, Station, airport_flags ),
+	OCL_SVAR( OC_TTD | OC_UINT8, Station, airport.type ),
+	OCL_SVAR( OC_TTO | OC_FILE_U16 | OC_VAR_U64, Station, airport.flags ),
 	OCL_NULL( 3 ),          ///< bus/truck status, blocked months, no longer in use
 	OCL_CNULL( OC_TTD, 1 ), ///< unknown
-	OCL_SVAR( OC_TTD | OC_FILE_U16 | OC_VAR_U64, Station, airport_flags ),
+	OCL_SVAR( OC_TTD | OC_FILE_U16 | OC_VAR_U64, Station, airport.flags ),
 	OCL_CNULL( OC_TTD, 2 ), ///< last_vehicle. now last_vehicle_type
 	OCL_CNULL( OC_TTD, 4 ), ///< junk at end of chunk
 
@@ -760,12 +771,12 @@ static bool LoadOldStation(LoadgameState *ls, int num)
 				st->string_id = _old_string_id + 0x2800; // custom name
 			}
 
-			if (HasBit(st->airport_flags, 8)) {
-				st->airport_type = 1; // large airport
-			} else if (HasBit(st->airport_flags, 6)) {
-				st->airport_type = 3; // oil rig
+			if (HasBit(st->airport.flags, 8)) {
+				st->airport.type = 1; // large airport
+			} else if (HasBit(st->airport.flags, 6)) {
+				st->airport.type = 3; // oil rig
 			} else {
-				st->airport_type = 0; // small airport
+				st->airport.type = 0; // small airport
 			}
 		} else {
 			st->string_id = RemapOldStringID(_old_string_id);
@@ -841,7 +852,7 @@ static bool LoadOldIndustry(LoadgameState *ls, int num)
 			i->random_colour = RemapTTOColour(i->random_colour);
 		}
 
-		IncIndustryTypeCount(i->type);
+		Industry::IncIndustryTypeCount(i->type);
 	} else {
 		delete i;
 	}
@@ -994,11 +1005,12 @@ static bool LoadOldCompany(LoadgameState *ls, int num)
 
 		if (num == 0) {
 			/* If the first company has no name, make sure we call it UNNAMED */
-			if (c->name_1 == 0)
+			if (c->name_1 == 0) {
 				c->name_1 = STR_SV_UNNAMED;
+			}
 		} else {
 			/* Beside some multiplayer maps (1 on 1), which we don't official support,
-			 * all other companys are an AI.. mark them as such */
+			 * all other companies are an AI.. mark them as such */
 			c->is_ai = true;
 		}
 
@@ -1098,7 +1110,7 @@ static bool LoadOldVehicleUnion(LoadgameState *ls, int num)
 		res = LoadChunk(ls, NULL, vehicle_empty_chunk);
 	} else {
 		switch (v->type) {
-			default: NOT_REACHED();
+			default: SlErrorCorrupt("Invalid vehicle type");
 			case VEH_TRAIN   : res = LoadChunk(ls, v, vehicle_train_chunk);    break;
 			case VEH_ROAD    : res = LoadChunk(ls, v, vehicle_road_chunk);     break;
 			case VEH_SHIP    : res = LoadChunk(ls, v, vehicle_ship_chunk);     break;
@@ -1129,15 +1141,15 @@ static const OldChunks vehicle_chunk[] = {
 	OCL_VAR ( OC_UINT16,   1, &_old_order ),
 
 	OCL_NULL ( 1 ), ///< num_orders, now calculated
-	OCL_SVAR(  OC_UINT8, Vehicle, cur_order_index ),
+	OCL_SVAR(  OC_UINT8, Vehicle, cur_implicit_order_index ),
 	OCL_SVAR(   OC_TILE, Vehicle, dest_tile ),
 	OCL_SVAR( OC_UINT16, Vehicle, load_unload_ticks ),
 	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Vehicle, date_of_last_service ),
 	OCL_SVAR( OC_UINT16, Vehicle, service_interval ),
 	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Vehicle, last_station_visited ),
 	OCL_SVAR( OC_TTD | OC_UINT8, Vehicle, tick_counter ),
-	OCL_SVAR( OC_TTD | OC_UINT16, Vehicle, max_speed ),
-	OCL_SVAR( OC_TTO | OC_FILE_U8 | OC_VAR_U16, Vehicle, max_speed ),
+	OCL_CNULL( OC_TTD, 2 ), ///< max_speed, now it is calculated.
+	OCL_CNULL( OC_TTO, 1 ), ///< max_speed, now it is calculated.
 
 	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32, Vehicle, x_pos ),
 	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32, Vehicle, y_pos ),
@@ -1149,7 +1161,7 @@ static const OldChunks vehicle_chunk[] = {
 
 	OCL_SVAR(  OC_UINT8, Vehicle, owner ),
 	OCL_SVAR(   OC_TILE, Vehicle, tile ),
-	OCL_SVAR( OC_UINT16, Vehicle, cur_image ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Vehicle, cur_image ),
 
 	OCL_NULL( 8 ),        ///< Vehicle sprite box, calculated automatically
 
@@ -1208,6 +1220,12 @@ static const OldChunks vehicle_chunk[] = {
 	OCL_END()
 };
 
+/**
+ * Load the vehicles of an old style savegame.
+ * @param ls  State (buffer) of the currently loaded game.
+ * @param num The number of vehicles to load.
+ * @return True iff loading went without problems.
+ */
 bool LoadOldVehicle(LoadgameState *ls, int num)
 {
 	/* Read the TTDPatch flags, because we need some info from it */
@@ -1298,7 +1316,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 		} else {
 			/* Read the vehicle type and allocate the right vehicle */
 			switch (ReadByte(ls)) {
-				default: NOT_REACHED();
+				default: SlErrorCorrupt("Invalid vehicle type");
 				case 0x00 /* VEH_INVALID */: v = NULL;                                        break;
 				case 0x10 /* VEH_TRAIN   */: v = new (_current_vehicle_id) Train();           break;
 				case 0x11 /* VEH_ROAD    */: v = new (_current_vehicle_id) RoadVehicle();     break;
@@ -1329,7 +1347,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 
 		v->next = (Vehicle *)(size_t)_old_next_ptr;
 
-		if (_cargo_count != 0) {
+		if (_cargo_count != 0 && CargoPacket::CanAllocateItem()) {
 			StationID source =    (_cargo_source == 0xFF) ? INVALID_STATION : _cargo_source;
 			TileIndex source_xy = (source != INVALID_STATION) ? Station::Get(source)->xy : 0;
 			v->cargo.Append(new CargoPacket(_cargo_count, _cargo_days, source, source_xy, source_xy));
@@ -1426,7 +1444,7 @@ static const OldChunks game_difficulty_chunk[] = {
 	OCL_SVAR( OC_FILE_U16 |  OC_VAR_U8, DifficultySettings, max_no_competitors ),
 	OCL_NULL( 2), // competitor_start_time
 	OCL_SVAR( OC_FILE_U16 |  OC_VAR_U8, DifficultySettings, number_towns ),
-	OCL_SVAR( OC_FILE_U16 |  OC_VAR_U8, DifficultySettings, number_industries ),
+	OCL_SVAR( OC_FILE_U16 |  OC_VAR_U8, DifficultySettings, industry_density ),
 	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, DifficultySettings, max_loan ),
 	OCL_SVAR( OC_FILE_U16 |  OC_VAR_U8, DifficultySettings, initial_interest ),
 	OCL_SVAR( OC_FILE_U16 |  OC_VAR_U8, DifficultySettings, vehicle_costs ),
@@ -1519,19 +1537,19 @@ static bool LoadTTDPatchExtraChunks(LoadgameState *ls, int num)
 					uint32 grfid = ReadUint32(ls);
 
 					if (ReadByte(ls) == 1) {
-						GRFConfig *c = CallocT<GRFConfig>(1);
-						c->grfid = grfid;
-						c->filename = strdup("TTDP game, no information");
+						GRFConfig *c = new GRFConfig("TTDP game, no information");
+						c->ident.grfid = grfid;
 
 						AppendToGRFConfigList(&_grfconfig, c);
-						DEBUG(oldloader, 3, "TTDPatch game using GRF file with GRFID %0X", BSWAP32(c->grfid));
+						DEBUG(oldloader, 3, "TTDPatch game using GRF file with GRFID %0X", BSWAP32(c->ident.grfid));
 					}
 					len -= 5;
-				};
+				}
 
 				/* Append static NewGRF configuration */
 				AppendStaticGRFConfigs(&_grfconfig);
-			} break;
+				break;
+			}
 
 			/* TTDPatch version and configuration */
 			case 0x3:
@@ -1553,6 +1571,8 @@ static bool LoadTTDPatchExtraChunks(LoadgameState *ls, int num)
 }
 
 extern TileIndex _cur_tileloop_tile;
+extern uint16 _disaster_delay;
+extern byte _trees_tick_ctr;
 static const OldChunks main_chunk[] = {
 	OCL_ASSERT( OC_TTD, 0 ),
 	OCL_ASSERT( OC_TTO, 0 ),
@@ -1650,7 +1670,7 @@ static const OldChunks main_chunk[] = {
 
 	OCL_ASSERT( OC_TTO, 0x496AC ),
 
-	OCL_VAR ( OC_UINT16,    1, &_vehicle_id_ctr_day ),
+	OCL_NULL ( 2 ), // _vehicle_id_ctr_day
 
 	OCL_CHUNK(  8, LoadOldSubsidy ),
 
@@ -1725,10 +1745,10 @@ static const OldChunks main_chunk[] = {
 
 bool LoadTTDMain(LoadgameState *ls)
 {
-	_read_ttdpatch_flags = false;
-	_ttdp_version = 0;
-
 	DEBUG(oldloader, 3, "Reading main chunk...");
+
+	_read_ttdpatch_flags = false;
+
 	/* Load the biggest chunk */
 	SmallStackSafeStackAlloc<byte, OLD_MAP_SIZE * 2> map3;
 	_old_map3 = map3.data;
@@ -1771,6 +1791,8 @@ bool LoadTTOMain(LoadgameState *ls)
 {
 	DEBUG(oldloader, 3, "Reading main chunk...");
 
+	_read_ttdpatch_flags = false;
+
 	SmallStackSafeStackAlloc<byte, 103 * sizeof(Engine)> engines; // we don't want to call Engine constructor here
 	_old_engines = (Engine *)engines.data;
 	SmallStackSafeStackAlloc<StringID, 800> vehnames;
