@@ -1,4 +1,4 @@
-/* $Id: vehicle_sl.cpp 19090 2010-02-10 21:06:05Z rubidium $ */
+/* $Id: vehicle_sl.cpp 22935 2011-09-15 18:56:43Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,13 +17,12 @@
 #include "../aircraft.h"
 #include "../station_base.h"
 #include "../effectvehicle_base.h"
-#include "../engine_base.h"
 
 #include "saveload.h"
 
 #include <map>
 
-/*
+/**
  * Link front and rear multiheaded engines to each other
  * This is done when loading a savegame
  */
@@ -151,7 +150,7 @@ void ConvertOldMultiheadToNew()
 						u->SetWagon();
 						u->SetFreeWagon();
 						break;
-					default: NOT_REACHED();
+					default: SlErrorCorrupt("Invalid train subtype");
 				}
 			}
 		}
@@ -165,7 +164,7 @@ void UpdateOldAircraft()
 	/* set airport_flags to 0 for all airports just to be sure */
 	Station *st;
 	FOR_ALL_STATIONS(st) {
-		st->airport_flags = 0; // reset airport
+		st->airport.flags = 0; // reset airport
 	}
 
 	Aircraft *a;
@@ -179,9 +178,10 @@ void UpdateOldAircraft()
 				continue;
 			}
 
-			AircraftLeaveHangar(a); // make airplane visible if it was in a depot for example
+			AircraftLeaveHangar(a, a->direction); // make airplane visible if it was in a depot for example
 			a->vehstatus &= ~VS_STOPPED; // make airplane moving
-			a->cur_speed = a->max_speed; // so aircraft don't have zero speed while in air
+			UpdateAircraftCache(a);
+			a->cur_speed = a->vcache.cached_max_speed; // so aircraft don't have zero speed while in air
 			if (!a->current_order.IsType(OT_GOTO_STATION) && !a->current_order.IsType(OT_GOTO_DEPOT)) {
 				/* reset current order so aircraft doesn't have invalid "station-only" order */
 				a->current_order.MakeDummy();
@@ -251,8 +251,7 @@ void AfterLoadVehicles(bool part_of_load)
 
 		if (part_of_load) v->fill_percent_te_id = INVALID_TE_ID;
 		v->first = NULL;
-		if (v->type == VEH_TRAIN) Train::From(v)->tcache.first_engine = INVALID_ENGINE;
-		if (v->type == VEH_ROAD)  RoadVehicle::From(v)->rcache.first_engine = INVALID_ENGINE;
+		if (v->IsGroundVehicle()) v->GetGroundVehicleCache()->first_engine = INVALID_ENGINE;
 	}
 
 	/* AfterLoadVehicles may also be called in case of NewGRF reload, in this
@@ -268,14 +267,19 @@ void AfterLoadVehicles(bool part_of_load)
 
 		FOR_ALL_VEHICLES(v) {
 			if (v->orders.old != NULL) {
-				if (CheckSavegameVersion(105)) { // Pre-105 didn't save an OrderList
+				if (IsSavegameVersionBefore(105)) { // Pre-105 didn't save an OrderList
 					if (mapping[v->orders.old] == NULL) {
 						/* This adds the whole shared vehicle chain for case b */
+
+						/* Creating an OrderList here is safe because the number of vehicles
+						 * allowed in these savegames matches the number of OrderLists. As
+						 * such each vehicle can get an OrderList and it will (still) fit. */
+						assert(OrderList::CanAllocateItem());
 						v->orders.list = mapping[v->orders.old] = new OrderList(v->orders.old, v);
 					} else {
 						v->orders.list = mapping[v->orders.old];
 						/* For old games (case a) we must create the shared vehicle chain */
-						if (CheckSavegameVersionOldStyle(5, 2)) {
+						if (IsSavegameVersionBefore(5, 2)) {
 							v->AddToShared(v->orders.list->GetFirstSharedVehicle());
 						}
 					}
@@ -297,14 +301,43 @@ void AfterLoadVehicles(bool part_of_load)
 		}
 	}
 
-	if (CheckSavegameVersion(105)) {
-		/* Before 105 there was no order for shared orders, thus it messed up horribly */
-		FOR_ALL_VEHICLES(v) {
-			if (v->First() != v || v->orders.list != NULL || v->previous_shared != NULL || v->next_shared == NULL) continue;
+	if (part_of_load) {
+		if (IsSavegameVersionBefore(105)) {
+			/* Before 105 there was no order for shared orders, thus it messed up horribly */
+			FOR_ALL_VEHICLES(v) {
+				if (v->First() != v || v->orders.list != NULL || v->previous_shared != NULL || v->next_shared == NULL) continue;
 
-			v->orders.list = new OrderList(NULL, v);
-			for (Vehicle *u = v; u != NULL; u = u->next_shared) {
-				u->orders.list = v->orders.list;
+				v->orders.list = new OrderList(NULL, v);
+				for (Vehicle *u = v; u != NULL; u = u->next_shared) {
+					u->orders.list = v->orders.list;
+				}
+			}
+		}
+
+		if (IsSavegameVersionBefore(157)) {
+			/* The road vehicle subtype was converted to a flag. */
+			RoadVehicle *rv;
+			FOR_ALL_ROADVEHICLES(rv) {
+				if (rv->subtype == 0) {
+					/* The road vehicle is at the front. */
+					rv->SetFrontEngine();
+				} else if (rv->subtype == 1) {
+					/* The road vehicle is an articulated part. */
+					rv->subtype = 0;
+					rv->SetArticulatedPart();
+				} else {
+					SlErrorCorrupt("Invalid road vehicle subtype");
+				}
+			}
+		}
+
+		if (IsSavegameVersionBefore(160)) {
+			/* In some old savegames there might be some "crap" stored. */
+			FOR_ALL_VEHICLES(v) {
+				if (!v->IsPrimaryVehicle() && v->type != VEH_DISASTER) {
+					v->current_order.Free();
+					v->unitnumber = 0;
+				}
 			}
 		}
 	}
@@ -314,22 +347,38 @@ void AfterLoadVehicles(bool part_of_load)
 	FOR_ALL_VEHICLES(v) {
 		assert(v->first != NULL);
 
-		if (v->type == VEH_TRAIN) {
-			Train *t = Train::From(v);
-			if (t->IsFrontEngine() || t->IsFreeWagon()) {
-				t->tcache.last_speed = t->cur_speed; // update displayed train speed
-				t->ConsistChanged(false);
+		switch (v->type) {
+			case VEH_TRAIN: {
+				Train *t = Train::From(v);
+				if (t->IsFrontEngine() || t->IsFreeWagon()) {
+					t->gcache.last_speed = t->cur_speed; // update displayed train speed
+					t->ConsistChanged(false);
+				}
+				break;
 			}
-		} else if (v->type == VEH_ROAD) {
-			RoadVehicle *rv = RoadVehicle::From(v);
-			if (rv->IsRoadVehFront()) {
-				RoadVehUpdateCache(rv);
+
+			case VEH_ROAD: {
+				RoadVehicle *rv = RoadVehicle::From(v);
+				if (rv->IsFrontEngine()) {
+					rv->gcache.last_speed = rv->cur_speed; // update displayed road vehicle speed
+					RoadVehUpdateCache(rv);
+					if (_settings_game.vehicle.roadveh_acceleration_model != AM_ORIGINAL) {
+						rv->CargoChanged();
+					}
+				}
+				break;
 			}
+
+			case VEH_SHIP:
+				Ship::From(v)->UpdateCache();
+				break;
+
+			default: break;
 		}
 	}
 
 	/* Stop non-front engines */
-	if (CheckSavegameVersion(112)) {
+	if (part_of_load && IsSavegameVersionBefore(112)) {
 		FOR_ALL_VEHICLES(v) {
 			if (v->type == VEH_TRAIN) {
 				Train *t = Train::From(v);
@@ -342,7 +391,7 @@ void AfterLoadVehicles(bool part_of_load)
 			}
 			/* trains weren't stopping gradually in old OTTD versions (and TTO/TTD)
 			 * other vehicle types didn't have zero speed while stopped (even in 'recent' OTTD versions) */
-			if ((v->vehstatus & VS_STOPPED) && (v->type != VEH_TRAIN || CheckSavegameVersionOldStyle(2, 1))) {
+			if ((v->vehstatus & VS_STOPPED) && (v->type != VEH_TRAIN || IsSavegameVersionBefore(2, 1))) {
 				v->cur_speed = 0;
 			}
 		}
@@ -354,8 +403,9 @@ void AfterLoadVehicles(bool part_of_load)
 				RoadVehicle *rv = RoadVehicle::From(v);
 				rv->roadtype = HasBit(EngInfo(v->First()->engine_type)->misc_flags, EF_ROAD_TRAM) ? ROADTYPE_TRAM : ROADTYPE_ROAD;
 				rv->compatible_roadtypes = RoadTypeToRoadTypes(rv->roadtype);
-			}
 				/* FALL THROUGH */
+			}
+
 			case VEH_TRAIN:
 			case VEH_SHIP:
 				v->cur_image = v->GetImage(v->direction);
@@ -428,7 +478,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		SLE_CONDNULL(5,                                                            0,  57),
 		     SLE_VAR(Vehicle, engine_type,           SLE_UINT16),
 
-		     SLE_VAR(Vehicle, max_speed,             SLE_UINT16),
+		SLE_CONDNULL(2,                                                            0,  151),
 		     SLE_VAR(Vehicle, cur_speed,             SLE_UINT16),
 		     SLE_VAR(Vehicle, subspeed,              SLE_UINT8),
 		     SLE_VAR(Vehicle, acceleration,          SLE_UINT8),
@@ -452,7 +502,8 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		     SLE_VAR(Vehicle, tick_counter,          SLE_UINT8),
 		 SLE_CONDVAR(Vehicle, running_ticks,         SLE_UINT8,                   88, SL_MAX_VERSION),
 
-		     SLE_VAR(Vehicle, cur_order_index,       SLE_UINT8),
+		     SLE_VAR(Vehicle, cur_implicit_order_index,  SLE_UINT8),
+		 SLE_CONDVAR(Vehicle, cur_real_order_index,  SLE_UINT8,                  158, SL_MAX_VERSION),
 		/* num_orders is now part of OrderList and is not saved but counted */
 		SLE_CONDNULL(1,                                                            0, 104),
 
@@ -522,8 +573,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		 SLE_CONDVAR(Vehicle, current_order_time,    SLE_UINT32,                  67, SL_MAX_VERSION),
 		 SLE_CONDVAR(Vehicle, lateness_counter,      SLE_INT32,                   67, SL_MAX_VERSION),
 
-		/* reserve extra space in savegame here. (currently 10 bytes) */
-		SLE_CONDNULL(10,                                                           2, SL_MAX_VERSION),
+		SLE_CONDNULL(10,                                                           2, 143), // old reserved space
 
 		     SLE_END()
 	};
@@ -544,8 +594,8 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		 SLE_CONDVAR(Train, wait_counter,        SLE_UINT16,                 136, SL_MAX_VERSION),
 
 		SLE_CONDNULL(2, 2, 19),
-		/* reserve extra space in savegame here. (currently 11 bytes) */
-		SLE_CONDNULL(11, 2, SL_MAX_VERSION),
+		 SLE_CONDVAR(Train, gv_flags,            SLE_UINT16,                 139, SL_MAX_VERSION),
+		SLE_CONDNULL(11, 2, 143), // old reserved space
 
 		     SLE_END()
 	};
@@ -562,10 +612,10 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		     SLE_VAR(RoadVehicle, reverse_ctr,          SLE_UINT8),
 
 		SLE_CONDNULL(2,                                                               6,  68),
+		 SLE_CONDVAR(RoadVehicle, gv_flags,             SLE_UINT16,                 139, SL_MAX_VERSION),
 		SLE_CONDNULL(4,                                                              69, 130),
 		SLE_CONDNULL(2,                                                               6, 130),
-		/* reserve extra space in savegame here. (currently 16 bytes) */
-		SLE_CONDNULL(16,                                                              2, SL_MAX_VERSION),
+		SLE_CONDNULL(16,                                                              2, 143), // old reserved space
 
 		     SLE_END()
 	};
@@ -575,8 +625,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		SLE_VEH_INCLUDE(),
 		     SLE_VAR(Ship, state, SLE_UINT8),
 
-		/* reserve extra space in savegame here. (currently 16 bytes) */
-		SLE_CONDNULL(16, 2, SL_MAX_VERSION),
+		SLE_CONDNULL(16, 2, 143), // old reserved space
 
 		     SLE_END()
 	};
@@ -598,8 +647,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 
 		 SLE_CONDVAR(Aircraft, turn_counter,          SLE_UINT8,                  136, SL_MAX_VERSION),
 
-		/* reserve extra space in savegame here. (currently 13 bytes) */
-		SLE_CONDNULL(13,                                                           2, SL_MAX_VERSION),
+		SLE_CONDNULL(13,                                                           2, 143), // old reserved space
 
 		     SLE_END()
 	};
@@ -618,7 +666,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		 SLE_CONDVAR(Vehicle, y_pos,                 SLE_INT32,                    6, SL_MAX_VERSION),
 		     SLE_VAR(Vehicle, z_pos,                 SLE_UINT8),
 
-		     SLE_VAR(Vehicle, cur_image,             SLE_UINT16),
+		     SLE_VAR(Vehicle, cur_image,             SLE_FILE_U16 | SLE_VAR_U32),
 		SLE_CONDNULL(5,                                                            0,  57),
 		     SLE_VAR(Vehicle, progress,              SLE_UINT8),
 		     SLE_VAR(Vehicle, vehstatus,             SLE_UINT8),
@@ -628,8 +676,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 
 		 SLE_CONDVAR(Vehicle, spritenum,             SLE_UINT8,                    2, SL_MAX_VERSION),
 
-		/* reserve extra space in savegame here. (currently 15 bytes) */
-		SLE_CONDNULL(15,                                                           2, SL_MAX_VERSION),
+		SLE_CONDNULL(15,                                                           2, 143), // old reserved space
 
 		     SLE_END()
 	};
@@ -658,7 +705,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		 SLE_CONDVAR(Vehicle, current_order.dest,    SLE_FILE_U8 | SLE_VAR_U16,    0,   4),
 		 SLE_CONDVAR(Vehicle, current_order.dest,    SLE_UINT16,                   5, SL_MAX_VERSION),
 
-		     SLE_VAR(Vehicle, cur_image,             SLE_UINT16),
+		     SLE_VAR(Vehicle, cur_image,             SLE_FILE_U16 | SLE_VAR_U32),
 		 SLE_CONDVAR(Vehicle, age,                   SLE_FILE_U16 | SLE_VAR_I32,   0,  30),
 		 SLE_CONDVAR(Vehicle, age,                   SLE_INT32,                   31, SL_MAX_VERSION),
 		     SLE_VAR(Vehicle, tick_counter,          SLE_UINT8),
@@ -666,8 +713,7 @@ const SaveLoad *GetVehicleDescription(VehicleType vt)
 		     SLE_VAR(DisasterVehicle, image_override,            SLE_UINT16),
 		     SLE_VAR(DisasterVehicle, big_ufo_destroyer_target,  SLE_UINT16),
 
-		/* reserve extra space in savegame here. (currently 16 bytes) */
-		SLE_CONDNULL(16,                                                           2, SL_MAX_VERSION),
+		SLE_CONDNULL(16,                                                           2, 143), // old reserved space
 
 		     SLE_END()
 	};
@@ -716,22 +762,23 @@ void Load_VEHS()
 			case VEH_EFFECT:   v = new (index) EffectVehicle();   break;
 			case VEH_DISASTER: v = new (index) DisasterVehicle(); break;
 			case VEH_INVALID: // Savegame shouldn't contain invalid vehicles
-			default: NOT_REACHED();
+			default: SlErrorCorrupt("Invalid vehicle type");
 		}
 
 		SlObject(v, GetVehicleDescription(vtype));
 
-		if (_cargo_count != 0 && IsCompanyBuildableVehicleType(v)) {
+		if (_cargo_count != 0 && IsCompanyBuildableVehicleType(v) && CargoPacket::CanAllocateItem()) {
 			/* Don't construct the packet with station here, because that'll fail with old savegames */
 			CargoPacket *cp = new CargoPacket(_cargo_count, _cargo_days, _cargo_source, _cargo_source_xy, _cargo_loaded_at_xy, _cargo_feeder_share);
 			v->cargo.Append(cp);
 		}
 
 		/* Old savegames used 'last_station_visited = 0xFF' */
-		if (CheckSavegameVersion(5) && v->last_station_visited == 0xFF)
+		if (IsSavegameVersionBefore(5) && v->last_station_visited == 0xFF) {
 			v->last_station_visited = INVALID_STATION;
+		}
 
-		if (CheckSavegameVersion(5)) {
+		if (IsSavegameVersionBefore(5)) {
 			/* Convert the current_order.type (which is a mix of type and flags, because
 			 *  in those versions, they both were 4 bits big) to type and flags */
 			v->current_order.flags = GB(v->current_order.type, 4, 4);
@@ -739,7 +786,7 @@ void Load_VEHS()
 		}
 
 		/* Advanced vehicle lists got added */
-		if (CheckSavegameVersion(60)) v->group_id = DEFAULT_GROUP;
+		if (IsSavegameVersionBefore(60)) v->group_id = DEFAULT_GROUP;
 	}
 }
 
@@ -752,5 +799,5 @@ static void Ptrs_VEHS()
 }
 
 extern const ChunkHandler _veh_chunk_handlers[] = {
-	{ 'VEHS', Save_VEHS, Load_VEHS, Ptrs_VEHS, CH_SPARSE_ARRAY | CH_LAST},
+	{ 'VEHS', Save_VEHS, Load_VEHS, Ptrs_VEHS, NULL, CH_SPARSE_ARRAY | CH_LAST},
 };
