@@ -1,4 +1,4 @@
-/* $Id: squirrel_helper.hpp 18862 2010-01-18 15:41:38Z rubidium $ */
+/* $Id: squirrel_helper.hpp 22886 2011-09-03 18:56:34Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,7 +12,7 @@
 #ifndef SQUIRREL_HELPER_HPP
 #define SQUIRREL_HELPER_HPP
 
-#include <squirrel.h>
+#include "squirrel.hpp"
 #include "../core/math_func.hpp"
 #include "../core/smallvec_type.hpp"
 #include "../economy_type.h"
@@ -118,6 +118,9 @@ namespace SQConvert {
 
 	template <> inline Array      *GetParam(ForceType<Array *>,      HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr)
 	{
+		/* Sanity check of the size. */
+		if (sq_getsize(vm, index) > UINT16_MAX) throw sq_throwerror(vm, _SC("an array used as parameter to a function is too large"));
+
 		SQObject obj;
 		sq_getstackobj(vm, index, &obj);
 		sq_pushobject(vm, obj);
@@ -883,6 +886,6 @@ namespace SQConvert {
 		}
 	}
 
-}; // namespace SQConvert
+} // namespace SQConvert
 
 #endif /* SQUIRREL_HELPER_HPP */
