@@ -1,4 +1,4 @@
-/* $Id: settings_gui.cpp 19555 2010-04-03 20:19:26Z rubidium $ */
+/* $Id: settings_gui.cpp 22650 2011-07-10 20:17:56Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -24,7 +24,6 @@
 #include "string_func.h"
 #include "widgets/dropdown_type.h"
 #include "widgets/dropdown_func.h"
-#include "openttd.h"
 #include "highscore.h"
 #include "base_media_base.h"
 #include "company_base.h"
@@ -32,10 +31,11 @@
 #include "viewport_func.h"
 #include "core/geometry_func.hpp"
 #include "ai/ai.hpp"
-#include <map>
+#include "language.h"
 
 #include "table/sprites.h"
 #include "table/strings.h"
+#include <map>
 
 static const StringID _units_dropdown[] = {
 	STR_GAME_OPTIONS_MEASURING_UNITS_IMPERIAL,
@@ -59,6 +59,12 @@ static const StringID _autosave_dropdown[] = {
 	INVALID_STRING_ID,
 };
 
+/**
+ * Fill a static array with consecutive stringIDs for use with a drop down.
+ * @param base First stringID.
+ * @param num  Number of stringIDs (must be at most 32).
+ * *return Pointer to the static buffer with stringIDs.
+ */
 static StringID *BuildDynamicDropdown(StringID base, int num)
 {
 	static StringID buf[32 + 1];
@@ -68,10 +74,11 @@ static StringID *BuildDynamicDropdown(StringID base, int num)
 	return buf;
 }
 
-int _nb_orig_names = SPECSTR_TOWNNAME_LAST - SPECSTR_TOWNNAME_START + 1;
-static StringID *_grf_names = NULL;
-static int _nb_grf_names = 0;
+int _nb_orig_names = SPECSTR_TOWNNAME_LAST - SPECSTR_TOWNNAME_START + 1; ///< Number of original town names.
+static StringID *_grf_names = NULL; ///< Pointer to town names defined by NewGRFs.
+static int _nb_grf_names = 0;       ///< Number of town names defined by NewGRFs.
 
+/** Allocate memory for the NewGRF town names. */
 void InitGRFTownGeneratorNames()
 {
 	free(_grf_names);
@@ -80,6 +87,11 @@ void InitGRFTownGeneratorNames()
 	for (StringID *s = _grf_names; *s != INVALID_STRING_ID; s++) _nb_grf_names++;
 }
 
+/**
+ * Get a town name.
+ * @param town_name Number of the wanted town name.
+ * @return Name of the town as string ID.
+ */
 static inline StringID TownName(int town_name)
 {
 	if (town_name < _nb_orig_names) return STR_GAME_OPTIONS_TOWN_NAME_ORIGINAL_ENGLISH + town_name;
@@ -88,6 +100,10 @@ static inline StringID TownName(int town_name)
 	return STR_UNDEFINED;
 }
 
+/**
+ * Get index of the current screen resolution.
+ * @return Index of the current screen resolution if it is a known resolution, #_num_resolutions otherwise.
+ */
 static int GetCurRes()
 {
 	int i;
@@ -169,7 +185,7 @@ struct GameOptionsWindow : Window {
 
 	GameOptionsWindow(const WindowDesc *desc) : Window()
 	{
-		this->opt = (_game_mode == GM_MENU) ? &_settings_newgame : &_settings_game;
+		this->opt = &GetGameSettings();
 		this->reload = false;
 
 		this->InitNested(desc);
@@ -190,8 +206,8 @@ struct GameOptionsWindow : Window {
 			case GOW_ROADSIDE_DROPDOWN:   SetDParam(0, STR_GAME_OPTIONS_ROAD_VEHICLES_DROPDOWN_LEFT + this->opt->vehicle.road_side); break;
 			case GOW_TOWNNAME_DROPDOWN:   SetDParam(0, TownName(this->opt->game_creation.town_name)); break;
 			case GOW_AUTOSAVE_DROPDOWN:   SetDParam(0, _autosave_dropdown[_settings_client.gui.autosave]); break;
-			case GOW_LANG_DROPDOWN:       SetDParam(0, SPECSTR_LANGUAGE_START + _dynlang.curr); break;
-			case GOW_RESOLUTION_DROPDOWN: SetDParam(0, GetCurRes() == _num_resolutions ? STR_RES_OTHER : SPECSTR_RESOLUTION_START + GetCurRes()); break;
+			case GOW_LANG_DROPDOWN:       SetDParamStr(0, _current_language->own_name); break;
+			case GOW_RESOLUTION_DROPDOWN: SetDParam(0, GetCurRes() == _num_resolutions ? STR_GAME_OPTIONS_RESOLUTION_OTHER : SPECSTR_RESOLUTION_START + GetCurRes()); break;
 			case GOW_SCREENSHOT_DROPDOWN: SetDParam(0, SPECSTR_SCREENSHOT_START + _cur_screenshot_format); break;
 			case GOW_BASE_GRF_DROPDOWN:   SetDParamStr(0, BaseGraphics::GetUsedSet()->name); break;
 			case GOW_BASE_GRF_STATUS:     SetDParam(0, BaseGraphics::GetUsedSet()->GetNumInvalid()); break;
@@ -201,11 +217,6 @@ struct GameOptionsWindow : Window {
 		}
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		switch (widget) {
@@ -299,7 +310,8 @@ struct GameOptionsWindow : Window {
 				}
 
 				ShowDropDownMenu(this, _driveside_dropdown, this->opt->vehicle.road_side, GOW_ROADSIDE_DROPDOWN, i, 0);
-			} break;
+				break;
+			}
 
 			case GOW_TOWNNAME_DROPDOWN: // Setup townname dropdown
 				ShowTownnameDropdown(this, this->opt->game_creation.town_name);
@@ -314,15 +326,20 @@ struct GameOptionsWindow : Window {
 
 				/* Sort language names */
 				LangList langs;
-				for (int i = 0; i < _dynlang.num; i++) langs[SPECSTR_LANGUAGE_START + i] = i;
+				int current_lang = 0;
+				for (int i = 0; i < (int)_languages.Length(); i++) {
+					if (&_languages[i] == _current_language) current_lang = i;
+					langs[SPECSTR_LANGUAGE_START + i] = i;
+				}
 
 				DropDownList *list = new DropDownList();
 				for (LangList::iterator it = langs.begin(); it != langs.end(); it++) {
 					list->push_back(new DropDownListStringItem((*it).first, (*it).second, false));
 				}
 
-				ShowDropDownList(this, list, _dynlang.curr, GOW_LANG_DROPDOWN);
-			} break;
+				ShowDropDownList(this, list, current_lang, GOW_LANG_DROPDOWN);
+				break;
+			}
 
 			case GOW_RESOLUTION_DROPDOWN: // Setup resolution dropdown
 				ShowDropDownMenu(this, BuildDynamicDropdown(SPECSTR_RESOLUTION_START, _num_resolutions), GetCurRes(), GOW_RESOLUTION_DROPDOWN, 0, 0);
@@ -331,7 +348,7 @@ struct GameOptionsWindow : Window {
 			case GOW_FULLSCREEN_BUTTON: // Click fullscreen on/off
 				/* try to toggle full-screen on/off */
 				if (!ToggleFullScreen(!_fullscreen)) {
-					ShowErrorMessage(STR_ERROR_FULLSCREEN_FAILED, INVALID_STRING_ID, 0, 0);
+					ShowErrorMessage(STR_ERROR_FULLSCREEN_FAILED, INVALID_STRING_ID, WL_ERROR);
 				}
 				this->SetWidgetLoweredState(GOW_FULLSCREEN_BUTTON, _fullscreen);
 				this->SetDirty();
@@ -381,7 +398,7 @@ struct GameOptionsWindow : Window {
 			case GOW_CURRENCY_DROPDOWN: // Currency
 				if (index == CUSTOM_CURRENCY_ID) ShowCustCurrency();
 				this->opt->locale.currency = index;
-				MarkWholeScreenDirty();
+				ReInitAllWindows();
 				break;
 
 			case GOW_DISTANCE_DROPDOWN: // Measuring units
@@ -411,7 +428,8 @@ struct GameOptionsWindow : Window {
 				break;
 
 			case GOW_LANG_DROPDOWN: // Change interface language
-				ReadLanguagePack(index);
+				ReadLanguagePack(&_languages[index]);
+				DeleteWindowByClass(WC_QUERY_STRING);
 				CheckForMissingGlyphsInLoadedLanguagePack();
 				UpdateAllVirtCoords();
 				ReInitAllWindows();
@@ -442,8 +460,14 @@ struct GameOptionsWindow : Window {
 		}
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data. @see GameOptionsInvalidationData
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
+		if (!gui_scope) return;
 		this->SetWidgetLoweredState(GOW_FULLSCREEN_BUTTON, _fullscreen);
 
 		bool missing_files = BaseGraphics::GetUsedSet()->GetNumMissing() == 0;
@@ -451,6 +475,8 @@ struct GameOptionsWindow : Window {
 
 		missing_files = BaseMusic::GetUsedSet()->GetNumInvalid() == 0;
 		this->GetWidget<NWidgetCore>(GOW_BASE_MUSIC_STATUS)->SetDataTip(missing_files ? STR_EMPTY : STR_GAME_OPTIONS_BASE_MUSIC_STATUS, STR_NULL);
+
+		InitGRFTownGeneratorNames();
 	}
 };
 
@@ -488,7 +514,7 @@ static const NWidgetPart _nested_game_options_widgets[] = {
 					NWidget(WWT_DROPDOWN, COLOUR_GREY, GOW_TOWNNAME_DROPDOWN), SetMinimalSize(150, 12), SetDataTip(STR_BLACK_STRING, STR_GAME_OPTIONS_TOWN_NAMES_DROPDOWN_TOOLTIP), SetFill(1, 0),
 				EndContainer(),
 				NWidget(WWT_FRAME, COLOUR_GREY), SetDataTip(STR_GAME_OPTIONS_LANGUAGE, STR_NULL),
-					NWidget(WWT_DROPDOWN, COLOUR_GREY, GOW_LANG_DROPDOWN), SetMinimalSize(150, 12), SetDataTip(STR_BLACK_STRING, STR_GAME_OPTIONS_LANGUAGE_TOOLTIP), SetFill(1, 0),
+					NWidget(WWT_DROPDOWN, COLOUR_GREY, GOW_LANG_DROPDOWN), SetMinimalSize(150, 12), SetDataTip(STR_BLACK_RAW_STRING, STR_GAME_OPTIONS_LANGUAGE_TOOLTIP), SetFill(1, 0),
 				EndContainer(),
 				NWidget(WWT_FRAME, COLOUR_GREY), SetDataTip(STR_GAME_OPTIONS_SCREENSHOT_FORMAT, STR_NULL),
 					NWidget(WWT_DROPDOWN, COLOUR_GREY, GOW_SCREENSHOT_DROPDOWN), SetMinimalSize(150, 12), SetDataTip(STR_BLACK_STRING, STR_GAME_OPTIONS_SCREENSHOT_FORMAT_TOOLTIP), SetFill(1, 0),
@@ -530,7 +556,7 @@ static const WindowDesc _game_options_desc(
 	_nested_game_options_widgets, lengthof(_nested_game_options_widgets)
 );
 
-
+/** Open the game options window. */
 void ShowGameOptions()
 {
 	DeleteWindowById(WC_GAME_OPTIONS, 0);
@@ -570,9 +596,6 @@ public:
 	{
 		this->InitNested(desc);
 
-		/* Copy current settings (ingame or in intro) to temporary holding place
-		 * change that when setting stuff, copy back on clicking 'OK' */
-		this->opt_mod_temp = (_game_mode == GM_MENU) ? _settings_newgame : _settings_game;
 		/* Setup disabled buttons when creating window
 		 * disable all other difficulty buttons during gameplay except for 'custom' */
 		this->SetWidgetsDisabledState(_game_mode != GM_MENU,
@@ -583,8 +606,9 @@ public:
 			WIDGET_LIST_END);
 		this->SetWidgetDisabledState(GDW_HIGHSCORE, _game_mode == GM_EDITOR || _networking); // highscore chart in multiplayer
 		this->SetWidgetDisabledState(GDW_ACCEPT, _networking && !_network_server); // Save-button in multiplayer (and if client)
-		this->LowerWidget(GDW_LVL_EASY + this->opt_mod_temp.difficulty.diff_level);
-		this->OnInvalidateData();
+
+		/* Read data */
+		this->OnInvalidateData(GOID_DIFFICULTY_CHANGED);
 	}
 
 	virtual void SetStringParameters(int widget) const
@@ -620,11 +644,6 @@ public:
 		}
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		if (widget >= GDW_OPTIONS_START) {
@@ -660,7 +679,7 @@ public:
 					AI::GetInfoList()->size() == 0 &&
 #endif /* ENABLE_AI */
 					this->opt_mod_temp.difficulty.max_no_competitors != 0) {
-				ShowErrorMessage(STR_WARNING_NO_SUITABLE_AI, INVALID_STRING_ID, 0, 0, true);
+				ShowErrorMessage(STR_WARNING_NO_SUITABLE_AI, INVALID_STRING_ID, WL_CRITICAL);
 			}
 			return;
 		}
@@ -682,7 +701,7 @@ public:
 				break;
 
 			case GDW_ACCEPT: { // Save button - save changes
-				GameSettings *opt_ptr = (_game_mode == GM_MENU) ? &_settings_newgame : &_settings_game;
+				GameSettings *opt_ptr = &GetGameSettings();
 
 				uint i;
 				GetSettingFromName("difficulty.diff_level", &i);
@@ -711,8 +730,25 @@ public:
 		}
 	}
 
-	virtual void OnInvalidateData(int data = 0)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data. @see GameOptionsInvalidationData
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
+		if (!gui_scope) return;
+
+		if (data == GOID_DIFFICULTY_CHANGED) {
+			/* Window was created or settings were changed on server. Reread everything. */
+
+			/* Copy current settings (ingame or in intro) to temporary holding place
+			 * change that when setting stuff, copy back on clicking 'OK' */
+			this->opt_mod_temp = GetGameSettings();
+
+			this->LowerWidget(GDW_LVL_EASY + this->opt_mod_temp.difficulty.diff_level);
+		}
+
 		uint i;
 		const SettingDesc *sd = GetSettingFromName("difficulty.max_no_competitors", &i);
 		for (i = 0; i < GAME_DIFFICULTY_NUM; i++, sd++) {
@@ -744,11 +780,11 @@ static NWidgetBase *MakeDifficultyOptionsWidgets(int *biggest_index)
 		NWidgetHorizontal *hor = new NWidgetHorizontal;
 
 		/* [<] button. */
-		NWidgetLeaf *leaf = new NWidgetLeaf(NWID_BUTTON_ARROW, COLOUR_YELLOW, widnum, AWV_DECREASE, STR_TOOLTIP_HSCROLL_BAR_SCROLLS_LIST);
+		NWidgetLeaf *leaf = new NWidgetLeaf(WWT_PUSHARROWBTN, COLOUR_YELLOW, widnum, AWV_DECREASE, STR_TOOLTIP_HSCROLL_BAR_SCROLLS_LIST);
 		hor->Add(leaf);
 
 		/* [>] button. */
-		leaf = new NWidgetLeaf(NWID_BUTTON_ARROW, COLOUR_YELLOW, widnum + 1, AWV_INCREASE, STR_TOOLTIP_HSCROLL_BAR_SCROLLS_LIST);
+		leaf = new NWidgetLeaf(WWT_PUSHARROWBTN, COLOUR_YELLOW, widnum + 1, AWV_INCREASE, STR_TOOLTIP_HSCROLL_BAR_SCROLLS_LIST);
 		hor->Add(leaf);
 
 		/* Some spacing between the text and the description */
@@ -812,6 +848,7 @@ static const WindowDesc _game_difficulty_desc(
 	_nested_game_difficulty_widgets, lengthof(_nested_game_difficulty_widgets)
 );
 
+/** Open the game-difficulty window. */
 void ShowGameDifficulty()
 {
 	DeleteWindowById(WC_GAME_OPTIONS, 0);
@@ -1044,7 +1081,7 @@ uint SettingEntry::Draw(GameSettings *settings_ptr, int left, int right, int bas
 {
 	if (cur_row >= max_row) return cur_row;
 
-	bool rtl = _dynlang.text_dir == TD_RTL;
+	bool rtl = _current_text_dir == TD_RTL;
 	int offset = rtl ? -4 : 4;
 	int level_width = rtl ? -LEVEL_WIDTH : LEVEL_WIDTH;
 
@@ -1124,7 +1161,7 @@ void SettingEntry::DrawSetting(GameSettings *settings_ptr, const SettingDesc *sd
 	bool editable = true;
 	bool disabled = false;
 
-	bool rtl = _dynlang.text_dir == TD_RTL;
+	bool rtl = _current_text_dir == TD_RTL;
 	uint buttons_left = rtl ? right - 19 : left;
 	uint text_left  = left + (rtl ? 0 : 25);
 	uint text_right = right - (rtl ? 25 : 0);
@@ -1138,7 +1175,7 @@ void SettingEntry::DrawSetting(GameSettings *settings_ptr, const SettingDesc *sd
 	if (sdb->cmd == SDT_BOOLX) {
 		static const Colours _bool_ctabs[2][2] = {{COLOUR_CREAM, COLOUR_RED}, {COLOUR_DARK_GREEN, COLOUR_GREEN}};
 		/* Draw checkbox for boolean-value either on/off */
-		bool on = (*(bool*)var);
+		bool on = ReadValue(var, sd->save.conv) != 0;
 
 		DrawFrameRect(buttons_left, button_y, buttons_left + 19, button_y + 8, _bool_ctabs[!!on][!!editable], on ? FR_LOWERED : FR_NONE);
 		SetDParam(0, on ? STR_CONFIG_SETTING_ON : STR_CONFIG_SETTING_OFF);
@@ -1250,8 +1287,6 @@ uint SettingsPage::Draw(GameSettings *settings_ptr, int left, int right, int bas
 
 
 static SettingEntry _settings_ui_display[] = {
-	SettingEntry("gui.vehicle_speed"),
-	SettingEntry("gui.status_long_date"),
 	SettingEntry("gui.date_format_in_default_names"),
 	SettingEntry("gui.population_in_label"),
 	SettingEntry("gui.measure_tooltip"),
@@ -1291,14 +1326,16 @@ static SettingEntry _settings_ui[] = {
 	SettingEntry(&_settings_ui_interaction_page, STR_CONFIG_SETTING_INTERACTION),
 	SettingEntry("gui.show_finances"),
 	SettingEntry("gui.errmsg_duration"),
+	SettingEntry("gui.hover_delay"),
 	SettingEntry("gui.toolbar_pos"),
+	SettingEntry("gui.statusbar_pos"),
 	SettingEntry("gui.pause_on_newgame"),
 	SettingEntry("gui.advanced_vehicle_list"),
 	SettingEntry("gui.timetable_in_ticks"),
 	SettingEntry("gui.timetable_arrival_departure"),
 	SettingEntry("gui.quick_goto"),
 	SettingEntry("gui.default_rail_type"),
-	SettingEntry("gui.always_build_infrastructure"),
+	SettingEntry("gui.disable_unsuitable_building"),
 	SettingEntry("gui.persistent_buildingtools"),
 	SettingEntry("gui.coloured_news_year"),
 };
@@ -1321,10 +1358,12 @@ static SettingEntry _settings_construction[] = {
 	SettingEntry("construction.build_on_slopes"),
 	SettingEntry("construction.autoslope"),
 	SettingEntry("construction.extra_dynamite"),
-	SettingEntry("construction.longbridges"),
+	SettingEntry("construction.max_bridge_length"),
+	SettingEntry("construction.max_tunnel_length"),
 	SettingEntry("station.never_expire_airports"),
 	SettingEntry("construction.freeform_edges"),
 	SettingEntry("construction.extra_tree_placement"),
+	SettingEntry("construction.command_pause_level"),
 };
 /** Construction sub-page */
 static SettingsPage _settings_construction_page = {_settings_construction, lengthof(_settings_construction)};
@@ -1339,8 +1378,6 @@ static SettingsPage _settings_stations_cargo_page = {_settings_stations_cargo, l
 
 static SettingEntry _settings_stations[] = {
 	SettingEntry(&_settings_stations_cargo_page, STR_CONFIG_SETTING_STATIONS_CARGOHANDLING),
-	SettingEntry("station.join_stations"),
-	SettingEntry("station.nonuniform_stations"),
 	SettingEntry("station.adjacent_stations"),
 	SettingEntry("station.distant_join_stations"),
 	SettingEntry("station.station_spread"),
@@ -1355,8 +1392,10 @@ static SettingsPage _settings_stations_page = {_settings_stations, lengthof(_set
 static SettingEntry _settings_economy_towns[] = {
 	SettingEntry("economy.bribe"),
 	SettingEntry("economy.exclusive_rights"),
+	SettingEntry("economy.fund_roads"),
 	SettingEntry("economy.town_layout"),
 	SettingEntry("economy.allow_town_roads"),
+	SettingEntry("economy.allow_town_level_crossings"),
 	SettingEntry("economy.found_town"),
 	SettingEntry("economy.mod_road_rebuild"),
 	SettingEntry("economy.town_growth_rate"),
@@ -1368,6 +1407,7 @@ static SettingsPage _settings_economy_towns_page = {_settings_economy_towns, len
 
 static SettingEntry _settings_economy_industries[] = {
 	SettingEntry("construction.raw_industry_construction"),
+	SettingEntry("construction.industry_platform"),
 	SettingEntry("economy.multiple_industry_per_town"),
 	SettingEntry("game_creation.oil_refinery_limit"),
 };
@@ -1434,10 +1474,10 @@ static SettingEntry _settings_vehicles_servicing[] = {
 static SettingsPage _settings_vehicles_servicing_page = {_settings_vehicles_servicing, lengthof(_settings_vehicles_servicing)};
 
 static SettingEntry _settings_vehicles_trains[] = {
+	SettingEntry("pf.reverse_at_signals"),
 	SettingEntry("vehicle.train_acceleration_model"),
 	SettingEntry("vehicle.train_slope_steepness"),
-	SettingEntry("vehicle.mammoth_trains"),
-	SettingEntry("gui.lost_train_warn"),
+	SettingEntry("vehicle.max_train_length"),
 	SettingEntry("vehicle.wagon_speed_limits"),
 	SettingEntry("vehicle.disable_elrails"),
 	SettingEntry("vehicle.freight_trains"),
@@ -1451,10 +1491,10 @@ static SettingEntry _settings_vehicles[] = {
 	SettingEntry(&_settings_vehicles_autorenew_page, STR_CONFIG_SETTING_VEHICLES_AUTORENEW),
 	SettingEntry(&_settings_vehicles_servicing_page, STR_CONFIG_SETTING_VEHICLES_SERVICING),
 	SettingEntry(&_settings_vehicles_trains_page, STR_CONFIG_SETTING_VEHICLES_TRAINS),
-	SettingEntry("order.gotodepot"),
 	SettingEntry("gui.new_nonstop"),
 	SettingEntry("gui.order_review_system"),
 	SettingEntry("gui.vehicle_income_warn"),
+	SettingEntry("gui.lost_vehicle_warn"),
 	SettingEntry("vehicle.never_expire_vehicles"),
 	SettingEntry("vehicle.max_trains"),
 	SettingEntry("vehicle.max_roadveh"),
@@ -1462,8 +1502,10 @@ static SettingEntry _settings_vehicles[] = {
 	SettingEntry("vehicle.max_ships"),
 	SettingEntry("vehicle.plane_speed"),
 	SettingEntry("vehicle.plane_crashes"),
-	SettingEntry("order.timetabling"),
 	SettingEntry("vehicle.dynamic_engines"),
+	SettingEntry("vehicle.roadveh_acceleration_model"),
+	SettingEntry("vehicle.roadveh_slope_steepness"),
+	SettingEntry("vehicle.smoke_amount"),
 };
 /** Vehicles sub-page */
 static SettingsPage _settings_vehicles_page = {_settings_vehicles, lengthof(_settings_vehicles)};
@@ -1497,11 +1539,13 @@ struct GameSettingsWindow : Window {
 	SettingEntry *valuewindow_entry; ///< If non-NULL, pointer to setting for which a value-entering window has been opened
 	SettingEntry *clicked_entry; ///< If non-NULL, pointer to a clicked numeric setting (with a depressed left or right button)
 
+	Scrollbar *vscroll;
+
 	GameSettingsWindow(const WindowDesc *desc) : Window()
 	{
 		static bool first_time = true;
 
-		settings_ptr = (_game_mode == GM_MENU) ? &_settings_newgame : &_settings_game;
+		settings_ptr = &GetGameSettings();
 
 		/* Build up the dynamic settings-array only once per OpenTTD session */
 		if (first_time) {
@@ -1514,9 +1558,11 @@ struct GameSettingsWindow : Window {
 		this->valuewindow_entry = NULL; // No setting entry for which a entry window is opened
 		this->clicked_entry = NULL; // No numeric setting buttons are depressed
 
-		this->InitNested(desc, 0);
+		this->CreateNestedTree(desc);
+		this->vscroll = this->GetScrollbar(SETTINGSEL_SCROLLBAR);
+		this->FinishInitNested(desc, 0);
 
-		this->vscroll.SetCount(_settings_main_page.Length());
+		this->vscroll->SetCount(_settings_main_page.Length());
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
@@ -1534,36 +1580,28 @@ struct GameSettingsWindow : Window {
 		if (widget != SETTINGSEL_OPTIONSPANEL) return;
 
 		_settings_main_page.Draw(settings_ptr, r.left + SETTINGTREE_LEFT_OFFSET, r.right - SETTINGTREE_RIGHT_OFFSET, r.top + SETTINGTREE_TOP_OFFSET,
-				this->vscroll.GetPosition(), this->vscroll.GetPosition() + this->vscroll.GetCapacity());
-	}
-
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
+				this->vscroll->GetPosition(), this->vscroll->GetPosition() + this->vscroll->GetCapacity());
 	}
 
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		if (widget != SETTINGSEL_OPTIONSPANEL) return;
 
-		int y = pt.y - this->GetWidget<NWidgetBase>(widget)->pos_y - SETTINGTREE_TOP_OFFSET;  // Shift y coordinate
-		if (y < 0) return;  // Clicked above first entry
-
-		byte btn = this->vscroll.GetPosition() + y / this->resize.step_height;  // Compute which setting is selected
-		if (y % this->resize.step_height > this->resize.step_height - 2) return;  // Clicked too low at the setting
+		uint btn = this->vscroll->GetScrolledRowFromWidget(pt.y, this, SETTINGSEL_OPTIONSPANEL, SETTINGTREE_TOP_OFFSET - 1);
+		if (btn == INT_MAX) return;
 
 		uint cur_row = 0;
 		SettingEntry *pe = _settings_main_page.FindEntry(btn, &cur_row);
 
 		if (pe == NULL) return;  // Clicked below the last setting of the page
 
-		int x = (_dynlang.text_dir == TD_RTL ? this->width - pt.x : pt.x) - SETTINGTREE_LEFT_OFFSET - (pe->level + 1) * LEVEL_WIDTH;  // Shift x coordinate
+		int x = (_current_text_dir == TD_RTL ? this->width - pt.x : pt.x) - SETTINGTREE_LEFT_OFFSET - (pe->level + 1) * LEVEL_WIDTH;  // Shift x coordinate
 		if (x < 0) return;  // Clicked left of the entry
 
 		if ((pe->flags & SEF_KIND_MASK) == SEF_SUBTREE_KIND) {
 			pe->d.sub.folded = !pe->d.sub.folded; // Flip 'folded'-ness of the sub-page
 
-			this->vscroll.SetCount(_settings_main_page.Length());
+			this->vscroll->SetCount(_settings_main_page.Length());
 			this->SetDirty();
 			return;
 		}
@@ -1622,11 +1660,12 @@ struct GameSettingsWindow : Window {
 							this->clicked_entry->SetButtons(0);
 						}
 						this->clicked_entry = pe;
-						this->clicked_entry->SetButtons((x >= 10) != (_dynlang.text_dir == TD_RTL) ? SEF_RIGHT_DEPRESSED : SEF_LEFT_DEPRESSED);
+						this->clicked_entry->SetButtons((x >= 10) != (_current_text_dir == TD_RTL) ? SEF_RIGHT_DEPRESSED : SEF_LEFT_DEPRESSED);
 						this->flags4 |= WF_TIMEOUT_BEGIN;
 						_left_button_clicked = false;
 					}
-				} break;
+					break;
+				}
 
 				default: NOT_REACHED();
 			}
@@ -1690,7 +1729,7 @@ struct GameSettingsWindow : Window {
 
 	virtual void OnResize()
 	{
-		this->vscroll.SetCapacityFromWidget(this, SETTINGSEL_OPTIONSPANEL, SETTINGTREE_TOP_OFFSET + SETTINGTREE_BOTTOM_OFFSET);
+		this->vscroll->SetCapacityFromWidget(this, SETTINGSEL_OPTIONSPANEL, SETTINGTREE_TOP_OFFSET + SETTINGTREE_BOTTOM_OFFSET);
 	}
 };
 
@@ -1702,9 +1741,9 @@ static const NWidgetPart _nested_settings_selection_widgets[] = {
 		NWidget(WWT_CAPTION, COLOUR_MAUVE), SetDataTip(STR_CONFIG_SETTING_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_MAUVE, SETTINGSEL_OPTIONSPANEL), SetMinimalSize(400, 174), EndContainer(),
+		NWidget(WWT_PANEL, COLOUR_MAUVE, SETTINGSEL_OPTIONSPANEL), SetMinimalSize(400, 174), SetScrollbar(SETTINGSEL_SCROLLBAR), EndContainer(),
 		NWidget(NWID_VERTICAL),
-			NWidget(WWT_SCROLLBAR, COLOUR_MAUVE, SETTINGSEL_SCROLLBAR),
+			NWidget(NWID_VSCROLLBAR, COLOUR_MAUVE, SETTINGSEL_SCROLLBAR),
 			NWidget(WWT_RESIZEBOX, COLOUR_MAUVE),
 		EndContainer(),
 	EndContainer(),
@@ -1717,6 +1756,7 @@ static const WindowDesc _settings_selection_desc(
 	_nested_settings_selection_widgets, lengthof(_nested_settings_selection_widgets)
 );
 
+/** Open advanced settings window. */
 void ShowGameSettings()
 {
 	DeleteWindowById(WC_GAME_OPTIONS, 0);
@@ -1743,7 +1783,7 @@ void DrawArrowButtons(int x, int y, Colours button_colour, byte state, bool clic
 	DrawSprite(SPR_ARROW_RIGHT, PAL_NONE, x + WD_IMGBTN_LEFT + 10, y + WD_IMGBTN_TOP);
 
 	/* Grey out the buttons that aren't clickable */
-	bool rtl = _dynlang.text_dir == TD_RTL;
+	bool rtl = _current_text_dir == TD_RTL;
 	if (rtl ? !clickable_right : !clickable_left) {
 		GfxFillRect(x +  1, y + 1, x +  1 + 8, y + 8, colour, FILLRECT_CHECKER);
 	}
@@ -1824,11 +1864,6 @@ struct CustomCurrencyWindow : Window {
 		}
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		int line = 0;
@@ -1957,8 +1992,8 @@ static const NWidgetPart _nested_cust_currency_widgets[] = {
 	NWidget(WWT_PANEL, COLOUR_GREY),
 		NWidget(NWID_VERTICAL, NC_EQUALSIZE), SetPIP(7, 3, 0),
 			NWidget(NWID_HORIZONTAL), SetPIP(10, 0, 5),
-				NWidget(NWID_BUTTON_ARROW, COLOUR_YELLOW, CUSTCURR_RATE_DOWN), SetDataTip(AWV_DECREASE, STR_CURRENCY_DECREASE_EXCHANGE_RATE_TOOLTIP),
-				NWidget(NWID_BUTTON_ARROW, COLOUR_YELLOW, CUSTCURR_RATE_UP), SetDataTip(AWV_INCREASE, STR_CURRENCY_INCREASE_EXCHANGE_RATE_TOOLTIP),
+				NWidget(WWT_PUSHARROWBTN, COLOUR_YELLOW, CUSTCURR_RATE_DOWN), SetDataTip(AWV_DECREASE, STR_CURRENCY_DECREASE_EXCHANGE_RATE_TOOLTIP),
+				NWidget(WWT_PUSHARROWBTN, COLOUR_YELLOW, CUSTCURR_RATE_UP), SetDataTip(AWV_INCREASE, STR_CURRENCY_INCREASE_EXCHANGE_RATE_TOOLTIP),
 				NWidget(NWID_SPACER), SetMinimalSize(5, 0),
 				NWidget(WWT_TEXT, COLOUR_BLUE, CUSTCURR_RATE), SetDataTip(STR_CURRENCY_EXCHANGE_RATE, STR_CURRENCY_SET_EXCHANGE_RATE_TOOLTIP), SetFill(1, 0),
 			EndContainer(),
@@ -1978,8 +2013,8 @@ static const NWidgetPart _nested_cust_currency_widgets[] = {
 				NWidget(WWT_TEXT, COLOUR_BLUE, CUSTCURR_SUFFIX), SetDataTip(STR_CURRENCY_SUFFIX, STR_CURRENCY_SET_CUSTOM_CURRENCY_SUFFIX_TOOLTIP), SetFill(1, 0),
 			EndContainer(),
 			NWidget(NWID_HORIZONTAL), SetPIP(10, 0, 5),
-				NWidget(NWID_BUTTON_ARROW, COLOUR_YELLOW, CUSTCURR_YEAR_DOWN), SetDataTip(AWV_DECREASE, STR_CURRENCY_DECREASE_CUSTOM_CURRENCY_TO_EURO_TOOLTIP),
-				NWidget(NWID_BUTTON_ARROW, COLOUR_YELLOW, CUSTCURR_YEAR_UP), SetDataTip(AWV_INCREASE, STR_CURRENCY_INCREASE_CUSTOM_CURRENCY_TO_EURO_TOOLTIP),
+				NWidget(WWT_PUSHARROWBTN, COLOUR_YELLOW, CUSTCURR_YEAR_DOWN), SetDataTip(AWV_DECREASE, STR_CURRENCY_DECREASE_CUSTOM_CURRENCY_TO_EURO_TOOLTIP),
+				NWidget(WWT_PUSHARROWBTN, COLOUR_YELLOW, CUSTCURR_YEAR_UP), SetDataTip(AWV_INCREASE, STR_CURRENCY_INCREASE_CUSTOM_CURRENCY_TO_EURO_TOOLTIP),
 				NWidget(NWID_SPACER), SetMinimalSize(5, 0),
 				NWidget(WWT_TEXT, COLOUR_BLUE, CUSTCURR_YEAR), SetDataTip(STR_JUST_STRING, STR_CURRENCY_SET_CUSTOM_CURRENCY_TO_EURO_TOOLTIP), SetFill(1, 0),
 			EndContainer(),
@@ -1996,6 +2031,7 @@ static const WindowDesc _cust_currency_desc(
 	_nested_cust_currency_widgets, lengthof(_nested_cust_currency_widgets)
 );
 
+/** Open custom currency window. */
 static void ShowCustCurrency()
 {
 	DeleteWindowById(WC_CUSTOM_CURRENCY, 0);
