@@ -1,4 +1,4 @@
-/* $Id: railtypes.h 19376 2010-03-08 22:19:39Z rubidium $ */
+/* $Id: railtypes.h 22652 2011-07-10 20:25:59Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,14 +7,16 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file railtypes.h
+/**
+ * @file railtypes.h
  * All the railtype-specific information is stored here.
  */
 
 #ifndef RAILTYPES_H
 #define RAILTYPES_H
 
-/** Global Railtype definition
+/**
+ * Global Railtype definition
  */
 static const RailtypeInfo _original_railtypes[] = {
 	/** Railway */
@@ -63,13 +65,10 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* Compatible railtypes */
 		RAILTYPES_RAIL | RAILTYPES_ELECTRIC,
 
-		/* main offset */
-		0,
-
 		/* bridge offset */
 		0,
 
-		/* custom ground offset */
+		/* fallback_railtype */
 		0,
 
 		/* curve speed advantage (multiplier) */
@@ -93,6 +92,19 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* map colour */
 		0x0A,
 
+		/* introduction date */
+		INVALID_DATE,
+
+		/* railtypes required for this to be introduced */
+		RAILTYPES_NONE,
+
+		/* introduction rail types */
+		RAILTYPES_RAIL,
+
+		/* sort order */
+		0 << 4 | 7,
+
+		{ NULL },
 		{ NULL },
 	},
 
@@ -146,13 +158,10 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* Compatible railtypes */
 		RAILTYPES_ELECTRIC | RAILTYPES_RAIL,
 
-		/* main offset */
-		0,
-
 		/* bridge offset */
 		0,
 
-		/* custom ground offset */
+		/* fallback_railtype */
 		0,
 
 		/* curve speed advantage (multiplier) */
@@ -176,6 +185,19 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* map colour */
 		0x0A,
 
+		/* introduction date */
+		INVALID_DATE,
+
+		/* railtypes required for this to be introduced */
+		RAILTYPES_NONE,
+
+		/* introduction rail types */
+		RAILTYPES_ELECTRIC,
+
+		/* sort order */
+		1 << 4 | 7,
+
+		{ NULL },
 		{ NULL },
 	},
 
@@ -225,13 +247,10 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* Compatible Railtypes */
 		RAILTYPES_MONO,
 
-		/* main offset */
-		82,
-
 		/* bridge offset */
 		16,
 
-		/* custom ground offset */
+		/* fallback_railtype */
 		1,
 
 		/* curve speed advantage (multiplier) */
@@ -255,6 +274,19 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* map colour */
 		0x0A,
 
+		/* introduction date */
+		INVALID_DATE,
+
+		/* railtypes required for this to be introduced */
+		RAILTYPES_NONE,
+
+		/* introduction rail types */
+		RAILTYPES_MONO,
+
+		/* sort order */
+		2 << 4 | 7,
+
+		{ NULL },
 		{ NULL },
 	},
 
@@ -304,13 +336,10 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* Compatible Railtypes */
 		RAILTYPES_MAGLEV,
 
-		/* main offset */
-		164,
-
 		/* bridge offset */
 		24,
 
-		/* custom ground offset */
+		/* fallback_railtype */
 		2,
 
 		/* curve speed advantage (multiplier) */
@@ -334,6 +363,19 @@ static const RailtypeInfo _original_railtypes[] = {
 		/* map colour */
 		0x0A,
 
+		/* introduction date */
+		INVALID_DATE,
+
+		/* railtypes required for this to be introduced */
+		RAILTYPES_NONE,
+
+		/* introduction rail types */
+		RAILTYPES_MAGLEV,
+
+		/* sort order */
+		3 << 4 | 7,
+
+		{ NULL },
 		{ NULL },
 	},
 };
