@@ -1,4 +1,4 @@
-/* $Id: tunnelbridge_cmd.cpp 20432 2010-08-09 21:58:52Z rubidium $ */
+/* $Id: tunnelbridge_cmd.cpp 22840 2011-08-25 13:30:26Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,20 +7,18 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file tunnelbridge_cmd.cpp
+/**
+ * @file tunnelbridge_cmd.cpp
  * This file deals with tunnels and bridges (non-gui stuff)
  * @todo seperate this file into two
  */
 
 #include "stdafx.h"
-#include "rail_map.h"
-#include "landscape.h"
-#include "unmovable_map.h"
+#include "newgrf_object.h"
 #include "viewport_func.h"
 #include "cmd_helper.h"
 #include "command_func.h"
 #include "town.h"
-#include "variables.h"
 #include "train.h"
 #include "ship.h"
 #include "roadveh.h"
@@ -31,17 +29,17 @@
 #include "tunnelbridge_map.h"
 #include "strings_func.h"
 #include "date_func.h"
-#include "functions.h"
+#include "clear_func.h"
 #include "vehicle_func.h"
 #include "sound_func.h"
 #include "tunnelbridge.h"
 #include "cheat_type.h"
 #include "elrail_func.h"
-#include "landscape_type.h"
 #include "pbs.h"
 #include "company_base.h"
-#include "engine_base.h"
 #include "newgrf_railtype.h"
+#include "object_base.h"
+#include "water.h"
 
 #include "table/sprites.h"
 #include "table/strings.h"
@@ -50,6 +48,9 @@
 BridgeSpec _bridge[MAX_BRIDGES];
 TileIndex _build_tunnel_endtile;
 
+/* Z position of the bridge sprites relative to bridge height (downwards) */
+static const int BRIDGE_Z_START = 3;
+
 /** Reset the data been eventually changed by the grf loaded. */
 void ResetBridges()
 {
@@ -67,28 +68,32 @@ void ResetBridges()
 	memcpy(&_bridge, &_orig_bridge, sizeof(_orig_bridge));
 }
 
-/** calculate the price factor for building a long bridge.
- * basically the cost delta is 1,1, 1, 2,2, 3,3,3, 4,4,4,4, 5,5,5,5,5, 6,6,6,6,6,6,  7,7,7,7,7,7,7,  8,8,8,8,8,8,8,8,
+/**
+ * Calculate the price factor for building a long bridge.
+ * Basically the cost delta is 1,1, 1, 2,2, 3,3,3, 4,4,4,4, 5,5,5,5,5, 6,6,6,6,6,6,  7,7,7,7,7,7,7,  8,8,8,8,8,8,8,8,
+ * @param length Length of the bridge.
+ * @return Price factor for the bridge.
  */
-int CalcBridgeLenCostFactor(int x)
+int CalcBridgeLenCostFactor(int length)
 {
-	int n;
-	int r;
+	if (length < 2) return length;
 
-	if (x < 2) return x;
-	x -= 2;
-	for (n = 0, r = 2;; n++) {
-		if (x <= n) return r + x * n;
-		r += n * n;
-		x -= n;
+	length -= 2;
+	int sum = 2;
+	for (int delta = 1;; delta++) {
+		for (int count = 0; count < delta; count++) {
+			if (length == 0) return sum;
+			sum += delta;
+			length--;
+		}
 	}
 }
 
 Foundation GetBridgeFoundation(Slope tileh, Axis axis)
 {
-	if ((tileh == SLOPE_FLAT) ||
-	    (((tileh == SLOPE_NE) || (tileh == SLOPE_SW)) && (axis == AXIS_X)) ||
-	    (((tileh == SLOPE_NW) || (tileh == SLOPE_SE)) && (axis == AXIS_Y))) return FOUNDATION_NONE;
+	if (tileh == SLOPE_FLAT ||
+			((tileh == SLOPE_NE || tileh == SLOPE_SW) && axis == AXIS_X) ||
+			((tileh == SLOPE_NW || tileh == SLOPE_SE) && axis == AXIS_Y)) return FOUNDATION_NONE;
 
 	return (HasSlopeHighestCorner(tileh) ? InclinedFoundation(axis) : FlatteningFoundation(tileh));
 }
@@ -161,7 +166,8 @@ static CommandCost CheckBridgeSlopeSouth(Axis axis, Slope *tileh, uint *z)
 	return CommandCost(EXPENSES_CONSTRUCTION, _price[PR_BUILD_FOUNDATION]);
 }
 
-/** Is a bridge of the specified type and length available?
+/**
+ * Is a bridge of the specified type and length available?
  * @param bridge_type Wanted type of bridge.
  * @param bridge_len  Wanted length of the bridge.
  * @return A succeeded (the requested bridge is available) or failed (it cannot be built) command.
@@ -169,7 +175,7 @@ static CommandCost CheckBridgeSlopeSouth(Axis axis, Slope *tileh, uint *z)
 CommandCost CheckBridgeAvailability(BridgeType bridge_type, uint bridge_len, DoCommandFlag flags)
 {
 	if (flags & DC_QUERY_COST) {
-		if (bridge_len <= (_settings_game.construction.longbridges ? 100U : 16U)) return CommandCost();
+		if (bridge_len <= _settings_game.construction.max_bridge_length) return CommandCost();
 		return_cmd_error(STR_ERROR_BRIDGE_TOO_LONG);
 	}
 
@@ -178,15 +184,15 @@ CommandCost CheckBridgeAvailability(BridgeType bridge_type, uint bridge_len, DoC
 	const BridgeSpec *b = GetBridgeSpec(bridge_type);
 	if (b->avail_year > _cur_year) return CMD_ERROR;
 
-	uint max = b->max_length;
-	if (max >= 16 && _settings_game.construction.longbridges) max = 100;
+	uint max = min(b->max_length, _settings_game.construction.max_bridge_length);
 
 	if (b->min_length > bridge_len) return CMD_ERROR;
 	if (bridge_len <= max) return CommandCost();
 	return_cmd_error(STR_ERROR_BRIDGE_TOO_LONG);
 }
 
-/** Build a Bridge
+/**
+ * Build a Bridge
  * @param end_tile end tile
  * @param flags type of operation
  * @param p1 packed start tile coords (~ dx)
@@ -201,13 +207,11 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 {
 	RailType railtype = INVALID_RAILTYPE;
 	RoadTypes roadtypes = ROADTYPES_NONE;
-	CommandCost cost(EXPENSES_CONSTRUCTION);
-	Owner owner;
 
 	/* unpack parameters */
 	BridgeType bridge_type = GB(p2, 0, 8);
 
-	if (p1 >= MapSize()) return CMD_ERROR;
+	if (!IsValidTile(p1)) return_cmd_error(STR_ERROR_BRIDGE_THROUGH_MAP_BORDER);
 
 	TransportType transport_type = Extract<TransportType, 15, 2>(p2);
 
@@ -254,12 +258,7 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 		CommandCost ret = CheckBridgeAvailability(bridge_type, bridge_len, flags);
 		if (ret.Failed()) return ret;
 	} else {
-		if (bridge_len > (_settings_game.construction.longbridges ? 100U : 16U)) return_cmd_error(STR_ERROR_BRIDGE_TOO_LONG);
-	}
-
-	/* retrieve landscape height and ensure it's on land */
-	if (IsWaterTile(tile_start) || IsWaterTile(tile_end)) {
-		return_cmd_error(STR_ERROR_ENDS_OF_BRIDGE_MUST_BOTH);
+		if (bridge_len > _settings_game.construction.max_bridge_length) return_cmd_error(STR_ERROR_BRIDGE_TOO_LONG);
 	}
 
 	uint z_start;
@@ -271,8 +270,12 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 	CommandCost terraform_cost_north = CheckBridgeSlopeNorth(direction, &tileh_start, &z_start);
 	CommandCost terraform_cost_south = CheckBridgeSlopeSouth(direction, &tileh_end,   &z_end);
 
+	/* Aqueducts can't be built of flat land. */
+	if (transport_type == TRANSPORT_WATER && (tileh_start == SLOPE_FLAT || tileh_end == SLOPE_FLAT)) return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
 	if (z_start != z_end) return_cmd_error(STR_ERROR_BRIDGEHEADS_NOT_SAME_HEIGHT);
 
+	CommandCost cost(EXPENSES_CONSTRUCTION);
+	Owner owner;
 	if (IsBridgeTile(tile_start) && IsBridgeTile(tile_end) &&
 			GetOtherBridgeEnd(tile_start) == tile_end &&
 			GetTunnelBridgeTransportType(tile_start) == transport_type) {
@@ -332,8 +335,7 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 		if (ret.Failed()) return ret;
 		cost = ret;
 
-		if (terraform_cost_north.Failed() || (terraform_cost_north.GetCost() != 0 && !allow_on_slopes))
-			return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
+		if (terraform_cost_north.Failed() || (terraform_cost_north.GetCost() != 0 && !allow_on_slopes)) return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
 		cost.AddCost(terraform_cost_north);
 
 		/* Try and clear the end landscape */
@@ -342,12 +344,9 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 		cost.AddCost(ret);
 
 		/* false - end tile slope check */
-		if (terraform_cost_south.Failed() || (terraform_cost_south.GetCost() != 0 && !allow_on_slopes))
-			return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
+		if (terraform_cost_south.Failed() || (terraform_cost_south.GetCost() != 0 && !allow_on_slopes)) return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
 		cost.AddCost(terraform_cost_south);
 
-		if (transport_type == TRANSPORT_WATER && (tileh_start == SLOPE_FLAT || tileh_end == SLOPE_FLAT)) return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
-
 		const TileIndex heads[] = {tile_start, tile_end};
 		for (int i = 0; i < 2; i++) {
 			if (MayHaveBridgeAbove(heads[i])) {
@@ -391,9 +390,12 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 					if (z_start < GetBridgeHeight(tile)) goto not_valid_below;
 					break;
 
-				case MP_UNMOVABLE:
-					if (!IsOwnedLand(tile)) goto not_valid_below;
+				case MP_OBJECT: {
+					const ObjectSpec *spec = ObjectSpec::GetByTile(tile);
+					if ((spec->flags & OBJECT_FLAG_ALLOW_UNDER_BRIDGE) == 0) goto not_valid_below;
+					if (GetTileMaxZ(tile) + spec->height * TILE_HEIGHT > z_start) goto not_valid_below;
 					break;
+				}
 
 				case MP_CLEAR:
 					break;
@@ -466,24 +468,28 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 		bridge_len += 2; // begin and end tiles/ramps
 
 		switch (transport_type) {
-			case TRANSPORT_ROAD: cost.AddCost(bridge_len * _price[PR_BUILD_ROAD] * 2); break;
+			case TRANSPORT_ROAD: cost.AddCost(bridge_len * _price[PR_BUILD_ROAD] * 2 * CountBits(roadtypes)); break;
 			case TRANSPORT_RAIL: cost.AddCost(bridge_len * RailBuildCost(railtype)); break;
 			default: break;
 		}
 
 		if (c != NULL) bridge_len = CalcBridgeLenCostFactor(bridge_len);
 
-		cost.AddCost((int64)bridge_len * _price[PR_BUILD_BRIDGE] * GetBridgeSpec(bridge_type)->price >> 8);
+		if (transport_type != TRANSPORT_WATER) {
+			cost.AddCost((int64)bridge_len * _price[PR_BUILD_BRIDGE] * GetBridgeSpec(bridge_type)->price >> 8);
+		} else {
+			/* Aqueducts use a separate base cost. */
+			cost.AddCost((int64)bridge_len * _price[PR_BUILD_AQUEDUCT]);
+		}
 
-		/* Aqueducts are a little more expensive. */
-		if (transport_type == TRANSPORT_WATER) cost.AddCost((int64)bridge_len * _price[PR_CLEAR_WATER]);
 	}
 
 	return cost;
 }
 
 
-/** Build Tunnel.
+/**
+ * Build Tunnel.
  * @param start_tile start tile of tunnel
  * @param flags type of operation
  * @param p1 bit 0-3 railtype or roadtypes
@@ -495,7 +501,6 @@ CommandCost CmdBuildBridge(TileIndex end_tile, DoCommandFlag flags, uint32 p1, u
 CommandCost CmdBuildTunnel(TileIndex start_tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
 	TransportType transport_type = Extract<TransportType, 8, 2>(p1);
-	CommandCost cost(EXPENSES_CONSTRUCTION);
 
 	RailType railtype = INVALID_RAILTYPE;
 	RoadTypes rts = ROADTYPES_NONE;
@@ -520,7 +525,7 @@ CommandCost CmdBuildTunnel(TileIndex start_tile, DoCommandFlag flags, uint32 p1,
 	DiagDirection direction = GetInclinedSlopeDirection(start_tileh);
 	if (direction == INVALID_DIAGDIR) return_cmd_error(STR_ERROR_SITE_UNSUITABLE_FOR_TUNNEL);
 
-	if (IsWaterTile(start_tile)) return_cmd_error(STR_ERROR_CAN_T_BUILD_ON_WATER);
+	if (HasTileWaterGround(start_tile)) return_cmd_error(STR_ERROR_CAN_T_BUILD_ON_WATER);
 
 	CommandCost ret = DoCommand(start_tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 	if (ret.Failed()) return ret;
@@ -547,6 +552,7 @@ CommandCost CmdBuildTunnel(TileIndex start_tile, DoCommandFlag flags, uint32 p1,
 	/* Number of tiles at which the cost increase coefficient per tile is halved */
 	int tiles_bump = 25;
 
+	CommandCost cost(EXPENSES_CONSTRUCTION);
 	Slope end_tileh;
 	for (;;) {
 		end_tile += delta;
@@ -576,25 +582,35 @@ CommandCost CmdBuildTunnel(TileIndex start_tile, DoCommandFlag flags, uint32 p1,
 	/* if the command fails from here on we want the end tile to be highlighted */
 	_build_tunnel_endtile = end_tile;
 
-	if (IsWaterTile(end_tile)) return_cmd_error(STR_ERROR_CAN_T_BUILD_ON_WATER);
+	if (tiles > _settings_game.construction.max_tunnel_length) return_cmd_error(STR_ERROR_TUNNEL_TOO_LONG);
+
+	if (HasTileWaterGround(end_tile)) return_cmd_error(STR_ERROR_CAN_T_BUILD_ON_WATER);
+
+	/* Clear the tile in any case */
+	ret = DoCommand(end_tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
+	if (ret.Failed()) return_cmd_error(STR_ERROR_UNABLE_TO_EXCAVATE_LAND);
+	cost.AddCost(ret);
 
 	/* slope of end tile must be complementary to the slope of the start tile */
 	if (end_tileh != ComplementSlope(start_tileh)) {
-		/* Check if there is a structure on the terraformed tile. Do not add the cost, that will be done by the terraforming
-		 * Note: Currently the town rating is also affected by this clearing-test. So effectivly the player is punished twice for clearing
-		 *       the tree on end_tile.
-		 */
-		ret = DoCommand(end_tile, 0, 0, DC_AUTO, CMD_LANDSCAPE_CLEAR);
-		if (ret.Failed()) return_cmd_error(STR_ERROR_UNABLE_TO_EXCAVATE_LAND);
+		/* Mark the tile as already cleared for the terraform command.
+		 * Do this for all tiles (like trees), not only objects. */
+		ClearedObjectArea *coa = FindClearedObject(end_tile);
+		if (coa == NULL) {
+			coa = _cleared_object_areas.Append();
+			coa->first_tile = end_tile;
+			coa->area = TileArea(end_tile, 1, 1);
+		}
 
+		/* Hide the tile from the terraforming command */
+		TileIndex old_first_tile = coa->first_tile;
+		coa->first_tile = INVALID_TILE;
 		ret = DoCommand(end_tile, end_tileh & start_tileh, 0, flags, CMD_TERRAFORM_LAND);
+		coa->first_tile = old_first_tile;
 		if (ret.Failed()) return_cmd_error(STR_ERROR_UNABLE_TO_EXCAVATE_LAND);
-	} else {
-		ret = DoCommand(end_tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
-		if (ret.Failed()) return ret;
+		cost.AddCost(ret);
 	}
 	cost.AddCost(_price[PR_BUILD_TUNNEL]);
-	cost.AddCost(ret);
 
 	/* Pay for the rail/road in the tunnel including entrances */
 	switch (transport_type) {
@@ -619,10 +635,15 @@ CommandCost CmdBuildTunnel(TileIndex start_tile, DoCommandFlag flags, uint32 p1,
 }
 
 
-static inline bool CheckAllowRemoveTunnelBridge(TileIndex tile)
+/**
+ * Are we allowed to remove the tunnel or bridge at \a tile?
+ * @param tile End point of the tunnel or bridge.
+ * @return A succeeded command if the tunnel or bridge may be removed, a failed command otherwise.
+ */
+static inline CommandCost CheckAllowRemoveTunnelBridge(TileIndex tile)
 {
 	/* Floods can remove anything as well as the scenario editor */
-	if (_current_company == OWNER_WATER || _game_mode == GM_EDITOR) return true;
+	if (_current_company == OWNER_WATER || _game_mode == GM_EDITOR) return CommandCost();
 
 	switch (GetTunnelBridgeTransportType(tile)) {
 		case TRANSPORT_ROAD: {
@@ -634,43 +655,57 @@ static inline bool CheckAllowRemoveTunnelBridge(TileIndex tile)
 			if (HasBit(rts, ROADTYPE_TRAM)) tram_owner = GetRoadOwner(tile, ROADTYPE_TRAM);
 
 			/* We can remove unowned road and if the town allows it */
-			if (road_owner == OWNER_TOWN && !(_settings_game.construction.extra_dynamite || _cheats.magic_bulldozer.value)) return CheckTileOwnership(tile);
+			if (road_owner == OWNER_TOWN && !(_settings_game.construction.extra_dynamite || _cheats.magic_bulldozer.value)) {
+				return CheckTileOwnership(tile);
+			}
 			if (road_owner == OWNER_NONE || road_owner == OWNER_TOWN) road_owner = _current_company;
 			if (tram_owner == OWNER_NONE) tram_owner = _current_company;
 
-			return CheckOwnership(road_owner, tile) && CheckOwnership(tram_owner, tile);
+			CommandCost ret = CheckOwnership(road_owner, tile);
+			if (ret.Succeeded()) ret = CheckOwnership(tram_owner, tile);
+			return ret;
 		}
 
 		case TRANSPORT_RAIL:
-		case TRANSPORT_WATER:
 			return CheckOwnership(GetTileOwner(tile));
 
+		case TRANSPORT_WATER: {
+			/* Always allow to remove aqueducts without owner. */
+			Owner aqueduct_owner = GetTileOwner(tile);
+			if (aqueduct_owner == OWNER_NONE) aqueduct_owner = _current_company;
+			return CheckOwnership(aqueduct_owner);
+		}
+
 		default: NOT_REACHED();
 	}
 }
 
+/**
+ * Remove a tunnel from the game, update town rating, etc.
+ * @param tile Tile containing one of the endpoints of the tunnel.
+ * @param flags Command flags.
+ * @return Succeeded or failed command.
+ */
 static CommandCost DoClearTunnel(TileIndex tile, DoCommandFlag flags)
 {
-	Town *t = NULL;
-	TileIndex endtile;
+	CommandCost ret = CheckAllowRemoveTunnelBridge(tile);
+	if (ret.Failed()) return ret;
 
-	if (!CheckAllowRemoveTunnelBridge(tile)) return CMD_ERROR;
+	TileIndex endtile = GetOtherTunnelEnd(tile);
 
-	endtile = GetOtherTunnelEnd(tile);
-
-	if (HasVehicleOnTunnelBridge(tile, endtile)) return CMD_ERROR;
+	ret = TunnelBridgeIsFree(tile, endtile);
+	if (ret.Failed()) return ret;
 
 	_build_tunnel_endtile = endtile;
 
+	Town *t = NULL;
 	if (IsTileOwner(tile, OWNER_TOWN) && _game_mode != GM_EDITOR) {
 		t = ClosestTownFromTile(tile, UINT_MAX); // town penalty rating
 
 		/* Check if you are allowed to remove the tunnel owned by a town
 		 * Removal depends on difficulty settings */
-		if (!CheckforTownRating(flags, t, TUNNELBRIDGE_REMOVE)) {
-			SetDParam(0, t->index);
-			return_cmd_error(STR_ERROR_LOCAL_AUTHORITY_REFUSES_TO_ALLOW_THIS);
-		}
+		CommandCost ret = CheckforTownRating(flags, t, TUNNELBRIDGE_REMOVE);
+		if (ret.Failed()) return ret;
 	}
 
 	/* checks if the owner is town then decrease town rating by RATING_TUNNEL_BRIDGE_DOWN_STEP until
@@ -712,31 +747,33 @@ static CommandCost DoClearTunnel(TileIndex tile, DoCommandFlag flags)
 }
 
 
+/**
+ * Remove a bridge from the game, update town rating, etc.
+ * @param tile Tile containing one of the endpoints of the bridge.
+ * @param flags Command flags.
+ * @return Succeeded or failed command.
+ */
 static CommandCost DoClearBridge(TileIndex tile, DoCommandFlag flags)
 {
-	DiagDirection direction;
-	TileIndexDiff delta;
-	TileIndex endtile;
+	CommandCost ret = CheckAllowRemoveTunnelBridge(tile);
+	if (ret.Failed()) return ret;
+
+	TileIndex endtile = GetOtherBridgeEnd(tile);
+
+	ret = TunnelBridgeIsFree(tile, endtile);
+	if (ret.Failed()) return ret;
+
+	DiagDirection direction = GetTunnelBridgeDirection(tile);
+	TileIndexDiff delta = TileOffsByDiagDir(direction);
+
 	Town *t = NULL;
-
-	if (!CheckAllowRemoveTunnelBridge(tile)) return CMD_ERROR;
-
-	endtile = GetOtherBridgeEnd(tile);
-
-	if (HasVehicleOnTunnelBridge(tile, endtile)) return CMD_ERROR;
-
-	direction = GetTunnelBridgeDirection(tile);
-	delta = TileOffsByDiagDir(direction);
-
 	if (IsTileOwner(tile, OWNER_TOWN) && _game_mode != GM_EDITOR) {
 		t = ClosestTownFromTile(tile, UINT_MAX); // town penalty rating
 
 		/* Check if you are allowed to remove the bridge owned by a town
 		 * Removal depends on difficulty settings */
-		if (!CheckforTownRating(flags, t, TUNNELBRIDGE_REMOVE)) {
-			SetDParam(0, t->index);
-			return_cmd_error(STR_ERROR_LOCAL_AUTHORITY_REFUSES_TO_ALLOW_THIS);
-		}
+		CommandCost ret = CheckforTownRating(flags, t, TUNNELBRIDGE_REMOVE);
+		if (ret.Failed()) return ret;
 	}
 
 	/* checks if the owner is town then decrease town rating by RATING_TUNNEL_BRIDGE_DOWN_STEP until
@@ -745,6 +782,8 @@ static CommandCost DoClearBridge(TileIndex tile, DoCommandFlag flags)
 		ChangeTownRating(t, RATING_TUNNEL_BRIDGE_DOWN_STEP, RATING_TUNNEL_BRIDGE_MINIMUM, flags);
 	}
 
+	Money base_cost = (GetTunnelBridgeTransportType(tile) != TRANSPORT_WATER) ? _price[PR_CLEAR_BRIDGE] : _price[PR_CLEAR_AQUEDUCT];
+
 	if (flags & DC_EXEC) {
 		/* read this value before actual removal of bridge */
 		bool rail = GetTunnelBridgeTransportType(tile) == TRANSPORT_RAIL;
@@ -782,9 +821,15 @@ static CommandCost DoClearBridge(TileIndex tile, DoCommandFlag flags)
 		}
 	}
 
-	return CommandCost(EXPENSES_CONSTRUCTION, (GetTunnelBridgeLength(tile, endtile) + 2) * _price[PR_CLEAR_BRIDGE]);
+	return CommandCost(EXPENSES_CONSTRUCTION, (GetTunnelBridgeLength(tile, endtile) + 2) * base_cost);
 }
 
+/**
+ * Remove a tunnel or a bridge from the game.
+ * @param tile Tile containing one of the endpoints.
+ * @param flags Command flags.
+ * @return Succeeded or failed command.
+ */
 static CommandCost ClearTile_TunnelBridge(TileIndex tile, DoCommandFlag flags)
 {
 	if (IsTunnel(tile)) {
@@ -794,8 +839,42 @@ static CommandCost ClearTile_TunnelBridge(TileIndex tile, DoCommandFlag flags)
 		if (flags & DC_AUTO) return_cmd_error(STR_ERROR_MUST_DEMOLISH_BRIDGE_FIRST);
 		return DoClearBridge(tile, flags);
 	}
+}
 
-	return CMD_ERROR;
+/**
+ * Draw a single pillar sprite.
+ * @param psid      Pillarsprite
+ * @param x         Pillar X
+ * @param y         Pillar Y
+ * @param z         Pillar Z
+ * @param w         Bounding box size in X direction
+ * @param h         Bounding box size in Y direction
+ * @param subsprite Optional subsprite for drawing halfpillars
+ */
+static inline void DrawPillar(const PalSpriteID *psid, int x, int y, int z, int w, int h, const SubSprite *subsprite)
+{
+	static const int PILLAR_Z_OFFSET = TILE_HEIGHT - BRIDGE_Z_START; ///< Start offset of pillar wrt. bridge (downwards)
+	AddSortableSpriteToDraw(psid->sprite, psid->pal, x, y, w, h, BB_HEIGHT_UNDER_BRIDGE - PILLAR_Z_OFFSET, z, IsTransparencySet(TO_BRIDGES), 0, 0, -PILLAR_Z_OFFSET, subsprite);
+}
+
+/**
+ * Draw two bridge pillars (north and south).
+ * @param z_bottom Bottom Z
+ * @param z_top    Top Z
+ * @param psid     Pillarsprite
+ * @param x        Pillar X
+ * @param y        Pillar Y
+ * @param w        Bounding box size in X direction
+ * @param h        Bounding box size in Y direction
+ * @return Reached Z at the bottom
+ */
+static int DrawPillarColumn(int z_bottom, int z_top, const PalSpriteID *psid, int x, int y, int w, int h)
+{
+	int cur_z;
+	for (cur_z = z_top; cur_z >= z_bottom; cur_z -= TILE_HEIGHT) {
+		DrawPillar(psid, x, y, cur_z, w, h, NULL);
+	}
+	return cur_z;
 }
 
 /**
@@ -811,50 +890,48 @@ static CommandCost ClearTile_TunnelBridge(TileIndex tile, DoCommandFlag flags)
  */
 static void DrawBridgePillars(const PalSpriteID *psid, const TileInfo *ti, Axis axis, bool drawfarpillar, int x, int y, int z_bridge)
 {
-	/* Do not draw bridge pillars if they are invisible */
-	if (IsInvisibilitySet(TO_BRIDGES)) return;
+	static const int bounding_box_size[2]  = {16, 2}; ///< bounding box size of pillars along bridge direction
+	static const int back_pillar_offset[2] = { 0, 9}; ///< sprite position offset of back facing pillar
 
-	SpriteID image = psid->sprite;
+	static const int INF = 1000; ///< big number compared to sprite size
+	static const SubSprite half_pillar_sub_sprite[2][2] = {
+		{ {  -14, -INF, INF, INF }, { -INF, -INF, -15, INF } }, // X axis, north and south
+		{ { -INF, -INF,  15, INF }, {   16, -INF, INF, INF } }, // Y axis, north and south
+	};
 
-	if (image != 0) {
-		/* "side" specifies the side the pillars stand on.
-		 * The length of the pillars is then set to the height of the bridge over the corners of this edge.
-		 *
-		 *                axis==AXIS_X  axis==AXIS_Y
-		 *   side==false      SW            NW
-		 *   side==true       NE            SE
-		 *
-		 * I have no clue, why this was done this way.
-		 */
-		bool side = HasBit(image, 0);
+	if (psid->sprite == 0) return;
 
-		/* "dir" means the edge the pillars stand on */
-		DiagDirection dir = AxisToDiagDir(axis);
-		if (side != (axis == AXIS_Y)) dir = ReverseDiagDir(dir);
+	/* Determine ground height under pillars */
+	DiagDirection south_dir = AxisToDiagDir(axis);
+	int z_front_north = ti->z;
+	int z_back_north = ti->z;
+	int z_front_south = ti->z;
+	int z_back_south = ti->z;
+	GetSlopeZOnEdge(ti->tileh, south_dir, &z_front_south, &z_back_south);
+	GetSlopeZOnEdge(ti->tileh, ReverseDiagDir(south_dir), &z_front_north, &z_back_north);
 
-		/* Determine ground height under pillars */
-		int front_height = ti->z;
-		int back_height = ti->z;
-		GetSlopeZOnEdge(ti->tileh, dir, &front_height, &back_height);
+	/* Shared height of pillars */
+	int z_front = max(z_front_north, z_front_south);
+	int z_back = max(z_back_north, z_back_south);
 
-		/* x and y size of bounding-box of pillars */
-		int w = (axis == AXIS_X ? 16 : 2);
-		int h = (axis == AXIS_X ? 2 : 16);
-		/* sprite position of back facing pillar */
-		int x_back = x - (axis == AXIS_X ? 0 : 9);
-		int y_back = y - (axis == AXIS_X ? 9 : 0);
+	/* x and y size of bounding-box of pillars */
+	int w = bounding_box_size[axis];
+	int h = bounding_box_size[OtherAxis(axis)];
+	/* sprite position of back facing pillar */
+	int x_back = x - back_pillar_offset[axis];
+	int y_back = y - back_pillar_offset[OtherAxis(axis)];
 
-		for (int cur_z = z_bridge; cur_z >= front_height || cur_z >= back_height; cur_z -= TILE_HEIGHT) {
-			/* Draw front facing pillar */
-			if (cur_z >= front_height) {
-				AddSortableSpriteToDraw(image, psid->pal, x, y, w, h, BB_HEIGHT_UNDER_BRIDGE - 5, cur_z, IsTransparencySet(TO_BRIDGES), 0, 0, -5);
-			}
+	/* Draw front pillars */
+	int bottom_z = DrawPillarColumn(z_front, z_bridge, psid, x, y, w, h);
+	if (z_front_north < z_front) DrawPillar(psid, x, y, bottom_z, w, h, &half_pillar_sub_sprite[axis][0]);
+	if (z_front_south < z_front) DrawPillar(psid, x, y, bottom_z, w, h, &half_pillar_sub_sprite[axis][1]);
 
-			/* Draw back facing pillar, but not the highest part directly under the bridge-floor */
-			if (drawfarpillar && cur_z >= back_height && cur_z < z_bridge - TILE_HEIGHT) {
-				AddSortableSpriteToDraw(image, psid->pal, x_back, y_back, w, h, BB_HEIGHT_UNDER_BRIDGE - 5, cur_z, IsTransparencySet(TO_BRIDGES), 0, 0, -5);
-			}
-		}
+	/* Draw back pillars, skip top two parts, which are hidden by the bridge */
+	int z_bridge_back = z_bridge - 2 * (int)TILE_HEIGHT;
+	if (drawfarpillar && (z_back_north <= z_bridge_back || z_back_south <= z_bridge_back)) {
+		bottom_z = DrawPillarColumn(z_back, z_bridge_back, psid, x_back, y_back, w, h);
+		if (z_back_north < z_back) DrawPillar(psid, x_back, y_back, bottom_z, w, h, &half_pillar_sub_sprite[axis][0]);
+		if (z_back_south < z_back) DrawPillar(psid, x_back, y_back, bottom_z, w, h, &half_pillar_sub_sprite[axis][1]);
 	}
 }
 
@@ -920,7 +997,6 @@ static void DrawBridgeTramBits(int x, int y, byte z, int offset, bool overlay, b
  */
 static void DrawTile_TunnelBridge(TileInfo *ti)
 {
-	SpriteID image;
 	TransportType transport_type = GetTunnelBridgeTransportType(ti->tile);
 	DiagDirection tunnelbridge_direction = GetTunnelBridgeDirection(ti->tile);
 
@@ -946,6 +1022,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 
 		bool catenary = false;
 
+		SpriteID image;
 		if (transport_type == TRANSPORT_RAIL) {
 			image = GetRailTypeInfo(GetRailType(ti->tile))->base_sprites.tunnel;
 		} else {
@@ -1000,7 +1077,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 
 		if (catenary) EndSpriteCombine();
 
-		/* Add helper BB for sprite sorting, that separate the tunnel from things beside of it */
+		/* Add helper BB for sprite sorting that separates the tunnel from things beside of it. */
 		AddSortableSpriteToDraw(SPR_EMPTY_BOUNDING_BOX, PAL_NONE, ti->x,              ti->y,              BB_data[6], BB_data[7], TILE_HEIGHT, ti->z);
 		AddSortableSpriteToDraw(SPR_EMPTY_BOUNDING_BOX, PAL_NONE, ti->x + BB_data[4], ti->y + BB_data[5], BB_data[6], BB_data[7], TILE_HEIGHT, ti->z);
 
@@ -1035,9 +1112,14 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 		}
 
 		if (!ice) {
-			DrawClearLandTile(ti, 3);
+			TileIndex next = ti->tile + TileOffsByDiagDir(tunnelbridge_direction);
+			if (ti->tileh != SLOPE_FLAT && ti->z == 0 && HasTileWaterClass(next) && GetWaterClass(next) == WATER_CLASS_SEA) {
+				DrawShoreTile(ti->tileh);
+			} else {
+				DrawClearLandTile(ti, 3);
+			}
 		} else {
-			DrawGroundSprite(SPR_FLAT_SNOW_DESERT_TILE + _tileh_to_sprite[ti->tileh], PAL_NONE);
+			DrawGroundSprite(SPR_FLAT_SNOW_DESERT_TILE + SlopeToSpriteOffset(ti->tileh), PAL_NONE);
 		}
 
 		/* draw ramp */
@@ -1099,7 +1181,8 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 }
 
 
-/** Compute bridge piece. Computes the bridge piece to display depending on the position inside the bridge.
+/**
+ * Compute bridge piece. Computes the bridge piece to display depending on the position inside the bridge.
  * bridges pieces sequence (middle parts).
  * Note that it is not covering the bridge heads, which are always referenced by the same sprite table.
  * bridge len 1: BRIDGE_PIECE_NORTH
@@ -1150,9 +1233,6 @@ void DrawBridgeMiddle(const TileInfo *ti)
 	 *
 	 */
 
-	/* Z position of the bridge sprites relative to bridge height (downwards) */
-	static const int BRIDGE_Z_START = 3;
-
 	if (!IsBridgeAbove(ti->tile)) return;
 
 	TileIndex rampnorth = GetNorthernBridgeEnd(ti->tile);
@@ -1191,7 +1271,7 @@ void DrawBridgeMiddle(const TileInfo *ti)
 	uint bridge_z = GetBridgeHeight(rampsouth);
 	uint z = bridge_z - BRIDGE_Z_START;
 
-	/* Add a bounding box, that separates the bridge from things below it. */
+	/* Add a bounding box that separates the bridge from things below it. */
 	AddSortableSpriteToDraw(SPR_EMPTY_BOUNDING_BOX, PAL_NONE, x, y, 16, 16, 1, bridge_z - TILE_HEIGHT + BB_Z_SEPARATOR);
 
 	/* Draw Trambits as SpriteCombine */
@@ -1263,7 +1343,7 @@ void DrawBridgeMiddle(const TileInfo *ti)
 
 			DrawGroundSpriteAt(image, pal, x - ti->x, y - ti->y, z - ti->z);
 		}
-	} else if (_settings_client.gui.bridge_pillars) {
+	} else {
 		/* draw pillars below for high bridges */
 		DrawBridgePillars(psid, ti, axis, drawfarpillar, x, y, z);
 	}
@@ -1412,59 +1492,61 @@ static void ChangeTileOwner_TunnelBridge(TileIndex tile, Owner old_owner, Owner
 	if (new_owner != INVALID_OWNER) {
 		SetTileOwner(tile, new_owner);
 	} else {
-		if (DoCommand(tile, 0, 0, DC_EXEC | DC_BANKRUPT, CMD_LANDSCAPE_CLEAR).Failed()) {
-			/* When clearing the bridge/tunnel failed there are still vehicles on/in
-			 * the bridge/tunnel. As all *our* vehicles are already removed, they
-			 * must be of another owner. Therefore this can't be rail tunnel/bridge.
-			 * In that case we can safely reassign the ownership to OWNER_NONE. */
-			assert(GetTunnelBridgeTransportType(tile) != TRANSPORT_RAIL);
+		if (GetTunnelBridgeTransportType(tile) == TRANSPORT_RAIL) {
+			/* Since all of our vehicles have been removed, it is safe to remove the rail
+			 * bridge / tunnel. */
+			CommandCost ret = DoCommand(tile, 0, 0, DC_EXEC | DC_BANKRUPT, CMD_LANDSCAPE_CLEAR);
+			assert(ret.Succeeded());
+		} else {
+			/* In any other case, we can safely reassign the ownership to OWNER_NONE. */
 			SetTileOwner(tile, OWNER_NONE);
 		}
 	}
 }
 
+/**
+ * Frame when the 'enter tunnel' sound should be played. This is the second
+ * frame on a tile, so the sound is played shortly after entering the tunnel
+ * tile, while the vehicle is still visible.
+ */
+static const byte TUNNEL_SOUND_FRAME = 1;
 
-static const byte _tunnel_fractcoord_1[4]    = {0x8E, 0x18, 0x81, 0xE8};
-static const byte _tunnel_fractcoord_2[4]    = {0x81, 0x98, 0x87, 0x38};
-static const byte _tunnel_fractcoord_3[4]    = {0x82, 0x88, 0x86, 0x48};
-static const byte _exit_tunnel_track[4]      = {1, 2, 1, 2};
-
-/** Get the trackdir of the exit of a tunnel */
-static const Trackdir _road_exit_tunnel_state[DIAGDIR_END] = {
-	TRACKDIR_X_SW, TRACKDIR_Y_NW, TRACKDIR_X_NE, TRACKDIR_Y_SE
-};
-static const byte _road_exit_tunnel_frame[4] = {2, 7, 9, 4};
-
-static const byte _tunnel_fractcoord_4[4]    = {0x52, 0x85, 0x98, 0x29};
-static const byte _tunnel_fractcoord_5[4]    = {0x92, 0x89, 0x58, 0x25};
-static const byte _tunnel_fractcoord_6[4]    = {0x92, 0x89, 0x56, 0x45};
-static const byte _tunnel_fractcoord_7[4]    = {0x52, 0x85, 0x96, 0x49};
+/**
+ * Frame when a vehicle should be hidden in a tunnel with a certain direction.
+ * This differs per direction, because of visibility / bounding box issues.
+ * Note that direction, in this case, is the direction leading into the tunnel.
+ * When entering a tunnel, hide the vehicle when it reaches the given frame.
+ * When leaving a tunnel, show the vehicle when it is one frame further
+ * to the 'outside', i.e. at (TILE_SIZE-1) - (frame) + 1
+ */
+extern const byte _tunnel_visibility_frame[DIAGDIR_END] = {12, 8, 8, 12};
 
 static VehicleEnterTileStatus VehicleEnter_TunnelBridge(Vehicle *v, TileIndex tile, int x, int y)
 {
 	int z = GetSlopeZ(x, y) - v->z_pos;
 
 	if (abs(z) > 2) return VETSB_CANNOT_ENTER;
+	/* Direction into the wormhole */
 	const DiagDirection dir = GetTunnelBridgeDirection(tile);
+	/* Direction of the vehicle */
+	const DiagDirection vdir = DirToDiagDir(v->direction);
+	/* New position of the vehicle on the tile */
+	byte pos = (DiagDirToAxis(vdir) == AXIS_X ? x : y) & TILE_UNIT_MASK;
+	/* Number of units moved by the vehicle since entering the tile */
+	byte frame = (vdir == DIAGDIR_NE || vdir == DIAGDIR_NW) ? TILE_SIZE - 1 - pos : pos;
 
 	if (IsTunnel(tile)) {
-		byte fc;
-		DiagDirection vdir;
-
 		if (v->type == VEH_TRAIN) {
 			Train *t = Train::From(v);
-			fc = (x & 0xF) + (y << 4);
-
-			vdir = DirToDiagDir(t->direction);
 
 			if (t->track != TRACK_BIT_WORMHOLE && dir == vdir) {
-				if (t->IsFrontEngine() && fc == _tunnel_fractcoord_1[dir]) {
+				if (t->IsFrontEngine() && frame == TUNNEL_SOUND_FRAME) {
 					if (!PlayVehicleSound(t, VSE_TUNNEL) && RailVehInfo(t->engine_type)->engclass == 0) {
 						SndPlayVehicleFx(SND_05_TRAIN_THROUGH_TUNNEL, v);
 					}
 					return VETSB_CONTINUE;
 				}
-				if (fc == _tunnel_fractcoord_2[dir]) {
+				if (frame == _tunnel_visibility_frame[dir]) {
 					t->tile = tile;
 					t->track = TRACK_BIT_WORMHOLE;
 					t->vehstatus |= VS_HIDDEN;
@@ -1472,23 +1554,22 @@ static VehicleEnterTileStatus VehicleEnter_TunnelBridge(Vehicle *v, TileIndex ti
 				}
 			}
 
-			if (dir == ReverseDiagDir(vdir) && fc == _tunnel_fractcoord_3[dir] && z == 0) {
+			if (dir == ReverseDiagDir(vdir) && frame == TILE_SIZE - _tunnel_visibility_frame[dir] && z == 0) {
 				/* We're at the tunnel exit ?? */
 				t->tile = tile;
-				t->track = (TrackBits)_exit_tunnel_track[dir];
+				t->track = DiagDirToDiagTrackBits(vdir);
 				assert(t->track);
 				t->vehstatus &= ~VS_HIDDEN;
 				return VETSB_ENTERED_WORMHOLE;
 			}
 		} else if (v->type == VEH_ROAD) {
 			RoadVehicle *rv = RoadVehicle::From(v);
-			fc = (x & 0xF) + (y << 4);
-			vdir = DirToDiagDir(v->direction);
 
 			/* Enter tunnel? */
 			if (rv->state != RVSB_WORMHOLE && dir == vdir) {
-				if (fc == _tunnel_fractcoord_4[dir] ||
-						fc == _tunnel_fractcoord_5[dir]) {
+				if (frame == _tunnel_visibility_frame[dir]) {
+					/* Frame should be equal to the next frame number in the RV's movement */
+					assert(frame == rv->frame + 1);
 					rv->tile = tile;
 					rv->state = RVSB_WORMHOLE;
 					rv->vehstatus |= VS_HIDDEN;
@@ -1498,48 +1579,45 @@ static VehicleEnterTileStatus VehicleEnter_TunnelBridge(Vehicle *v, TileIndex ti
 				}
 			}
 
-			if (dir == ReverseDiagDir(vdir) && (
-						/* We're at the tunnel exit ?? */
-						fc == _tunnel_fractcoord_6[dir] ||
-						fc == _tunnel_fractcoord_7[dir]
-					) &&
-					z == 0) {
+			/* We're at the tunnel exit ?? */
+			if (dir == ReverseDiagDir(vdir) && frame == TILE_SIZE - _tunnel_visibility_frame[dir] && z == 0) {
 				rv->tile = tile;
-				rv->state = _road_exit_tunnel_state[dir];
-				rv->frame = _road_exit_tunnel_frame[dir];
+				rv->state = DiagDirToDiagTrackdir(vdir);
+				rv->frame = frame;
 				rv->vehstatus &= ~VS_HIDDEN;
 				return VETSB_ENTERED_WORMHOLE;
 			}
 		}
 	} else { // IsBridge(tile)
-
-		if (v->IsPrimaryVehicle() && v->type != VEH_SHIP) {
+		if (v->type != VEH_SHIP) {
 			/* modify speed of vehicle */
 			uint16 spd = GetBridgeSpec(GetBridgeType(tile))->speed;
 
 			if (v->type == VEH_ROAD) spd *= 2;
-			if (v->cur_speed > spd) v->cur_speed = spd;
+			Vehicle *first = v->First();
+			first->cur_speed = min(first->cur_speed, spd);
 		}
 
-		if (DirToDiagDir(v->direction) == dir) {
-			switch (dir) {
-				default: NOT_REACHED();
-				case DIAGDIR_NE: if ((x & 0xF) != 0)             return VETSB_CONTINUE; break;
-				case DIAGDIR_SE: if ((y & 0xF) != TILE_SIZE - 1) return VETSB_CONTINUE; break;
-				case DIAGDIR_SW: if ((x & 0xF) != TILE_SIZE - 1) return VETSB_CONTINUE; break;
-				case DIAGDIR_NW: if ((y & 0xF) != 0)             return VETSB_CONTINUE; break;
-			}
+		if (vdir == dir) {
+			/* Vehicle enters bridge at the last frame inside this tile. */
+			if (frame != TILE_SIZE - 1) return VETSB_CONTINUE;
 			switch (v->type) {
 				case VEH_TRAIN: {
 					Train *t = Train::From(v);
 					t->track = TRACK_BIT_WORMHOLE;
-					ClrBit(t->flags, VRF_GOINGUP);
-					ClrBit(t->flags, VRF_GOINGDOWN);
-				} break;
-
-				case VEH_ROAD:
-					RoadVehicle::From(v)->state = RVSB_WORMHOLE;
+					ClrBit(t->gv_flags, GVF_GOINGUP_BIT);
+					ClrBit(t->gv_flags, GVF_GOINGDOWN_BIT);
 					break;
+				}
+
+				case VEH_ROAD: {
+					RoadVehicle *rv = RoadVehicle::From(v);
+					rv->state = RVSB_WORMHOLE;
+					/* There are no slopes inside bridges / tunnels. */
+					ClrBit(rv->gv_flags, GVF_GOINGUP_BIT);
+					ClrBit(rv->gv_flags, GVF_GOINGDOWN_BIT);
+					break;
+				}
 
 				case VEH_SHIP:
 					Ship::From(v)->state = TRACK_BIT_WORMHOLE;
@@ -1548,33 +1626,36 @@ static VehicleEnterTileStatus VehicleEnter_TunnelBridge(Vehicle *v, TileIndex ti
 				default: NOT_REACHED();
 			}
 			return VETSB_ENTERED_WORMHOLE;
-		} else if (DirToDiagDir(v->direction) == ReverseDiagDir(dir)) {
+		} else if (vdir == ReverseDiagDir(dir)) {
 			v->tile = tile;
 			switch (v->type) {
 				case VEH_TRAIN: {
 					Train *t = Train::From(v);
 					if (t->track == TRACK_BIT_WORMHOLE) {
-						t->track = (DiagDirToAxis(dir) == AXIS_X ? TRACK_BIT_X : TRACK_BIT_Y);
+						t->track = DiagDirToDiagTrackBits(vdir);
 						return VETSB_ENTERED_WORMHOLE;
 					}
-				} break;
+					break;
+				}
 
 				case VEH_ROAD: {
 					RoadVehicle *rv = RoadVehicle::From(v);
 					if (rv->state == RVSB_WORMHOLE) {
-						rv->state = _road_exit_tunnel_state[dir];
+						rv->state = DiagDirToDiagTrackdir(vdir);
 						rv->frame = 0;
 						return VETSB_ENTERED_WORMHOLE;
 					}
-				} break;
+					break;
+				}
 
 				case VEH_SHIP: {
 					Ship *ship = Ship::From(v);
 					if (ship->state == TRACK_BIT_WORMHOLE) {
-						ship->state = (DiagDirToAxis(dir) == AXIS_X ? TRACK_BIT_X : TRACK_BIT_Y);
+						ship->state = DiagDirToDiagTrackBits(vdir);
 						return VETSB_ENTERED_WORMHOLE;
 					}
-				} break;
+					break;
+				}
 
 				default: NOT_REACHED();
 			}
