@@ -1,4 +1,4 @@
-/* $Id: fullscreen.mm 19012 2010-02-04 22:17:41Z yexo $ */
+/* $Id: fullscreen.mm 22992 2011-10-04 20:12:02Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -84,7 +84,7 @@ static int CDECL ModeSorter(const OTTD_Point *p1, const OTTD_Point *p2)
 	return 0;
 }
 
-uint QZ_ListModes(OTTD_Point *modes, uint max_modes, CGDirectDisplayID display_id, int display_depth)
+uint QZ_ListModes(OTTD_Point *modes, uint max_modes, CGDirectDisplayID display_id, int device_depth)
 {
 	CFArrayRef mode_list  = CGDisplayAvailableModes(display_id);
 	CFIndex    num_modes = CFArrayGetCount(mode_list);
@@ -99,7 +99,7 @@ uint QZ_ListModes(OTTD_Point *modes, uint max_modes, CGDirectDisplayID display_i
 		CFNumberRef number = (const __CFNumber*)CFDictionaryGetValue(onemode, kCGDisplayBitsPerPixel);
 		CFNumberGetValue(number, kCFNumberSInt32Type, &bpp);
 
-		if (bpp != display_depth) continue;
+		if (bpp != device_depth) continue;
 
 		number = (const __CFNumber*)CFDictionaryGetValue(onemode, kCGDisplayWidth);
 		CFNumberGetValue(number, kCFNumberSInt32Type, &intvalue);
@@ -149,22 +149,11 @@ bool QZ_CanDisplay8bpp()
 }
 
 class FullscreenSubdriver: public CocoaSubdriver {
-	int                display_width;
-	int                display_height;
-	int                display_depth;
-	int                screen_pitch;
-	void              *screen_buffer;
-	void              *pixel_buffer;
-
 	CGDirectDisplayID  display_id;         ///< 0 == main display (only support single display)
 	CFDictionaryRef    cur_mode;           ///< current mode of the display
 	CFDictionaryRef    save_mode;          ///< original mode of the display
 	CGDirectPaletteRef palette;            ///< palette of an 8-bit display
 
-	#define MAX_DIRTY_RECTS 100
-	Rect dirty_rects[MAX_DIRTY_RECTS];
-	int num_dirty_rects;
-
 
 	/* Gamma functions to try to hide the flash from a res switch
 	 * Fade the display from normal to black
@@ -248,8 +237,8 @@ class FullscreenSubdriver: public CocoaSubdriver {
 
 		if (refreshRate == 0) return;
 
-		double linesPerSecond = refreshRate * this->display_height;
-		double target = this->display_height;
+		double linesPerSecond = refreshRate * this->device_height;
+		double target = this->device_height;
 
 		/* Figure out the first delay so we start off about right */
 		double position = CGDisplayBeamPosition(this->display_id);
@@ -264,10 +253,10 @@ class FullscreenSubdriver: public CocoaSubdriver {
 	bool SetVideoMode(int w, int h)
 	{
 		/* Define this variables at the top (against coding style) because
-		 * otherwise GCC barfs at the goto's jumping over variable initialization. */
+		 * otherwise GCC 4.2 barfs at the goto's jumping over variable initialization. */
 		NSRect screen_rect;
-		NSPoint pt;
 		int gamma_error;
+		NSPoint mouseLocation;
 
 		/* Destroy any previous mode */
 		if (this->pixel_buffer != NULL) {
@@ -277,14 +266,14 @@ class FullscreenSubdriver: public CocoaSubdriver {
 
 		/* See if requested mode exists */
 		boolean_t exact_match;
-		this->cur_mode = CGDisplayBestModeForParameters(this->display_id, this->display_depth, w, h, &exact_match);
+		this->cur_mode = CGDisplayBestModeForParameters(this->display_id, this->device_depth, w, h, &exact_match);
 
 		/* If the mode wasn't an exact match, check if it has the right bpp, and update width and height */
 		if (!exact_match) {
 			int bpp;
 			CFNumberRef number = (const __CFNumber*) CFDictionaryGetValue(this->cur_mode, kCGDisplayBitsPerPixel);
 			CFNumberGetValue(number, kCFNumberSInt32Type, &bpp);
-			if (bpp != this->display_depth) {
+			if (bpp != this->device_depth) {
 				DEBUG(driver, 0, "Failed to find display resolution");
 				goto ERR_NO_MATCH;
 			}
@@ -296,6 +285,14 @@ class FullscreenSubdriver: public CocoaSubdriver {
 			CFNumberGetValue(number, kCFNumberSInt32Type, &h);
 		}
 
+		/* Capture the main screen */
+		CGDisplayCapture(this->display_id);
+
+		/* Store the mouse coordinates relative to the total screen */
+		mouseLocation = [ NSEvent mouseLocation ];
+		mouseLocation.x /= this->device_width;
+		mouseLocation.y /= this->device_height;
+
 		/* Fade display to zero gamma */
 		OTTD_QuartzGammaTable gamma_table;
 		gamma_error = this->FadeGammaOut(&gamma_table);
@@ -312,20 +309,29 @@ class FullscreenSubdriver: public CocoaSubdriver {
 			goto ERR_NO_SWITCH;
 		}
 
-		this->screen_buffer = CGDisplayBaseAddress(this->display_id);
-		this->screen_pitch  = CGDisplayBytesPerRow(this->display_id);
+		/* Since CGDisplayBaseAddress and CGDisplayBytesPerRow are no longer available on 10.7,
+		 * disable until a replacement can be found. */
+		if (MacOSVersionIsAtLeast(10, 7, 0)) {
+			this->window_buffer = NULL;
+			this->window_pitch  = NULL;
+		} else {
+#if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7)
+			this->window_buffer = CGDisplayBaseAddress(this->display_id);
+			this->window_pitch  = CGDisplayBytesPerRow(this->display_id);
+#endif
+		}
 
-		this->display_width = CGDisplayPixelsWide(this->display_id);
-		this->display_height = CGDisplayPixelsHigh(this->display_id);
+		this->device_width  = CGDisplayPixelsWide(this->display_id);
+		this->device_height = CGDisplayPixelsHigh(this->display_id);
 
 		/* Setup double-buffer emulation */
-		this->pixel_buffer = malloc(this->display_width * this->display_height * this->display_depth / 8);
+		this->pixel_buffer = malloc(this->device_width * this->device_height * this->device_depth / 8);
 		if (this->pixel_buffer == NULL) {
 			DEBUG(driver, 0, "Failed to allocate memory for double buffering");
 			goto ERR_DOUBLEBUF;
 		}
 
-		if (this->display_depth == 8 && !CGDisplayCanSetPalette(this->display_id)) {
+		if (this->device_depth == 8 && !CGDisplayCanSetPalette(this->display_id)) {
 			DEBUG(driver, 0, "Not an indexed display mode.");
 			goto ERR_NOT_INDEXED;
 		}
@@ -333,6 +339,9 @@ class FullscreenSubdriver: public CocoaSubdriver {
 		/* If we don't hide menu bar, it will get events and interrupt the program */
 		HideMenuBar();
 
+		/* Hide the OS cursor */
+		CGDisplayHideCursor(this->display_id);
+
 		/* Fade the display to original gamma */
 		if (!gamma_error) FadeGammaIn(&gamma_table);
 
@@ -342,15 +351,20 @@ class FullscreenSubdriver: public CocoaSubdriver {
 		 * We can hack around this bug by setting the screen rect ourselves.
 		 * This hack should be removed if/when the bug is fixed.
 		 */
-		screen_rect = NSMakeRect(0, 0, this->display_width, this->display_height);
+		screen_rect = NSMakeRect(0, 0, this->device_width, this->device_height);
 		[ [ NSScreen mainScreen ] setFrame:screen_rect ];
 
-		pt = [ NSEvent mouseLocation ];
-		pt.y = this->display_height - pt.y;
-		if (this->MouseIsInsideView(&pt)) QZ_HideMouse();
-
 		this->UpdatePalette(0, 256);
 
+		/* Move the mouse cursor to approx the same location */
+		CGPoint display_mouseLocation;
+		display_mouseLocation.x = mouseLocation.x * this->device_width;
+		display_mouseLocation.y = this->device_height - (mouseLocation.y * this->device_height);
+
+		_cursor.in_window = true;
+
+		CGDisplayMoveCursorToPoint(this->display_id, display_mouseLocation);
+
 		return true;
 
 		/* Since the blanking window covers *all* windows (even force quit) correct recovery is crucial */
@@ -364,8 +378,8 @@ ERR_NO_SWITCH:
 ERR_NO_CAPTURE:
 		if (!gamma_error) this->FadeGammaIn(&gamma_table);
 ERR_NO_MATCH:
-		this->display_width = 0;
-		this->display_height = 0;
+		this->device_width = 0;
+		this->device_height = 0;
 
 		return false;
 	}
@@ -378,7 +392,12 @@ ERR_NO_MATCH:
 
 		/* Restore original screen resolution/bpp */
 		CGDisplaySwitchToMode(this->display_id, this->save_mode);
+
 		CGReleaseAllDisplays();
+
+		/* Bring back the cursor */
+		CGDisplayShowCursor(this->display_id);
+
 		ShowMenuBar();
 
 		/* Reset the main screen's rectangle
@@ -386,8 +405,6 @@ ERR_NO_MATCH:
 		NSRect screen_rect = NSMakeRect(0, 0, CGDisplayPixelsWide(this->display_id), CGDisplayPixelsHigh(this->display_id));
 		[ [ NSScreen mainScreen ] setFrame:screen_rect ];
 
-		QZ_ShowMouse();
-
 		/* Destroy the pixel buffer */
 		if (this->pixel_buffer != NULL) {
 			free(this->pixel_buffer);
@@ -396,8 +413,8 @@ ERR_NO_MATCH:
 
 		if (!gamma_error) this->FadeGammaIn(&gamma_table);
 
-		this->display_width = 0;
-		this->display_height = 0;
+		this->device_width  = CGDisplayPixelsWide(this->display_id);
+		this->device_height = CGDisplayPixelsHigh(this->display_id);
 	}
 
 public:
@@ -413,9 +430,9 @@ public:
 
 		if (bpp == 8) this->palette = CGPaletteCreateDefaultColorPalette();
 
-		this->display_width  = 0;
-		this->display_height = 0;
-		this->display_depth  = bpp;
+		this->device_width  = CGDisplayPixelsWide(this->display_id);
+		this->device_height = CGDisplayPixelsHigh(this->display_id);
+		this->device_depth  = bpp;
 		this->pixel_buffer   = NULL;
 
 		this->num_dirty_rects = MAX_DIRTY_RECTS;
@@ -429,11 +446,11 @@ public:
 	virtual void Draw(bool force_update)
 	{
 		const uint8 *src   = (uint8 *)this->pixel_buffer;
-		uint8 *dst         = (uint8 *)this->screen_buffer;
-		uint pitch         = this->screen_pitch;
-		uint width         = this->display_width;
+		uint8 *dst         = (uint8 *)this->window_buffer;
+		uint pitch         = this->window_pitch;
+		uint width         = this->device_width;
 		uint num_dirty     = this->num_dirty_rects;
-		uint bytesperpixel = this->display_depth / 8;
+		uint bytesperpixel = this->device_depth / 8;
 
 		/* Check if we need to do anything */
 		if (num_dirty == 0) return;
@@ -442,8 +459,8 @@ public:
 			num_dirty = 1;
 			this->dirty_rects[0].left   = 0;
 			this->dirty_rects[0].top    = 0;
-			this->dirty_rects[0].right  = this->display_width;
-			this->dirty_rects[0].bottom = this->display_height;
+			this->dirty_rects[0].right  = this->device_width;
+			this->dirty_rects[0].bottom = this->device_height;
 		}
 
 		WaitForVerticalBlank();
@@ -475,7 +492,7 @@ public:
 
 	virtual void UpdatePalette(uint first_color, uint num_colors)
 	{
-		if (this->display_depth != 8) return;
+		if (this->device_depth != 8) return;
 
 		for (uint32_t index = first_color; index < first_color + num_colors; index++) {
 			/* Clamp colors between 0.0 and 1.0 */
@@ -492,13 +509,13 @@ public:
 
 	virtual uint ListModes(OTTD_Point *modes, uint max_modes)
 	{
-		return QZ_ListModes(modes, max_modes, this->display_id, this->display_depth);
+		return QZ_ListModes(modes, max_modes, this->display_id, this->device_depth);
 	}
 
 	virtual bool ChangeResolution(int w, int h)
 	{
-		int old_width  = this->display_width;
-		int old_height = this->display_height;
+		int old_width  = this->device_width;
+		int old_height = this->device_height;
 
 		if (SetVideoMode(w, h)) return true;
 
@@ -514,12 +531,12 @@ public:
 
 	virtual int GetWidth()
 	{
-		return this->display_width;
+		return this->device_width;
 	}
 
 	virtual int GetHeight()
 	{
-		return this->display_height;
+		return this->device_height;
 	}
 
 	virtual void *GetPixelBuffer()
@@ -539,14 +556,14 @@ public:
 	virtual NSPoint GetMouseLocation(NSEvent *event)
 	{
 		NSPoint pt = [ NSEvent mouseLocation ];
-		pt.y = this->display_height - pt.y;
+		pt.y = this->device_height - pt.y;
 
 		return pt;
 	}
 
 	virtual bool MouseIsInsideView(NSPoint *pt)
 	{
-		return pt->x >= 0 && pt->y >= 0 && pt->x < this->display_width && pt->y < this->display_height;
+		return pt->x >= 0 && pt->y >= 0 && pt->x < this->device_width && pt->y < this->device_height;
 	}
 
 	virtual bool IsActive()
@@ -557,6 +574,15 @@ public:
 
 CocoaSubdriver *QZ_CreateFullscreenSubdriver(int width, int height, int bpp)
 {
+	/* OSX 10.7 doesn't support this way of the fullscreen driver. If we end up here
+	 * OpenTTD was compiled without SDK 10.7 available and - and thus we don't support
+	 * fullscreen mode in OSX 10.7 or higher, as necessary elements for this way have
+	 * been removed from the API.
+	 */
+	if (MacOSVersionIsAtLeast(10, 7, 0)) {
+		return NULL;
+	}
+
 	FullscreenSubdriver *ret = new FullscreenSubdriver(bpp);
 
 	if (!ret->ChangeResolution(width, height)) {
