@@ -1,4 +1,4 @@
-/* $Id: wnd_quartz.mm 19084 2010-02-10 17:37:47Z smatz $ */
+/* $Id: wnd_quartz.mm 22992 2011-10-04 20:12:02Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -33,7 +33,6 @@
 #include "cocoa_v.h"
 #include "../../core/math_func.hpp"
 #include "../../gfx_func.h"
-#include "../../functions.h"
 
 /**
  * Important notice regarding all modifications!!!!!!!
@@ -45,76 +44,14 @@
 
 class WindowQuartzSubdriver;
 
-
-/* Subclass of NSWindow to fix genie effect and support resize events  */
-@interface OTTD_QuartzWindow : NSWindow {
-	WindowQuartzSubdriver *driver;
-}
-
+/* Subclass of OTTD_CocoaView to fix Quartz rendering */
+@interface OTTD_QuartzView : OTTD_CocoaView
 - (void)setDriver:(WindowQuartzSubdriver*)drv;
-
-- (void)miniaturize:(id)sender;
-- (void)display;
-- (void)setFrame:(NSRect)frameRect display:(BOOL)flag;
-- (void)appDidHide:(NSNotification*)note;
-- (void)appWillUnhide:(NSNotification*)note;
-- (void)appDidUnhide:(NSNotification*)note;
-- (id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)styleMask backing:(NSBackingStoreType)backingType defer:(BOOL)flag;
-@end
-
-/* Delegate for our NSWindow to send ask for quit on close */
-@interface OTTD_QuartzWindowDelegate : NSObject {
-	WindowQuartzSubdriver *driver;
-}
-
-- (void)setDriver:(WindowQuartzSubdriver*)drv;
-
-- (BOOL)windowShouldClose:(id)sender;
-@end
-
-/* Subclass of NSView to fix Quartz rendering */
-@interface OTTD_QuartzView : NSView {
-	WindowQuartzSubdriver *driver;
-}
-
-- (void)setDriver:(WindowQuartzSubdriver*)drv;
-
-- (void)drawRect:(NSRect)rect;
-- (BOOL)isOpaque;
+- (void)drawRect:(NSRect)invalidRect;
 @end
 
 class WindowQuartzSubdriver: public CocoaSubdriver {
-	int device_width;
-	int device_height;
-
-	int window_width;
-	int window_height;
-
-	int buffer_depth;
-
-	void *pixel_buffer;
-	void *image_buffer;
-
-	OTTD_QuartzWindow *window;
-
-	#define MAX_DIRTY_RECTS 100
-	Rect dirty_rects[MAX_DIRTY_RECTS];
-	int num_dirty_rects;
-
-	uint32 palette[256];
-
-public:
-	bool active;
-	bool setup;
-
-	OTTD_QuartzView *qzview;
-	CGContextRef cgcontext;
-
 private:
-	void GetDeviceInfo();
-
-	bool SetVideoMode(int width, int height);
-
 	/**
 	 * This function copies 8bpp pixels from the screen buffer in 32bpp windowed mode.
 	 *
@@ -125,6 +62,9 @@ private:
 	 */
 	void BlitIndexedToView32(int left, int top, int right, int bottom);
 
+	virtual void GetDeviceInfo();
+	virtual bool SetVideoMode(int width, int height);
+
 public:
 	WindowQuartzSubdriver(int bpp);
 	virtual ~WindowQuartzSubdriver();
@@ -138,10 +78,11 @@ public:
 	virtual bool ChangeResolution(int w, int h);
 
 	virtual bool IsFullscreen() { return false; }
+	virtual bool ToggleFullscreen(); /* Full screen mode on OSX 10.7 */
 
 	virtual int GetWidth() { return window_width; }
 	virtual int GetHeight() { return window_height; }
-	virtual void *GetPixelBuffer() { return buffer_depth == 8 ? pixel_buffer : image_buffer; }
+	virtual void *GetPixelBuffer() { return buffer_depth == 8 ? pixel_buffer : window_buffer; }
 
 	/* Convert local coordinate to window server (CoreGraphics) coordinate */
 	virtual CGPoint PrivateLocalToCG(NSPoint *p);
@@ -178,147 +119,12 @@ static CGColorSpaceRef QZ_GetCorrectColorSpace()
 }
 
 
-@implementation OTTD_QuartzWindow
-
-- (void)setDriver:(WindowQuartzSubdriver*)drv
-{
-	driver = drv;
-}
-
-
-/* we override these methods to fix the miniaturize animation/dock icon bug */
-- (void)miniaturize:(id)sender
-{
-	/* make the alpha channel opaque so anim won't have holes in it */
-	driver->SetPortAlphaOpaque ();
-
-	/* window is hidden now */
-	driver->active = false;
-
-	QZ_ShowMouse();
-
-	[ super miniaturize:sender ];
-}
-
-- (void)display
-{
-	/* This method fires just before the window deminaturizes from the Dock.
-	 * We'll save the current visible surface, let the window manager redraw any
-	 * UI elements, and restore the surface. This way, no expose event
-	 * is required, and the deminiaturize works perfectly.
-	 */
-
-	driver->SetPortAlphaOpaque();
-
-	/* save current visible surface */
-	[ self cacheImageInRect:[ driver->qzview frame ] ];
-
-	/* let the window manager redraw controls, border, etc */
-	[ super display ];
-
-	/* restore visible surface */
-	[ self restoreCachedImage ];
-
-	/* window is visible again */
-	driver->active = true;
-}
-
-- (void)setFrame:(NSRect)frameRect display:(BOOL)flag
-{
-	[ super setFrame:frameRect display:flag ];
-
-	/* Don't do anything if the window is currently being created */
-	if (driver->setup) return;
-
-	if (!driver->WindowResized()) error("Cocoa: Failed to resize window.");
-}
-
-- (void)appDidHide:(NSNotification*)note
-{
-	driver->active = false;
-}
-
-
-- (void)appWillUnhide:(NSNotification*)note
-{
-	driver->SetPortAlphaOpaque ();
-
-	/* save current visible surface */
-	[ self cacheImageInRect:[ driver->qzview frame ] ];
-}
-
-- (void)appDidUnhide:(NSNotification*)note
-{
-	/* restore cached image, since it may not be current, post expose event too */
-	[ self restoreCachedImage ];
-
-	driver->active = true;
-}
-
-
-- (id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)styleMask backing:(NSBackingStoreType)backingType defer:(BOOL)flag
-{
-	/* Make our window subclass receive these application notifications */
-	[ [ NSNotificationCenter defaultCenter ] addObserver:self selector:@selector(appDidHide:) name:NSApplicationDidHideNotification object:NSApp ];
-
-	[ [ NSNotificationCenter defaultCenter ] addObserver:self selector:@selector(appDidUnhide:) name:NSApplicationDidUnhideNotification object:NSApp ];
-
-	[ [ NSNotificationCenter defaultCenter ] addObserver:self selector:@selector(appWillUnhide:) name:NSApplicationWillUnhideNotification object:NSApp ];
-
-	return [ super initWithContentRect:contentRect styleMask:styleMask backing:backingType defer:flag ];
-}
-
-@end
-
-@implementation OTTD_QuartzWindowDelegate
-
-- (void)setDriver:(WindowQuartzSubdriver*)drv
-{
-	driver = drv;
-}
-
-- (BOOL)windowShouldClose:(id)sender
-{
-	HandleExitGameRequest();
-
-	return NO;
-}
-
-- (void)windowDidBecomeKey:(NSNotification*)aNotification
-{
-	driver->active = true;
-}
-
-- (void)windowDidResignKey:(NSNotification*)aNotification
-{
-	driver->active = false;
-}
-
-- (void)windowDidBecomeMain:(NSNotification*)aNotification
-{
-	driver->active = true;
-}
-
-- (void)windowDidResignMain:(NSNotification*)aNotification
-{
-	driver->active = false;
-}
-
-@end
-
 @implementation OTTD_QuartzView
 
 - (void)setDriver:(WindowQuartzSubdriver*)drv
 {
 	driver = drv;
 }
-
-
-- (BOOL)isOpaque
-{
-	return YES;
-}
-
 - (void)drawRect:(NSRect)invalidRect
 {
 	if (driver->cgcontext == NULL) return;
@@ -400,10 +206,14 @@ static CGColorSpaceRef QZ_GetCorrectColorSpace()
 
 void WindowQuartzSubdriver::GetDeviceInfo()
 {
-	/* Initialize the video settings; this data persists between mode switches */
+	/* Initialize the video settings; this data persists between mode switches
+	 * and gather some information that is useful to know about the display */
+
+#	if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_6
+	/* This way is deprecated as of OSX 10.6 but continues to work.Thus use it
+	 * always, unless allowed to skip compatibility with 10.5 and earlier */
 	CFDictionaryRef cur_mode = CGDisplayCurrentMode(kCGDirectMainDisplay);
 
-	/* Gather some information that is useful to know about the display */
 	CFNumberGetValue(
 		(const __CFNumber*)CFDictionaryGetValue(cur_mode, kCGDisplayWidth),
 		kCFNumberSInt32Type, &this->device_width
@@ -413,6 +223,29 @@ void WindowQuartzSubdriver::GetDeviceInfo()
 		(const __CFNumber*)CFDictionaryGetValue(cur_mode, kCGDisplayHeight),
 		kCFNumberSInt32Type, &this->device_height
 	);
+#	else
+	/* Use the new API when compiling for OSX 10.6 or later */
+	CGDisplayModeRef cur_mode = CGDisplayCopyDisplayMode(kCGDirectMainDisplay);
+	if (cur_mode == NULL) { return; }
+
+	this->device_width = CGDisplayModeGetWidth(cur_mode);
+	this->device_height = CGDisplayModeGetHeight(cur_mode);
+
+	CGDisplayModeRelease(cur_mode);
+#	endif
+}
+
+/** Switch to full screen mode on OSX 10.7
+ * @return Whether we switched to full screen
+ */
+bool WindowQuartzSubdriver::ToggleFullscreen()
+{
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7)
+	[this->window toggleFullScreen:this->window];
+	return true;
+#else
+	return false;
+#endif
 }
 
 bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
@@ -427,7 +260,7 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
 
 	/* Check if we should recreate the window */
 	if (this->window == nil) {
-		OTTD_QuartzWindowDelegate *delegate;
+		OTTD_CocoaWindowDelegate *delegate;
 
 		/* Set the window style */
 		unsigned int style = NSTitledWindowMask;
@@ -435,7 +268,7 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
 		style |= NSResizableWindowMask;
 
 		/* Manually create a window, avoids having a nib file resource */
-		this->window = [ [ OTTD_QuartzWindow alloc ]
+		this->window = [ [ OTTD_CocoaWindow alloc ]
 							initWithContentRect:contentRect
 							styleMask:style
 							backing:NSBackingStoreBuffered
@@ -447,6 +280,29 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
 			return false;
 		}
 
+		/* Add built in full-screen support when available (OS X 10.7 and higher)
+		 * This code actually compiles for 10.5 and later, but only makes sense in conjunction
+		 * with the quartz fullscreen support as found only in 10.7 and later
+		 */
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7)
+		if ([this->window respondsToSelector:@selector(toggleFullScreen:)]) {
+			/* Constants needed to build on pre-10.7 systems. Source: NSWindow documentation. */
+			const int NSWindowCollectionBehaviorFullScreenPrimary = 1 << 7;
+			const int NSWindowFullScreenButton = 7;
+
+			NSWindowCollectionBehavior behavior = [this->window collectionBehavior];
+			behavior |= NSWindowCollectionBehaviorFullScreenPrimary;
+			[window setCollectionBehavior:behavior];
+
+			NSButton* fullscreenButton =
+			[this->window standardWindowButton:NSWindowFullScreenButton];
+			[fullscreenButton setAction:@selector(toggleFullScreen:)];
+			[fullscreenButton setTarget:this->window];
+
+			[this->window setCollectionBehavior: NSWindowCollectionBehaviorFullScreenPrimary];
+		}
+#endif
+
 		[ this->window setDriver:this ];
 
 		char caption[50];
@@ -456,24 +312,26 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
 		[ this->window setMiniwindowTitle:nsscaption ];
 		[ nsscaption release ];
 
+		[ this->window setContentMinSize:NSMakeSize(64.0f, 64.0f) ];
+
 		[ this->window setAcceptsMouseMovedEvents:YES ];
 		[ this->window setViewsNeedDisplay:NO ];
 
 		[ this->window useOptimizedDrawing:YES ];
 
-		delegate = [ [ OTTD_QuartzWindowDelegate alloc ] init ];
+		delegate = [ [ OTTD_CocoaWindowDelegate alloc ] init ];
 		[ delegate setDriver:this ];
 		[ this->window setDelegate:[ delegate autorelease ] ];
 	} else {
 		/* We already have a window, just change its size */
 		[ this->window setContentSize:contentRect.size ];
 
-		// Ensure frame height - title bar height >= view height
+		/* Ensure frame height - title bar height >= view height */
 		contentRect.size.height = Clamp(height, 0, [ this->window frame ].size.height - 22 /* 22 is the height of title bar of window*/);
 
-		if (this->qzview != nil) {
+		if (this->cocoaview != nil) {
 			height = contentRect.size.height;
-			[ this->qzview setFrameSize:contentRect.size ];
+			[ this->cocoaview setFrameSize:contentRect.size ];
 		}
 	}
 
@@ -483,19 +341,19 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
 	[ this->window center ];
 
 	/* Only recreate the view if it doesn't already exist */
-	if (this->qzview == nil) {
-		this->qzview = [ [ OTTD_QuartzView alloc ] initWithFrame:contentRect ];
-		if (this->qzview == nil) {
-			DEBUG(driver, 0, "Could not create the Quickdraw view.");
+	if (this->cocoaview == nil) {
+		this->cocoaview = [ [ OTTD_QuartzView alloc ] initWithFrame:contentRect ];
+		if (this->cocoaview == nil) {
+			DEBUG(driver, 0, "Could not create the Quartz view.");
 			this->setup = false;
 			return false;
 		}
 
-		[ this->qzview setDriver:this ];
+		[ this->cocoaview setDriver:this ];
 
-		[ this->qzview setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable ];
-		[ this->window setContentView:qzview ];
-		[ this->qzview release ];
+		[ (NSView*)this->cocoaview setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable ];
+		[ this->window setContentView:cocoaview ];
+		[ this->cocoaview release ];
 		[ this->window makeKeyAndOrderFront:nil ];
 	}
 
@@ -511,7 +369,7 @@ void WindowQuartzSubdriver::BlitIndexedToView32(int left, int top, int right, in
 {
 	const uint32 *pal   = this->palette;
 	const uint8  *src   = (uint8*)this->pixel_buffer;
-	uint32       *dst   = (uint32*)this->image_buffer;
+	uint32       *dst   = (uint32*)this->window_buffer;
 	uint          width = this->window_width;
 	uint          pitch = this->window_width;
 
@@ -528,13 +386,13 @@ WindowQuartzSubdriver::WindowQuartzSubdriver(int bpp)
 	this->window_width  = 0;
 	this->window_height = 0;
 	this->buffer_depth  = bpp;
-	this->image_buffer  = NULL;
+	this->window_buffer  = NULL;
 	this->pixel_buffer  = NULL;
 	this->active        = false;
 	this->setup         = false;
 
 	this->window = nil;
-	this->qzview = nil;
+	this->cocoaview = nil;
 
 	this->cgcontext = NULL;
 
@@ -543,14 +401,12 @@ WindowQuartzSubdriver::WindowQuartzSubdriver(int bpp)
 
 WindowQuartzSubdriver::~WindowQuartzSubdriver()
 {
-	QZ_ShowMouse();
-
 	/* Release window mode resources */
 	if (this->window != nil) [ this->window close ];
 
 	CGContextRelease(this->cgcontext);
 
-	free(this->image_buffer);
+	free(this->window_buffer);
 	free(this->pixel_buffer);
 }
 
@@ -587,12 +443,10 @@ void WindowQuartzSubdriver::Draw(bool force_update)
 
 		/* Normally drawRect will be automatically called by Mac OS X during next update cycle,
 		 * and then blitting will occur. If force_update is true, it will be done right now. */
-		[ this->qzview setNeedsDisplayInRect:dirtyrect ];
-		if (force_update) [ this->qzview displayIfNeeded ];
+		[ this->cocoaview setNeedsDisplayInRect:dirtyrect ];
+		if (force_update) [ this->cocoaview displayIfNeeded ];
 	}
 
-	//DrawResizeIcon();
-
 	this->num_dirty_rects = 0;
 }
 
@@ -643,7 +497,7 @@ CGPoint WindowQuartzSubdriver::PrivateLocalToCG(NSPoint *p)
 {
 
 	p->y = this->window_height - p->y;
-	*p = [ this->qzview convertPoint:*p toView:nil ];
+	*p = [ this->cocoaview convertPoint:*p toView:nil ];
 
 	*p = [ this->window convertBaseToScreen:*p ];
 	p->y = this->device_height - p->y;
@@ -658,7 +512,7 @@ CGPoint WindowQuartzSubdriver::PrivateLocalToCG(NSPoint *p)
 NSPoint WindowQuartzSubdriver::GetMouseLocation(NSEvent *event)
 {
 	NSPoint pt = [ event locationInWindow ];
-	pt = [ this->qzview convertPoint:pt fromView:nil ];
+	pt = [ this->cocoaview convertPoint:pt fromView:nil ];
 
 	pt.y = this->window_height - pt.y;
 
@@ -667,7 +521,7 @@ NSPoint WindowQuartzSubdriver::GetMouseLocation(NSEvent *event)
 
 bool WindowQuartzSubdriver::MouseIsInsideView(NSPoint *pt)
 {
-	return [ qzview mouse:*pt inRect:[ this->qzview bounds ] ];
+	return [ cocoaview mouse:*pt inRect:[ this->cocoaview bounds ] ];
 }
 
 
@@ -677,7 +531,7 @@ bool WindowQuartzSubdriver::MouseIsInsideView(NSPoint *pt)
  */
 void WindowQuartzSubdriver::SetPortAlphaOpaque()
 {
-	uint32 *pixels = (uint32*)this->image_buffer;
+	uint32 *pixels = (uint32*)this->window_buffer;
 	uint32  pitch  = this->window_width;
 
 	for (int y = 0; y < this->window_height; y++)
@@ -688,20 +542,20 @@ void WindowQuartzSubdriver::SetPortAlphaOpaque()
 
 bool WindowQuartzSubdriver::WindowResized()
 {
-	if (this->window == nil || this->qzview == nil) return true;
+	if (this->window == nil || this->cocoaview == nil) return true;
 
-	NSRect newframe = [ this->qzview frame ];
+	NSRect newframe = [ this->cocoaview frame ];
 
 	this->window_width = newframe.size.width;
 	this->window_height = newframe.size.height;
 
 	/* Create Core Graphics Context */
-	free(this->image_buffer);
-	this->image_buffer = (uint32*)malloc(this->window_width * this->window_height * 4);
+	free(this->window_buffer);
+	this->window_buffer = (uint32*)malloc(this->window_width * this->window_height * 4);
 
 	CGContextRelease(this->cgcontext);
 	this->cgcontext = CGBitmapContextCreate(
-		this->image_buffer,        // data
+		this->window_buffer,        // data
 		this->window_width,        // width
 		this->window_height,       // height
 		8,                         // bits per component
