@@ -1,4 +1,4 @@
-/* $Id: water_cmd.cpp 22653 2011-07-10 20:29:19Z rubidium $ */
+/* $Id: water_cmd.cpp 23403 2011-12-03 21:31:44Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -205,12 +205,17 @@ static CommandCost DoBuildLock(TileIndex tile, DiagDirection dir, DoCommandFlag
 {
 	CommandCost cost(EXPENSES_CONSTRUCTION);
 
+	int delta = TileOffsByDiagDir(dir);
+	CommandCost ret = EnsureNoVehicleOnGround(tile);
+	if (ret.Succeeded()) ret = EnsureNoVehicleOnGround(tile + delta);
+	if (ret.Succeeded()) ret = EnsureNoVehicleOnGround(tile - delta);
+	if (ret.Failed()) return ret;
+
 	/* middle tile */
-	CommandCost ret = DoCommand(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
+	ret = DoCommand(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 	if (ret.Failed()) return ret;
 	cost.AddCost(ret);
 
-	int delta = TileOffsByDiagDir(dir);
 	/* lower tile */
 	WaterClass wc_lower = IsWaterTile(tile - delta) ? GetWaterClass(tile - delta) : WATER_CLASS_CANAL;
 
@@ -647,7 +652,8 @@ static void DrawWaterTileStruct(const TileInfo *ti, const WaterDrawTileStruct *w
 /** Draw a lock tile. */
 static void DrawWaterLock(const TileInfo *ti)
 {
-	const WaterDrawTileStruct *wdts = _lock_display_seq[GetSection(ti->tile)];
+	int section = GetSection(ti->tile);
+	const WaterDrawTileStruct *wdts = _lock_display_seq[section];
 
 	/* Draw ground sprite. */
 	SpriteID image = wdts++->image;
@@ -675,7 +681,8 @@ static void DrawWaterLock(const TileInfo *ti)
 	if (base == 0) {
 		/* If no custom graphics, use defaults. */
 		base = SPR_LOCK_BASE;
-		zoffs = ti->z > wdts[3].delta_y ? 24 : 0;
+		uint8 z_threshold = section >= 8 ? 8 : 0;
+		zoffs = ti->z > z_threshold ? 24 : 0;
 	}
 
 	DrawWaterTileStruct(ti, wdts, base, zoffs, PAL_NONE, CF_LOCKS);
